// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda;

import org.isam.util.xml.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;

public class ProfileManager
{
    public Enumeration
    getBootstrapServices( String profileName )
        {
            Profile p = (Profile) profiles.get(profileName);

            return new StringTokenizer(
                (p==null) ? "" : p.getBootstrapServices(),
                ",",
                false);
        }
    
    public String
    getServiceProperty( String profileName, String serviceName, String propName )
        {
            Profile p = (Profile) profiles.get(profileName);

            return ( p == null ) ? null : p.getServiceProperty(serviceName, propName);
        }
    
    public String
    getServiceProperty( String serviceName, String propName )
        {
            String activeProfileName = Exehda.getActiveProfile();

            return getServiceProperty(activeProfileName, serviceName, propName);
        }

    public String
    getProperty( String profileName, String propName )
        {
            Profile p = (Profile) profiles.get(profileName);
            return ( p == null ) ? null : p.getGlobalProperty( propName);
        }
    
    public String
    getProperty( String propName )
        {
            String activeProfileName = Exehda.getActiveProfile();

            return getProperty(activeProfileName, propName);
        }

    public void
    addProfileData(XmlElement profileData)
        {
            if ( !"PROFILE".equals(profileData.getTag()) ) return;

            Profile profile = new Profile(profileData.getAttribute("name", "noname"));
            
                // for each profile inner element
            Enumeration profEnum = profileData.getChildren();
            while ( profEnum.hasMoreElements() ) {
                XmlElement elem = (XmlElement) profEnum.nextElement();

                if ( "SERVICE".equals(elem.getTag()) ) {
                        // service definition
                    XmlElement service = elem;
                    String serviceName = service.getAttribute("name", null);
                    
                    profile.setServiceProperty(serviceName,
                                               "impl",
                                               service.getAttribute("impl", "none"));
                    
                    profile.setServiceProperty(serviceName,
                                               "loadPolicy",
                                               service.getAttribute("loadPolicy", "none"));
                    
                        // for each <PROP> element
                    Enumeration propEnum = service.getChildren();
                    while ( propEnum.hasMoreElements() ) {
                        XmlElement prop = (XmlElement) propEnum.nextElement();

                        if ( !"PROP".equals(prop.getTag()) ) continue;

                        profile.setServiceProperty(serviceName,
                                                   prop.getAttribute("name", ""),
                                                   prop.getAttribute("value", ""));
                    } // for each property

                    String loadPolicy = profile.getServiceProperty(serviceName, "loadPolicy");
                    
                    if ( "boot".equals(loadPolicy)) {
                        profile.addBootstrapService(serviceName);
                    }
                }
                else if ("PROP".equals(elem.getTag())) {
                        // profile global property definition
                    profile.setGlobalProperty(elem.getAttribute("name", ""),
                                              elem.getAttribute("value", ""));
                }
                         
            } // for each profile element


            profiles.put( profile.getProperty("name"), profile);
        }

    public void
    removeProfileData( String profileName )
        {
            profiles.remove(profileName);
        }

    private Hashtable profiles = new Hashtable();
}

class Profile
    extends java.util.Properties
{
    public Profile(String name)
        {
            put("name", name);
            put("bootstrapServices","");
        }

    public void
    setGlobalProperty(String propName, String value)
        {
             put("_global."+propName, value);
        }

    public String
    getGlobalProperty( String propName )
        {
            return getProperty("_global."+propName);
        }

    public void
    setServiceProperty(String serviceName, String propName, String value)
        {
            put("_services."+serviceName+"."+propName, value);
        }

    public String
    getServiceProperty(String serviceName, String propName)
        {
            return getProperty("_services."+serviceName+"."+propName);
        }

    public void
    addBootstrapService( String serviceName )
        {
            String bss = getBootstrapServices();

            if ( bss.indexOf(serviceName) > 0 )
                return;
            else {
                bss = "".equals(bss) ? serviceName : (bss + "," + serviceName);
                put("bootstrapServices", bss);
            }
        }

    public String
    getBootstrapServices()
        {
            return getProperty("bootstrapServices");
        }
}
