// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.CellId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.ResourceTypeConstraint;
import org.isam.exehda.AttributeSimpleMatchConstraint;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Worb;

import org.isam.exehda.services.cib.backend.CibBackEnd;
import org.isam.exehda.services.cib.backend.CibBackEndException;
import org.isam.exehda.services.cib.backend.CibBackEndConf;


import java.util.*;


/**
 * Cell Information Base Front-end (general implementation).
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-07-28 17:14:34 -0300 (Thu, 28 Jul 2005) $ $Revision: 1034 $
 */
public class CibImpl
    extends AbstractService
    implements CellInformationBase, CibBackEndConf
{
    final static CellId LOCAL_CELL = HostId.getLocalHost().getCell();
    final static NameSpace NS_LOCAL_CELL = LOCAL_CELL.toNameSpace();
    
    final static String DFLT_CIB_BACKEND_IMPL = "org.isam.exehda.services.cib.backend.ldap.LdapBackEnd";
    final static String DFLT_CIB_NEIGHBORHOOD = "";
    
    final static String PROP_CIB_BACKEND_PREFIX = "backEnd.";
    final static String PROP_CIB_BACKEND_IMPL = "backEnd.impl";
    
    final static String PROP_CIB_PASSWORD = "password";
    final static String PROP_CIB_NEIGHBORHOOD = "neighborhood";
    

    
    public void
    start()
        throws Exception
        {
            try {
                // storage back-end
            String backEndImpl = Exehda.getServiceProperty(
                CellInformationBase.SERVICE_NAME,
                PROP_CIB_BACKEND_IMPL,
                DFLT_CIB_BACKEND_IMPL);            

            try {
                    // load and initialize selected back-end
                this.backEnd = (CibBackEnd) Class.forName(backEndImpl).newInstance();
                this.backEnd.init(this);
            }
            catch ( Exception e ) {
                log(Sev.CRITICAL, "start()", e);
                throw new NullPointerException("Failed to initilize the storage backend: "+e);
            }

                // initialize static neighborhood
            initNeighborhood( Exehda.getServiceProperty( CellInformationBase.SERVICE_NAME,
                                                         PROP_CIB_NEIGHBORHOOD,
                                                         DFLT_CIB_NEIGHBORHOOD));

            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            worb.exportService(this,
                               CellInformationBase.class,
                               CellInformationBase.SERVICE_NAME);
            }
            catch (Exception e) { e.printStackTrace(); throw e; } 
        }

    public void
    stop()
        {
                //
                // TODO: unexport service
                //
        }

    public String
    getProperty(String p, String defVal)
        {
            return Exehda.getServiceProperty( CellInformationBase.SERVICE_NAME,
                                              PROP_CIB_BACKEND_PREFIX + p,
                                              defVal);
        }

//     public ResourceName[]
//     select(ResourceConstraint sc, int maxResults)
//         {
//             log(Sev.DEBUG, "select", "selecting resources that match constraints={0}", sc);
//             return backEnd.select(sc, null, maxResults);
//         }
    
//     private ResourceName[]
//     select(ResourceConstraint sc, NameSpace[] ns, int maxResults)
//         {
//             log(Sev.DEBUG, "select", "selecting resources that match constraints={0}", sc);
//             int totalResults = 0;
//             List resultList = new LinkedList();
//             for(int i = 0; i < ns.length; i++){
//                 if((getCell(ns[i]).equals(cellName))||(getCell(ns[i]) == null)){
// 		   // FIX ME: should forward ns[i] to backend
//                    ResourceName[] rs = backEnd.select(sc,null,maxResults);
//                    resultList.add(rs);
//                    totalResults += rs.length;
//                 }
//                 else{
//                     Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
//                     String contactAddress = findCellContactAddress(getCell(ns[i]));
//                     if(contactAddress == null){
//                         return(null);
//                     }
//                     CellInformationBase cibCell = (CellInformationBase)
//                     worb.lookupService(contactAddress, CellInformationBase.class);
// 		    // FIX ME: should forward ns[i] to remote cell
//                     ResourceName[] rs = cibCell.select(sc,maxResults);
//                     resultList.add(rs);
//                     totalResults += rs.length;
//                 }
//             }
//             ResourceName[] results;
//             if((totalResults > maxResults)&&(maxResults != -1)){
//                 totalResults = maxResults;
//             }
//             results = new ResourceName[totalResults];
            
//             for(int i = 0; i < totalResults;){
//                 Iterator it = resultList.iterator();
//                 ResourceName[] rs = (ResourceName[])it.next();
//                 for(int j = 0;((j<rs.length)&&(i<totalResults));i++){
//                     results[i] = rs[j];
//                 }
//             }
         
//             return (results);
//         }

    public ResourceName[]
    select(ResourceConstraint sc, NameSpace ns, int maxResults) 
        {
            try {
                    // defaults to the scope of the local cell
                if ( ns == null ) ns = NS_LOCAL_CELL;

                    // check if the target namespace within the scope of the local cell
                if ( isLocalNameSpace(ns) ) {
                        // forward the request to the backend
                    return backEnd.select(sc, ns, maxResults);
                }
                    // otherwise (target namespace not within the scope of local cell)
                else {
                        // lookup the CIB that handles the namespace where the resource is defined
                    CellInformationBase remCib = lookupRemoteCib(ns);
                        // forward the request to that CIB
                    return remCib.select(sc, ns, maxResults);
                }
             }
            catch ( CibBackEndException cbe ) {
                log (Sev.ERROR, "select()", cbe);
                return null;
            }
        }
    
    
    public ResourceName[]
    selectByName(String n, NameSpace ns, int maxResults)
        {
            log(Sev.DEBUG, "selectByName()",
                "selecting resources with simple name={0} in name space={1}", n, ns);

                //
                // XXX: hardcoded attribute name "ID"
                //
            return select( new AttributeSimpleMatchConstraint("ID",n), ns, maxResults );
        }
    
    public ResourceName[]
    selectByType(String t, NameSpace ns, int maxResults)
        {
            log(Sev.DEBUG, "selectByType",
                "selecting resource of type={0} in name space={1}", t, ns);

            return select( new ResourceTypeConstraint(t), ns, maxResults);
        }
    
    public String
    getAttribute(ResourceName r, String a)
        {
            log(Sev.TRACE, "getAttribute", "attr={0}, resource is {1}", a, r);
            
            try {                
                    // if the resource belongs to the local cell
                if ( isLocalResource(r) ) {
                        // access the local backend
                    return backEnd.getAttribute(r,a);
                }
                    // resource doesn't belong to the local cell
                else {
                        // lookup the CIB that handles the given resource
                    CellInformationBase remCib = lookupRemoteCib(r);
                        // forward the request to that CIB
                    return remCib.getAttribute(r, a);
                }
            }
            catch ( CibBackEndException cbe ) {
                    //
                    // TO DO: better exception handling; some exceptions should be
                    // propagated to the caller
                    //
                log (Sev.ERROR, "getAttribute()", cbe);
                return null;
            }
        }
    
    public String
    setAttribute(ResourceName r, String a, String v)
        {
            log(Sev.TRACE, "setAttribute", "attr={0}, resource is {1}", a, r);

            try {                
                    // if the resource belongs to the local cell
                if ( isLocalResource(r) ) {
                        // access the local backend
                    return backEnd.setAttribute(r,a,v);
                }
                    // resource doesn't belong to the local cell
                else {
                        // lookup the CIB that handles the given resource
                    CellInformationBase remCib = lookupRemoteCib(r);
                        // forward the request to that CIB
                    return remCib.setAttribute(r,a,v);
                }
            }
            catch ( CibBackEndException cbe ) {
                    //
                    // TO DO: better exception handling; some exceptions should be
                    // propagated to the caller
                    //
                log (Sev.ERROR, "setAttribute()", cbe);
                return null;
            }
        }
    
    public String[]
    getAttributes(ResourceName r)
        {
            log(Sev.DEBUG, "getAttributes()", "resource is {0}", r);

            try {                
                    // if the resource belongs to the local cell
                if ( isLocalResource(r) ) {
                        // access the local backend
                    return backEnd.getAttributes(r);
                }
                    // resource doesn't belong to the local cell
                else {
                        // lookup the CIB that handles the given resource
                    CellInformationBase remCib = lookupRemoteCib(r);
                        // forward the request to that CIB
                    return remCib.getAttributes(r);
                }
            }
            catch ( CibBackEndException cbe ) {
                    //
                    // TO DO: better exception handling; some exceptions should be
                    // propagated to the caller
                    //
                log (Sev.ERROR, "getAttributes", cbe);
                return null;
            }
        }
    
    public void
    addResource(ResourceName r)
        {
            log(Sev.DEBUG, "addResource", "Adding resource {0}", r);

            try {                
                    // if the resource belongs to the local cell
                if ( isLocalResource(r) ) {
                        // access the local backend
                    backEnd.addResource(r);
                }
                    // resource doesn't belong to the local cell
                else {
                        // lookup the CIB that handles the given resource
                    CellInformationBase remCib = lookupRemoteCib(r);
                        // forward the request to that CIB
                    remCib.addResource(r);
                }
            }
            catch ( CibBackEndException cbe ) {
                    //
                    // TO DO: better exception handling; some exceptions should be
                    // propagated to the caller
                    //
                log (Sev.ERROR, "addResource", cbe);
                return;
            }
        }
    
    public void
    removeResource(ResourceName r)
        {
            try {                
                    // if the resource belongs to the local cell
                if ( isLocalResource(r) ) {
                        // access the local backend
                    backEnd.removeResource(r);
                }
                    // resource doesn't belong to the local cell
                else {
                        // lookup the CIB that handles the given resource
                    CellInformationBase remCib = lookupRemoteCib(r);
                        // forward the request to that CIB
                    remCib.removeResource(r);
                }
            }
            catch ( CibBackEndException cbe ) {
                    //
                    // TO DO: better exception handling; some exceptions should be
                    // propagated to the caller
                    //
                log (Sev.ERROR, "removeResource()", cbe);
                return;
            }
        }

        ////////////////////////////////////////////////////////////////////////////
        // PRIVATE STUFF
        ////////////////////////////////////////////////////////////////////////////

        /**
         * Returns the cell information base that currelty handle the given namespace
         *
         * @param ns a <code>NameSpace</code> value
         * @return a <code>CellInformationBase</code> value
         */
    private final CellInformationBase
    lookupRemoteCib(NameSpace ns)
        {
                // backup original parameter
            final NameSpace ons = ns;
            
                // strip inner namespaces leaving only the cell ns
            while ( ! "cell".equals(ns.getType()) ) {
                if ( (ns = ns.getParent()) == null ) {
                        // namespace defined outside any cell
                        //
                        // XXX: by now, abort operation
                        //
                    throw new IllegalArgumentException(
                        "Provided namespace do not include cell information :"+ons);
                }
            }
            
                // check if the cell for the given namespace was already learned
            String ca = (String) neighborByNameSpace.get(ns);
            
            if ( ca == null ) {
                    // not yet known, trigger neighborhood search
                ca = searchNeighboorhood(ns);
            }

            return (CellInformationBase) ((Worb) Exehda.getService(Worb.SERVICE_NAME))
                .lookupService(ca, CellInformationBase.class);
        }

    
        /**
         * Returns the cell information base that currelty handle the given resource
         *
         * @param rn a <code>ResourceName</code> value
         * @return a <code>CellInformationBase</code> value
         */
    private final CellInformationBase
    lookupRemoteCib(ResourceName rn)
        {
            NameSpace ns = rn.getNameSpace();

            if ( (ns == null) && (rn.getType() == "cell") ) {
                ns = new NameSpace(rn.getCommonName(), null);
            }

            if ( ns != null ) {
                return lookupRemoteCib(ns);
            }
            else {
                    // XXX:
                throw new IllegalArgumentException(
                    "Only 'cell' resources are allowed to have no parent name space: "+rn);
            }
        }

    
        /**
         * Returns whether the given resource is handled (catalogued) by the local CIB.
         *
         * @param rn a <code>ResourceName</code> value
         * @return a <code>boolean</code> value
         */
    private final boolean
    isLocalResource(ResourceName r)
        {            
            return ( LOCAL_CELL.equals(r) || isLocalNameSpace(r.getNameSpace()) );
        }


    
        /**
         * Returns whether the given namespace is handled (served) by the local CIB.
         *
         * @param ns a <code>NameSpace</code> value
         * @return a <code>boolean</code> value
         */
    private final boolean
    isLocalNameSpace(NameSpace ns)
        {
            return ( NS_LOCAL_CELL.equals(ns) || NS_LOCAL_CELL.contains(ns) );
        }
        

    
        /**
         * Initialize the cell static neighborhood. If <code>staticNeighboors</code> in a
         * non-empty string it parse as a semi-colon separated list of neighbors
         * (actually, contact addresses of remote CIBs) that must be used as the static
         * neighborhood of this cell. Otherwise, the neighborhood is read from the
         * backend, which is queryied for resources of type <code>neighbor</code>.
         *
         * @param staticNeighbors a semi-colon separated list of contact addresses of remote CIBs
         */
    private final void
    initNeighborhood(String staticNeighbors)
        throws CibBackEndException
        {

            if ( staticNeighbors.startsWith("+;")
                 || staticNeighbors.endsWith(";+") ) {

                    //
                    // TO DO: merge neighborhood lists
                    //
                throw new NullPointerException(
                    "Merging of bootstrap neighborhood lists not yet implemented.");
                
//                     // read neighborhood from the backend
//                 ResourceName[] nrn = backEnd.select(
//                     new ResourceTypeConstraint("neighbor"), NS_LOCAL_CELL, -1);


            }

                // load static neighborhood
            StringTokenizer st = new StringTokenizer(staticNeighbors, ";");

            for (int i=0; st.hasMoreTokens(); i++) {
                String nca = st.nextToken().trim();
                    // if non-empty contact address
                if ( nca.length() > 0 ) {

                        // register a new candidate neighbor
                    candidateNeighborhood.addElement(nca);

                        // register the neighbor in the cib
                    ResourceName nrn = new ResourceName(
                        "neighbor", "static#"+i, new NameSpace(
                            "neighborhood", "static", NS_LOCAL_CELL));

                    try {
                        backEnd.addResource(nrn);
                    }
                    catch (Exception e) {}
                    
                    backEnd.setAttribute(nrn, "contactAddress", nca);
                }
            }
        }

    
        /**
         * Updates the dynamic neighborhood of this cell with a new known neighbor.
         *
         * @param ca contact address of the CIB on the new neighbor.
         *
         * @return true if the cell's neighborhood has been modified, false otherwise
         */
    private final boolean
    registerNeighbor(String ca)
        {
            final String MTD = "registerNeighbor()";
            
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            
            CellInformationBase rcib = (CellInformationBase)
                worb.lookupService(ca, CellInformationBase.class);

            try {
                    // find out which cell is that
                    // XXX: assuming there is just one cell
                ResourceName rcell = rcib.selectByType("cell", null, -1)[0];

                    // XXX: assume cells are the outermost namespace, i.e. they have no
                    // parent namespaces
                NameSpace rns = new NameSpace(rcell.getCommonName(), null);

            
                String oldca = (String) neighborByNameSpace.put(rns, ca);
                if (  oldca == null ) {
                    log(Sev.INFO, MTD,
                        "Registering new neighbor ns={0}, ca={1}", rns, ca);

                        // register neighbor for later exploring
                    activeNeighborhood.addElement(ca);
                    return true;
                }
                else if ( oldca.equals(ca) ) {
                        // do nothing, it is already there
                    return false;
                }
                else {
                        //
                        // FIX ME: concerning to security, perhaps it should never allow to
                        // overwrite CAs
                        //
                
                    log(Sev.WARNING, MTD,
                        "Overwritting neighbor ns={0}, ca={1} with ca={2}", rns, oldca, ca);

                    activeNeighborhood.removeElement(oldca);
                    activeNeighborhood.addElement(ca);
                
                    return true;
                }
            }
            catch (Exception e) {
                log(Sev.WARNING, MTD, "Failed to contact neighbor {0}. Skipping...", ca);
                log(Sev.WARNING, MTD, e);
                return false;
            }
        }

    
        /**
         * Search the neighborhood for a cell that handle the given name space, returning
         * the contact address of the cellular CIB on that cell.
         *
         * @param ns a <code>NameSpace</code> value
         * @return a <code>String</code> value
         */
    private final String
    searchNeighboorhood( NameSpace ns )
        {
            log(Sev.DEBUG, "searchNeighborhood()", "ns={0}", ns);
            
            while ( true ) {
                for (int i=candidateNeighborhood.size()-1; i>=0; i--) {
                    String ca = (String) candidateNeighborhood.remove(i);

                    if ( registerNeighbor(ca) ) {

                        
                            // neighborhood has changed, try ns lookup again
                        ca = (String) neighborByNameSpace.get(ns);
                        if ( ca != null ) {
                                // found, return it
                            return ca;
                        }
                    }
                    
                    log(Sev.DEBUG, "searchNeighborhood()",
                            "ns={0}, neighborhood={1}, ca={2}", ns, neighborByNameSpace, ca);
                }
                    // candidate list is empty, and a suitable cell has not been found
            
                    //
                    // TO DO: build a new candidate list from neighbors of neigbors
                    //
                throw new NullPointerException("Recursive neighborhood search not yet implemented.");
            }
        }

    
    private CibBackEnd backEnd = null;
    
        /**
         * A map from NameSpaces to CellInformationBase contactAddresses
         *
         */
    private Hashtable  neighborByNameSpace = new Hashtable();
    
        /**
         * Optimization for iterating through active (in use) neighboor CAs
         *
         */
    private Vector activeNeighborhood = new Vector();

    
        /**
         * Known to exist but still unexplored neighboors
         *
         */
    private Vector candidateNeighborhood = new Vector();
}
