// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//



package org.isam.exehda.services;

import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.ResourceConstraint;


/**
 * The cell's resource catalog.
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-02-24 02:31:44 -0300 (Thu, 24 Feb 2005) $ $Revision: 586 $
 */
public interface CellInformationBase
{
    public final static String SERVICE_NAME = "cib";

        /**
         * Returns the resources of the local cell that satisfy the given filter. The
         * search is restricted to the given name space and will return at most the
         * number specified of results (notice, it would return less than that).
         *
         * @param filter a <code>ResourceConstraint</code> value
         * @param ns a <code>NameSpace</code> value
         * @param maxResults an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] select( ResourceConstraint filter, NameSpace ns, int maxResults);    
    
        /**
         * Shortcut for selecting resources whose instance indentifier match the given
         * name.
         *
         * @param n a <code>String</code> value
         * @param ns a <code>NameSpace</code> value
         * @param maxResults an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] selectByName(String n, NameSpace ns, int maxResults);

        /**
         * Shortcut for selecting resources of a given type.
         *
         * @param t a <code>String</code> value
         * @param ns a <code>NameSpace</code> value
         * @param maxResults an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] selectByType(String t, NameSpace ns, int maxResults);
    
        /**
         * Queries the value of a resource's attribute.
         *
         * @param r a <code>ResourceName</code> value
         * @param a a <code>String</code> value
         * @return a <code>String</code> value
         */
    public String getAttribute(ResourceName r, String a);
    
        /**
         * Modifies the value of a resource's attribute.
         *
         * @param r a <code>ResourceName</code> value
         * @param a a <code>String</code> value
         * @param v a <code>String</code> value
         * @return a <code>String</code> value
         */
    public String setAttribute(ResourceName r, String a, String v);
    
        /**
         * Queries which attributes are defined for the given resource.
         *
         * @param r a <code>ResourceName</code> value
         * @return a <code>String[]</code> value
         */
    public String[] getAttributes(ResourceName r);
    
        /**
         * Adds a resource to the cell information base catalog.
         *
         * @param r a <code>ResourceName</code> value
         */
    public void addResource(ResourceName r);
    
        /**
         * Deletes a resource from the cell information base catalog.
         *
         * @param r a <code>ResourceName</code> value
         */
    public void removeResource(ResourceName r);
}



