/*
 * DiscovererClientImpl.java
 *
 * Created on February 17, 2005, 10:15 AM
 */

package org.isam.exehda.services.discoverer;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.Discoverer;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ResourceName;

/**
 *
 * @author  frainer
 */
public class DiscovererClientImpl
    extends AbstractService
    implements Discoverer
{
    final static String CIB_ATT_DISCOVERER_CONTACT = "contactAddress";
    
    private Discoverer cellDiscoverer;
    
    /** Creates a new instance of DiscovererClientImpl */
    public DiscovererClientImpl() {
    }
    
    public void start() throws java.lang.Exception {

        ResourceName cdsname = new ResourceName(
            "service",
            Discoverer.SERVICE_NAME,
            HostId.getLocalHost().getCell().toNameSpace());

        String cdsuri = ((CellInformationBase) Exehda.getService(CellInformationBase.SERVICE_NAME))
            .getAttribute(cdsname, CIB_ATT_DISCOVERER_CONTACT);


        if ( cdsuri != null ) {
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            cellDiscoverer = (Discoverer)
                worb.lookupService(cdsuri, Discoverer.class);
        }
        else {
            throw new NullPointerException(
                "Cellular instance of the Discoverer service not found");
        }

    }
    
    public void stop() throws java.lang.Exception {
    }
    
    /**
    * Finds a resource that matches the given query.
    *
    * @param q a <code>Query</code> containing the query
    * @return a array of <code>ResourceDescriptor</code>, representing the matching resources found.
    */
    public ResourceDescriptor[] find(Query q) {
        return(cellDiscoverer.find(q));
    }
    
    /**
    * Registers a new resource that is equivalent to the one
    * described in the xml passed as a parameter.
    *
    * @param xmlDesc a <code>String</code> containing a xml document that describes the attributes of a resource
    * @return a new <code>ResourceName</code> that represents the resource added.
    */
    public ResourceDescriptor register(String xmlDesc) {
        return(cellDiscoverer.register(xmlDesc));
    }
    
    /**
    * Removes a previously added resource.
    *
    * @param r a <code>ResourceDescriptor</code> that represents the resource that is to be removed
    * @return void
    */
    public void remove(ResourceDescriptor r) {
        cellDiscoverer.remove(r);
    }
    
    /**
    * Renews a previously added resource.
    *
    * @param r a <code>ResourceDescriptor</code> that represents the resource that is to be renewed
    * @return void
    */
    public void renew(ResourceDescriptor r) {
        cellDiscoverer.renew(r);
    }    
}
