/*
 * AdvertiseManagerProxy.java
 *
 * Created on 20 de Maio de 2004, 17:56
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.services.DiscovererService;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.Worb;
import org.isam.exehda.Exehda;
import org.isam.util.xml.XmlElement;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.UnknownHostException;
import java.io.StringReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

/**
 *
 * @author  egon
 */
public class AdvertiseManagerProxy {

    private DatagramSocket unicastSocket;
    private String unicastAddress;
    private Integer unicastPort;
    
    private MulticastSocket multicastProxySocket;
    private String multicastProxyAddress;
    private Integer multicastProxyPort;
    
    private String multicastBaseAddress;
    private Integer multicastBasePort;
    
    private Hashtable discovererBaseHst;
    private Object lock;
    
    private Logger logger; 
    private Worb worb;
    
    private AdvertiseListenerProxy advertiseListenerProxy;
    private AckListenerProxy ackListenerProxy;

    
    /**
     * Creates a new instance of AdvertiseManagerProxy
     */
    public AdvertiseManagerProxy(String unicastAddress, Integer unicastPort, String multicastProxyAddress, Integer multicastProxyPort, String multicastBaseAddress, Integer multicastBasePort, Hashtable discovererBaseHst, Object lock) {

        this.logger = (Logger) Exehda.getService(Logger.SERVICE_NAME);
        this.worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);           
        
        this.unicastAddress = unicastAddress;
        this.unicastPort = unicastPort;        
        this.multicastProxyAddress = multicastProxyAddress;
        this.multicastProxyPort = multicastProxyPort;
        this.multicastBaseAddress = multicastBaseAddress;
        this.multicastBasePort = multicastBasePort;
        
        this.discovererBaseHst = discovererBaseHst;
        this.lock = lock;
        
	try {
            //Create unicast socket for sending advertise and receiving acks from Bases
	    this.unicastSocket = new DatagramSocket(this.unicastPort.intValue(), InetAddress.getByName(this.unicastAddress));
            
            //Create multicast socket for receiving advertises from Bases
            InetAddress group = InetAddress.getByName(this.multicastProxyAddress);
            this.multicastProxySocket = new MulticastSocket(this.multicastProxyPort.intValue());
            this.multicastProxySocket.joinGroup(group);
	} catch(UnknownHostException uhe) {
            uhe.printStackTrace();
	    System.exit(1);
	} catch(IOException ioe) {
            ioe.printStackTrace();
	    System.exit(1);
	}
        
        try {
            this.advertise();
        } catch(IOException ioe) {
            ioe.printStackTrace();
	    System.exit(1);
	} 
        
        this.advertiseListenerProxy = new AdvertiseListenerProxy();
        this.advertiseListenerProxy.start();
        
        this.ackListenerProxy = new AckListenerProxy();
        this.ackListenerProxy.start();
    }

    /**
     * advertise
     */
    public void advertise() throws IOException {
                
	try {
            String msg = this.buildAdvertise().toString();
            InetAddress group = InetAddress.getByName(this.multicastBaseAddress);
            DatagramPacket packetSend = new DatagramPacket(msg.getBytes(), msg.length(), group, this.multicastBasePort.intValue());
            this.unicastSocket.send(packetSend);    
        } catch(IOException ioe) {
            ioe.printStackTrace();
       	    throw ioe;
        }        
    }
    
    /**
     * buildAdvertise
     */
    private XmlElement buildAdvertise() {
    
        //create advertise xml document
        XmlElement xmlRootAdvertise = XmlElement.createDocRoot();
        XmlElement xmlAdvertise = XmlElement.createElement("CONTROL");
        
        XmlElement xmlMessage = XmlElement.createElement("MESSAGE");
        XmlElement xmlSource = XmlElement.createElement("SOURCE");
        XmlElement xmlIP = XmlElement.createElement("IP");        
        XmlElement xmlPort = XmlElement.createElement("PORT");
        
        XmlElement xmlMessageValue = XmlElement.createPCData("Advertise");
        XmlElement xmlSourceValue = XmlElement.createPCData("Proxy");
        XmlElement xmlIPValue = XmlElement.createPCData(this.unicastAddress);  
        XmlElement xmlPortValue = XmlElement.createPCData(this.unicastPort.toString());
        
        xmlMessage.addChild(xmlMessageValue);
        xmlSource.addChild(xmlSourceValue);
        xmlIP.addChild(xmlIPValue);
        xmlPort.addChild(xmlPortValue);

        xmlAdvertise.addChild(xmlMessage);
        xmlAdvertise.addChild(xmlSource);
        xmlAdvertise.addChild(xmlIP);
        xmlAdvertise.addChild(xmlPort);

        xmlRootAdvertise.addChild(xmlAdvertise);
        
        return xmlRootAdvertise;
    }
            
    /**
     * AdvertiseListenerProxy
     */
    class AdvertiseListenerProxy extends Thread {

        /**
         * run
         */
        public void run() {
        
            try {
	        while (true) {
                
                    // Wait for advertises from Discoverer Bases
                    byte[] buf = new byte[1000];
                    DatagramPacket packetRecv = new DatagramPacket(buf, buf.length);
                    multicastProxySocket.receive(packetRecv);

                    //get information from the xml received
                    XmlElement xmlReceivedRoot = XmlElement.parse(new StringReader(new String(packetRecv.getData())));

                    Vector vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/MESSAGE", null);
                    XmlElement xmlRecMessage = (XmlElement) vec.firstElement();
                    String message = (String) xmlRecMessage.getContents();              

                    vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/CONTACTADDRESS", null);
                    XmlElement xmlRecContactAddress = (XmlElement) vec.firstElement();
                    String remoteContactAddress = (String) xmlRecContactAddress.getContents();

                    synchronized(lock) {
                        if (discovererBaseHst.get(remoteContactAddress) == null) {

                            DiscovererService discovererTmp = (DiscovererService) worb.lookupService(remoteContactAddress, DiscovererService.class);
                            discovererBaseHst.put(remoteContactAddress, discovererTmp);
                        }
                    }
                }            
            } catch (IOException ioe) {
                ioe.printStackTrace();
                System.exit(1);
            }
        }        
    }
    
    /**
     * AckListenerProxy
     */
    class AckListenerProxy extends Thread {
    
        /**
         * run
         */ 
        public void run() {
        
            try {
	        while (true) {
                
                    // Wait for acks from Discoverer Bases
                    byte[] buf = new byte[1000];
                    DatagramPacket packetRecv = new DatagramPacket(buf, buf.length);
                    unicastSocket.receive(packetRecv);

                    //get information from the xml received
                    XmlElement xmlReceivedRoot = XmlElement.parse(new StringReader(new String(packetRecv.getData())));

                    Vector vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/MESSAGE", null);
                    XmlElement xmlRecMessage = (XmlElement) vec.firstElement();
                    String message = (String) xmlRecMessage.getContents();              

                    vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/CONTACTADDRESS", null);
                    XmlElement xmlRecContactAddress = (XmlElement) vec.firstElement();
                    String remoteContactAddress = (String) xmlRecContactAddress.getContents();

                    synchronized(lock) {
                        if (discovererBaseHst.get(remoteContactAddress) == null) {

                            DiscovererService discovererTmp = (DiscovererService) worb.lookupService(remoteContactAddress, DiscovererService.class);
                            discovererBaseHst.put(remoteContactAddress, discovererTmp);
                        }
                    }
                }            
            } catch (IOException ioe) {
                ioe.printStackTrace();
                System.exit(1);
            }
        }        
    }
}