/*
 * DiscovererBase.java
 *
 * Created on 20 de Maio de 2004, 14:11
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.DiscovererService;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Worb;
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.util.xml.XmlElement;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Vector;


/**
 *
 * @author  egon
 */
public class DiscovererBase extends AbstractService implements DiscovererService {

    private CellInformationBase cellInformationBase;
    private Worb worb;
    private AdvertiseManagerBase advertiseManagerBase;
    private ResourceCataloguer resourceCataloguer;
    private LeaseManager leaseManager;
    private ResourceMatcher resourceMatcher;
    private NeighborhoodBroker neighborhoodBroker;
    private PreferenceManager preferenceManager;
    private NameSpace nameSpace;
    
    private String unicastAddress;
    private Integer unicastPort;
    private String multicastProxyAddress;
    private Integer multicastProxyPort;
    private String multicastBaseAddress;
    private Integer multicastBasePort;
    private Hashtable discovererBaseHst;
    private Object lock;
    private Object lockLease;
    private Vector recentRequests;
    private String contactAddress;
    
    private static long REQUEST_TABLE_SIZE = 10;
    
        
    /**
     * Creates a new instance of DiscovererBase
     */
    public DiscovererBase() {
        
    }

    /**
     * start
     */
    public void start() throws Exception {

        //Get configuration properties
        this.unicastAddress = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "unicastAddress", null);
        this.unicastPort = new Integer(Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "unicastPort", null));
        this.multicastProxyAddress = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "multicastProxyAddress", null);
        this.multicastProxyPort = new Integer(Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "multicastProxyPort", null));
        this.multicastBaseAddress = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "multicastBaseAddress", null);
        this.multicastBasePort = new Integer(Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "multicastBasePort", null));
        this.contactAddress = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "contactAddress", null);

        this.discovererBaseHst = new Hashtable();
        this.lock = new Object();
        this.lockLease = new Object();
        this.recentRequests = new Vector();
        
        this.getInformedBases();
        
        //this.nameSpace = "service:" + DiscovererService.SERVICE_NAME + ",host:" + HostId.getLocalHost();
        this.nameSpace = new NameSpace("host", Integer.toString(HostId.getLocalHost().getId()), NameSpace.DEFAULT_NS);
        
        //Export service
        this.worb = (Worb) Exehda.getService("worb");
        this.worb.exportService(this, DiscovererService.class, DiscovererService.SERVICE_NAME);

        //Get CIB reference
        this.cellInformationBase = (CellInformationBase) Exehda.getService(CellInformationBase.SERVICE_NAME);
        
        this.registerServiceOnCIB();
        
        //Create advertise listener
        this.advertiseManagerBase = new AdvertiseManagerBase(this.unicastAddress, this.unicastPort, this.multicastProxyAddress, this.multicastProxyPort, this.multicastBaseAddress, this.multicastBasePort, this.discovererBaseHst, this.lock, this.contactAddress);
        
        //Create lease manager
        this.leaseManager = new LeaseManager(this.cellInformationBase, this.nameSpace, this.lockLease);
        this.leaseManager.start();
        
        //Create resource cataloguer
        this.resourceCataloguer = new ResourceCataloguer(this.cellInformationBase, this.nameSpace, this.leaseManager, this.lockLease);
        
        //Create resource matcher
        this.resourceMatcher = new ResourceMatcher(this.cellInformationBase, this.nameSpace);
        
        //Create neighborhood broker
        this.neighborhoodBroker = new NeighborhoodBroker(this.cellInformationBase, this.nameSpace);
       
        //Create preference manager
        this.preferenceManager = new PreferenceManager();
        
        log(Sev.DEBUG, "start", "DiscovererBase started!!!!");
    }

    /**
     * stop
     */
    public void stop() throws Exception {
    }

    /**
     * registerResource
     */
    public ResourceName registerResource(XmlElement xmlDescription) throws Exception {

        ResourceName rn = null;
        
        if (this.worb.getClientHost() == null) {        //call from local resource
            log(Sev.DEBUG, "registerResource", "Chamada local...");
            rn = this.resourceCataloguer.catalogue(xmlDescription, true);
            
            Enumeration enumeration = discovererBaseHst.elements();
            while(enumeration.hasMoreElements()) {
                DiscovererService discovererTmp = (DiscovererService) enumeration.nextElement();
                discovererTmp.registerResource(xmlDescription);
            } 
        } else {                                        //call from Worb
            log(Sev.DEBUG, "registerResource", "Chamada pelo Worb...");
            rn = this.resourceCataloguer.catalogue(xmlDescription, false);
        }
                
        return rn;
    }

    /**
     * removeResource
     */
    public void removeResource(ResourceName rn) {

        if (this.worb.getClientHost() == null) {        //call from local resource
            log(Sev.DEBUG, "removeResource", "Chamada local...");
            this.cellInformationBase.removeResource(rn);
            
            Enumeration enumeration = discovererBaseHst.elements();
            while(enumeration.hasMoreElements()) {
                DiscovererService discovererTmp = (DiscovererService) enumeration.nextElement();
                discovererTmp.removeResource(rn);
            } 
        } else {                                        //call from Worb
            log(Sev.DEBUG, "removeResource", "Chamada pelo Worb...");
            this.cellInformationBase.removeResource(rn);
        }
    }
    
    /**
     * renewResource
     */
    public void renewResource(ResourceName rn, XmlElement xmlDescriptor, boolean isModified) throws Exception {
   
        if (this.worb.getClientHost() == null) {        //call from local resource
            log(Sev.DEBUG, "renewResource", "Chamada local...");
            this.resourceCataloguer.renew(rn, xmlDescriptor, isModified);
            
            Enumeration enumeration = discovererBaseHst.elements();
            while(enumeration.hasMoreElements()) {
                DiscovererService discovererTmp = (DiscovererService) enumeration.nextElement();
                discovererTmp.renewResource(rn, xmlDescriptor, isModified);
            } 
        } else {                                        //call from Worb
            log(Sev.DEBUG, "renewResource", "Chamada pelo Worb...");
            this.resourceCataloguer.renew(rn, xmlDescriptor, isModified);
        }
    }

    /**
     * requestResource
     */
    public XmlElement requestResource(XmlElement xmlRequest) throws Exception {
        
        //Get request ID
        String requestID = this.getRequestID(xmlRequest);

        //Process query if necessary
        XmlElement xmlResults = null;
        if (this.recentRequests.contains(requestID)) {
            xmlResults = null;
        } else {
            this.updateRequestTable(requestID);            
            
            xmlResults = this.resourceMatcher.match(xmlRequest);
            xmlResults = this.neighborhoodBroker.propagateRequest(xmlRequest, xmlResults);
            
            xmlResults = this.finishQueryProcessing(xmlRequest, xmlResults);
        }
        return xmlResults;
    }
    
    /**
     * finishQueryProcessing
     */
    private XmlElement finishQueryProcessing(XmlElement xmlRequest, XmlElement xmlResults) {
    
        log(Sev.DEBUG, "finishQueryProcessing", "##### ENTROU #####");
        
        //Apply default preferences
        xmlResults = this.preferenceManager.applyDefaultSelection(xmlResults);
        
        //Sort using preferences
        String requestType = this.getRequestType(xmlRequest);
        xmlResults = this.preferenceManager.sortResults(xmlResults, requestType); 
        
        //Narrow results
        int requestMaxResults = this.getRequestMaxResults(xmlRequest);
        xmlResults = this.resourceMatcher.narrowResults(xmlResults, requestMaxResults);
        
        return xmlResults;
    }
    
    /**
     * getRequestID
     */
    private String getRequestID(XmlElement xmlRequest) {
    
        Vector vec = XmlElement.selectByPath(xmlRequest, "/QUERY/ID", null);
        XmlElement xmlID = (XmlElement) vec.firstElement();
        String requestID = (String) xmlID.getContents();
        
        return requestID;
    }
    
    /**
     * getRequestType
     */
    private String getRequestType(XmlElement xmlRequest) {
    
        Vector vec = XmlElement.selectByPath(xmlRequest, "/QUERY/TYPE", null);
        XmlElement xmlType = (XmlElement) vec.firstElement();
        String requestType = (String) xmlType.getContents();
        
        return requestType;
    }
    
    /**
     * getRequestMaxResults
     */
    private int getRequestMaxResults(XmlElement xmlRequest) {
    
        Vector vec = XmlElement.selectByPath(xmlRequest, "/QUERY/MAXRESULTS", null);
        XmlElement xmlMaxResults = (XmlElement) vec.firstElement();
        int requestMaxResults = Integer.parseInt((String) xmlMaxResults.getContents());
        
        return requestMaxResults;
    }
    
    /**
     * updateRequestsTable
     */
    private void updateRequestTable(String requestID) {
        
        //Implements a LRU algorithm
        this.recentRequests.add(0, requestID);
        if (this.recentRequests.size() > this.REQUEST_TABLE_SIZE) {
            this.recentRequests.remove(this.recentRequests.size() - 1);
        }
    }
    
    /**
     * getInformedBases
     */
    private void getInformedBases() {
    
        String informedBases = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "informedBases", null);
        if ((informedBases == null) || informedBases.equals("")) {
            return;
        }
        
        Worb worb = (Worb) Exehda.getService("worb");
        StringTokenizer st = new StringTokenizer(informedBases, ";");
        
        while(st.hasMoreTokens()) {
        
            String token = st.nextToken();
            synchronized(this.lock) {

                if (this.discovererBaseHst.get(token) == null) {
                    
                    try {
                        DiscovererService discovererTmp = (DiscovererService) worb.lookupService(token, DiscovererService.class);
                        this.discovererBaseHst.put(token, discovererTmp);
                    } catch(Exception e) {
                        log(Sev.ERROR, "getInformedBases", "Failed trying to lookup informed contact address: " + token);
                        e.printStackTrace();
                    }
                }
            }
            System.out.println("\n#################\nDB -- getInformedBases\n"+this.discovererBaseHst.toString()+"###########");
        }   
    }
    
    /**
     * registerServiceOnCIB
     */
    private void registerServiceOnCIB() {
    
        ResourceName[] rnList = this.cellInformationBase.selectByName(DiscovererService.SERVICE_NAME, NameSpace.DEFAULT_NS, -1);
        ResourceName rnTmp = null;

        if (rnList.length == 0) {
            rnTmp = new ResourceName("service", DiscovererService.SERVICE_NAME, NameSpace.DEFAULT_NS);
            this.cellInformationBase.addResource(rnTmp);
        } else {
            rnTmp = rnList[0];
        }
        
        boolean done = false;
        while(!done) {
            String attrTmp = this.cellInformationBase.getAttribute(rnTmp, "contactAddress");
            if (attrTmp == null) {
                attrTmp = this.contactAddress;
            } else {
                attrTmp = attrTmp + ";" + this.contactAddress;
            }
            this.cellInformationBase.setAttribute(rnTmp, "contactAddress", attrTmp);
            
            StringTokenizer st = new StringTokenizer(this.cellInformationBase.getAttribute(rnTmp, "contactAddress"), ";");
            while (st.hasMoreTokens()) {
                if (st.nextToken().equals(this.contactAddress)) {
                    done = true;
                }
            }
        }
    }
    
    /**
     * setOrderBy
     */
    public void setOrderBy(String type, String attribute, String order) {
        
        this.preferenceManager.setOrderBy(type, attribute, order);
    }
    
    /**
     * setDefaultSelection
     */
    public void setDefaultSelection(XmlElement xmlResults, String defaultName) {
        
        this.preferenceManager.setDefaultSelection(xmlResults, defaultName);
    }
}
