/*
 * DispatcherThread.java
 *
 * Created on February 15, 2005, 5:44 PM
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.Exehda;
import org.isam.exehda.ResourceName;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.DiscovererService;
import org.isam.exehda.services.ContextManager;
import org.isam.exehda.services.ContextManager.Context;
import org.isam.exehda.services.ContextManager.ContextListener;
import org.isam.exehda.services.ContextManager.ContextState;
import org.isam.util.xml.XmlElement;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 *
 * @author  egon
 */
public class DispatcherThread extends Thread implements ContextListener {
    
    private Vector registerQueue;
    private Vector removeQueue;
    private String ctx_connectivity;
    private Hashtable discovererBaseHst;
    private Logger logger;
    
    
    /** 
     * Creates a new instance of DispatcherThread 
     */
    public DispatcherThread(Hashtable discovererBaseHst, Context context) {
        
        this.logger = (Logger) Exehda.getService(Logger.SERVICE_NAME);
        this.discovererBaseHst = discovererBaseHst;
        this.registerQueue = new Vector();
        this.removeQueue = new Vector();
        
        ContextManager contextManager = (ContextManager) Exehda.getService(ContextManager.SERVICE_NAME);
        contextManager.addContextListener(this, context);
        
        while (this.ctx_connectivity == null) {
            synchronized(this) {
                try{
                    this.wait(1000);
                } catch(Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
    
    /**
     * run
     */
    public void run() {
    
        while(true) {           
            try {
                if (this.registerQueue.size() > 0) {
                    this.registerResource();
                }
            
                if (this.removeQueue.size() > 0) {
                    this.removeResource();
                }
            
                synchronized(this) {
                    if ((this.registerQueue.size() == 0) 
                     && (this.removeQueue.size() == 0)) {
                         this.wait();
                    }
                }
            } catch(Exception e) {
                e.printStackTrace();
                System.exit(1);
            }
        }
    }
    
    /**
     * contextChanged
     */
    public void contextChanged(ContextState prevContext, ContextState currContext) {
    
        if (currContext.getType().getName().equalsIgnoreCase("connectivity")) {
            this.ctx_connectivity = currContext.getValue();
        }
    }
    
    /**
     * enqueueOnRegisterResource
     */
    public synchronized void enqueueOnRegisterResource(XmlElement xmlDescription) {
        
        this.registerQueue.add(xmlDescription);
        this.notify();
    }
    
    /**
     * enqueueOnRemoveResource
     */
    public synchronized void enqueueOnRemoveResource(ResourceName rn) {
        
        this.removeQueue.add(rn);
        this.notify();
    }
    
    /**
     * registerResource
     */
    private void registerResource() throws Exception {
    
        this.logger.log(Logger.Sev.DEBUG, "DispatcherThread", "registerResource", "##### ctx_connectivity = " + this.ctx_connectivity, null);
        
        synchronized(this) {
            while (!this.ctx_connectivity.equalsIgnoreCase("online")) {
                this.logger.log(Logger.Sev.DEBUG, "DispatcherThread", "registerResource", "##### waiting...", null);
                this.wait(1000);
            }
        }
        
        XmlElement xmlDescriptor = (XmlElement) this.registerQueue.remove(0);
        Enumeration enumeration = discovererBaseHst.elements();
        while(enumeration.hasMoreElements()) {
            DiscovererService discovererTmp = (DiscovererService) enumeration.nextElement();
            discovererTmp.registerResource(xmlDescriptor);
        }
    }
    
    /**
     * removeResource
     */
    private void removeResource() throws Exception {
    
        synchronized(this) {
            while (!this.ctx_connectivity.equalsIgnoreCase("online")) {
                this.wait(1000);
            }
        }
        
        ResourceName rn = (ResourceName) this.removeQueue.remove(0);
        Enumeration enumeration = discovererBaseHst.elements();
        while(enumeration.hasMoreElements()) {
            DiscovererService discovererTmp = (DiscovererService) enumeration.nextElement();
            discovererTmp.removeResource(rn);
        }
    }
}
