/*
 * LeaseRenewer.java
 *
 * Created on 20 de Maio de 2004, 14:37
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.Exehda;
import org.isam.exehda.ResourceName;
import org.isam.exehda.services.DiscovererService;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.Logger.Sev;
import org.isam.util.xml.XmlElement;
import java.io.File;
import java.io.FileReader;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 *
 * @author  egon
 */
public class LeaseRenewer extends Thread {

    private Logger logger;
    private DiscovererService discoverer;
    private String descriptorFileName;
    private ResourceName rn;
    private File file;
    private XmlElement xmlDescriptor;
    private XmlElement xmlLease;
    private long lease;
    private long lastModifiedTime;
    private Object stopLock;
    private boolean stopNow;
    
    
    /** 
     * Creates a new instance of LeaseRenewer 
     */
    public LeaseRenewer(DiscovererService discoverer, String descriptorFileName, ResourceName rn) throws java.io.FileNotFoundException,java.io.IOException {
        
        try {
            this.logger = (Logger) Exehda.getService(Logger.SERVICE_NAME);
            this.discoverer = discoverer;
            this.descriptorFileName = descriptorFileName;
            this.rn = rn;
            this.file = new File(this.descriptorFileName);
            this.lastModifiedTime = System.currentTimeMillis();
            this.stopNow = false;
            this.xmlDescriptor = XmlElement.parse(new FileReader(this.file));                
            
            Vector vec = XmlElement.selectByPath(this.xmlDescriptor,"/RESOURCE/LEASE",null);
            this.xmlLease = (XmlElement)vec.firstElement();        
            this.lease = (new Long(xmlLease.getContents().toString())).longValue();
            
            vec = XmlElement.selectByPath(this.xmlDescriptor,"/RESOURCE/NAME",null);
            XmlElement xmlName = (XmlElement)vec.firstElement();
            xmlName.clearContents();
            xmlName.addChild(XmlElement.createPCData(rn.getSimpleName()));
        }
        catch (java.io.FileNotFoundException fnfe){
            fnfe.printStackTrace();
            throw new java.io.FileNotFoundException("File not Found!!!");
        }
        catch (java.io.IOException ioe){
            ioe.printStackTrace();
            throw new java.io.IOException("I/O Error!!!");
        }
    }
    
    /**
     * run
     */
    public void run() {
    
        while (true){
            try {
                System.gc();
                this.sleep(this.lease);
                System.gc();
                
                if(!this.sendRenew()) {
                     return;
                }
            } catch(Exception e){
                e.printStackTrace();
                this.logger.log(Logger.Sev.ERROR,"LeaseRenewer","run","Could not renew resource on Base!!!", null);
            }
        }
    }
    
    /**
     * stopLeaseRenewer
     */
    public synchronized void stopLeaseRenewer() {
        this.stopNow = true;
    }
    
    /**
     * sendRenew
     */
    public synchronized boolean sendRenew() throws Exception {

        if (!this.stopNow) {
            long initialTime = System.currentTimeMillis();

            boolean isModified = this.checkIsModified();
            this.discoverer.renewResource(this.rn, this.xmlDescriptor, isModified);

            long finalTime = System.currentTimeMillis();    
            long deltaTime = finalTime - initialTime;
            logger.log(Logger.Sev.DEBUG,"LeaseRenewer","run","###### Lease renewed in = " + deltaTime + " ms", null);

            return true;
        } else {
            return false;
        }
        
    }
    
    /**
     * checkIsModified
     */
    private boolean checkIsModified() throws Exception {
        
        if (this.file.lastModified() > this.lastModifiedTime) {
            
            this.lastModifiedTime = System.currentTimeMillis();
            this.xmlDescriptor=XmlElement.parse(new FileReader(this.file));            
            
            Vector vec = XmlElement.selectByPath(this.xmlDescriptor,"/RESOURCE/LEASE",null);
            XmlElement xmlLease = (XmlElement)vec.firstElement();
            this.lease = (new Long(xmlLease.getContents().toString())).longValue();
            
            if (verifyName(rn, this.xmlDescriptor)){
                
                vec = XmlElement.selectByPath(this.xmlDescriptor,"/RESOURCE/NAME",null);
                XmlElement xmlName = (XmlElement)vec.firstElement();
                xmlName.clearContents();
                xmlName.addChild(XmlElement.createPCData(rn.getSimpleName()));
                return true;
            } else {
                throw new Exception("Resource name cannot be modified!");
            }
        } else {
            return false;
        }  
    }
    
    /**
     * verifyName
     */
    private boolean verifyName(ResourceName rn, XmlElement xml) {

        String [] name = rn.getSimpleName().split(":");
        Vector vec = XmlElement.selectByPath(xml, "/RESOURCE/NAME", null);
        XmlElement xmlName = (XmlElement) vec.firstElement();

        if (name[0].equals(xmlName.getContents().toString())){
            return true;
        } else {
            return false;
        }
    }
}
