/*
 * QueryBuilder.java
 *
 * Created on 23 de Junho de 2004, 16:33
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.HostId;
import org.isam.util.xml.XmlElement;
import java.util.Vector;

/**
 *
 * @author  guilherme
 */
public class QueryBuilder {
    
    private static final String EQUALITY = "equ";
    private static final String GREATER = "gre";
    private static final String GREATER_OR_EQUAL = "geq";
    private static final String INEQUALITY = "neq";
    private static final String LESS = "les";
    private static final String LESS_OR_EQUAL = "leq";
    private static final String MULTIVALUE = "mul";
    private static final String RANGE = "ran";
    
    private static final int DEFAULT_TTL = 1;
    private static final int DEFAULT_TIMEOUT = 0;
    private static final int DEFAULT_MAXRESULTS = 0;

    private boolean hasBranches;
    private boolean isBranch;

    private XmlElement xmlQuery;    
    private XmlElement querySon;


    /**
     * Creates a new instance of QueryBuilder 
     */
    public QueryBuilder(String resourceType, int searchTimeToLive, int maxResults, int timeout) throws Exception {

        this.isBranch=false;
        this.hasBranches = false;
        
        this.xmlQuery = XmlElement.createDocRoot();
        this.querySon = XmlElement.createElement("QUERY");
        
        this.addMainAttributes(resourceType, searchTimeToLive, maxResults, timeout);
        this.xmlQuery.addChild(querySon);
    }

    /**
     * Creates a new instance of QueryBuilder 
     */
    public QueryBuilder(String resourceType, int searchTimeToLive, int maxResults) throws Exception {
        this(resourceType, searchTimeToLive, maxResults, DEFAULT_TIMEOUT);
    }

    /**
     * Creates a new instance of QueryBuilder 
     */
    public QueryBuilder(String resourceType, int searchTimeToLive) throws Exception {
        this(resourceType, searchTimeToLive, DEFAULT_MAXRESULTS, DEFAULT_TIMEOUT);
    }
   
    /**
     * Creates a new instance of QueryBuilder 
     */
    public QueryBuilder(String resourceType) throws Exception {

        this(resourceType, DEFAULT_TTL, DEFAULT_MAXRESULTS, DEFAULT_TIMEOUT);
    }
    
    /**
     * Creates a new instance of QueryBuilder 
     */
    private QueryBuilder(boolean isBranch) {
        
        this.isBranch = isBranch;
    }
    
    /**
     * Creates a new instance of QueryBuilder 
     */
    public QueryBuilder() {
        
        this.isBranch=false;
        this.hasBranches = false;

        this.xmlQuery = XmlElement.createDocRoot();
        this.querySon = XmlElement.createElement("QUERY");
        
        this.xmlQuery.addChild(querySon);
    }
    
    /**
     * addMainAttributes
     */    
    protected void addMainAttributes(String resourceType, int searchTimeToLive, int maxResults, int timeout) throws Exception {
        
        XmlElement xmlID = XmlElement.createElement("ID");
        XmlElement xmlType = XmlElement.createElement("TYPE");
        XmlElement xmlTTL = XmlElement.createElement("TTL");
        XmlElement xmlMaxResults = XmlElement.createElement("MAXRESULTS");
        XmlElement xmlTimeout = XmlElement.createElement("TIMEOUT");
                
        XmlElement xmlIDValue = XmlElement.createPCData(System.currentTimeMillis() + ":" + HostId.getLocalHost().getId());
        XmlElement xmlTypeValue = XmlElement.createPCData(resourceType);
        XmlElement xmlTTLValue = XmlElement.createPCData(Integer.toString(searchTimeToLive));
        XmlElement xmlMaxResultsValue = XmlElement.createPCData(Integer.toString(maxResults));
        XmlElement xmlTimeoutValue = XmlElement.createPCData(Integer.toString(timeout));
        
        xmlID.addChild(xmlIDValue);
        xmlType.addChild(xmlTypeValue);
        xmlTTL.addChild(xmlTTLValue);
        xmlMaxResults.addChild(xmlMaxResultsValue);
        xmlTimeout.addChild(xmlTimeoutValue);
        
        querySon.addChild(xmlID);
        querySon.addChild(xmlType);
        querySon.addChild(xmlTTL);
        querySon.addChild(xmlMaxResults);
        querySon.addChild(xmlTimeout);
    }
    
    /**
     * addMainAttributes
     */    
    protected void addMainAttributes(String resourceType, int searchTimeToLive, int maxResults) throws Exception {

        addMainAttributes(resourceType, searchTimeToLive, maxResults, this.DEFAULT_TIMEOUT);
    }
    
    /**
     * addMainAttributes
     */    
    protected void addMainAttributes(String resourceType, int searchTimeToLive) throws Exception {

        addMainAttributes(resourceType, searchTimeToLive, this.DEFAULT_MAXRESULTS, this.DEFAULT_TIMEOUT);
    }
    
    /**
     * addMainAttributes
     */    
    protected void addMainAttributes(String resourceType) throws Exception {
        
        this.addMainAttributes(resourceType, this.DEFAULT_TTL, this.DEFAULT_MAXRESULTS, this.DEFAULT_TIMEOUT);
    }
 
    /**
     * addEqualityCriteria
     */
    public void addEqualityCriteria(String name, Object value){

        XmlElement xmlCriteria = XmlElement.createElement("CRITERIA");

        xmlCriteria.setAttribute("name", name);
        xmlCriteria.setAttribute("op", EQUALITY);        
        xmlCriteria.setAttribute("value", value.toString());        

        XmlElement xmlCrit;
        java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
           
        if (vec1.size() > 0) {
            xmlCrit = (XmlElement) vec1.firstElement();
            xmlCrit.addChild(xmlCriteria);        
        }
        else {
            this.xmlQuery.addChild(xmlCriteria);
        }
    }
    
    /**
     * addInequalityCriteria
     */
    public void addInequalityCriteria(String name, Object value){
        
        XmlElement xmlCriteria = XmlElement.createElement("CRITERIA");

        xmlCriteria.setAttribute("name", name);
        xmlCriteria.setAttribute("op", INEQUALITY);        
        xmlCriteria.setAttribute("value", value.toString());        

        XmlElement xmlCrit;
        java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
           
        if (vec1.size() > 0) {
            xmlCrit = (XmlElement) vec1.firstElement();
            xmlCrit.addChild(xmlCriteria);        
        }
        else {
            this.xmlQuery.addChild(xmlCriteria);
        }
    }
    
    /**
     * addGreaterCriteria
     */
    public void addGreaterCriteria(String name, Object value){
        
        XmlElement xmlCriteria = XmlElement.createElement("CRITERIA");

        xmlCriteria.setAttribute("op", GREATER);        
        xmlCriteria.setAttribute("value", value.toString());        
        xmlCriteria.setAttribute("name", name);

        XmlElement xmlCrit;
        java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
           
        if (vec1.size() > 0) {
            xmlCrit = (XmlElement) vec1.firstElement();
            xmlCrit.addChild(xmlCriteria);        
        }
        else {
            this.xmlQuery.addChild(xmlCriteria);
        }
    }
    
    /**
     * addGreaterOrEqualCriteria
     */
    public void addGreaterOrEqualCriteria(String name, Object value){
        
        XmlElement xmlCriteria = XmlElement.createElement("CRITERIA");

        xmlCriteria.setAttribute("name", name);
        xmlCriteria.setAttribute("op", GREATER_OR_EQUAL);        
        xmlCriteria.setAttribute("value", value.toString());        

        XmlElement xmlCrit;
        java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
           
        if (vec1.size() > 0) {
            xmlCrit = (XmlElement) vec1.firstElement();
            xmlCrit.addChild(xmlCriteria);        
        }
        else {
            this.xmlQuery.addChild(xmlCriteria);
        }
    }
    
    /**
     * addLessCriteria
     */
    public void addLessCriteria(String name, Object value){
        
        XmlElement xmlCriteria = XmlElement.createElement("CRITERIA");

        xmlCriteria.setAttribute("name", name);
        xmlCriteria.setAttribute("op", LESS);        
        xmlCriteria.setAttribute("value", value.toString());        

        XmlElement xmlCrit;
        java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
           
        if (vec1.size() > 0) {
            xmlCrit = (XmlElement) vec1.firstElement();
            xmlCrit.addChild(xmlCriteria);        
        }
        else {
            this.xmlQuery.addChild(xmlCriteria);
        }
    }
    
    /**
     * addLessOrEqualCriteria
     */
    public void addLessOrEqualCriteria(String name, Object value){
        
        XmlElement xmlCriteria = XmlElement.createElement("CRITERIA");

        xmlCriteria.setAttribute("name", name);
        xmlCriteria.setAttribute("op", LESS_OR_EQUAL);        
        xmlCriteria.setAttribute("value", value.toString());        

        XmlElement xmlCrit;
        java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
           
        if (vec1.size() > 0) {
            xmlCrit = (XmlElement) vec1.firstElement();
            xmlCrit.addChild(xmlCriteria);        
        }
        else {
            this.xmlQuery.addChild(xmlCriteria);
        }
    }
    
    /**
     * addMuiltiValueCriteria
     */
    public void addMultiValueCriteria(String name, Object[] value){
        
        XmlElement xmlCriteria = XmlElement.createElement("CRITERIA");

        xmlCriteria.setAttribute("name", name);
        xmlCriteria.setAttribute("op", MULTIVALUE);        

        String auxValue = new String();
        for (int i=0; i<value.length; i++){
            if (i!=0){
                auxValue=auxValue+";"+value[i].toString();
            }
            else{
                auxValue=value[i].toString();
            }
        }
        
        xmlCriteria.setAttribute("value", auxValue);        

        XmlElement xmlCrit;
        java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
           
        if (vec1.size() > 0) {
            xmlCrit = (XmlElement) vec1.firstElement();
            xmlCrit.addChild(xmlCriteria);        
        }
        else {
            this.xmlQuery.addChild(xmlCriteria);
        }
    }
    
    /**
     * addRangeCriteria
     */
    public void addRangeCriteria(String name, Object valueIni, Object valueEnd){

        XmlElement xmlCriteria = XmlElement.createElement("CRITERIA");

        xmlCriteria.setAttribute("name", name);
        xmlCriteria.setAttribute("op", RANGE);        
        xmlCriteria.setAttribute("value", valueIni.toString()+";"+valueEnd.toString());        

        XmlElement xmlCrit;
        java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
           
        if (vec1.size() > 0) {
            xmlCrit = (XmlElement) vec1.firstElement();
            xmlCrit.addChild(xmlCriteria);        
        }
        else {
            this.xmlQuery.addChild(xmlCriteria);
        }
    }
    
    /**
     * toXmlElement
     */
    public XmlElement toXmlElement(){
        
        return this.xmlQuery;
    }
    
    /**
     * toString
     */
    public String toString(){
        
        return this.toXmlElement().toString();
    }
    
    /**
     * addNewBranch
     */
    public QueryBuilder addNewBranch() {

        XmlElement xmlBranches;
        if (!this.isBranch) {
            if (!this.hasBranches){
                java.util.Vector vec1 = XmlElement.selectByPath(this.xmlQuery, "/QUERY", null);
                xmlBranches = (XmlElement) vec1.firstElement();
                xmlBranches.addChild(XmlElement.createElement("BRANCHES"));
                this.hasBranches = true;
            }
            java.util.Vector vec = XmlElement.selectByPath(this.xmlQuery, "/QUERY/BRANCHES", null);
            xmlBranches = (XmlElement) vec.firstElement();
        }
        else {
            if (!this.hasBranches){
                xmlBranches = this.xmlQuery;
                xmlBranches.addChild(XmlElement.createElement("BRANCHES"));
                this.hasBranches = true;
            }
            java.util.Vector vec = XmlElement.selectByPath(this.xmlQuery, "/BRANCHES", null);
            xmlBranches = (XmlElement) vec.firstElement();
        }
        
        QueryBuilder branches = new QueryBuilder(true);
        branches.xmlQuery = XmlElement.createElement("BRANCH");
        xmlBranches.addChild(branches.xmlQuery);
        return branches;
    }
    
    /*
    public QueryBuilder clear() throws Exception {

        QueryBuilder query=this;
        if (this.isBranch){
            this.xmlQuery.clearContents();
        }
        else {
            java.util.Vector vec = XmlElement.selectByPath(query.xmlQuery, "/QUERY/TTL", null);
            if (vec.size() > 0){
                XmlElement xmlTTL = (XmlElement) vec.firstElement();
                vec = XmlElement.selectByPath(query.xmlQuery, "/QUERY/TYPE", null);
                XmlElement xmlType = (XmlElement) vec.firstElement();
                query = new QueryBuilder(xmlType.getContents().toString(),java.lang.Integer.parseInt(xmlTTL.getContents().toString()));
            }
            else{
                query = new QueryBuilder();
            }

        }
        return query;        
    }
    */
    
    /*
    public static QueryBuilder removeMainAtributes(QueryBuilder queryMain){
 
        
        System.out.println("\n\n###############RemoveMain -> this antes\n"+queryMain.toString());
        QueryBuilder query = new QueryBuilder();
        Vector vec = XmlElement.selectByPath(queryMain.xmlQuery, "/QUERY/CRITERIA", null);
        System.out.println("\n\n###############RemoveMain -> this depois criteria\n"+queryMain.toString());
        Vector vec2 = XmlElement.selectByPath(queryMain.xmlQuery, "/QUERY/BRANCHES", null);
        System.out.println("\n\n###############RemoveMain -> this depois branches\n"+queryMain.toString());
        
        for (int i=0;i< vec.size();i++){
            query.querySon.addChild((XmlElement)vec.elementAt(i));
        }
        
        System.out.println("\n\n###############RemoveMain -> query\n"+query.toString());

        System.out.println("\n\n###############RemoveMain -> vecBranchesSize"+vec.size());
        if (vec2.size()>0){
            query.querySon.addChild((XmlElement)vec2.firstElement());
        }
        
        return query;
    }
    */
}