// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos;

import org.isam.exehda.HostId;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.Collector.Monitor;
import org.isam.exehda.services.Collector.Sensor;
import org.isam.exehda.services.Collector.SensorParameter;
import org.isam.exehda.services.Collector.MonitoringData;

import java.util.Hashtable;


/**
 * Primos native monitor implementation.
 *
 * @todo create views of a sensor for each consumer
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-06-18 22:22:36 -0300 (Sat, 18 Jun 2005) $ $Revision: 930 $
 */
public class NativeMonitor
    implements Monitor
{
    private final String MONITOR_NAME = "PrimosNativeMonitor";
    
    static class NativeSensor
    {
        private final int T_INT = 0;
        private final int T_FLOAT = 1;
        private final int T_STRING = 2;
        
        long key;
        String name;
        Class type;
        boolean enabled;
        boolean staticSensor;
        int intVal;
        boolean booleanVal;
        float floatVal;
        String stringVal;
        
        public NativeSensor(long key, int type, String name)
            {
                this.key = key;
                this.name = name;

                switch (type) {
                    case T_INT:
                        this.type = Integer.TYPE;
                        break;
                    case T_FLOAT:
                        this.type = Float.TYPE;
                        break;
                    case T_STRING:
                        this.type = String.class;
                        break;
                }
            }

        public String getName()
            {
                return name;
            }

        public Class getType()
            {
                return type;
            }

            /**
             * Describe <code>getParameterNames</code> method here.
             *
             * @todo Not yet implemented
             *
             * @return a <code>String[]</code> value
             */
        public String[] getParameterNames()
            {
                    // FIX ME:
                return new String[0];
            }

            /**
             * Describe <code>getParameterTypes</code> method here.
             *
             * @todo Not yet implemented
             *
             * @return a <code>Class[]</code> value
             */
        public Class[] getParameterTypes()
            {
                    // FIX ME:
                return new Class[0];
            }

        public boolean isEnabled()
            {
                return enabled;
            }
        
        public boolean isStatic()
            {
                return staticSensor;
            }
        
        public void setEnabled(boolean enable)
            {
                    // FIX ME: do something useful
                this.enabled = enable;
            }
        
        public void setParameter(String p, Object v)
            {
                    // FIX ME: currently there are no native parameters, so do nothing
            }

        public Object getParameter(String p)
            {
                    // FIX ME: currently there are no native parameters
                throw new IllegalArgumentException("No such sensor parameter");
            }

        public void probe( MonitoringData data )
            {
                if ( Integer.TYPE == type ) {
                    data.setInt(intVal);
                }
                else if ( Float.TYPE == type ) {
                    data.setDouble(floatVal);
                }
                else {
                    data.setString(stringVal);
                }
            }
//         public MetaMonitor.RawSensorData probe()
//             {
//                 return this;
//             }
        
//         public String stringValue()
//             {
//                 if ( Integer.TYPE == type ) {
//                     return Integer.toString(intVal);
//                 }
//                 else if ( Float.TYPE == type ) {
//                     return Float.toString(floatVal);
//                 }
//                 else {
//                     return stringVal;
//                 }
//             }
    }

        // PUBLIC METHODS ///////////////////////////

    public NativeMonitor()
        {
            NameSpace nsLocalHost = HostId.getLocalHost().toNameSpace();
            
            this.name = new ResourceName("monitor",MONITOR_NAME, nsLocalHost);
            this.nspace = new NameSpace(name.getCommonName(), nsLocalHost);
            this.natSensorsByName = new Hashtable();
            loadSensors();
        }
    
    public ResourceName
    getName()
        {
            return this.name;
        }

    public Sensor[]
    getSensors()
        {
            Sensor[] sa  = new Sensor[nativeSensors.length];
                
            for (int i=0; i< sa.length; i++) {
                sa[i] = makeSensor(nativeSensors[i]);
            }
            return sa;
        }
    
    public Sensor
    getSensor( String name )
        {
            return makeSensor(findNativeSensor(name));
        }
                
    public SensorParameter[] getSensorParameters( Sensor s )
        {
            NativeSensor nsen = findNativeSensor(s);

            String[] spn = nsen.getParameterNames();
            Class[] spt = nsen.getParameterTypes();
            SensorParameter[] sparams = new SensorParameter[spn.length];
            
            for (int i=0; i<sparams.length; i++) {
                sparams[i] = new SensorParameter(spn[i], spt[i], s);
            }

            return sparams;
        }
        
    public Object getSensorParameter( SensorParameter p )
        {
            NativeSensor nsen = findNativeSensor(p.getSensor());
            return nsen.getParameter(p.getName());
        }

    public void setSensorParameter( SensorParameter p, Object v )
        {
            NativeSensor nsen = findNativeSensor(p.getSensor());
            nsen.setParameter(p.getName(), v);
        }

    public void setSensorEnabled( Sensor s, boolean enable )
        {
            NativeSensor nsen = findNativeSensor(s);
            nsen.setEnabled(enable);
        }

    public boolean isSensorEnabled( Sensor s )
        {
            NativeSensor nsen = findNativeSensor(s);
            return nsen.isEnabled();
        }

    public MonitoringData probeSensor( Sensor s, MonitoringData d )
        {
            NativeSensor nsen = findNativeSensor(s);

                // TODO: check if d is valid and reuse d
            if ( d == null ) {
                d = new MonitoringData(s.getName());
            }
            
            nsen.probe(d);
            return d;
        }

    public void installSensor( String name, String impl, Hashtable params )
        {
                // TO DO: 
            throw new NullPointerException("Operation not supported.");
        }

    public void
    quantumExpired()
        {
            updateRawSensorData(nativeSensors);
        }

        // PRIVATE METHODS //////////////////////
    private void
    loadSensors()
        {
                // FIX ME: segmentation fault if getNativeSensors is called multiple times
            if ( nativeSensors == null ) {
                nativeSensors = getNativeSensors();
            }

            natSensorsByName.clear();
            for (int i=0; i< nativeSensors.length; i++) {
                natSensorsByName.put(nativeSensors[i].getName(),
                                     nativeSensors[i]);
            }
        }

    private final Sensor
    makeSensor(NativeSensor natsen)
        {
            ResourceName sname = new ResourceName(
                "sensor",natsen.getName(), this.nspace);
            
                // FIX ME: use indirect key instead of rsen better isolation
            return  new Sensor(sname, natsen.getType(), natsen); 
        }

    private final NativeSensor
    findNativeSensor(String name)
        {
            NativeSensor nsen =  (NativeSensor) natSensorsByName.get(name);

            if ( nsen != null ) {
                return nsen;
            }
            else {
                throw new NullPointerException("No such sensor");
            }
        }

    private final NativeSensor
    findNativeSensor(Sensor s)
        {
            NativeSensor nsen = (NativeSensor) s.getKey();
            
            if ( nsen != null ) {
                return nsen;
            }
            else {
                throw new NullPointerException("No such sensor");
            }
        }
    

        // NATIVE METHODS ///////////////////////

    static {
        System.loadLibrary("PrimosNativeMonitor");
    }
    
    private native NativeSensor[] getNativeSensors();
    private native void updateRawSensorData(NativeSensor[] s);

        // ATTRIBUTES ///////////////////////////

    private ResourceName   name;
    private NameSpace      nspace;
//    private Sensor[]       sensors;
    private Hashtable      natSensorsByName;
    private static NativeSensor[] nativeSensors = null;;
}
