// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos.objseed;

import org.isam.exehda.ObjectId;
import org.isam.exehda.services.ObjectSeed.Activator;
import org.isam.exehda.services.ObjectSeed.MarshaledOX;
import org.isam.exehda.services.Logger.Sev;


/**
 * Intercepts activation/de-activation calls and print usefull debugging messages;
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-04-14 21:07:43 -0300 (Thu, 14 Apr 2005) $ $Revision: 770 $
 */
class DebugActivator
    extends AbstractActivator
{
    public DebugActivator( Activator chainedActivator )
        {
            this.chainedActivator = chainedActivator;
        }
    
    public void activate( ObjectId oxID, Object obj, MarshaledOX extState )
        throws Exception
        {
            log(Sev.DEBUG, "activate()",
                "Activating object id={0}, instance={1}", oxID, obj);

            if ( chainedActivator != null ) {
                chainedActivator.activate(oxID, obj, extState);
            }
        }
    
    public void deactivate( ObjectId oxID, Object obj, MarshaledOX oxState )
        throws Exception
        {
            log(Sev.DEBUG, "deactivate()",
                "Deactivating object id={0}, instance={1}", oxID, obj);

            if ( chainedActivator != null ) {
                chainedActivator.deactivate(oxID, obj, oxState );
            }
        }

    private Activator chainedActivator;
}
