// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib.backend.ldap;


import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.ResourceConstraint.ResourceInfo;
import org.isam.exehda.CompositeResourceConstraint;
import org.isam.exehda.AttributeConstraint;
import org.isam.exehda.AttributeSimpleMatchConstraint;
import org.isam.exehda.AttributeDefinedConstraint;
import org.isam.exehda.services.cib.backend.CibBackEnd;
import org.isam.exehda.services.cib.backend.CibBackEndException;
import org.isam.exehda.services.cib.backend.CibBackEndConf;
import javax.naming.*;
import javax.naming.directory.*;
import javax.naming.ldap.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Properties;
import java.util.List;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Enumeration;

/**
 *
 * @todo This documentation is out-dated.
 *
 *
 * Ldab Install and Configuration Instructions:
 *
 *
 * @author frainer. Last modified by $Author: lucc $
 * @version $Date: 2005-07-08 13:35:00 -0300 (Fri, 08 Jul 2005) $ $Revision: 970 $
 */
public class LdapBackEnd
    implements CibBackEnd
{
    final static String CIB_NS_TYPE_CELL = "cell";

    public LdapBackEnd()
        {
            this.sessionFactory = null;
        }

    
    public void
    init( CibBackEndConf conf )
        throws CibBackEndException
        {
                //
                // FIX ME: property names in-line; default values
                //
            String serverURI = conf.getProperty("ldap.server",
                                                "ldap://localhost");
            
            this. rootDn = conf.getProperty("ldap.rootDn",
                                             "dc=CELL");
            
            String user = conf.getProperty("ldap.user",
                                           "cn=Manager,dc=CELL");
            
            String passwd = conf.getProperty("ldap.passwd", "*****");

                // create session factory
            this.sessionFactory = new LdapSessionFactory(serverURI, rootDn, user, passwd);

            try {
                    // check access to the LDAP server
                LdapSession sTest = sessionFactory.createSession();
                sessionFactory.releaseSession(sTest);
            }
            catch (Exception e) {
                throw new CibBackEndException("LdapBackend sanity check failed", e);
            }

                // init root LdapResourceTypeHandler
            initRootResourceTypeHandler();
        }

    
        /**
         * Adds a new resource to the cib, if it doesn't already exist .
         *
         * @param  r  The ResourceName that is to be added
         */
    public synchronized void
    addResource(ResourceName r)
        throws CibBackEndException
        {
            try {
                LdapSession ldap = sessionFactory.createSession();
            
                rootHandler.createNode(ldap, r);
            
                sessionFactory.releaseSession(ldap);
            }
            catch (NamingException ne) {
                throw new CibBackEndException("addResource() failed", ne);
            }
        }
    
    
        /**
         * Returns the value of the especified attribute in the especified resource.
         *
         * @param r The resource
         * @param a The name of the attribute  
         *        
         * @return the value of the attribute     
         */
    public synchronized String
    getAttribute( ResourceName r, String a )
        throws CibBackEndException
        {
            try {
                LdapSession ldap = sessionFactory.createSession();

                String result = rootHandler.getNodeAttribute(ldap, r, a);
       
                sessionFactory.releaseSession(ldap);
            
                return result;
            }
            catch ( NamingException ne ) {
                throw new CibBackEndException("getAttributed() failed", ne);
            }
        }
    
        /**
         * Returns all the names of the attribues of a given resource 
         *
         * @param r The name of the resource 
         *        
         * @return an array of string cointaining all the names of the attributes of the
         * resource
         */
    public synchronized String[]
    getAttributes ( ResourceName r )
        throws CibBackEndException
        {
            try {
                LdapSession ldap = sessionFactory.createSession();

                String[] result = rootHandler.listNodeAttributes(ldap, r);
            
                sessionFactory.releaseSession(ldap);
            
                return result;
            }
            catch ( NamingException ne ) {
                throw new CibBackEndException("getAttributes() failed", ne);
            }
        }
    
        /**
         * Removes a resource from the cib.
         *
         * @param r The name of the resource that will be removed
         */
    public synchronized void
    removeResource( ResourceName r )
        throws CibBackEndException
        {
            try {
                LdapSession ldap = sessionFactory.createSession();

                rootHandler.deleteNode(ldap, r);
            
                sessionFactory.releaseSession(ldap);
            }
            catch ( NamingException ne ) {
                throw new CibBackEndException("removeResources() failed", ne);
            }
        }
    
        /**
         * Searches the cib for resources that match a certain ResourceConstraint
         *
         * @param rc The constraints the returned resources must satisfy
         * @param ns Scope of the search
         * @param maxResults The maximum number of results that are to be returned
         *
         * @return An array containing the ResourceNames of the cib entries that matched
         * the ResourceConstraint
         */
    public synchronized ResourceName[]
    select( ResourceConstraint rc, NameSpace ns, int maxResults )
        throws CibBackEndException
        {            
            try {
                String filter = rootHandler.toSearchFilter(rc);

//                System.out.println("Search filter: "+filter);
                                   
                    //
                    // TO DO: perhaps reuse DnBuffer
                    //
                String scope = rootHandler.encode(ns, null).toString();

                LdapSession ldap = sessionFactory.createSession();
                
                    //
                    // XXX: the '+' bellow allow use to access operation attributes too; e.g.
                    // modification time
                    //
                NamingEnumeration ne = ldap.search(scope, filter, new String[] { "*", "+" } );

                Vector tmp = new Vector( maxResults>0 ? maxResults : 20 );
            
                while ( ne.hasMore() ) {
                    SearchResult sr = (SearchResult) ne.next();
                    Attributes atts = sr.getAttributes();
                    
                        // re-build full dn
                    String dn = sr.getName();
                    if ( "".equals(dn) ) {
                        dn = scope;
                    }
                    else {
                        dn += "," + scope;
                    }

                        // second stage filtering
                    LdapResourceInfo ri = rootHandler.toLdapResourceInfo(dn, atts);

//                    System.out.println("select(): dn="+dn+", rn="+ri.getName());
                    
                    if ( rc.check(ri) == ResourceConstraint.OK ) {
                        tmp.add(ri.getName());
                    }
                }
                                               
                sessionFactory.releaseSession(ldap);

                    // limit answer size to maxResults
                maxResults = ( maxResults < 0 ) ? tmp.size()
                    : ( maxResults < tmp.size() ? maxResults : tmp.size() );

                ResourceName[] results = new ResourceName[maxResults];
                for (int i=0; i<results.length; i++) {
                    results[i] = (ResourceName) tmp.elementAt(i);
                }

                return results;
            }
            catch (NamingException ne) {
                throw new CibBackEndException("select() failed", ne);
            }
    }

    
        /**
         * Sets the value of a given attribute in a given resource
         *
         * @param r the resource which "owns" the attribute that is to be set
         * @param a the name of the attribute that is to be set
         * @param v the value to which the attribute is to set        
         *
         * @return a string containing tha value to which the attribute was set    
         */
    public synchronized String
    setAttribute(ResourceName r, String a, String v)
        throws CibBackEndException
        {
            try {
            LdapSession ldap = sessionFactory.createSession();
            
            rootHandler.setNodeAttribute(ldap, r, a, v);

                //
                // FIX ME: do we realy need to return the new value??
                //
            v = rootHandler.getNodeAttribute(ldap, r, a);
            
            sessionFactory.releaseSession(ldap);
            
            return v;
            
        }
            catch ( NamingException ne ) {
                throw new CibBackEndException("setAttribute() failed", ne);
            }
        }

        ///////////////////////////////////////////////////////////////////////
        // PRIVATE STUFF
        ///////////////////////////////////////////////////////////////////////

    
        /**
         * Install the root resource type handler.
         *
         * <p>In future the root handler will be installed in the LDAP itseft
         * (i.e. serialized to an LDAP entry) and loaded upon backend
         * initialization. This way it would be very straitforward to support different
         * mapping strategies for each ldap repository. Each repository <quote>will
         * know</quote> about its own strategy.
         *
         * <p>Currently, the root handler creation is hard-coded in the ldap backend. So,
         * only one mapping strategy is supported by now.
         *
         */
    private void
    initRootResourceTypeHandler()
        {
                //
                // TO DO: read the root handler from ldap
                //
//                 // load resource type handle table from the server
//             LdapSession session = sessionFactory.createSession();
//             ...
//             sessionFactory.releaseSession(session);
            
            CompositeLRTH rh = new CompositeLRTH();
            rh.addHandler(new DefaultLRTH());
            rh.addHandler(new CellLRTH());
            rh.addHandler(new HostLRTH());
            rh.addHandler(new ServiceLRTH());
            rh.addHandler(new ApplicationLRTH());

            this.rootHandler = rh;
            LdapResourceTypeHandler.setRootHandler(rh);
        }

        /**
         * Finds the name of the Cell to which the NameSpace belongs
         *
         * @param rn the name space
         *         
         * @return a String with the name of the cell to which the given NameSpace
         * belongs to. Null if it doesn't belong to any cell.
         */
    private String
    getCell(NameSpace ns)
        {
            while ( ns != null ) {
                if ( CIB_NS_TYPE_CELL.equals(ns.getType()) ) {
                    return ns.getSimpleName();
                }
                ns = ns.getParent();
            }
            return null;
        }
    
        /**
         * Finds the name of the Cell to which the ResourceName belongs
         *
         * @param  rn  the resource name 
         *         
         * @return a String with the name of the cell to which the given ResourceName
         * belongs to. Null if it doesn't belong to any cell.
         */
    private String
    getCell(ResourceName rn)
        {
            return getCell(rn.getNameSpace());
        }    


    private LdapResourceTypeHandler rootHandler;
    private LdapSessionFactory sessionFactory;
    private Hashtable typeConversion;
    private Hashtable typeBackConversion;
    private Hashtable attributeConversion;
    private Hashtable attributeBackConversion;
    private String rootDn;
    
}
