// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.oxm;

import org.isam.exehda.Exehda;
import org.isam.exehda.ObjectId;
import org.isam.exehda.HostId;
import org.isam.exehda.CellId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.ResourceBroker;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.Executor;

/**
 * Cellular instance of the ResourceBroker service.
 *
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-06-23 18:42:54 -0300 (Thu, 23 Jun 2005) $ $Revision: 959 $ 
 */
public class ResourceBrokerCell
    extends AbstractService
    implements ResourceBroker
{
    final static CellId LOCAL_CELL = HostId.getLocalHost().getCell();
    
    public void
    start()
        {
                // export service to worb
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            worb.exportService(this,
                               ResourceBroker.class,
                               ResourceBroker.SERVICE_NAME);
        }

    public void
    stop()
        {
                //
                // TO DO: unexport service
                //
        }
   
        /**
         * Busca por recursos que atendam o conjunto de restrições especificado em query.
         *
         * @param query a <code>ResourceConstraint[]</code> value
         * @param scope 0: celula local, 1: vizinhança nível 1, 2: viz. n.2...
         * @param num numero *máximo* de recursos a retornar, pode retornar menos
         * @return nomes do recursos encontrados que satisfazem o critério ou um array de 
         *         tamanho zero caso nenhum recurso tenha sido encontrado
         */
    public ResourceName[] findResource( ResourceConstraint[] query, int scope, int num )
        {
                // XXX: inicialmente assume scope=1 e num=infinito
                                
                // pesquisa na celula local
            ResourceName[] result = localSearch(query, num);

                // filtra recursos retornados em result usando a regras de acesso                
                // retorna subconjunto de result visivel para o chamador
            return filter(result);
        }
    
        /**
         * Busca por recursos que atendam o conjunto de restries especificado em query e 
         * marca como alocados.
         *
         * @param req a <code>ResourceConstraint[]</code> value
         * @param scope an <code>int</code> value
         * @param num an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] allocResource( ResourceConstraint[] req, int scope, int num )
        {
            ResourceName[] result = null;
            
                // dependendo do tipo do recurso procurado
            if ( isHostQuery(query) ) {
                    // pesquisa por nodo de processamento                   
                    // roteia requisio para o Scheduler
                    
                    // filtra resultado
//                 result = filter(...);
            }
            else {
                    // pesquisa por outro tipo de recurso
                    // roteia requisio para a cib (ou para o discoverer)
                    
                    // filtra resultado
//                 result = filter(...);
            }
/*            
                // TO DO: marca recurso como allocado
*/

                // retorna
            return result;    
        }
    
    
        ///////////////////////////////////////////////////////////////////////
        // PRIVATE STUFF
        ///////////////////////////////////////////////////////////////////////

        /**
         * Facilidade para acessar o servio Worb.
         *
         */
    private final Worb
    getServiceWorb()
        {
            return (Worb) Exehda.getService(Worb.SERVICE_NAME);
        }

        /**
         * Facilidade para acessar o servio CIB.
         *
         */
    private final CellInformationBase
    getServiceCib()
        {
            return (CellInformationBase) Exehda.getService(CellInformationBase.SERVICE_NAME);
        }
        
        /**
         * Facilidade para acessar o servio Executor.
         *
         */
    private final Executor
    getServiceExecutor()
        {
            return (Executor) Exehda.getService(Executor.SERVICE_NAME);
        }        
        
        /**
         * Testa o tipo da pesquisa, se nodo de processamento ou recurso especializado
         *
         * @return true, se nodo de processamento, falso caso contrrio
         */
    private final boolean
    isHostQuery(ResourceConstraint[] query)
        {
                // verifica se entre as restrições definidas em query está tipo=host
/*                                
            for (int i=0; i< query.lentgh; i++) {
                ResourceConstraint c = query[i];
                if ( c instanceof ResourceTypeConstraint ) {
                    String t = ((ResourceTypeConstraint) c).getType();
                    if ( "host".equals(t) ) return true;
                }
            }
            return false;   
*/
            throw new NullPointerException("Not yet implemented");
        }
        
        /**
         * Pesquisa na base da célula local buscando por recursos que atenda o critério 
         * definido
         *
         */
    private final ResourceName[]
    localSearch( ResourceConstraint[] query, int num )
        {
                // TO DO: pesquisa na CIB local
            throw new NullPointerException("Not yet implemented.");
        }
        
        
    private final ResourceName[]
    filter( ResourceName[] r )
        { 
/*        
                // contexto de execução atual: nodo chamador, celula onda a app iniciou e a própria applicação
            ApplicationId app = getServiceExecutor().currentApplication();
            CellId  homeAppCell = app.getBaseHost().getCell();
            HostId caller = getServiceWorb().getClientHost();
            
                // filtra 'r' considerando as regras aplicáveis ao contexto atual
                // nesse momento talves apenas o homeAppCell
*/                
            throw new NullPointerException("Not yet implemented.");
        }


        /**
         * Tabela de Vector contendo os nodos permitidos conforme o usuário ou célula.
         * Quem não estiver na tabela não vê nenhum recurso da célula.
         *
         */
    private java.util.Hashtable resourceViewById;
}
