package dimi;

import java.rmi.RemoteException;

import dimi.message.MessageListener;
import dimi.configuration.ChannelConfiguration;
import dimi.configuration.DisseminationUnitConfiguration;
import dimi.exception.CantCreateChannelException;
import dimi.exception.CantFindChannelException;
import dimi.exception.CantFindDBaseException;
import dimi.exception.CantEnterChannelException;
import dimi.name.ChannelName;
import dimi.name.ConsumerEntryPointName;
import dimi.name.DisseminationUnitName;
import dimi.name.DNodeName;
import dimi.name.BadformedDimiNameException;

/**
 * Implementacao de um DNode.
 * @author mcmoraes
 */
public class DNodeImpl extends DisseminationUnitImpl implements DNode,DisseminationService {

    //atributos - INICIO
    private DBase dbase = null;
    private long consumerCounter;
    //atributos - FIM

    //construtores - INICIO
    /**
     * Creates a new instance of DBase
     * @param name o nome do DNode
     * @param disseminationUnitConfiguration a configuracao do DNode
     * @param dbase o DBase deste DNode
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public DNodeImpl(DNodeName name,DisseminationUnitConfiguration disseminationUnitConfiguration,DBase dbase) throws RemoteException{
        super(name,disseminationUnitConfiguration);
        setChannelManager(new DNodeChannelManager(this));
        this.dbase = dbase;
    }
    //construtores - FIM
    
    //interface DNode - INICIO
    /**
     * Getter for property dbase.
     * @return Value of property dbase.
     */
    public DBase getDbase() {
        return dbase;
    }    
    //interface DNode - FIM
    
    //interface DisseminationService - INICIO
    /**
     * Entra em um canal e assume um ponto de entrada de consumidor como filho nesse canal.
     * @return o ponto de entrada de consumo no canal
     * @param desiredPeerEntryPointsNumber o numero desejado de pais no canal
     * @param messageListener o MessageListener de consumo
     * @param channelName o nome do canal
     * @throws CantFindDBaseException se nao foi possivel encontrar o DBase indicado
     * @throws CantFindChannelException se o canal nao for encontrado
     * @throws CantEnterChannelException se nao for possivel entrar no canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public final ConsumerEntryPoint enterChannel(ChannelName channelName, int desiredPeerEntryPointsNumber,MessageListener messageListener) throws RemoteException, CantFindChannelException,CantFindDBaseException,CantEnterChannelException{
        ConsumerEntryPointName consumerEntryPointName = null;
        try{
            consumerEntryPointName = new ConsumerEntryPointName(channelName.toString() + "/" + nextConsumerLocalName());
            System.out.println("new consumer entry point name:" + consumerEntryPointName);
        }
        catch(BadformedDimiNameException bfdne){bfdne.printStackTrace();}
        ConsumerEntryPoint consumer = new ConsumerEntryPointImpl(consumerEntryPointName,messageListener);
        getChannelManager().enterChannel(channelName,desiredPeerEntryPointsNumber,consumer);
        return consumer;
    }
    
    /**
     * Cria um novo canal.
     * @return o ponto de entrada para o produtor do canal
     * @param channelConfiguration a configuracao do canal
     * @throws CantCreateChannelException se nao foi possivel criar o canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public final ProducerEntryPoint createChannel(ChannelConfiguration channelConfiguration) throws CantCreateChannelException,RemoteException{
        return ((DNodeChannelManager)getChannelManager()).createChannel(channelConfiguration);
    }
    //interface DisseminationService - FIM

    //interface DisseminationUnit - INICIO
    /**
     * Finaliza o servico do DNode.
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public final void destroy() throws RemoteException{
    }
    //interface DisseminationUnit - FIM
   
    private synchronized String nextConsumerLocalName(){
        return "consumer" + String.valueOf(consumerCounter++);
    }
    
}
