package dimi;

import dimi.configuration.ChannelConfiguration;
import dimi.message.MessageListener;
import dimi.exception.CantCreateChannelException;
import dimi.exception.CantFindChannelException;
import dimi.exception.CantFindDBaseException;
import dimi.exception.CantEnterChannelException;
import dimi.name.DisseminationUnitName;
import dimi.name.ChannelName;

import java.rmi.RemoteException;
import java.rmi.Remote;

/**
 * A interfae do servico de disseminacao oferecida aos usuarios.
 * @author  mcmoraes
 */
public interface DisseminationService extends Remote {
 
    /**
     * Cria um novo canal.
     * @return o ponto de entrada para o produtor do canal
     * @param channelConfiguration a configuracao do canal
     * @throws CantCreateChannelException se nao foi possivel criar o canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public ProducerEntryPoint createChannel(ChannelConfiguration channelConfiguration) throws  RemoteException,CantCreateChannelException;
    
    /**
     * Entra em um canal e assume um ponto de entrada de consumidor como filho nesse canal.
     * @return o ponto de entrada de consumo no canal
     * @param desiredEntryPointsNumber o numero desejado de pais no canal
     * @param messageListener o MessageListener de consumo
     * @param channelName o nome do canal
     * @throws CantFindChannelException se o canal nao for encontrado
     * @throws CantFindPeerException se o ponto de entrada nao for encontrado
     * @throws CantEnterChannelException se nao for possivel entrar no canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public ConsumerEntryPoint enterChannel(ChannelName channelName, int desiredEntryPointsNumber, MessageListener messageListener) throws RemoteException,CantFindChannelException,CantFindDBaseException,CantEnterChannelException;
    
}
