package dimi.gui.simple;

import java.util.Timer;
import java.util.TimerTask;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import java.rmi.RemoteException;

import dimi.channel.DisseminationUnitChannel;
import dimi.channel.Channel;
import dimi.channel.connection.DisseminationUnitConnection;

/**
 * Um frame para visualizar um canal.
 * @author mauricio
 */
public class ChannelIFrame extends javax.swing.JInternalFrame {
    
    private Timer timer = new Timer();
    private Updater updater = new Updater();
    
    private DisseminationUnitChannel channel = null;
    
    static int CHANNEL_TAB_INDEX = 0;
    static int CONNECTIONS_TAB_INDEX = 1;
    
    private int ROOT_NAME_PROPERTIES_LIST_INDEX = 0;
    private void putRootNameOnPropertiesList(){
        propertiesListModel.add(ROOT_NAME_PROPERTIES_LIST_INDEX, "Root Name: " + channel.getNames()[Channel.ROOT_NAME]);
    }
    
    private int LOCAL_NAME_PROPERTIES_LIST_INDEX = 1;
    private void putLocalNameOnPropertiesList(){
        propertiesListModel.add(LOCAL_NAME_PROPERTIES_LIST_INDEX, "Local Name: " + channel.getNames()[Channel.LOCAL_NAME]);
    }

    private int DESCRIPTION_PROPERTIES_LIST_INDEX = 2;
    private void putDescriptionOnPropertiesList(){
        propertiesListModel.add(DESCRIPTION_PROPERTIES_LIST_INDEX, "Description: " + channel.getConfiguration().getDescription());
    }

    private int PRIORITY_PROPERTIES_LIST_INDEX = 3;
    private void putPriorityOnPropertiesList(){
        propertiesListModel.add(PRIORITY_PROPERTIES_LIST_INDEX, "Priority: " + channel.getConfiguration().getPriority());
    }

    private int FILTER_PROPERTIES_LIST_INDEX = 4;
    private void putFilterOnPropertiesList(){
        propertiesListModel.add(FILTER_PROPERTIES_LIST_INDEX, "Filter: " + channel.getConfiguration().getFilter());
    }

    private int HEART_BEAT_PERIOD_PROPERTIES_LIST_INDEX = 5;
    private void putHeartBeatPeriodOnPropertiesList(){
        propertiesListModel.add(HEART_BEAT_PERIOD_PROPERTIES_LIST_INDEX, "Heart-Beat Period: " + channel.getConfiguration().getHeartBeatPeriod());
    }

    private int MAXIMUM_SILENT_INTERVAL_PROPERTIES_LIST_INDEX = 6;
    private void putMaximumSilentIntervalOnPropertiesList(){
        propertiesListModel.add(MAXIMUM_SILENT_INTERVAL_PROPERTIES_LIST_INDEX, "Maximum Silent Interval: " + channel.getConfiguration().getMaximumSilentInterval());
    }

    private DefaultListModel propertiesListModel = new DefaultListModel();
    private DefaultListModel inputConnectionsListModel = new DefaultListModel();
    private DefaultListModel outputConnectionsListModel = new DefaultListModel();
    private DefaultListModel messageBufferListModel = new DefaultListModel();

    /**
     * Creates new form ChannelIFrame
     * @param channel o canal ser observado
     */
    public ChannelIFrame(DisseminationUnitChannel channel) {
        initComponents();
        this.channel = channel;
        this.channelTPane.setTitleAt(0,"Channel");
        //
        this.propertiesList.setModel(propertiesListModel);
        this.inputConnectionsList.setModel(inputConnectionsListModel);
        this.outputConnectionsList.setModel(outputConnectionsListModel);
        this.messageBufferList.setModel(messageBufferListModel);
        //
        this.channelTPane.addTab("Connections",connectionsPanel);
        this.connectionsDesktop.putClientProperty("JDesktopPane.dragMode", "outline");
        //
        channelRootNameLabel.setText(channel.getNames()[Channel.ROOT_NAME].toString());
        channelLocalNameLabel.setText(channel.getNames()[Channel.LOCAL_NAME].toString());
        putRootNameOnPropertiesList();
        putLocalNameOnPropertiesList();
        putDescriptionOnPropertiesList();
        putPriorityOnPropertiesList();
        putFilterOnPropertiesList();
        putHeartBeatPeriodOnPropertiesList();
        putMaximumSilentIntervalOnPropertiesList();
        
        inputConnectionsList.addListSelectionListener(new InputConnectionsListSelectionListener());
        outputConnectionsList.addListSelectionListener(new OutputConnectionsListSelectionListener());
        
        timer.schedule(updater,DBaseGUI.UPDATER_PERIOD, DBaseGUI.UPDATER_PERIOD);
    }
    
    class Updater extends TimerTask{
        public void run(){
            if(channel.isActive()){
                DBaseGUI.updateDefaultListModel(inputConnectionsListModel,channel.getConnectionManager().getInputConnections());
                DBaseGUI.updateDefaultListModel(outputConnectionsListModel,channel.getConnectionManager().getOutputConnections());
            }
            else{
                dispose();
            }
        }
    }
    
    private Map connectionsIFrameMap = new TreeMap();
    class InputConnectionsListSelectionListener implements ListSelectionListener{
        public void valueChanged(ListSelectionEvent evt) {
            if (evt.getValueIsAdjusting() == false) {
                if (inputConnectionsList.getSelectedIndex() != -1) {
                    channelTPane.setSelectedIndex(CONNECTIONS_TAB_INDEX);
                    DisseminationUnitConnection connection = (DisseminationUnitConnection)inputConnectionsList.getSelectedValue();
                    ConnectionIFrame frame = (ConnectionIFrame)connectionsIFrameMap.get(connection.toString());
                    if(frame == null || frame.isClosed()){
                        frame = new ConnectionIFrame(connection);
                        frame.setBounds(0, 0, DBaseGUI.screenSize.width/2, DBaseGUI.screenSize.height/4);
                        connectionsIFrameMap.put(connection.toString(),frame);
                        connectionsDesktop.add(frame);
                    }
                    //:( - swing juca! - BEGIN
                    else{
                        frame.setVisible(false);
                    }
                    try {
                        frame.setSelected(true);
                    }
                    catch (java.beans.PropertyVetoException pve) {
                        pve.printStackTrace();
                    }
                    frame.setVisible(true);
                    //:( - swing juca! - END
                }
            }
        }
    }
    class OutputConnectionsListSelectionListener implements ListSelectionListener{
        public void valueChanged(ListSelectionEvent evt) {
            if (evt.getValueIsAdjusting() == false) {
                if (outputConnectionsList.getSelectedIndex() != -1) {
                    channelTPane.setSelectedIndex(CONNECTIONS_TAB_INDEX);
                    DisseminationUnitConnection connection = (DisseminationUnitConnection)outputConnectionsList.getSelectedValue();
                    ConnectionIFrame frame = (ConnectionIFrame)connectionsIFrameMap.get(connection.toString());
                    if(frame == null || frame.isClosed()){
                        frame = new ConnectionIFrame(connection);
                        frame.setBounds(0, 0, DBaseGUI.screenSize.width/2, DBaseGUI.screenSize.height/2);
                        connectionsIFrameMap.put(connection,frame);
                        connectionsDesktop.add(frame);
                    }
                    //:( - swing juca! - BEGIN
                    else{
                        frame.setVisible(false);
                    }
                    try {
                        frame.setSelected(true);
                    }
                    catch (java.beans.PropertyVetoException pve) {
                        pve.printStackTrace();
                    }
                    frame.setVisible(true);
                    //:( - swing juca! - END
                }
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        connectionsPanel = new javax.swing.JPanel();
        connectionsDesktop = new javax.swing.JDesktopPane();
        JPanel910 = new javax.swing.JPanel();
        jPanel13 = new javax.swing.JPanel();
        getOutChannelButton = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        channelTPane = new javax.swing.JTabbedPane();
        jPanel11 = new javax.swing.JPanel();
        JPanel903 = new javax.swing.JPanel();
        JScrollPane904 = new javax.swing.JScrollPane();
        propertiesList = new javax.swing.JList();
        JLabel906 = new javax.swing.JLabel();
        JPanel904 = new javax.swing.JPanel();
        JScrollPane905 = new javax.swing.JScrollPane();
        inputConnectionsList = new javax.swing.JList();
        JLabel907 = new javax.swing.JLabel();
        jPanel12 = new javax.swing.JPanel();
        JScrollPane906 = new javax.swing.JScrollPane();
        outputConnectionsList = new javax.swing.JList();
        JLabel908 = new javax.swing.JLabel();
        JPanel905 = new javax.swing.JPanel();
        JScrollPane907 = new javax.swing.JScrollPane();
        messageBufferList = new javax.swing.JList();
        JLabel909 = new javax.swing.JLabel();
        JPanel906 = new javax.swing.JPanel();
        JScrollPane908 = new javax.swing.JScrollPane();
        partialAscendenceList = new javax.swing.JList();
        JLabel910 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        channelRootNameLabel = new javax.swing.JLabel();
        channelLocalNameLabel = new javax.swing.JLabel();

        connectionsPanel.setLayout(new java.awt.BorderLayout());

        connectionsPanel.setBorder(new javax.swing.border.EtchedBorder());
        connectionsPanel.add(connectionsDesktop, java.awt.BorderLayout.CENTER);

        setClosable(true);
        setIconifiable(true);
        setMaximizable(true);
        setResizable(true);
        addInternalFrameListener(new javax.swing.event.InternalFrameListener() {
            public void internalFrameOpened(javax.swing.event.InternalFrameEvent evt) {
            }
            public void internalFrameClosing(javax.swing.event.InternalFrameEvent evt) {
                formInternalFrameClosing(evt);
            }
            public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt) {
                formInternalFrameClosed(evt);
            }
            public void internalFrameIconified(javax.swing.event.InternalFrameEvent evt) {
            }
            public void internalFrameDeiconified(javax.swing.event.InternalFrameEvent evt) {
            }
            public void internalFrameActivated(javax.swing.event.InternalFrameEvent evt) {
            }
            public void internalFrameDeactivated(javax.swing.event.InternalFrameEvent evt) {
            }
        });

        JPanel910.setLayout(new java.awt.BorderLayout());

        jPanel13.setLayout(new java.awt.BorderLayout());

        jPanel13.setBorder(new javax.swing.border.EtchedBorder());
        getOutChannelButton.setText("Get Out of this Channel!");
        getOutChannelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                getOutChannelButtonActionPerformed(evt);
            }
        });

        jPanel13.add(getOutChannelButton, java.awt.BorderLayout.CENTER);

        JPanel910.add(jPanel13, java.awt.BorderLayout.SOUTH);

        getContentPane().add(JPanel910, java.awt.BorderLayout.SOUTH);

        jPanel1.setLayout(new java.awt.BorderLayout());

        channelTPane.setFont(new java.awt.Font("Serif", 3, 12));
        jPanel11.setLayout(new java.awt.GridLayout(1, 0));

        jPanel11.setBorder(new javax.swing.border.EtchedBorder());
        JPanel903.setLayout(new java.awt.BorderLayout());

        propertiesList.setFont(new java.awt.Font("Default", 0, 10));
        propertiesList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JScrollPane904.setViewportView(propertiesList);

        JPanel903.add(JScrollPane904, java.awt.BorderLayout.CENTER);

        JLabel906.setFont(new java.awt.Font("Serif", 2, 12));
        JLabel906.setText("Properties:");
        JPanel903.add(JLabel906, java.awt.BorderLayout.NORTH);

        jPanel11.add(JPanel903);

        JPanel904.setLayout(new java.awt.BorderLayout());

        inputConnectionsList.setFont(new java.awt.Font("Default", 0, 10));
        inputConnectionsList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JScrollPane905.setViewportView(inputConnectionsList);

        JPanel904.add(JScrollPane905, java.awt.BorderLayout.CENTER);

        JLabel907.setFont(new java.awt.Font("Serif", 2, 12));
        JLabel907.setText("Input Connections:");
        JPanel904.add(JLabel907, java.awt.BorderLayout.NORTH);

        jPanel11.add(JPanel904);

        jPanel12.setLayout(new java.awt.BorderLayout());

        outputConnectionsList.setFont(new java.awt.Font("Default", 0, 10));
        outputConnectionsList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JScrollPane906.setViewportView(outputConnectionsList);

        jPanel12.add(JScrollPane906, java.awt.BorderLayout.CENTER);

        JLabel908.setFont(new java.awt.Font("Serif", 2, 12));
        JLabel908.setText("Output Connections:");
        jPanel12.add(JLabel908, java.awt.BorderLayout.NORTH);

        jPanel11.add(jPanel12);

        JPanel905.setLayout(new java.awt.BorderLayout());

        messageBufferList.setFont(new java.awt.Font("Default", 0, 10));
        messageBufferList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JScrollPane907.setViewportView(messageBufferList);

        JPanel905.add(JScrollPane907, java.awt.BorderLayout.CENTER);

        JLabel909.setFont(new java.awt.Font("Serif", 2, 12));
        JLabel909.setText("Buffer:");
        JPanel905.add(JLabel909, java.awt.BorderLayout.NORTH);

        jPanel11.add(JPanel905);

        JPanel906.setLayout(new java.awt.BorderLayout());

        partialAscendenceList.setFont(new java.awt.Font("Default", 0, 10));
        partialAscendenceList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JScrollPane908.setViewportView(partialAscendenceList);

        JPanel906.add(JScrollPane908, java.awt.BorderLayout.CENTER);

        JLabel910.setFont(new java.awt.Font("Serif", 2, 12));
        JLabel910.setText("PartialAscendenceList");
        JPanel906.add(JLabel910, java.awt.BorderLayout.NORTH);

        jPanel11.add(JPanel906);

        channelTPane.addTab("tab1", jPanel11);

        jPanel1.add(channelTPane, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

        jPanel2.setLayout(new java.awt.BorderLayout());

        channelRootNameLabel.setFont(new java.awt.Font("Dialog", 3, 12));
        channelRootNameLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        channelRootNameLabel.setText("CHANNEL ROOT NAME");
        jPanel2.add(channelRootNameLabel, java.awt.BorderLayout.NORTH);

        channelLocalNameLabel.setFont(new java.awt.Font("Dialog", 3, 12));
        channelLocalNameLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        channelLocalNameLabel.setText("CHANNEL LOCAL NAME");
        jPanel2.add(channelLocalNameLabel, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel2, java.awt.BorderLayout.NORTH);

        pack();
    }//GEN-END:initComponents

    private void getOutChannelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_getOutChannelButtonActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_getOutChannelButtonActionPerformed

    private void formInternalFrameClosed(javax.swing.event.InternalFrameEvent evt) {//GEN-FIRST:event_formInternalFrameClosed
    }//GEN-LAST:event_formInternalFrameClosed

    private void formInternalFrameClosing(javax.swing.event.InternalFrameEvent evt) {//GEN-FIRST:event_formInternalFrameClosing
        updater.cancel();
        Iterator it = null;
        it = connectionsIFrameMap.values().iterator();
        while(it.hasNext()){
            ConnectionIFrame frame = (ConnectionIFrame)it.next();
            
        }
    }//GEN-LAST:event_formInternalFrameClosing
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel JLabel906;
    private javax.swing.JLabel JLabel907;
    private javax.swing.JLabel JLabel908;
    private javax.swing.JLabel JLabel909;
    private javax.swing.JLabel JLabel910;
    private javax.swing.JPanel JPanel903;
    private javax.swing.JPanel JPanel904;
    private javax.swing.JPanel JPanel905;
    private javax.swing.JPanel JPanel906;
    private javax.swing.JPanel JPanel910;
    private javax.swing.JScrollPane JScrollPane904;
    private javax.swing.JScrollPane JScrollPane905;
    private javax.swing.JScrollPane JScrollPane906;
    private javax.swing.JScrollPane JScrollPane907;
    private javax.swing.JScrollPane JScrollPane908;
    private javax.swing.JLabel channelLocalNameLabel;
    private javax.swing.JLabel channelRootNameLabel;
    private javax.swing.JTabbedPane channelTPane;
    private javax.swing.JDesktopPane connectionsDesktop;
    private javax.swing.JPanel connectionsPanel;
    private javax.swing.JButton getOutChannelButton;
    private javax.swing.JList inputConnectionsList;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel11;
    private javax.swing.JPanel jPanel12;
    private javax.swing.JPanel jPanel13;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JList messageBufferList;
    private javax.swing.JList outputConnectionsList;
    private javax.swing.JList partialAscendenceList;
    private javax.swing.JList propertiesList;
    // End of variables declaration//GEN-END:variables
    
}
