package dimi.message.buffer;

import dimi.message.Message;
import dimi.channel.connection.DisseminationUnitConnection;
/**
 * Buffer de mensagens de uma conexao.
 * @author mcmoraes
 */
public class ConnectionMessageBuffer extends MessageBufferImpl{
    
    //atributos - INICIO
    private ConnectionMessageBufferMediator mediator = null;
    private DisseminationUnitConnection connection = null;
    //atributos - FIM

    //construtores - INICIO
    /**
     * Creates a new instance of ConnectionMessageBuffer
     * @param connection a conexao que deve ser dona do novo buffer
     */
    public ConnectionMessageBuffer(DisseminationUnitConnection connection) {
        this.mediator = (ConnectionMessageBufferMediator)connection.getConnectionManager().getChannel().getChannelManager().getDisseminationUnitImpl().getMessageBuffer().getMediator();
        this.connection = connection;
    }
    //construtores - FIM

    //setters e getters - INICIO
    /**
     * Getter for property mediator.
     * @return Value of property connection.
     */
    public final MessageBufferMediator getMediator() {
        return mediator;
    }

    /**
     * Getter for property connection.
     * @return Value of property connection.
     */
    public final DisseminationUnitConnection getConnection() {
        return connection;
    }
    //setters e getters - FIM
    
    /**
     * Adiciona uma mensagem ao buffer.
     * @param message a mensagem a ser adicionada ao buffer
     */    
    public final synchronized void add(Message message) {
        mediator.addToConnectionMessageBuffer(this, message);
    }
    
    /**
     * Compara dois buffers de conexao. Dois buffers de conexao sao
     * iguais se as conexoes que os possuirem forem iguais.
     * @param connectionMessageBuffer o buffer de conexao a ser comparado
     * @return true, se forem iguais; false, caso contrario
     */    
    public boolean equals(Object connectionMessageBuffer){
        if(connectionMessageBuffer == null ||
            !(connectionMessageBuffer instanceof ConnectionMessageBuffer)){
            return false;
        }
        ConnectionMessageBuffer other = (ConnectionMessageBuffer)connectionMessageBuffer;
        return connection.equals(other.getConnection());
    }
   

}
