package dimi.message.buffer;

import dimi.message.Message;
import dimi.message.buffer.MessageBufferImpl;
import java.util.Iterator;
import dimi.DisseminationUnitImpl;

/**
 * Buffer utilizado por um disseminador. Possui a caracteristica
 * de possuir tamanho limitado (em bytes).
 * @author mcmoraes
 */
public abstract class DisseminationUnitMessageBuffer extends MessageBufferImpl{
    
    /**
     * Tamanho maximo (em bytes) do buffer.
     */    
    protected long maximumSize;
    /**
     * Tamanho corrente (em bytes) do buffer.
     */    
    protected long currentSize;
    /**
     * O disseminador que possui este buffer.
     */    
    protected DisseminationUnitImpl disseminationUnitImpl = null;
    
    protected DisseminationUnitMessageBufferMediator mediator = null;
    
    /**
     * Constroi um novo buffer de mensagens de um disseminador.
     * @param disseminator o disseminador dono do novo buffer
     * @param maximumSize o tamanho maximo (em bytes) do novo buffer
     */    
    protected DisseminationUnitMessageBuffer(DisseminationUnitImpl disseminationUnitImpl, long maximumSize) {
        this.maximumSize = maximumSize;
        this.currentSize = 0;
        this.mediator = new MessageBufferMediatorImpl();
        this.disseminationUnitImpl = disseminationUnitImpl;
    }

    /**
     * remove a proxima mensagem do buffer
     * @return a proxima mensagem do buffer
     */    
    public Message remove() {
        Message message = null;
        if(!buffer.isEmpty()){
            message = super.remove();
            currentSize -= message.getHeader().getContentSize();
            mediator.removeFromDisseminatorMessageBuffer(message);
        }
        return message;
    }
    
    /**
     * Adiciona uma mensagem ao buffer.
     * @param message a mensagem a ser adicionada
     */    
    public abstract void add(Message message);
    
    /**
     * Getter for property disseminator.
     * @return Value of property disseminator.
     */
    public DisseminationUnitImpl getDisseminationUnit() {
        return disseminationUnitImpl;
    }
            
    public final MessageBufferMediator getMediator() {
        return mediator;
    }

}
