package dimi.message.buffer;

import dimi.message.Message;
import dimi.name.ChannelName;

import java.util.Map;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

/**
 * Mediador da interacao entre os buffers do disseminador e das conexoes.
 * @author mcmoraes
 */
public class MessageBufferMediatorImpl implements ConnectionMessageBufferMediator, DisseminationUnitMessageBufferMediator{
    
    private DisseminationUnitMessageBuffer disseminationUnitMessageBuffer = null;
    
    //nome de canal --> mapa de mensagens do canal --> lista de buffers que contem a mensagen
    private Map channelMessageConnectionMessageBufferMap = null;
    
    /** Cria uma nova conexao de MessageBufferMediator*/
    public MessageBufferMediatorImpl() {
        this.channelMessageConnectionMessageBufferMap = new Hashtable();
    }
    
    public final synchronized void addChannel(ChannelName channelName){
        this.channelMessageConnectionMessageBufferMap.put(channelName, new Hashtable());
    }
    /**
     * Adiciona uma mensagem a um buffer de conexao e a coloca,
     * se necessario, no buffer do disseminador.
     * @param connectionMessageBuffer o buffer de conexao que deve armazenar a mensagem
     * @param message a mensagem
     */    
    public final synchronized void addToConnectionMessageBuffer(ConnectionMessageBuffer connectionMessageBuffer,
                                            Message message){
        
        connectionMessageBuffer.buffer.add(message);
        ChannelName channelName = message.getHeader().getChannelName();
        Map messageMap = (Map)channelMessageConnectionMessageBufferMap.get(channelName);
        List messageBuffers = (List)messageMap.get(message);
        if(messageBuffers == null){
            disseminationUnitMessageBuffer.add(message);
            messageBuffers = new ArrayList();
            messageMap.put(message,messageBuffers);
        }
        messageBuffers.add(connectionMessageBuffer);
        
    }
    
    /**
     * Remove uma mensagem de um buffer de conexao e remove esta mensagem,
     * se necessario, do buffer do disseminador.
     * @param connectionMessageBuffer o buffer de conexao do qual a mensagem sera retirada
     * @param message a mensagem a ser retirada
     */    
    public final synchronized void removeFromConnectionMessageBuffer(ConnectionMessageBuffer connectionMessageBuffer,
                                                Message message){
                                                    
        connectionMessageBuffer.buffer.remove(message);
        ChannelName channelName = message.getHeader().getChannelName();
        Map messageMap = (Map)channelMessageConnectionMessageBufferMap.get(channelName);
        List messageBuffers = (List)messageMap.get(message);
        messageBuffers.remove(connectionMessageBuffer);
        if(messageBuffers.isEmpty()){
            disseminationUnitMessageBuffer.remove(message);
        }
        
    }
    
    /**
     * !!!REVER!!! Remove uma mensagem de um buffer de disseminador e remove esta mensagem,
     * dos buffers de conexao na qual estiver armazenada. !!!REVER!!!
     * @param message a mensagem a ser retirada
     */    
    public final synchronized void removeFromDisseminatorMessageBuffer(Message message) {
        
        ChannelName channelName = message.getHeader().getChannelName();
        Map messageMap = (Map)channelMessageConnectionMessageBufferMap.get(channelName);
        List messageBuffers = (List)messageMap.get(message);
        Iterator it = messageBuffers.iterator();
        while(it.hasNext()){
            ConnectionMessageBuffer buffer = (ConnectionMessageBuffer)it.next();
            buffer.remove(message);
        }
        messageMap.remove(message);
    }
        
}
