package dimi.util.indexer;

import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.rmi.NotBoundException;
import java.rmi.Naming;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.MalformedURLException;
import java.rmi.AlreadyBoundException;
import java.rmi.server.UnicastRemoteObject;

/**
 * Nomeador de objetos distribuidos, localizados em uma mesma maquina.
 */ 
public class IndexerImpl extends UnicastRemoteObject implements Indexer{

    private InetAddress localhost = null;
    private String name = null;
    private long index = 0;

    /**
     * Construtor vazio.
     * @throws RemoteException if failed to export object
     * @throws UnknownHostException if no IP address for the host could be found.
     */    
    public IndexerImpl() throws RemoteException, UnknownHostException{
        super();
        name = "//"+InetAddress.getLocalHost().getHostName()+"/"+Indexer.class.getName();
    }

    /**
     * Retorna um nome unico nesse nomeador a partir de um nome
     * de base.
     * @return o nome unico.
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public synchronized long getNext() throws RemoteException{
        return index++;
    }
    
    /**
     * Disponibiliza o nomeador no registry com o nome
     * "//'InetAddress.getLocalHost().getHostName()'/'Namer.class'".
     * @throws RemoteException if failed to export object
     * @throws MalformedURLException Se o nome do nomeador esta mal formado.
     */    
    public void start() throws RemoteException,MalformedURLException{
        
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }
        Naming.rebind(name,this);
    }

    /**
     * Unbound o nomeador do registry.
     * @throws RemoteException if failed to export object
     * @throws MalformedURLException Se o nome do nomeador esta mal formado.
     * @throws NotBoundException Se o nomeador nao esta bound.
     */    
    public void stop() throws RemoteException,MalformedURLException,NotBoundException{
        Naming.unbind(name);
    }

}
