/*
 * AdvertiseManagerBase.java
 *
 * Created on 17 de Junho de 2004, 14:26
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.services.DiscovererService;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.Worb;
import org.isam.exehda.Exehda;
import org.isam.util.xml.XmlElement;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.UnknownHostException;
import java.io.StringReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

/**
 *
 * @author  egon
 */
public class AdvertiseManagerBase {

    private DatagramSocket unicastSocket;
    private String unicastAddress;
    private Integer unicastPort;
    
    private String multicastProxyAddress;
    private Integer multicastProxyPort;
    
    private MulticastSocket multicastBaseSocket;
    private String multicastBaseAddress;
    private Integer multicastBasePort;
    
    private Hashtable discovererBaseHst;
    private Object lock;
    private String contactAddress;
    
    private Logger logger; 
    private Worb worb;

    private AdvertiseListenerBase advertiseListenerBase;
    private AckListenerBase ackListenerBase;

    
    /**
     * Creates a new instance of AdvertiseManagerBase
     */
    public AdvertiseManagerBase(String unicastAddress, Integer unicastPort, String multicastProxyAddress, Integer multicastProxyPort, String multicastBaseAddress, Integer multicastBasePort, Hashtable discovererBaseHst, Object lock, String contactAddress) throws UnknownHostException, IOException {

        this.logger = (Logger) Exehda.getService(Logger.SERVICE_NAME);
        this.worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);           
        
        this.unicastAddress = unicastAddress;
        this.unicastPort = unicastPort;        
        this.multicastProxyAddress = multicastProxyAddress;
        this.multicastProxyPort = multicastProxyPort;
        this.multicastBaseAddress = multicastBaseAddress;
        this.multicastBasePort = multicastBasePort;
        
        this.discovererBaseHst = discovererBaseHst;
        this.lock = lock;
        this.contactAddress = contactAddress;
        
	try {
            //Create unicast socket for sending advertise and receiving acks from Bases
	    this.unicastSocket = new DatagramSocket(this.unicastPort.intValue(), InetAddress.getByName(this.unicastAddress));
            
            //Create multicast socket for receiving advertises from Bases and Proxies
            InetAddress group = InetAddress.getByName(this.multicastBaseAddress);
            this.multicastBaseSocket = new MulticastSocket(this.multicastBasePort.intValue());
            this.multicastBaseSocket.joinGroup(group);
	} catch(UnknownHostException uhe) {
            uhe.printStackTrace();
	    throw(uhe);
	} catch(IOException ioe) {
            ioe.printStackTrace();
	    throw(ioe);
	}
        
        try {
            this.advertise();
        } catch(IOException ioe) {
            ioe.printStackTrace();
	    System.exit(1);
	} 
        
        this.advertiseListenerBase = new AdvertiseListenerBase();
        this.advertiseListenerBase.start();
        
        this.ackListenerBase = new AckListenerBase();
        this.ackListenerBase.start();
    }

    /**
     * advertise
     */
    public void advertise() throws IOException {
                
	try {
            String msg = this.buildAdvertise().toString();
            
            InetAddress group = InetAddress.getByName(this.multicastBaseAddress);
            DatagramPacket packetSend = new DatagramPacket(msg.getBytes(), msg.length(), group, this.multicastBasePort.intValue());
            this.unicastSocket.send(packetSend);
        
            group = InetAddress.getByName(this.multicastProxyAddress);
            packetSend = new DatagramPacket(msg.getBytes(), msg.length(), group, this.multicastProxyPort.intValue());
            this.unicastSocket.send(packetSend);
        } catch(IOException ioe) {
            ioe.printStackTrace();
       	    throw ioe;
        }        
    }
    
    /**
     * buildAdvertise
     */
    private XmlElement buildAdvertise() {
    
        //creates advertise xml document
        XmlElement xmlRootAdvertise = XmlElement.createDocRoot();
        XmlElement xmlAdvertise = XmlElement.createElement("CONTROL");
        
        XmlElement xmlMessage = XmlElement.createElement("MESSAGE");
        XmlElement xmlSource = XmlElement.createElement("SOURCE");
        XmlElement xmlContactAddress = XmlElement.createElement("CONTACTADDRESS");        
        XmlElement xmlIP = XmlElement.createElement("IP");        
        XmlElement xmlPort = XmlElement.createElement("PORT");
        
        XmlElement xmlMessageValue = XmlElement.createPCData("Advertise");
        XmlElement xmlSourceValue = XmlElement.createPCData("Base");
        XmlElement xmlContactAddressValue = XmlElement.createPCData(this.contactAddress);  
        XmlElement xmlIPValue = XmlElement.createPCData(this.unicastAddress);
        XmlElement xmlPortValue = XmlElement.createPCData(this.unicastPort.toString());
        
        xmlMessage.addChild(xmlMessageValue);
        xmlSource.addChild(xmlSourceValue);
        xmlContactAddress.addChild(xmlContactAddressValue);
        xmlIP.addChild(xmlIPValue);
        xmlPort.addChild(xmlPortValue);
        
        xmlAdvertise.addChild(xmlMessage);
        xmlAdvertise.addChild(xmlSource);
        xmlAdvertise.addChild(xmlContactAddress);
        xmlAdvertise.addChild(xmlIP);
        xmlAdvertise.addChild(xmlPort);

        xmlRootAdvertise.addChild(xmlAdvertise);
        
        return xmlRootAdvertise;
    }
        
    /**
     * buildAck
     */
    private XmlElement buildAck() {
    
        //creates ack xml document
        XmlElement xmlRootAck = XmlElement.createDocRoot();
        XmlElement xmlAck = XmlElement.createElement("CONTROL");

        XmlElement xmlMessage = XmlElement.createElement("MESSAGE");
        XmlElement xmlContactAddress = XmlElement.createElement("CONTACTADDRESS");        

        XmlElement xmlMessageValue = XmlElement.createPCData("Ack");
        XmlElement xmlContactAddressValue = XmlElement.createPCData(this.contactAddress);

        xmlMessage.addChild(xmlMessageValue);
        xmlContactAddress.addChild(xmlContactAddressValue);

        xmlAck.addChild(xmlMessage);
        xmlAck.addChild(xmlContactAddress);         

        xmlRootAck.addChild(xmlAck);
        
        return xmlRootAck;
    }
        
    /**
     * AdvertiseListenerBase
     */
    class AdvertiseListenerBase extends Thread {

        /**
         * run
         */
        public void run() {
        
            try {
	        while (true) {
                
                    // Wait for advertises from Discoverer Bases or Proxies
                    byte[] buf = new byte[1000];
                    DatagramPacket packetRecv = new DatagramPacket(buf, buf.length);
                    multicastBaseSocket.receive(packetRecv);

                    //get information from the xml received
                    XmlElement xmlReceivedRoot = XmlElement.parse(new StringReader(new String(packetRecv.getData())));

                    Vector vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/MESSAGE", null);
                    XmlElement xmlRecMessage = (XmlElement) vec.firstElement();
                    String message = (String) xmlRecMessage.getContents();              

                    vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/SOURCE", null);
                    XmlElement xmlRecSource = (XmlElement) vec.firstElement();
                    String source = (String) xmlRecSource.getContents();
                    
                    if (source.equalsIgnoreCase("Base")) {
                    
                        vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/CONTACTADDRESS", null);
                        XmlElement xmlRecContactAddress = (XmlElement) vec.firstElement();
                        String remoteContactAddress = (String) xmlRecContactAddress.getContents();
                        
                        synchronized(lock) {
                            if ((discovererBaseHst.get(remoteContactAddress) == null) 
                             && (!remoteContactAddress.equalsIgnoreCase(contactAddress))) {

                                DiscovererService discovererTmp = (DiscovererService) worb.lookupService(remoteContactAddress, DiscovererService.class);
                                discovererBaseHst.put(remoteContactAddress, discovererTmp);
                            }
                        }
                    }

                    vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/IP", null);
                    XmlElement xmlRecIP = (XmlElement) vec.firstElement();
                    String remoteIP = (String) xmlRecIP.getContents();

                    vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/PORT", null);
                    XmlElement xmlRecPort = (XmlElement) vec.firstElement();
                    String remotePort = (String) xmlRecPort.getContents();

                    //Send an ack
                    String msg = buildAck().toString();
                    DatagramPacket packetSend = new DatagramPacket(msg.getBytes(), msg.length(), InetAddress.getByName(remoteIP), new Integer(remotePort).intValue());
                    multicastBaseSocket.send(packetSend);
                }            
            } catch (IOException ioe) {
                ioe.printStackTrace();
                System.exit(1);
            }
        }        
    }
    
    /**
     * AckListenerBase
     */
    class AckListenerBase extends Thread {
    
        /**
         * run
         */
        public void run() {
        
            try {
	        while (true) {
                
                    // Wait for acks from Discoverer Bases
                    byte[] buf = new byte[1000];
                    DatagramPacket packetRecv = new DatagramPacket(buf, buf.length);
                    unicastSocket.receive(packetRecv);

                    //get information from the xml received
                    XmlElement xmlReceivedRoot = XmlElement.parse(new StringReader(new String(packetRecv.getData())));

                    Vector vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/MESSAGE", null);
                    XmlElement xmlRecMessage = (XmlElement) vec.firstElement();
                    String message = (String) xmlRecMessage.getContents();              

                    vec = XmlElement.selectByPath(xmlReceivedRoot, "/CONTROL/CONTACTADDRESS", null);
                    XmlElement xmlRecContactAddress = (XmlElement) vec.firstElement();
                    String remoteContactAddress = (String) xmlRecContactAddress.getContents();

                    synchronized(lock) {
                        if ((discovererBaseHst.get(remoteContactAddress) == null) 
                             && (!remoteContactAddress.equalsIgnoreCase(contactAddress))) {

                                DiscovererService discovererTmp = (DiscovererService) worb.lookupService(remoteContactAddress, DiscovererService.class);
                                discovererBaseHst.put(remoteContactAddress, discovererTmp);
                        }
                    }
                }            
            } catch (IOException ioe) {
                ioe.printStackTrace();
                System.exit(1);
            }
        }        
    }
}