/*
 * PreferenceManager.java
 *
 * Created on February 22, 2005, 6:22 PM
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.Exehda;
import org.isam.exehda.services.Logger;
import org.isam.util.xml.XmlElement;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.io.BufferedReader;
import java.io.StringReader;
import java.io.IOException;
import java.util.Vector;
import java.util.StringTokenizer;

/**
 *
 * @author  egon
 */
public class PreferenceManager {

    private static String URL_PREFERENCES_FILE = "avu://isam@cell/perdis-prefs.xml";
    private Logger logger;
    
    /** 
     * Creates a new instance of PreferenceManager 
     */
    public PreferenceManager() {
        
        this.logger = (Logger) Exehda.getService(Logger.SERVICE_NAME);
    }
    
    /**
     * sortResults
     */
    public XmlElement sortResults(XmlElement xmlResults, String type) {
    
        this.logger.log(Logger.Sev.DEBUG, "PreferenceManager", "sortResults", "######## ENTROU ########", null);
        
        Vector orderBy = this.getOrderBy(type);
        if (orderBy == null) {
            return xmlResults;
        }
        
        String attribute = (String) orderBy.get(0);
        String order = (String) orderBy.get(1);
                
        Vector vecSrc = XmlElement.selectByPath(xmlResults,"/RESOURCE",null);
        Vector newVecSrc = new Vector();

        for (int i = 0; i < vecSrc.size(); i++){
            
            //Get the value of the order attribute for one resource
            XmlElement xmlResource = (XmlElement) vecSrc.elementAt(i);
            Vector propsVec = XmlElement.selectByPath(xmlResource, "/PROPERTIES/PROPERTY", null);
            String strValueTmp = null;
            
            for (int j = 0; j < propsVec.size(); j++) {
            
                XmlElement xmlPropTmp = (XmlElement) propsVec.elementAt(j);
                String strPropTmp = xmlPropTmp.getAttribute("name","");
                
                if (strPropTmp.equalsIgnoreCase(attribute)) {
                    strValueTmp = xmlPropTmp.getAttribute("value","");
                    break;
                }
            }
            
            //Insert the resource in the right place on the new vector
            boolean found = false;
            
            if (strValueTmp != null) {
                
                for (int k = 0; k < newVecSrc.size(); k++) {
                
                    //Get the value of the order attribute for another resource
                    XmlElement xmlResource2 = (XmlElement) newVecSrc.elementAt(k);
                    Vector propsVec2 = XmlElement.selectByPath(xmlResource2, "/PROPERTIES/PROPERTY", null);
                    String strValueTmp2 = null;
            
                    for (int m = 0; m < propsVec2.size(); m++) {
            
                        XmlElement xmlPropTmp2 = (XmlElement) propsVec2.elementAt(m);
                        String strPropTmp2 = xmlPropTmp2.getAttribute("name","");
                
                        if (strPropTmp2.equalsIgnoreCase(attribute)) {
                            strValueTmp2 = xmlPropTmp2.getAttribute("value","");
                            break;
                        }
                    }
                
                    //Compare values to find the right index for the first resource
                    if (strValueTmp2 == null) {
                        newVecSrc.add(k, xmlResource);
                        found = true;
                        break;
                    } else {
                        
                        int comp = 0;
                        
                        try {
                            comp = new Long(strValueTmp).compareTo(new Long(strValueTmp2));
                        } catch (NumberFormatException nfe) {
                            comp = strValueTmp.compareTo(strValueTmp2);
                        }
                        
                        if (order.equalsIgnoreCase("desc")) {
                            if (comp >= 0) {
                                newVecSrc.add(k, xmlResource);
                                found = true;
                                break;    
                            }
                        } else {
                            if (comp <= 0) {
                                newVecSrc.add(k, xmlResource);
                                found = true;
                                break;    
                            }
                        }
                    }
                }
            }
            
            if (!found) {
                newVecSrc.add(xmlResource);
            }
        }
        
        //Build the new QueryResult
        XmlElement newXmlResults = XmlElement.createDocRoot();
        newXmlResults = XmlElement.createElement("QUERYRESULT");
        
        for (int i = 0; i < newVecSrc.size(); i++) {
            newXmlResults.addChild((XmlElement) newVecSrc.elementAt(i));
        }
        
        return newXmlResults;
    }
    
    /** 
     *  getOrderBy
     */
    private Vector getOrderBy(String type) {
    
        XmlElement xmlPrefs = this.readFileFromAVU();
        if (xmlPrefs == null) {
            return null;
        }
        
        Vector orderPropertyAndType = null;
        Vector vecOrder = XmlElement.selectByPath(xmlPrefs, "/PREFERENCES/ORDERBY/ORDER", null);

        for (int i = 0; i < vecOrder.size(); i++) {

            XmlElement xmlElement = (XmlElement) vecOrder.get(i);
            String resourceTypeTmp = xmlElement.getAttribute("resourceType", "");

            if (type.equalsIgnoreCase(resourceTypeTmp)) {                    
                orderPropertyAndType = new Vector();
                orderPropertyAndType.add(xmlElement.getAttribute("property", ""));
                orderPropertyAndType.add(xmlElement.getAttribute("orderType", ""));
                break;
            }
        }
        
        return orderPropertyAndType;
    }
    
    /** 
     *  setOrderBy
     */
    public void setOrderBy(String type, String attribute, String order) {

        this.logger.log(Logger.Sev.DEBUG, "PreferenceManager", "setOrderBy", "######## ENTROU ########", null);
        
        XmlElement xmlPrefs = this.readFileFromAVU();
        if (xmlPrefs == null) {
            return;
        }

        boolean found = false;
        Vector vecOrder = XmlElement.selectByPath(xmlPrefs, "/PREFERENCES/ORDERBY/ORDER", null);
        
        for (int i = 0; i < vecOrder.size(); i++) {

            XmlElement xmlOrder = (XmlElement) vecOrder.get(i);
            String resourceTypeTmp = xmlOrder.getAttribute("resourceType", "");

            if (type.equalsIgnoreCase(resourceTypeTmp)) {                    
                
                found = true;
                xmlOrder.setAttribute("property", attribute);
                xmlOrder.setAttribute("orderType", order);
                break;
            }
        }
        
        if (!found) {
            Vector vecOrderBy = XmlElement.selectByPath(xmlPrefs, "/PREFERENCES/ORDERBY", null);
            XmlElement xmlOrderBy = (XmlElement) vecOrderBy.firstElement();
            
            XmlElement xmlOrder = XmlElement.createElement("ORDER");
            xmlOrder.setAttribute("resourceType", type);
            xmlOrder.setAttribute("property", attribute);
            xmlOrder.setAttribute("orderType", order);
                
            xmlOrderBy.addChild(xmlOrder);
        }
        
        this.writeFileToAVU(xmlPrefs);
    }
    
    /**
     * readFileFromAVU
     */
    private XmlElement readFileFromAVU() {

        this.logger.log(Logger.Sev.DEBUG, "PreferenceManager", "readFileFromAVU", "######## ENTROU ########", null);
        
        XmlElement xmlPrefs = null;
        
        try {
            URL url = new URL(this.URL_PREFERENCES_FILE);
            URLConnection urlConnection = url.openConnection();
            urlConnection.setRequestProperty("Isam-Avu-Operation","read");

            BufferedReader br = new BufferedReader(new InputStreamReader(urlConnection.getInputStream()));
            String str = "";
            int aux;

            while((aux = br.read()) != -1) {
                str += (char) aux;
            }
            br.close();

            if (str.trim().equals("")) { 
                xmlPrefs = this.createPreferencesSkeleton();
                this.writeFileToAVU(xmlPrefs);
            } else {
                xmlPrefs = XmlElement.parse(new StringReader(str));
            }
            
        } catch(MalformedURLException mue) {
            mue.printStackTrace();
            return null;
        } catch(IOException ioe) {
            xmlPrefs = this.createPreferencesSkeleton();
            this.writeFileToAVU(xmlPrefs);
        }

        return xmlPrefs;
    }
    
    /**
     * writeFileToAVU
     */
    private void writeFileToAVU(XmlElement file) {
        
        this.logger.log(Logger.Sev.DEBUG, "PreferenceManager", "writeFileToAVU", "######## ENTROU ########", null);
        this.logger.log(Logger.Sev.DEBUG, "PreferenceManager", "writeFileToAVU", file.toString(), null);

         try {
            URL url = new URL(this.URL_PREFERENCES_FILE);
            URLConnection urlConnection = url.openConnection();
            urlConnection.setRequestProperty("Isam-Avu-Operation","write");
            
            BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(urlConnection.getOutputStream()));

            bw.write(file.toString());
	    bw.flush();
	    bw.close();
        
            urlConnection.getInputStream().read();
            
            this.logger.log(Logger.Sev.DEBUG, "PreferenceManager", "writeFileToAVU", "######## SAIU ########", null);
        } catch(MalformedURLException mue) {
            mue.printStackTrace();
        } catch(IOException ioe) {
           ioe.printStackTrace();
        }
    }
    
    /**
     * createPreferencesSkeleton
     */
    private XmlElement createPreferencesSkeleton() {
    
        XmlElement xmlDocRoot = XmlElement.createDocRoot();
        
        XmlElement xmlPreferences = XmlElement.createElement("PREFERENCES");
        XmlElement xmlResultSets = XmlElement.createElement("RESULTSETS");
        XmlElement xmlOrderBy = XmlElement.createElement("ORDERBY");

        xmlPreferences.addChild(xmlResultSets);
        xmlPreferences.addChild(xmlOrderBy);
        xmlDocRoot.addChild(xmlPreferences);
        
        return xmlDocRoot;
    }
    
    /**
     * applyDefaultSelection
     */
    public XmlElement applyDefaultSelection(XmlElement xmlResults) {

        this.logger.log(Logger.Sev.DEBUG, "PreferenceManager", "applyDefaultSelection", "######## ENTROU ########", null);
        
        String defaultResource = this.getDefaultSelection(xmlResults);
        if (defaultResource != null) {
            
            Vector vecResource = XmlElement.selectByPath(xmlResults,"/RESOURCE",null);
            XmlElement xmlResource = null;
            
            for (int i = 0; i < vecResource.size(); i++) {
                
                xmlResource = (XmlElement) vecResource.get(i);
                
                Vector vecName = XmlElement.selectByPath(xmlResource,"/NAME",null);
                XmlElement xmlName = (XmlElement) vecName.firstElement();
                String nameStr = (String) xmlName.getContents();
                
                if (nameStr.equalsIgnoreCase(defaultResource)) {
                    break;
                }
            }
                
            XmlElement newXmlResults = XmlElement.createDocRoot();
            newXmlResults = XmlElement.createElement("QUERYRESULT");    
            newXmlResults.addChild(xmlResource);
                
            return newXmlResults;                
        } else {
            return xmlResults;
        }
    }
    
    /** 
     *  getDefaultSelection
     */
    private String getDefaultSelection(XmlElement xmlResults) {
    
        XmlElement xmlPrefs = this.readFileFromAVU();
        if (xmlPrefs == null) {
            return null;
        }
        
        Vector resourceNamesFromResults = this.getResourceNamesFromResults(xmlResults);
        Vector resourceNamesFromPreferences = this.getResourceNamesFromPreferences(xmlPrefs, this.getTypeFromResults(xmlResults));
        
        if ((resourceNamesFromResults.size() > 0) 
         && (resourceNamesFromPreferences.size() > 0)) {
        
            String defaultResource = (String) resourceNamesFromPreferences.firstElement();
        
            if (resourceNamesFromPreferences.containsAll(resourceNamesFromResults) 
             && resourceNamesFromResults.contains(defaultResource)) {
            
                 return defaultResource;
            } else {
                return null;
            }
        } else {
            return null;
        }
    }
    
    /**
     * setDefaultSelection
     */
    public void setDefaultSelection(XmlElement xmlResults, String defaultName) {
    
        XmlElement xmlPrefs = this.readFileFromAVU();
        if (xmlPrefs == null) {
            return;
        }
        
        String resourceType = this.getTypeFromResults(xmlResults);
        if (resourceType == null) {
            return;
        }
        
        boolean found = false;
        Vector vecResultSet = XmlElement.selectByPath(xmlPrefs, "/PREFERENCES/RESULTSETS/RESULTSET", null);
        
        for (int i = 0; i < vecResultSet.size(); i++) {

            XmlElement xmlResultSet = (XmlElement) vecResultSet.get(i);
            String resourceTypeTmp = xmlResultSet.getAttribute("resourceType", "");
            
            if (resourceType.equalsIgnoreCase(resourceTypeTmp)) {
                
                found = true;                
                
                xmlResultSet.clearContents();
                xmlResultSet.setAttribute("resourceType", resourceTypeTmp);
                
                Vector resourceNamesFromResults = this.getResourceNamesFromResults(xmlResults);
                for (int j = 0; j < resourceNamesFromResults.size(); j++) {
                    
                    String resourceName = (String) resourceNamesFromResults.get(j);
                    XmlElement xmlResource = XmlElement.createElement("RESOURCE");
                    xmlResource.setAttribute("name", resourceName);
                    
                    if (defaultName.equalsIgnoreCase(resourceName)) {
                        xmlResource.setAttribute("default", "yes");
                    }
                    
                    xmlResultSet.addChild(xmlResource);
                }
                
                break;
            }
        }
        
        if (!found) {
            
            Vector vecResultSets = XmlElement.selectByPath(xmlPrefs, "/PREFERENCES/RESULTSETS", null);
            XmlElement xmlResultSets = (XmlElement) vecResultSets.firstElement();
            
            XmlElement xmlResultSet = XmlElement.createElement("RESULTSET");
            xmlResultSet.setAttribute("resourceType", resourceType);
                
            Vector resourceNamesFromResults = this.getResourceNamesFromResults(xmlResults);
            for (int j = 0; j < resourceNamesFromResults.size(); j++) {

                String resourceName = (String) resourceNamesFromResults.get(j);
                XmlElement xmlResource = XmlElement.createElement("RESOURCE");
                xmlResource.setAttribute("name", resourceName);

                if (defaultName.equalsIgnoreCase(resourceName)) {
                    xmlResource.setAttribute("default", "yes");
                }
                
                xmlResultSet.addChild(xmlResource);
            }
            xmlResultSets.addChild(xmlResultSet);
        }
        
        this.writeFileToAVU(xmlPrefs);
    }
    
    /**
     * getResourceNamesFromResults
     */
    private Vector getResourceNamesFromResults(XmlElement xmlResults) {
    
        Vector resourceNamesList = new Vector();
        
        Vector vecName = XmlElement.selectByPath(xmlResults, "/RESOURCE/NAME", null);
        for (int i = 0; i < vecName.size(); i++) {
            
            XmlElement xmlName = (XmlElement) vecName.elementAt(i);
            resourceNamesList.add(xmlName.getContents());
        }
        
        return resourceNamesList;
    }
    
    /**
     * getTypeFromResults
     */
    private String getTypeFromResults(XmlElement xmlResults) {
    
        String resourceType = null;
        Vector vecType = XmlElement.selectByPath(xmlResults, "/RESOURCE/TYPE", null);
        
        if (vecType.size() > 0) {
            XmlElement xmlType = (XmlElement) vecType.firstElement();
            resourceType = (String) xmlType.getContents();
        }

        return resourceType;
    }
    
    /**
     * getResourceNamesFromPreferences
     */
    private Vector getResourceNamesFromPreferences(XmlElement xmlPrefs, String resourceType) {
    
        Vector resourceNamesList = new Vector();
        
        Vector vecResultSet = XmlElement.selectByPath(xmlPrefs, "/PREFERENCES/RESULTSETS/RESULTSET", null);
        for (int i = 0; i < vecResultSet.size(); i++) {
            
            XmlElement xmlResultSet = (XmlElement) vecResultSet.elementAt(i);
            String resourceTypeTmp = xmlResultSet.getAttribute("resourceType", "");
            
            if (!resourceTypeTmp.equalsIgnoreCase(resourceType)) {

                continue;
            } else {
                
                Vector vecResource = XmlElement.selectByPath(xmlResultSet, "/RESOURCE", null);                
                for (int j = 0; j < vecResource.size(); j++) {
                
                    XmlElement xmlResource = (XmlElement) vecResource.get(j);
                    String resourceDefault = xmlResource.getAttribute("default", "");
                    String resourceName = xmlResource.getAttribute("name", "");
                    
                    //The default option is put at first
                    if (resourceDefault.equalsIgnoreCase("yes")) {
                        resourceNamesList.add(0, resourceName);
                    } else {
                        resourceNamesList.add(resourceName);
                    }   
                }
            }
        }
        
        return resourceNamesList;
    }
}