// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import java.util.Vector;
import java.util.NoSuchElementException;
import java.io.Serializable;

public interface CCManager
{
    public final String SERVICE_NAME = "cc-manager";
    
    public static class Tuple
    {
        private Vector v;

        private Tuple( Tuple t )
            {
                v = (Vector) t.v.clone();
            }

        public Tuple()
            {
                v = new Vector();
            }
        
        public Tuple add(Object item)
            {
                v.add(item); return this;
            }
        
        public Object get(int idx) throws NoSuchElementException
            {
                return v.get(idx);
            }
        
        public int length()
            {
                return v.size();
            }
        
        public boolean matches(Tuple pattern, int startIndex)
            {
                    // if the pattern if longer that the tuple, matching fails
                if (pattern.length()>this.length()){
                    return false;
                }
                    // for each element in the pattern, starting at startIndex,
                for (int i=startIndex; i<pattern.length(); i++) {       
                        // compare it to the corresponding element in the tuple 
                    if ( !this.get(i).equals(pattern.get(i)) ){
                            // matching fails
                        return false;
                    }
                }
                    // successfull matching
                return true;
            }
        
        public Object clone()
            {
                return  new Tuple(this);
            }
    }
    

        // TODO: implement Externalizable
    public static class TupleSpace implements Serializable
    {
        public final static long UNBOUND = -1L;
        
        private transient CCManager manager;
        private Object id;
        
        public TupleSpace(Object id)
            {
                this.id = id;
                this.manager = null;
            }

        public void setManager(CCManager m)
            {
                if ( manager == null ) {
                    manager = m;
                }
            }

        public Object getId()
            {
                return id;
            }
        
        public Tuple in(Tuple pattern, boolean remove, long timeout)
            {
                return manager.in(id, pattern, remove, timeout);
            }

        public void out(Tuple t)
            {
                manager.out(id, t);
            }
    }
    
    TupleSpace createSpace();
    
    void destroySpace(TupleSpace space);
    
    Tuple in(Object spaceId, Tuple pattern, boolean remove, long timeout);
    
//     Tuple read(Object spaceName, Tuple pattern, long timeout);
    
    void out(Object spaceId, Tuple t);
    
    TupleSpace joinSpace(Object spaceName);
    
    void leaveSpace(TupleSpace space);
}
