// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Worb;
import java.util.Hashtable;

public class CibImpl
    extends AbstractService
    implements CellInformationBase
{
    final static String SERVICE_NAME = "cib";
    private Hashtable attrs;

    public CibImpl()
        {
            attrs = new Hashtable();
        }
    
    public void
    start()
        {
            Worb worb = (Worb) Exehda.getService("worb");
            worb.exportService(this, CellInformationBase.class, SERVICE_NAME);
        }

    public void
    stop()
        {
                //
                // TODO: unexport service
                //
        }
    
    public void
    addHost(HostId h, String contactAddr)
        {
            log( Sev.DEBUG, "addHost", "entering h={0}, ca={1}", h, contactAddr);
            String key= h.toString() + "#contactAddress";

            attrs.put(h.toString(), "online");
            attrs.put(key, contactAddr);
        }
    
    public void
    removeHost(HostId h)
        {
            attrs.put(h.toString(), "online");
        }
    
    public String
    getHostAttribute(HostId h, String attrName)
        {
            log( Sev.DEBUG, "getHostAttribute", "entering h={0}, ca={1}", h, attrName);
            String key= h.toString() + "#"+ attrName;
            
            return (String) attrs.get(key);
        }
    
    public void
    setHostAttribute(HostId h, String attrName, String attrValue)
        {
            String key= h.toString() + "#"+ attrName;
            
            attrs.put(key, attrValue);
        }

    public void
    addApplication(ApplicationId app, String codeBase)
        {
            log( Sev.DEBUG, "addApplication", "entering app={0}, codebase={1}", app, codeBase);
            String key = "APP::"+app+"#"+"codeBase";
            
            attrs.put(key, codeBase);
        }
    
    public String
    getAppAttribute(ApplicationId app, String attrName)
        {
            log( Sev.DEBUG, "getAppAttribute", "entering app={0}, attrName={1}", app, attrName);
            String key = "APP::"+app+"#"+attrName;

            return (String) attrs.get(key);
        }
}
