// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Worb;
import java.util.Hashtable;

public class CibImplClient
    extends AbstractService
    implements CellInformationBase
{
    final static String SERVICE_NAME = "cib";
    final static String PROP_CIB_URI = "contactAddress";
    private CellInformationBase cibCell;

    public CibImplClient()
        {
        }
    
    public void
    start() throws Exception
        {
            Worb worb = (Worb) Exehda.getService("worb");

            String cibCellUri = Exehda.getServiceProperty(SERVICE_NAME,
                                                          PROP_CIB_URI,
                                                          "//localhost:1980/cib");

            cibCell = (CellInformationBase)
                worb.lookupService(cibCellUri, CellInformationBase.class);

                //
                // TODO: perhaps, monitor should do node registering
                //
                // register this node
            cibCell.addHost(
                HostId.getLocalHost(),
                "//"+HostId.getLocalHost().getInetAddress().getHostAddress()+":1980");
        }

    public void
    stop()
        {
                //
                // TODO: unexport service
                //
        }
    
    public void
    addHost(HostId h, String contactAddr)
        {
            cibCell.addHost(h, contactAddr);
        }
    
    public void
    removeHost(HostId h)
        {
            cibCell.removeHost(h);
        }
    
    public String
    getHostAttribute(HostId h, String attrName)
        {
            return cibCell.getHostAttribute(h, attrName);
        }
    
    public void
    setHostAttribute(HostId h, String attrName, String attrValue)
        {
            cibCell.setHostAttribute(h, attrName, attrValue);
        }

    public void
    addApplication(ApplicationId app, String codeBase)
        {
            cibCell.addApplication(app, codeBase);
        }
    
    public String
    getAppAttribute(ApplicationId app, String attrName)
        {
            return cibCell.getAppAttribute(app, attrName);
        }
}
