// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.logging;

import org.isam.exehda.Exehda;
import org.isam.exehda.services.Service;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.Logger;

import java.text.MessageFormat;
import java.util.Date;

import java.io.PrintStream;
import java.io.FileOutputStream;


public class BasicLogger
    implements Logger, Service
{
    final static String SERVICE_NAME = "logger";

    final static String[] SEV_NAMES = {
        "CRITICAL",
        "ERROR",
        "WARNING",
        "INFO",
        "DEBUG",
        "TRACE"
    };

    int enabledLogSeverityLevel;
    String logHeaderFormat;
    PrintStream logStream;
    Date now;

    public BasicLogger()
        {
            now = new Date();
        }
    
    public void
    start()
        throws java.io.IOException
        {
            enabledLogSeverityLevel = Exehda.getServiceProperty(
                SERVICE_NAME,
                "logLevel",
                Integer.MAX_VALUE);
            logHeaderFormat = Exehda.getServiceProperty(
                SERVICE_NAME,
                "headerFormat",
                "[{0,date,short} {0, time} {1}] at {2}.{3}:" );

            String outputFileName = Exehda.getServiceProperty(
                SERVICE_NAME,
                "output",
                null );

            if ( outputFileName == null ) {
                logStream = System.err;
            }
            else {
                logStream = new PrintStream(
                    new FileOutputStream(outputFileName), false);
            }
        }

    public void
    stop()
        {
            if ( logStream != System.err ) logStream.close();
        }

    public void
    log( int severity, String className, String method, String msg, Object[] params )
        {
            if ( severity <= enabledLogSeverityLevel ) {
                logStream.println();
                logStream.println( getHeader(severity, className, method)
                                    + (( params == null )
                                       ? msg : MessageFormat.format(msg, params)));
                logStream.flush();
                                    
            }
        }

    public void
    log( int severity, String className, String method, Throwable t )
        {
            if ( severity <= enabledLogSeverityLevel ) {
                logStream.println();
                logStream.print(getHeader(severity, className, method));
                t.printStackTrace(logStream);
                logStream.flush();
                                    
            }
        }

    private String
    getHeader( int severity, String className, String method )
        {
            String sevStr = SEV_NAMES[ Math.min( (severity-1) / 1000, 5 ) ] ;

            now.setTime(System.currentTimeMillis());
            
            return MessageFormat.format(logHeaderFormat,
                                        new Object[] {
                                            now, sevStr, className, method
                                        });
        }
}

