// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos;

import org.isam.exehda.Exehda;
import org.isam.exehda.services.Monitor;
import org.isam.exehda.services.Monitor.Sensor;
import org.isam.exehda.services.Monitor.SensorParameter;
// import org.isam.exehda.services.Executor;
// import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.AbstractService;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

public class MetaMonitor
    extends AbstractService
    implements Monitor
{
    public interface RawMonitor
    {
        public void quantumExpired();
        public RawSensor[] getSensors();
    }

    public interface RawSensorData
    {
//         public Class getType();
//         public boolean isBoolean();
//         public boolean isInt();
//         public boolean isFloat();
//         public boolean isString();
//         public boolean booleanValue();
//         public int intValue();
//         public float floatValue();
        public String stringValue();
    }
    
    public interface RawSensor
    {
        public String getName();
        public Class getType();
        public String[] getParameterNames();
        public Class[]  getParameterTypes();
        public boolean isEnabled();
        public boolean isStatic();
        public void setEnabled(boolean enable);
        public void setParameter(String p, Object v);
        public Object getParameter(String p);
        public RawSensorData probe();
    }

    private RawMonitor   natmon;
    private RawMonitor[] appmon;
    private Hashtable    sensorByName;
    private Vector       enabledSensors;
    
    public
    MetaMonitor()
        {
            natmon = null;
            appmon = null;
            sensorByName = new Hashtable();
            enabledSensors = new Vector();
        }

    public void
    start()
        {
            natmon = new NativeMonitor();

            RawSensor[] rs = natmon.getSensors();
            for (int i=0; i<rs.length; i++) {
                String sname = rs[i].getName();

                sensorByName.put(sname, rs[i]);
                if (rs[i].isEnabled()) {
                    enabledSensors.add(rs[i]);
                }
            }
        }

    public void
    stop()
        {
        }

    public Sensor[]
    getSensors()
        {
                // TO DO: better implementation, cachin sensors[]
            Sensor[] sensors = new Sensor[ sensorByName.size() ];

            Enumeration senum = sensorByName.elements();
            for ( int i=0; senum.hasMoreElements(); i++ ) {
                RawSensor rs = (RawSensor) senum.nextElement();
                sensors[i] = new Sensor(rs.getName(), rs.getType());
            }

            return sensors;
        }
    
    public void
    setSensorEnabled(Sensor s, boolean enable)
        {
            RawSensor rs = findRawSensor(s.getName());

            if ( rs.isEnabled() != enable ) {
                rs.setEnabled(enable);

                if (enable) {
                    enabledSensors.add(rs);
                }
                else {
                    enabledSensors.remove(rs);
                }
            }
        }

    public boolean
    isSensorEnabled(Sensor s)
        {
            RawSensor rs = findRawSensor(s.getName());

            return rs.isEnabled();
        }

    public void
    setSensorParameter(SensorParameter p, Object v)
        {
            RawSensor rs = findRawSensor(p.getSensor().getName());

            if ( checkParameterAssignAllowed(p,v) ) {
                rs.setParameter(p.getName(), v);
            }
        }

    public Object
    getSensorParameter(SensorParameter p)
        {
            RawSensor rs = findRawSensor(p.getSensor().getName());

            return rs.getParameter(p.getName());
        }

    public SensorParameter[]
    getSensorParameters(Sensor s)
        {
                // TO DO: better implementation
            RawSensor rs = findRawSensor(s.getName());
            
            String[] pnames = rs.getParameterNames();
            Class[] ptypes = rs.getParameterTypes();

            SensorParameter[] params = new SensorParameter[pnames.length];

            for (int i=0; i< params.length; i++) {
                params[i] = new SensorParameter(s, pnames[i], ptypes[i]);
            }

            return params;
        }

    public Object
    probeSensor(Sensor s)
        {
            RawSensor rs = findRawSensor(s.getName());

            RawSensorData data = rs.probe();

            if ( String.class == rs.getType() ) {
                return data.stringValue();
            }
            else {
                    // TO DO:
                throw new NullPointerException("Not yet implemented");
            }
                
        }
    
    private final RawSensor
    findRawSensor(String sName)
        {
            RawSensor rs = (RawSensor) sensorByName.get(sName);

            if ( rs == null ) {
                throw new IllegalArgumentException("Unknown sensor "+sName);
            }
            else {
                return rs;
            }
        }

    private final boolean
    checkParameterAssignAllowed(SensorParameter p, Object v)
        {
                // TO DO:
            throw new NullPointerException("Not yet implemented");
        }
}












