// Copyright 2002 Rodrigo Araujo Real
//		  Luciano Cavalheiro da Silva
//		  Iara Augustin
//		  Adenauer Correa Yamin
//                Gustavo Frainer
//		  Claudio Fernando Resin Geyer
//
// This file is part of TiPS.
//
// TiPS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TiPS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TiPS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: DataManagerImpl.java 83 2003-12-17 18:04:29Z rreal $
//


/**
 * @file   DataManagerImpl.java
 * @author Rodrigo Real
 * @date   Fri Aug  1 14:04:08 2003
 * 
 * @brief  One implementation of the DataManager of TiPS.
 *         $Id: DataManagerImpl.java 83 2003-12-17 18:04:29Z rreal $
 * 
 */


package org.isam.exehda.services.tips;

import java.util.*;

public class DataManagerImpl
    implements DataManager
{
        /** Associations */
    Hashtable hostTable;
    private final int histSize = 5;
    private long start_time;

    public DataManagerImpl() 
        {
            hostTable = new Hashtable();
            start_time = System.currentTimeMillis();
            System.out.println(";;;;Hostid;Timestamp;Sensor;Constancy;Utility");
        }
    
    public Object getSensorValue(String hostid, String sensorName)
        {
            Hashtable sensorTable = (Hashtable)hostTable.get(hostid);
            Vector sv = (Vector) sensorTable.get(sensorName);
            if ( sv == null) {
                return(null);
            } // end of if ()
            
            return(sv.lastElement());
        }
    
    public Object getSensorHistory(String hostid, String sensorName)
        {
            Hashtable sensorTable = (Hashtable)hostTable.get(hostid);
            Vector hist = new Vector(2);
            hist.add(sensorTable.get(sensorName+"tstamp"));
            hist.add(sensorTable.get(sensorName));
            return(hist);
        }
    

    public synchronized void setSensorValue(String hostid, String sensorName, Object value)
        {
            Vector sv;
            Vector tstamp;
            
            if (hostTable.containsKey(hostid)) {
                Hashtable sensorTable = (Hashtable) hostTable.get(hostid);
                if (sensorTable.containsKey(sensorName)) {
                    sv = (Vector) sensorTable.get(sensorName);
                    tstamp = (Vector) sensorTable.get(sensorName+"tstamp");
                    sv.add(value);
                    tstamp.add(new Long(System.currentTimeMillis()));
                    
                    if (sv.size() > histSize) {
                        sv.removeElementAt(0);
                        tstamp.removeElementAt(0);
                    }                    
                } else {
                    sv = new Vector(histSize);
                    sv.add(value);
                    tstamp = new Vector(histSize);
                    tstamp.add(new Long(System.currentTimeMillis()));
                    
                    sensorTable.put(sensorName,sv);
                    sensorTable.put(sensorName+"tstamp",tstamp);
                }
            } else {
                Hashtable sensorTable = new Hashtable();
                sv = new Vector(histSize);
                sv.add(value);
                tstamp = new Vector(histSize);
                tstamp.add(new Long(System.currentTimeMillis()));

                sensorTable.put(sensorName,sv);
                sensorTable.put(sensorName+"tstamp",tstamp);
                hostTable.put(hostid,sensorTable);
            }
//            dump();
            summary();
        }

    public boolean hasConstancy(String hostid, String sensorName)
        {
            Hashtable sensorTable = (Hashtable)hostTable.get(hostid);
            return(sensorTable.containsKey(sensorName+"constancy"));
        }

    
    public Double getConstancy(String hostid, String sensorName)
        {
            return((Double)getSensorValue(hostid,sensorName+"constancy"));
        }
    
    public void setConstancy(String hostid, String sensorName, Double value)
        {
            setSensorValue(hostid,sensorName+"constancy",value);
        }
    

    public Double getUtility(String hostid)
        {
            return((Double)getSensorValue(hostid,"utility"));
        }
    
    public synchronized void setUtility(String hostid, Double value)
        {
            setSensorValue(hostid,"utility",value);
        }
        
    public synchronized Hashtable getUtilityTable()
        {
            Hashtable hostUt = new Hashtable();
            Double ut;
            String hostid;
            Set hostidSet = hostTable.keySet();
            Iterator hostIt = hostidSet.iterator();
            
            while(hostIt.hasNext()) {
                hostid = (String) hostIt.next();
                ut = getUtility(hostid);
                hostUt.put(hostid,ut);
            }
            
            return(hostUt);
        }

    public synchronized void dump() 
        {
            Set hostidSet;
            Set sensorNameSet;
            String hostKey;
            String sensorKey;
            Iterator hostIt;
            Iterator sensorIt;

            hostidSet = hostTable.keySet();
            hostIt = hostidSet.iterator();

            while(hostIt.hasNext()) {
                hostKey = (String) hostIt.next();
                Hashtable sensorTable = (Hashtable) hostTable.get(hostKey);
                System.out.println(hostKey);
                sensorNameSet = sensorTable.keySet();
                sensorIt = sensorNameSet.iterator();

                while(sensorIt.hasNext()) {
                    sensorKey = (String) sensorIt.next();
                    System.out.println("    "+sensorKey+ " = "
                                       + sensorTable.get(sensorKey));
                }
            }
        }

    public synchronized void summary() 
        {
            Set hostidSet;
            Set sensorNameSet;
            String hostKey;
            String sensorKey;
            Iterator hostIt;
            Iterator sensorIt;

            double utility;
            double constancy;
            double sensorVal;
            
            hostidSet = hostTable.keySet();
            hostIt = hostidSet.iterator();

            while(hostIt.hasNext()) {
                hostKey = (String) hostIt.next();
                Hashtable sensorTable = (Hashtable) hostTable.get(hostKey);
                sensorNameSet = sensorTable.keySet();
                sensorIt = sensorNameSet.iterator();

                try {
                    sensorVal = ((Double)getSensorValue(hostKey, "CPU_OCCUP_IDLE")).doubleValue();
                    constancy = ((Double)getSensorValue(hostKey, "CPU_OCCUP_IDLEconstancy")).doubleValue();
                    utility = ((Double)getSensorValue(hostKey, "utility")).doubleValue();
                } catch (NullPointerException e) {
                    sensorVal = constancy = utility =0;
                } // end of try-catch

//                System.out.println(";;;;"+hostKey + ";" + (int)((System.currentTimeMillis() - start_time)/1000) +  ";" + sensorVal + ";" +  constancy + ";" + utility );
                System.out.println(";;;;"+hostKey + ";" + System.currentTimeMillis() +  ";" + sensorVal + ";" +  constancy + ";" + utility );
                
            }
        }

}

