// Copyright 2002 Rodrigo Araujo Real
//		  Luciano Cavalheiro da Silva
//		  Iara Augustin
//		  Adenauer Correa Yamin
//                Gustavo Frainer
//		  Claudio Fernando Resin Geyer
//
// This file is part of TiPS.
//
// TiPS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TiPS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TiPS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: LearnerStd.java 83 2003-12-17 18:04:29Z rreal $
//




package org.isam.exehda.services.tips;

import java.util.*;

public class LearnerStd
    extends Thread
    implements Learner
{
        /** Associations */
    private DataManager dataM;
    private Double constancy;

    private int threashold;
    private int count;
    private final int period = 1000;

    private Vector hosts;
    private BayesNetwork bn;

    
    public LearnerStd(DataManager dataM, BayesNetwork bn) 
        {
            this.dataM = dataM;
            threashold = 10;
            count = 0;
            hosts = new Vector();
            this.bn = bn;
            start();
        }


    private double
    average(Vector values) 
        {
            int i;
            double average=0;

            for (i=0;i<values.size();i++) {
                average = ((Double)values.elementAt(i)).doubleValue();
            }

            average = average / values.size();
            
            return(average);
        }

    private double
    std(Vector values) 
        {
            double average = average(values);
            Vector dev = new Vector(values.size());
            double val;
            double std=0;
            int i;
            

                // Calculating deviations
            for (i=0;i<values.size();i++) {
                val = ((Double)values.elementAt(i)).doubleValue();
                dev.add(i, new Double(val - average));
            }
                
            for (i=0;i<dev.size();i++) {
                val = ((Double)dev.elementAt(i)).doubleValue() * ((Double)dev.elementAt(i)).doubleValue();
                std += val;
            }
            std = Math.sqrt(std / dev.size());
            System.out.println("=======>>>> Std = " + std);
            
            return(std);
        }
    
    
        /** 
         * Operation
         *
         *  
         */
    public void
    run() 
        {

            String sensorName = new String("CPU_OCCUP_IDLE");
            int i;

            double lastValue;
            double newValue;
            double constancy;
            Vector sv;
            Vector tstamp;
            Vector hist;
            String hostid;
            
            System.out.println("Starting Learner (CVS $Rev: 83 $) ...");
            
            while(true) {
//                System.out.println("Learner looping ...");
                
                    //loops throw all registered hosts to adjust constancy
                for (i = 0 ; i < hosts.size() ; i++) {
                    long tdelta;
                    
                    hostid = (String)hosts.elementAt(i);
                    hist = (Vector) dataM.getSensorHistory(hostid,sensorName);
                    tstamp = (Vector)hist.get(0);
                    sv = (Vector) hist.get(1);

                    if ( tstamp.size() >= 1) {
                        tdelta = System.currentTimeMillis() - 
                            ((Long) tstamp.elementAt(tstamp.size()-1)).longValue();
                    } else {
                        tdelta = 0;
                    }
                    
                    if ( sv.size() >= 2 ) {
                        newValue = ((Double)sv.elementAt(sv.size()-1)).doubleValue();
                        lastValue = ((Double) sv.elementAt(sv.size()-2)).doubleValue();                        
                        constancy = ((Double)dataM.getConstancy(hostid, sensorName)).doubleValue();

                    } else if ( sv.size() == 1) {
                        newValue = ((Double)sv.elementAt(sv.size()-1)).doubleValue();
                        lastValue = newValue;
                        if ( dataM.hasConstancy(hostid, sensorName)) {
                            constancy = ((Double)dataM.getConstancy(hostid, sensorName)).doubleValue();
                        } else {
                            bn.eval(hostid, sensorName);
                            constancy=0;
                        }
                        
                    } else {
                        newValue = 0;
                        lastValue = newValue;
//                        dataM.setConstancy(hostid, sensorName, new Double(0));
                        constancy=0;
                    }
                    
                    if ( (tdelta <= period) &&
                          std(sv) > 10 ) {
                            // subtract 0.1 from constancy
                        System.out.println("Constancy ---");
                        constancy -= 0.1;
                        if (constancy < 0)
                            constancy = 0;
                        dataM.setConstancy(hostid, sensorName, new Double(constancy));
                        bn.eval(hostid, sensorName);
                        count=0;
                    } else {
                        count++;
                    }
                    if (count == 5) {
                            // add 0.1 to constancy
                        System.out.println("Constancy +++");
                        constancy += 0.1;
                        if (constancy > 1)
                            constancy=1;
                        dataM.setConstancy(hostid, sensorName, new Double(constancy));
                        bn.eval(hostid, sensorName);
                        count =0; 
                   }
                }

                try {
                Thread.sleep(period);
                } catch (Exception e) {}

            }
        }
    
    public void setInfo(String hostid, String sensorName)
        {
            if (!hosts.contains(hostid)) {
                hosts.add(hostid);
                System.out.println("Learner added host: " + hostid);
            }

            if (!dataM.hasConstancy(hostid,sensorName)) {
                dataM.setConstancy(hostid, sensorName, new Double(0));
            }
        }
}

