// Copyright 2002 Rodrigo Araujo Real
//		  Luciano Cavalheiro da Silva
//		  Iara Augustin
//		  Adenauer Correa Yamin
//		  Claudio Fernando Resin Geyer
//
// This file is part of ISAM-CS.
//
// ISAM-CS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// ISAM-CS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: TSocketCollector.java 83 2003-12-17 18:04:29Z rreal $
//

package org.isam.exehda.services.tips;

import isam.cs.*;
import isam.cs.sensors.*;
import exehda.cc.Tuple;
import java.util.Hashtable;
import java.net.*;
import java.io.*;


/**
 * Provides a collector implementation that gathers monitoring information from host
 * monitors connected through sockets. This component is compliant with the monitoring
 * architeture and data formats implemented by PRIMOS. Specifically, monitoring data is
 * published in <EVENTS ts="timestamp"></EVENTS> blocks. Each sensor inside the block
 * follows the format <S s="sensorname" v="sensorvalue" />.
 * 
 */
public class TSocketCollector
    extends SocketCollector
{
        /** Associations */
    TInfoDispatcher tid;
    
    
        /**
         * Initilizes a collector object that listens for monitoring data on the
         * specified port and whose output is directed to the specified space.
         *
         * @param port  tcp bind port
         * @param space tuple space
         * @return 
         */

    public TSocketCollector ( int port )
        {
            super(port,null);
        }

    public void setNext (TInfoDispatcher tid ) 
        {
            this.tid = tid;
        }
    

    public void
    pushData ( String sensorName, String nodeName, Object value )
        {
            Tuple tpl = new Tuple(); // data tuple
            Long counter;

            tpl.add(sensorName).add(nodeName).add(value);
            
            System.out.println("Collector: pushData() Data: "+tpl);
            tid.push(tpl);
        }
}

