// Copyright 2002 Rodrigo Araujo Real
//		  Luciano Cavalheiro da Silva
//		  Iara Augustin
//		  Adenauer Correa Yamin
//                Gustavo Frainer
//		  Claudio Fernando Resin Geyer
//
// This file is part of TiPS. TiPS is a Probabilistic Scheduler
//
// TiPS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TiPS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TiPS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: Tips.java 85 2003-12-17 18:12:46Z rreal $
//

package org.isam.exehda.services.tips;

import java.util.*;
import java.rmi.server.UnicastRemoteObject;
import java.lang.reflect.*;
import org.isam.exehda.Exehda;
import org.isam.exehda.services.Service;

    /**
    * TiPS most important class. This class instantiate and keeps the
    * references to all the components of the scheduler. It also
    * permits the substitution of some of these components.
    */
public class Tips
    implements Service
{

    private DataManager dataM;
    private Learner l;
    private BayesNetwork bn;
    private UtilityGen ug;
    final int COLLECTOR_PORT = 29902;
    final String SERVICE_NAME = "scheduler";
    
    
    public Tips()
        {            
            System.out.println("Starting TiPS (CVS $Revision: 85 $) ...");

        }

    public Tips(Properties prop)
        {            
            System.out.println("Starting TiPS (CVS $Revision: 85 $) ...");
        }

    private static void start_registry() 
        {
            try {
                java.rmi.registry.LocateRegistry.createRegistry(1099);
                System.out.println("RMIRegistry criado.");
            } catch (java.rmi.RemoteException re){
                System.out.println("RMIRegistry j existente.");
            }
        }

    public void start()
        throws java.rmi.RemoteException, ClassNotFoundException, InstantiationException,
        IllegalAccessException, InvocationTargetException, NoSuchMethodException
        {

            start_registry();
            
            TSocketCollector collector  = new TSocketCollector(COLLECTOR_PORT);
            
                // Starting the data manager
            Class dataMClass =  Class.forName ("org.isam.exehda.services.tips.DataManagerImpl");
            DataManager dataM = (DataManager) dataMClass.newInstance();

                // Starting the utility generator
            Class ugClass = Class.forName(Exehda.getServiceProperty(SERVICE_NAME,"utilitygen","org.isam.exehda.services.tips.UtilityGenImpl2"));
            Constructor ugConst = ugClass.getDeclaredConstructor(new Class[] {DataManager.class} );
            ug = (UtilityGen) ugConst.newInstance(new Object[] {dataM});            

                // Starting the BayesNetwork
            Class bnClass = Class.forName(Exehda.getServiceProperty(SERVICE_NAME,"bayesnetwork","org.isam.exehda.services.tips.BayesNetworkImpl"));
            System.out.println("Loading " + (bnClass.getInterfaces()[0]).toString());
            Constructor bnConst = bnClass.getDeclaredConstructor(new Class[] {DataManager.class, UtilityGen.class} );
            bn = (BayesNetwork) bnConst.newInstance(new Object[] {dataM,ug});

                // Starting the Learner
            Class lClass = Class.forName(Exehda.getServiceProperty(SERVICE_NAME,"learner","org.isam.exehda.services.tips.LearnerImpl"));
            Constructor lConst = lClass.getDeclaredConstructor(new Class[] {DataManager.class, BayesNetwork.class} );
            l = (Learner) lConst.newInstance(new Object[] {dataM,bn});

                // Starting the dispatcher which communicates the
                // arrival of new data to the related components
            TInfoDispatcher  tid = new TInfoDispatcher(dataM, l,bn);
            collector.setNext(tid);
            
                // Launching the decision maker, which replies to queries for new nodes
            TDecMaker sc = new TDecMakerImpl(dataM,bn);
            try {
                java.rmi.Naming.rebind("Tips", sc);
            }catch(Exception e){
                throw new java.rmi.RemoteException("TDecMaker",e);
            }

                // Launching components manager, which permits the substitution of classes
            CompManager cm = new CompManagerImpl(this);
            try {
                java.rmi.Naming.rebind("CompManager", cm);
            }catch(Exception e){
                throw new java.rmi.RemoteException("CompManager",e);
            }
        }

    public void stop() 
        {}

    
        /** 
         * Permits the substitution at running time of the class
         * responsible for the processing of the bayesian network
         * 
         * @param new_bn the new BayesNetwork object already instantiated
         */
    public void setBayesNetwork(BayesNetwork new_bn) 
        {
            bn = new_bn;
        }

        /** 
         * Permits the substitution at running time of the class
         * responsible for the learning process
         * 
         * @param new_l the new Learner object already instantiated
         */
    public void setLearner(Learner new_l) 
        {
            l = new_l;
        }

        /** 
         * Permits the substitution at running time of the class
         * responsible for the utility generation
         * 
         * @param new_ug the new UtilityGen object already instantiated
         */
    public void setUtilityGen(UtilityGen new_ug) 
        {
            ug = new_ug;
        }
}
