// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda;

import java.util.Properties;
import org.isam.exehda.services.ServicesManager;
import org.isam.perv.IsamClassLoader;
import org.isam.util.xml.*;

public class Exehda
{
    private static String CFG_URL_PROP = "config.url";
    static ServicesManager servicesManager;
    static volatile Properties exehdaProps;

    static {
            // init middleware props
        exehdaProps = new Properties();
        try {
                // load middleware profiles
            String customURL = System.getProperty("exehda."+CFG_URL_PROP);
            java.net.URL configURL = null;
            if ( customURL == null) {
                
                ClassLoader cl = IsamClassLoader.getInstance();
                configURL = cl.getResource(
                    "org/isam/exehda/exehda-services.xml");
            }
            else {
                configURL = new java.net.URL(customURL);
            }

            Exehda.setProperty(CFG_URL_PROP, configURL.toString());
            
            ProfileManager.mergeProfile(
                XmlElement.parse(
                    new java.io.InputStreamReader(
                        configURL.openStream())));

                // init services manager
            servicesManager = new ServicesManager();
        
                //
                // TODO: fill in default props
                //
        }
        catch (Exception e) {
            System.err.println("EXEHDA initialization failed.");
            e.printStackTrace();
            System.exit(-1);
        }
    }

    public static void
    bootstrap( String profileName )
        {
            System.out.println("Bootstraping EXEHDA services [profile:"
                               + profileName
                               +"]...");
            
            if ( servicesManager.bootstrapServices(profileName) ) {
                System.out.println("EXEHDA is ready.");
            }
            else {
                System.out.println("EXEHDA initialization failed! Exiting...");
            }
        }

    public static void
    shutdown()
        {
            System.out.println("Shuting down EXEHDA services...");
            System.out.println("Done.");
        }

    public static Object
    getService( String serviceName )
        {
            return null;
        }

    public static String
    getProperty( String propName, String defaultValue )
        {
            String r = exehdaProps.getProperty(propName);

            return r == null ? defaultValue : r;
        }

    public static String
    getServiceProperty( String serviceName, String propName, String defaultValue)
        {
            String profile = servicesManager.getActiveProfile();

            propName = profile + ".services." + serviceName + "." + propName;

            String r = exehdaProps.getProperty(propName);

            return r == null ? defaultValue : r;
        }

    public static void
    setProperty( String propName, String propValue )
        {
            exehdaProps.put(propName, propValue);
        }

    public static void
    main( String[] args )
        {
            String optProfile = "node";

            try {
            for ( int idx = 0; idx < args.length; idx++ ) {
                String opt = args[idx];

                if ( "--profile".equals(opt) ) {
                    optProfile = args[++idx];
                }
            }
            }
            catch (ArrayIndexOutOfBoundsException e) {
                System.err.println("Missing parameter: " + e);
                System.exit(-1);   
            }

            bootstrap(optProfile);

            try {
                System.in.read();
            }
            catch (Exception e) {
                    // empty
            }

            shutdown();
            System.exit(0);
        }
}


