// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda;

import org.isam.util.xml.*;
import java.util.Enumeration;

public class ProfileManager
{
    private static void
    setProfileProperty(String profile, String propName, String propValue)
        {
            Exehda.setProperty(profile+"."+propName,
                               propValue);
        }

    private static void
    setServiceProperty(String profile, String svc, String propName, String propValue)
        {
            Exehda.setProperty(profile+".service."+svc+"."+propName,
                               propValue);
        }
    
    public static void
    mergeProfile(XmlElement profileData)
        {
            if ( ! "EXEHDA".equals(profileData.getTag()) ) {
                throw new NullPointerException("Invalid profile");
            }

                // for each <PROFILE> element 
            Enumeration profileEnum = profileData.getChildren();
            while ( profileEnum.hasMoreElements() ) {
                XmlElement profile = (XmlElement) profileEnum.nextElement();
                    
                if ( !"PROFILE".equals(profile.getTag()) ) continue;
                
                String profileName = profile.getAttribute("name",
                                                          null); 
                String svcsToBoot = "";

                    // for each <SERVICE> element
                Enumeration svcEnum = profile.getChildren();
                while ( svcEnum.hasMoreElements() ) {
                    XmlElement service = (XmlElement) svcEnum.nextElement();

                    if ( !"SERVICE".equals(service.getTag()) ) continue;

                    String serviceName = service.getAttribute("name",
                                                               null);
                    String serviceImpl = service.getAttribute("impl",
                                                              null);
                    setServiceProperty(profileName,
                                       serviceName,
                                       "impl", serviceImpl);
                    
                    String loadPolicy = service.getAttribute("loadPolicy",
                                                             null);

                    setServiceProperty(profileName,
                                       serviceName,
                                       "loadPolicy", loadPolicy);
                    
                    if ( "boot".equals(loadPolicy)) {
                        svcsToBoot = ("".equals(svcsToBoot))
                            ? serviceName
                            : svcsToBoot + "," + serviceName;
                    }
                    
                        // for each <PROP> element
                    Enumeration propEnum = service.getChildren();
                    while ( propEnum.hasMoreElements() ) {
                        XmlElement prop = (XmlElement) propEnum.nextElement();

                        if ( !"PROP".equals(prop.getTag()) ) continue;

                        String propName = prop.getAttribute("name",
                                                            null);
                        String propValue = prop.getAttribute("value",
                                                            null);

                        setServiceProperty(profileName,
                                           serviceName,
                                           propName, propValue);
                    } // for each property

                    
                } // for each service

                setProfileProperty(profileName,
                                   "bootstrapServices", svcsToBoot);
            } // for each profile
        }
}
