
package org.isam.exehda.services;

import java.io.*;
import org.isam.exehda.services.HttpService;
import org.isam.exehda.services.HttpService.ServiceRequest;
import org.isam.exehda.services.HttpService.ServiceResponse;

public class AvuService
    extends HttpService
{
    static final String PROP_AVU_OPERATION = "Isam-Avu-Operation";
    static final String PROP_AVU_RESOURCE  = "Resource";
    static final String PROP_AVU_USER      = "Isam-User";
    static final String CONTENT_LENGTH     = "Content-Length";

    static final int    AVU_OP_READ        = 0;
    static final int    AVU_OP_WRITE       = 1;

    static final String AVU_ROOT           = "./";

    static final int    AVU_CELL_INDEX        = 1;
    static final int    AVU_USER_INDEX        = 0;

    static final int    DEFAULT_AVU_PORT = 1977;

    public
    AvuService()
        {
            this(DEFAULT_AVU_PORT);
        }
    
    public
    AvuService( int port )
        {
            super(port);
        }

    protected void
    handleRequest( ServiceRequest req )
        throws IOException
        {
            switch ( getAvuOperation(req) ) {
                case AVU_OP_READ:
                    handleAvuRead(req);
                    break;
                case AVU_OP_WRITE:
                    handleAvuWrite(req);
                    break;
                default:
                    System.err.println("Opps! Unknown AVU operation.");
            }
                // op read
                // check if is a valid file name( do not start with .. for example)
                // open local file
                // output HTTP response header
                // output file contents
        }

    protected boolean
    checkAccess( String fileName, String userName, int op )
        {
                // TODO: check if its a valid filename
            
                // grant read access to all users
            if ( op == AVU_OP_READ ) {
                return true;
            }
            else {
		// grant all the rest to
                return true;
            }
        }

    protected int
    getAvuOperation( ServiceRequest req )
        {
                // defaults to 'read'
            int op = AVU_OP_READ;
            String opString = req.getProperty(PROP_AVU_OPERATION);
	    System.out.println("Operation == " + opString);
            if ( "write".equals(opString) ) {
                op = AVU_OP_WRITE;
            }

            return op;
        }

    protected String[]
    getUserCellPair( String userName)
        {
            String []userCellPair = new String[2];

	    int atIndex = userName.indexOf("@");
	    //TODO: Throw Exception
	    userCellPair[AVU_USER_INDEX] = userName.substring(0,atIndex);
	    userCellPair[AVU_CELL_INDEX] = userName.substring(atIndex+1);

	    return(userCellPair);

        }

    protected void
    handleAvuRead( ServiceRequest req )
        throws IOException
        {
            String fileName = req.getProperty(PROP_AVU_RESOURCE);
            String userName = req.getProperty(PROP_AVU_USER);

	    String[] userCellPair = getUserCellPair(userName);

	   

            if ( checkAccess(fileName, userName, AVU_OP_READ) ) {

		File file = new File(AVU_ROOT
				 + userCellPair[AVU_CELL_INDEX]
				 + "/"
				 + userCellPair[AVU_USER_INDEX]
				 + ""
				 + fileName);

                ServiceResponse resp =
                    new ServiceResponse(req,
                                        ServiceResponse.OK);

                resp.putHeader("Content-Length", ""+file.length());

		byte[] buff = new byte[4096];
                InputStream ins =
                    new FileInputStream(file);

                int nr;

                while ( (nr = ins.read(buff)) != -1 ) {
                    resp.putData(buff, 0, nr);
                }
                
                resp.close();
            }
            else {
                ServiceResponse resp =
                    new ServiceResponse( req,
                                         ServiceResponse.BAD_REQUEST);
                resp.close();
            }
        }

    protected void
    handleAvuWrite( ServiceRequest req )
	throws IOException{
	
	String fileName = req.getProperty(PROP_AVU_RESOURCE);
	String userName = req.getProperty(PROP_AVU_USER);
	int bytesToRead = Integer.parseInt(req.getProperty(CONTENT_LENGTH));
	
	String[] userCellPair = getUserCellPair(userName);
	
	if ( checkAccess(fileName, userName, AVU_OP_WRITE) ) {
	
	    java.net.Socket clientSocket = req.getClientSocket();

	    InputStream ins = clientSocket.getInputStream();
	    
	    byte[] buff = new byte[4096];
	    OutputStream outs =
		new FileOutputStream(AVU_ROOT
				    + userCellPair[AVU_CELL_INDEX]
				    + "/"
				    + userCellPair[AVU_USER_INDEX]
				    + ""
				    + fileName);
	    
	    int nr = 0;
	    int bytesRead = 0;
	   
	    while (bytesRead < bytesToRead) {
		nr = ins.read(buff);
		outs.write(buff, 0, nr);
		bytesRead += nr;
	    }
	    
	    outs.close();
	    //ins.close();
	   
	    ServiceResponse resp =
		new ServiceResponse(req,
				    ServiceResponse.OK);
	   
	    
	    resp.close();
	}
	else {
	    ServiceResponse resp =
		new ServiceResponse( req,
				     ServiceResponse.BAD_REQUEST);
	    resp.close();
	}
    }

    public static void
    main( String args[] )
        throws Exception
        {
            AvuService service = new AvuService(1977);

            service.start();

            Thread.currentThread().sleep(120000);
            
            service.stop();
        }
}
    
