// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import java.io.*;
import org.isam.exehda.services.HttpService;
import org.isam.exehda.services.HttpService.ServiceRequest;
import org.isam.exehda.services.HttpService.ServiceResponse;

public class BdaService
    extends HttpService
{
    static final String PROP_BDA_OPERATION = "Isam-Bda-Operation";
    static final String PROP_BDA_RESOURCE  = "Resource";
    static final String PROP_BDA_USER      = "Isam-User";

    static final int    BDA_OP_READ        = 0;
    static final int    BDA_OP_WRITE       = 1;

    static final String BDA_ROOT           = "./";

    static final int DEFAULT_BDA_PORT = 1978;

    public
    BdaService()
        {
            this(DEFAULT_BDA_PORT);
        }
    
    public
    BdaService( int port )
        {
            super(port);
        }

    protected void
    handleRequest( ServiceRequest req )
        throws IOException
        {
            switch ( getBdaOperation(req) ) {
                case BDA_OP_READ:
                    handleBdaRead(req);
                    break;
                case BDA_OP_WRITE:
                    handleBdaWrite(req);
                    break;
                default:
                    System.err.println("Opps! Unknown BDA operation.");
            }
                // op read
                // check if is a valid file name( do not start with .. for example)
                // open local file
                // output HTTP response header
                // output file contents
        }

    protected boolean
    checkAccess( String fileName, String userName, int op )
        {
                // TODO: check if its a valid filename
            
                // grant read access to all users
            if ( op == BDA_OP_READ ) {
                return true;
            }
            else {
                return false;
            }
        }

    protected int
    getBdaOperation( ServiceRequest req )
        {
                // defaults to 'read'
            int op = BDA_OP_READ;
            String opString = req.getProperty(PROP_BDA_OPERATION);
            if ( "write".equals(opString) ) {
                op = BDA_OP_WRITE;
            }

            return op;
        }

    protected void
    handleBdaRead( ServiceRequest req )
        throws IOException
        {
            String fileName = req.getProperty(PROP_BDA_RESOURCE);
            String userName = req.getProperty(PROP_BDA_USER);

            if ( checkAccess(fileName, userName, BDA_OP_READ) ) {
                File file = new File(BDA_ROOT+fileName);
                
                ServiceResponse resp =
                    new ServiceResponse(req,
                                        ServiceResponse.OK);
                resp.putHeader("Content-Length", ""+file.length());

                byte[] buff = new byte[4096];
                InputStream ins = new FileInputStream(file);

                int nr;

                while ( (nr = ins.read(buff)) != -1 ) {
                    resp.putData(buff, 0, nr);
                }
                
                resp.close();
            }
            else {
                ServiceResponse resp =
                    new ServiceResponse( req,
                                         ServiceResponse.BAD_REQUEST);
                resp.close();
            }
        }

    protected void
    handleBdaWrite( ServiceRequest req )
        {
        }

    public static void
    main( String args[] )
        throws Exception
        {
            BdaService service = new BdaService(1978);

            service.start();

            Thread.currentThread().sleep(120000);
            
            service.stop();
        }
}
    
