// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import org.isam.perv.IsamClassLoader;
import org.isam.exehda.Exehda;
import org.isam.util.xml.*;
import java.util.StringTokenizer;


public class ServicesManager
{
    private String activeProfile;
    
    public ServicesManager()
        {
            activeProfile = null;
        }

    public boolean
    bootstrapServices( String profileName )
        {
            activeProfile = profileName;
            StringTokenizer services = new StringTokenizer(
                Exehda.getProperty(profileName +".bootstrapServices",""),
                ",",
                false);

            while ( services.hasMoreTokens() ) {
                String svcName = services.nextToken();

                System.err.print("Starting service "+svcName+"...");
                
                if ( !startService( svcName )) {
                    System.out.println("failed.");
                    return false;
                }

                System.out.println("ok.");
            }
            return true;
         
        }
    
    public boolean
    startService( String serviceName )
        {
            try {
                ClassLoader cl = IsamClassLoader.getInstance();

                String serviceImpl = Exehda.getProperty(
                    activeProfile + ".service." + serviceName + ".impl",
                    null);

                if ( serviceImpl == null ) {
                    throw new ClassNotFoundException(
                        "Implementation not defined for service"
                        + serviceName
                        + " in profile " + activeProfile);
                }
            
                Service svc = (Service) cl.loadClass(serviceImpl).newInstance();
            

                svc.start();

                return true;
            }
            catch (Exception e) {
                e.printStackTrace();
                return false;
            }
        }

    public boolean
    stopService( String serviceName )
        {
            return false;
        }

    public Object
    getServiceProxy( String serviceName )
        {
            throw new NullPointerException("Method not yet implemented");
        }

    public String
    getActiveProfile()
        {
            return activeProfile;
        }
}
