package org.isam.exehda.services.dhc.client;

import org.isam.exehda.services.dhc.*;
import org.isam.exehda.services.dhc.server.*;
import org.isam.exehda.*;
import org.isam.util.xml.*;
import java.net.*;
import java.rmi.*;
import java.io.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;


/*
 * KConfigMaker - Retrieves configurations from server
 */

public class KConfigMaker {

    int timeout;
    int waitTime;
    String primosdPath;
    UDPServer hostDaemon;

    /*creates new KConfigMakes with the given parameters*/
    public KConfigMaker(int timeout, String primosdPath){
	this.timeout = timeout;
	this.primosdPath = primosdPath;
    }

    /*creates new KConfigMakes with default parameters*/
    public KConfigMaker(){
	this.timeout = 1000;
	this.primosdPath = "./";
    }
  
    /**start the configuration service in the node**/
    public void start(){
	getConfiguration();
	try{
	    File file = new File("KConf.Host");
	    //if KConf.Host exists launch the process
	    if(file.exists()){
		BufferedReader br = new BufferedReader(new FileReader(file));
		String host = br.readLine();
		hostDaemon = new UDPServer(host);
		hostDaemon.start();
	    }
	}catch(Exception e){e.printStackTrace();}
    }

    /**stop the configuration service in the node**/
    public void stop(){
	try{
	    hostDaemon.makeStop();
	    hostDaemon.interrupt(); 
	}catch(Exception e){e.printStackTrace();}
    }

 
    
    /*Processes the given xml string producing and primos-d.propreties file*/

    public void processXML(String xmlString){
	try{
	    Reader read = new InputStreamReader(new ByteArrayInputStream(xmlString.getBytes()));
	    XmlElement element = XmlElement.parse(read);
	    
	    ProfileManager.mergeProfile(element);
	}catch(Exception e){e.printStackTrace();}
    }
       
    /*looks for a server and tries to get the configuration from it*/
    public void getConfiguration(){
	UDPClient client = new UDPClient(timeout);
	String host = null;
	BufferedReader br =  new BufferedReader ( new InputStreamReader(System.in));
	HostQuerier querier = null;
	String hostName = "Unknown";
	String ipAddr = "0.0.0.0";
	String mac = "";

	//tries to find a configuration server
	while(host == null){
	    host = client.queryHost();
	 
	    if (host == null){
		try{
		    if(querier == null){
			querier = new HostQuerier();
			querier.start();
		    }
		  
		    host = querier.getHost();
	
			
		}catch(Exception e){
		    e.printStackTrace();}
	    }
	}
	if(querier != null){querier.makeStop();}
	//if a host was found...
	if(host != ""){
	    try {
		InetAddress localhost = InetAddress.getLocalHost();
		hostName = localhost.getHostName();
		ipAddr = localhost.getHostAddress();
	    }
	    catch (java.net.UnknownHostException uhe){
		System.out.println("Could not find host. Exiting");
		System.exit(0);
	;
	    }
	    
	    
	    
	    try {
	
		String look = "//"+host+"/Knoppix-ConfigServer";
		ConfigServer server = (ConfigServer) Naming.lookup(look);
		String propreties = server.getConfiguration(hostName,ipAddr,mac);
		/*processa xml*/
		processXML(propreties);
		/*spawn new listener*/
		BufferedWriter bw = new BufferedWriter(new FileWriter("KConf.Host"));
		bw.write(host,0,host.length());
		bw.newLine();
		bw.close();
	    }catch(Exception e){
	    e.printStackTrace();}
	}
	
    }
}
