package org.isam.exehda.services.dhc.server;

import org.isam.exehda.services.dhc.*;
import org.isam.exehda.services.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.util.*;
import java.net.*;
import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;
import java.io.*;


/*
 * ConfigServerImpl - Implementation of the Exehda/Knoppix Cd 
 * Configuration server
 */


public class ConfigServerImpl extends HttpService implements ConfigServer, Service {
    /**properties**/
    static final String PROP_DHC_HOSTNAME = "Isam-Dhc-Hostname";
    static final String PROP_DHC_IP  = "Isam-Dhc-IpAddr";
    static final String PROP_DHC_MAC      = "Isam-Dhc-MacAddr";
    /**defaults**/
    static final int DEFAULT_DHC_PORT = 1976;

    Hashtable skeletons;
    List rules;
    UDPServer server;
    String directory;

    /**Initializes server**/
    public ConfigServerImpl (){
	 super(DEFAULT_DHC_PORT);
	//directory = Exehda.getServiceProperty("dhc","rulesDirectory");
	directory = "./";
	makeRules(directory+"/"+"rules.isam");
	//makeSkeletons();
    }

    

    /**Start the service**/
    public void start(){
	super.start();
	String ipAddr = "" ;
	try {
	    UnicastRemoteObject.exportObject(this);
	    
	    // register the object within the nameservice        
	    Naming.rebind("Knoppix-ConfigServer", this);
	    
	    
	    try {
		InetAddress localhost = InetAddress.getLocalHost();
		ipAddr = localhost.getHostAddress();
	    }
	    catch (java.net.UnknownHostException uhe){
		uhe.printStackTrace();
	    }
	    
	    server = new UDPServer(ipAddr);
	    server.start();
	   
	}catch(Exception e){e.printStackTrace();}
    
    }

    /**stop the service**/
    public void stop(){
	try{
	    UnicastRemoteObject.unexportObject(this,true);
	    server.makeStop();
	    server.interrupt(); 
	}catch(Exception e){e.printStackTrace();}
    }


    /**make the rules from file**/
    private void makeRules(String file){
	rules = new LinkedList();
	skeletons = new Hashtable();
	try{
	    DocumentBuilder db = DocumentBuilderFactory.newInstance().newDocumentBuilder();
	   Document  xml = db.parse(file);
	    
	   NodeList nodeList = xml.getElementsByTagName("RULE");

	   if (nodeList != null) {

	       for (int loopIndex = 0; loopIndex < nodeList.getLength();loopIndex++ ) {
		   
		   NamedNodeMap attributes = nodeList.item(loopIndex).getAttributes();
		   Attr type = (Attr)attributes.getNamedItem("attribute");
		   Attr match = (Attr)attributes.getNamedItem("match");
		   Attr use = (Attr)attributes.getNamedItem("use");
		   skeletons.put(use.getValue(),new Skeleton(directory+"/"+use.getValue()));
		   rules.add(new Rule(type.getValue(),match.getValue(),use.getValue()));
	       }
	       
	       
	   }
	}catch(Exception e){e.printStackTrace();}	
    }

    
    /**Looks up the correct skeleton in the rules list**/
    protected Skeleton getSkeleton(String hostName,String ipAddr,String mac){
	Iterator it = rules.iterator();
	Rule r;
	
	while(it.hasNext()){
	    r = (Rule)it.next();
	    if(r.match(hostName,ipAddr,mac)){
		return((Skeleton)skeletons.get(r.getSkeleton()));
	    }
	}
	return(null);
    }

    /**handle a request for a profile**/
    protected void
	handleRequest( ServiceRequest req )
        throws IOException
    {
	String hostName = req.getProperty(PROP_DHC_HOSTNAME);
	String ipAddr = req.getProperty(PROP_DHC_IP);
	String mac = req.getProperty(PROP_DHC_MAC);

	String profile = getConfiguration(hostName,ipAddr,mac);
	ServiceResponse resp =
	    new ServiceResponse(req,
				ServiceResponse.OK);
	resp.putHeader("Content-Length", ""+profile.getBytes().length);
	
	byte[] buff = new byte[4096];
	InputStream ins = new ByteArrayInputStream(profile.getBytes());
	
	int nr;
	
	while ( (nr = ins.read(buff)) != -1 ) {
	    resp.putData(buff, 0, nr);
	}
	
	resp.close();
    }
    
    /**Get the correct configuration for the machine***/
    public String 
	getConfiguration(String hostName, String ipAddr, String mac) 
	throws RemoteException{
	String s;
	Skeleton skel = getSkeleton(hostName,ipAddr,mac);
	if (skel != null){
	    s = skel.genConfig(hostName,ipAddr,mac);
	}
	else{
	    System.out.println("Could no find a matching rule");
	    s= "NoMatch";
	}
	return(s);
    }
    

}
