// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos;

import org.isam.perv.IsamClassLoader;
import org.isam.exehda.services.Service;
import org.isam.util.xml.*;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Vector;
import java.io.StringReader;
import java.io.IOException;

public class Executor
    implements Service, org.isam.exehda.services.Executor
{
    public void
    start()
        {
                // empty
        }
    
    public void
    stop() throws java.lang.Exception
        {
                // empty
        }
    
    public void
    startApplication(XmlElement launchDescriptor)
        {
            String codeBaseURL = null;
            String mainClassName = null;
            String[] appArgs = null;
            String ownerId = null;
            

            Enumeration docEnum = launchDescriptor.getChildren();
            while ( docEnum.hasMoreElements() ) {
                XmlElement e = (XmlElement) docEnum.nextElement();
                String tag = e.getTag();
                
                if ( "code".equals(tag) ) {
                    Enumeration codeEnum = e.getChildren();
                    while ( codeEnum.hasMoreElements() ) {
                        XmlElement ce = (XmlElement) codeEnum.nextElement();

                        if ( "main".equals(ce.getTag()) ) {
                            mainClassName = ce.getAttribute("class", null);
                        }
                        else if ( "jar".equals(ce.getTag()) ) {
                            codeBaseURL = ce.getAttribute("href", null);
                        }
                    }
                }
                else if ( "parameters".equals(tag) ) {
                    Enumeration paramEnum = e.getChildren();
                    Vector v = new Vector();

                    while ( paramEnum.hasMoreElements() ) {
                        XmlElement p = (XmlElement) paramEnum.nextElement();

                        if ( "param".equals(p.getTag()) ) {
                            v.addElement((String) p.getContents());                    
                        }
                        else {
                                //
                                // TODO: better error handling
                                //
                            System.err.println(p);
                        }
                    }

                    appArgs = new String[v.size()];
                    for ( int i=0; i<appArgs.length; i++ ) {
                        appArgs[i] = (String) v.elementAt(i);
                    }
                }
                else if ( "owner".equals(tag) ) {
                    ownerId = e.getAttribute("id", "no-owner");
                }
            }

            launch(codeBaseURL, mainClassName, appArgs);
        }

    private void
    launch( String codeBaseURL, String mainClassName, String[] args)
        {
            try {
                ClassLoader appClassLoader = new IsamClassLoader(codeBaseURL);
                Class mainClass = appClassLoader.loadClass(mainClassName);
                Class[] mainArgTypes = new Class[] { String[].class };

                Method mainMethod = mainClass.getMethod("main", mainArgTypes);

                mainMethod.invoke(null, new Object[] { args });
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
}
