// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda;


/**
 * Uniquely identifies an EXEHDA Object (formelly an OX) in the system. Unicity is
 * guaranteed by using the ID of the host were the object was first instantiated, what it
 * supposed to be unique in the system, as the parent id and localy generated unique 32
 * bits long interger generated at that host as local id.
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2004-12-16 17:46:04 -0200 (Thu, 16 Dec 2004) $ $Revision: 496 $
 */
public final class ObjectId
    extends HierarchicalId
{
        /**
         * Creates a new <code>ObjectId</code> instance.
         *
         * Assumes HostId.getLocalHost() as the parent ID.
         *
         */
    public ObjectId()
        {
            super(HostId.getLocalHost());
            instanceId = seed++;
        }
    
        /**
         * Returns the base host of this OX (i.e. the host were the OX was first
         * instantiated).
         *
         * Notice, such host may be different from the OX's current location since the OX
         * may have been migrated to another host.
         *
         * @return the OX's base host
         */
    public HostId
    getHost()
        {
            return (HostId) getParentId();
        }


        /**
         * @inheritDoc
         */
    public boolean
    equals(Object o)
        {
            if ( o != null
                 && o instanceof ObjectId ) {
                ObjectId oid = (ObjectId) o;

                return oid.instanceId == instanceId
                    && getParentId().equals(oid.getParentId());
            }
            return false;
        }

    
        /**
         * @inheritDoc
         */
    public int
    hashCode()
        {
            return instanceId;
        }

        /**
         * @inheritDoc
         */
    protected final String
    getType()
        {
            return "ox";
        }
    
        /**
         * @inheritDoc
         */
    protected final Object
    getLocalId()
        {
            return Integer.toHexString(instanceId);
        }
    

        /** local id of this object inside the host */
    private int    instanceId;

    private static int seed = (int) System.currentTimeMillis();
}

