// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import java.util.Vector;
import java.util.NoSuchElementException;
import java.io.Serializable;

public interface CCManager
{
    public final String SERVICE_NAME = "cc-manager";

    public static class Tuple implements java.io.Serializable
    {
        private Vector v;

            /**
             * Constructor
             *
             */
        private Tuple( Tuple t )
            {
                v = (Vector) t.v.clone();
            }
        
            /**
             * Constructor
             *
             */
        public Tuple()
            {
                v = new Vector();
            }
        
            /**
             * Operation
             *
             * @param elem
             * @return Tuple
             */
        public Tuple add(Object item)
            {
                v.add(item); return this;
            }
        
            /**
             * Operation
             *
             * @param idx
             * @return Object
             */
        public Object get(int idx) throws NoSuchElementException
            {
                return v.get(idx);
            }
        
            /**
             * Operation
             *
             * @return int
             */
        public int length()
            {
                return v.size();
            }

            /**
             * Operation
             *
             * @param pattern
             * @param startIndex
             * @return boolean
             */
        public boolean matches(Tuple pattern, int startIndex)
            {
                    // if the pattern if longer that the tuple, matching fails
                if (pattern.length()>this.length()){
                    return false;
                }
                    // for each element in the pattern, starting at startIndex,
                for (int i=startIndex; i<pattern.length(); i++) {       
                        // compare it to the corresponding element in the tuple 
                    if ( !this.get(i).equals(pattern.get(i)) ){
                            // matching fails
                        return false;
                    }
                }
                    // successfull matching
                return true;
            }
        
            /**
             * Operation
             *
             * @return Object
             */
        public Object clone()
            {
                return  new Tuple(this);
            }
        
            /**
             * Operation
             *
             * @return String
             */
        public String toString()
            {
                return v.toString();
            }
    }
    

        // TODO: implement Externalizable
    public static class TupleSpace implements Serializable
    {
        public final static long UNBOUND = -1L;
        
        private transient CCManager manager;
        private Object id;

            /**
             * Operation
             *
             * @param id
             * @return 
             */
        public TupleSpace(Object id)
            {
                this.id = id;
                this.manager = null;
            }

            /**
             * Operation
             *
             * @return Object
             */
        public void setManager(CCManager m)
            {
                if ( manager == null ) {
                    manager = m;
                }
            }

            /**
             * Operation
             *
             * @return Object
             */
        public Object getId()
            {
                return id;
            }
        
            /**
             * Operation
             *
             * @param pattern
             * @param remove
             * @param timeout
             * @return Tuple
             */
        public Tuple in(Tuple pattern, boolean remove, long timeout)
            {
                return manager.in(id, pattern, remove, timeout);
            }

            /**
             * Operation
             *
             * @param Tuple
             */
        public void out(Tuple t)
            {
                manager.out(id, t);
            }
    }

        /**
         * Operation
         *
         * @return TupleSpace
         */
    public TupleSpace createSpace();
    
        /**
         * Operation
         *
         * @param space
         */
    public void destroySpace ( TupleSpace space );
    
        /**
         * Operation
         *
         * @param spaceId
         * @return TupleSpace
         */
    public TupleSpace joinSpace ( Object spaceId );
    
        /**
         * Operation
         *
         * @param space
         */
    public void leaveSpace ( TupleSpace space );
    
        /**
         * Operation
         *
         * @param spaceId
         * @param pattern
         * @param remove
         * @param timeout
         * @return Tuple
         */
    public Tuple in ( Object spaceId, Tuple pattern, boolean remove, long timeout );
    
        /**
         * Operation
         *
         * @param spaceId
         * @param t
         */
    public void out ( Object spaceId, Tuple t );
}
