// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//



package org.isam.exehda.services;

import java.util.Enumeration;
import java.util.Vector;


public interface CellInformationBase
{
    public final static String SERVICE_NAME = "cib";
    
    public static class NameComponent
        implements java.io.Serializable
    {
            /**
             * Constructor
             *
             * @todo change type-instance separator from ':' to '='
             *
             * @param nc
             */
        public NameComponent(String nc)
            {
                int idx = nc.indexOf(':');

                if ( idx < 1 ) {
                    throw new IllegalArgumentException("Malformed name compoment: "+nc);
                }
                String t = nc.substring(0,idx);
                String v = nc.substring(idx+1);
                if ( t == null || v == null ) {
                    throw new IllegalArgumentException("Both type and value must not be null");
                }
                this.type = t.trim();
                this.value = v.trim();
            }
    
            /**
             * Constructor
             *
             * @param t
             * @param v
             */
        public NameComponent(String t, String v)
            {
                if ( t == null || v == null ) {
                    throw new IllegalArgumentException("Both type and value must not be null");
                }
                this.type = t.trim(); this.value = v.trim();
            }

            /**
             * Describe <code>getType</code> method here.
             *
             * @return a <code>String</code> value
             */
        public String getType()
            {
                return type;
            }
    
            /**
             * Describe <code>getValue</code> method here.
             *
             * @return a <code>String</code> value
             */
        public String getValue()
            {
                return value;
            }
    
            /**
             * Describe <code>equals</code> method here.
             *
             * @param o an <code>Object</code> value
             * @return a <code>boolean</code> value
             */
        public boolean equals( Object o )
            {
                if ( (o != null ) && (o instanceof NameComponent) ) {
                    NameComponent nc = (NameComponent) o;

                    if ( type.equals(nc.type) && value.equals(nc.value) ) {
                        return true;
                    }
                }
                return false;
            }
    
            /**
             * Describe <code>hashCode</code> method here.
             *
             * @return an <code>int</code> value
             */
        public int hashCode()
            {
                return type.hashCode() ^ value.hashCode();
            }
    
            /**
             * Describe <code>toString</code> method here.
             *
             * @return a <code>String</code> value
             */
        public String toString()
            {
                return "["+type+':'+value+']';
            }
        
        private String type;
        private String value;
    }

    public static class NameSpace
        implements java.io.Serializable
    {
        public static final NameSpace DEFAULT_NS = new NameSpace("DEFAULT", "DEFAULT" , null);
        
            /**
             * Creates a new <code>NameSpace</code> instance.
             *
             * @param cn a <code>NameComponent</code> value
             * @param parent a <code>NameSpace</code> value
             */
        public NameSpace(NameComponent cn, NameSpace parent)
            {
                this.cn = cn;
                this.parent = parent;

            }
        
            /// TODO: delayed parent parsing
//         private NameSpace (String ns)
//             {
//                 int idx1 = ns.indexOf(':');
//                 int idx2 = ns.indexOf(',',idx1);

//                 if ( idx2 < 0 ) idx2 = ns.length();

//                 String t = ns.substring(0,idx1);
//                 String v = ns.substring(idx1+1,idx2);

//                 this(new NameComponent(t,v), ns.substring(idx2));
//             }
        
            /**
             * Creates a new <code>NameSpace</code> instance.
             *
             * @param t a <code>String</code> value
             * @param v a <code>String</code> value
             * @param parent a <code>NameSpace</code> value
             */
        public NameSpace(String t, String v, NameSpace parent)
            {
                this.cn = new NameComponent(t,v);
                this.parent = parent;
            }
        
            /**
             * Describe <code>getParent</code> method here.
             *
             * @return a <code>NameSpace</code> value
             */
        public NameSpace getParent()
            {
                return parent;
            }
        
            /**
             * Describe <code>equals</code> method here.
             *
             * @param o an <code>Object</code> value
             * @return a <code>boolean</code> value
             */
        public boolean equals( Object o )
            {
                if ( ( o != null ) && (o instanceof NameSpace) ) {
                    NameSpace ns2 = (NameSpace) o;
                    
                    if (cn == null) {
                        if (ns2.cn != null) {
                            return false;
                        }
                    }
                    else if ( ! cn.equals(ns2.cn) ) {
                        return false;
                    }

                    if (parent == null) {
                        if (ns2.parent != null ) {
                            return false;
                        }
                    }
                    else if ( ! parent.equals(ns2.parent) ) {
                        return false;
                    }

                    return true;
                }
                return false;
            }
        
            /**
             * Describe <code>hashCode</code> method here.
             *
             * @return an <code>int</code> value
             */
        public int hashCode()
            {
                return cn.hashCode();
            }
        
            /**
             * Describe <code>toString</code> method here.
             *
             * @return a <code>String</code> value
             */
        public String toString()
            {
                return (cn == null)
                    ? ""
                    : (parent == null) ? cn.toString() : cn.toString()+','+parent.toString();
            }

            /**
             * Describe <code>contains</code> method here.
             *
             * @param ns a <code>NameSpace</code> value
             * @return a <code>boolean</code> value
             */
        public boolean contains(NameSpace ns)
            {
                while ( ns != null ) {
                    ns = ns.getParent();
                    
                    if ( this.equals(ns) ) return true;
                }
                return false;
            }

            /**
             * Describe <code>getName</code> method here.
             *
             * @return a <code>NameComponent</code> value
             */
        public NameComponent getName()
            {
                return this.cn;
            }
        
        NameComponent cn;
        NameSpace parent;
    }

    public static class ResourceName
        implements java.io.Serializable
    {
        
            /**
             * Creates a new <code>ResourceName</code> instance.
             *
             * @param cn a <code>String</code> value
             */
        public ResourceName(String cn)
            {
                this(cn, NameSpace.DEFAULT_NS);
            }

            /**
             * Creates a new <code>ResourceName</code> instance.
             *
             * @param type a <code>String</code> value
             * @param instance a <code>String</code> value
             */
        public ResourceName(String type, String instance)
            {
                this(type, instance, NameSpace.DEFAULT_NS);
            }

            /**
             * Creates a new <code>ResourceName</code> instance.
             *
             * @param type a <code>String</code> value
             * @param instance a <code>String</code> value
             * @param ns a <code>NameSpace</code> value
             */
        public ResourceName(String type, String instance, NameSpace ns)
            {
                if ( ns == null ) {
                    throw new IllegalArgumentException("Resources must belong to a name space.");
                }
                this.canonicalName = new NameComponent(type, instance);
                this.nameSpace = ns;
            }

            /**
             * Creates a new <code>ResourceName</code> instance.
             *
             * @param cn a <code>String</code> value
             * @param ns a <code>NameSpace</code> value
             */
        public ResourceName(String cn, NameSpace ns)
            {
                if ( ns == null ) {
                    throw new IllegalArgumentException("Resources must belong to a name space.");
                }
                this.canonicalName = new NameComponent(cn);
                this.nameSpace = ns;
            }
        
            /**
             * Describe <code>getSimpleName</code> method here.
             *
             * @return a <code>String</code> value
             */
        public String getSimpleName()
            {
                return canonicalName.getValue();
            }
        
            /**
             * Describe <code>getCanonicalName</code> method here.
             *
             * @return a <code>NameComponent</code> value
             */
        public NameComponent getCanonicalName()
            {
                return canonicalName;
            }
        
            /**
             * Describe <code>getNameSpace</code> method here.
             *
             * @return a <code>NameSpace</code> value
             */
        public NameSpace getNameSpace()
            {
                return nameSpace;
            }
        
            /**
             * Describe <code>getDN</code> method here.
             *
             * @return a <code>String</code> value
             */
        public String getDN()
            {
                return canonicalName.toString()+','+nameSpace.toString();
            }
        
            /**
             * Describe <code>equals</code> method here.
             *
             * @param o an <code>Object</code> value
             * @return a <code>boolean</code> value
             */
        public boolean equals(Object o) {
            if ( o instanceof ResourceName ) {
                ResourceName rn = (ResourceName) o;

                if ( this.canonicalName.equals(rn.canonicalName)
                     && this.nameSpace.equals(rn.nameSpace)) {
                    return true;
                }
            }
            return false;
        }
        
            /**
             * Describe <code>toString</code> method here.
             *
             * @return a <code>String</code> value
             */
        public String toString()
            {
                return "{cn="+canonicalName+", ns="+nameSpace+'}';
            }
        
            /**
             * Describe <code>hashCode</code> method here.
             *
             * @return an <code>int</code> value
             */
        public int hashCode()
            {
                return canonicalName.hashCode();
            }

        private NameComponent canonicalName;
        private NameSpace nameSpace;
    }


    public static interface Resource
    {
            /**
             * Describe <code>getName</code> method here.
             *
             * @return a <code>ResourceName</code> value
             */
        public ResourceName getName();
        
            /**
             * Describe <code>getAttribute</code> method here.
             *
             * @param n a <code>String</code> value
             * @return a <code>String</code> value
             */
        public String getAttribute( String n );
    }

    public static abstract class SelectionConstraint
        implements java.io.Serializable
    {
            /**
             * Describe <code>getInstance</code> method here.
             *
             * @param xmldef a <code>String</code> value
             * @return a <code>SelectionConstraint</code> value
             */
        public static SelectionConstraint getInstance( String xmldef )
            {
                throw new NullPointerException("Not yet implemented");
            }

            /**
             * Describe <code>match</code> method here.
             *
             * @param r a <code>Resource</code> value
             * @return a <code>boolean</code> value
             */
        public abstract boolean match( Resource r );
    }

    public static class CompositeSelectionConstraint
        extends SelectionConstraint
    {
        public static final int M_AND = 0;
        public static final int M_OR = 1;
        public static final int M_XOR = 2;

            /**
             * Creates a new <code>CompositeSelectionConstraint</code> instance.
             *
             * @param mode an <code>int</code> value
             */
        public CompositeSelectionConstraint(int mode)
            {
                this.mode = mode;
                constraints = new Vector();
            }
        
            /**
             * Describe <code>addConstraint</code> method here.
             *
             * @param c a <code>SelectionConstraint</code> value
             * @param mode an <code>int</code> value
             */
        public void addConstraint(SelectionConstraint c)
            {
                constraints.addElement(c);
            }
        
            /**
             * Describe <code>match</code> method here.
             *
             * @param r a <code>Resource</code> value
             * @return a <code>boolean</code> value
             */
        public boolean match( Resource r )
            {
                Enumeration elements = constraints.elements();
                if ( mode == M_AND ) {
                    while ( elements.hasMoreElements() ) {
                        SelectionConstraint sc = (SelectionConstraint)elements.nextElement();
                        if ( !sc.match(r) ) {
                            return false ;
                        }
                        return(true);
                    }
                }
                if ( mode == M_OR ) {
                    while ( elements.hasMoreElements() ) {
                        SelectionConstraint sc = (SelectionConstraint)elements.nextElement();
                        if ( sc.match(r) ) {
                            return(true);
                        }
                        return(false);
                    }
                }
                
                if ( mode == M_XOR ) {
                    boolean and = true;
                    boolean or = false;
                    while ( elements.hasMoreElements() ) {
                        SelectionConstraint sc = (SelectionConstraint)elements.nextElement();
                        if ( sc.match(r) ) {
                            or = true;
                        }
                        else {
                            and = false;
                        }
                        if ( (!and) && (or) ) {
                            return(true);
                        }
                        else {
                            return(false);
                        }
                    }
                }
                
                return(false);
            }
                
        public Enumeration constraints()
            {
                return (constraints.elements());
            }
        
        public int getMode()
            {
                return (mode);
            }

        private int mode;
        private Vector constraints;
        
    }

    public class AttributeDefinedConstraint
        extends SelectionConstraint
    {
            /**
             * Creates a new <code>AttributeDefinedConstraint</code> instance.
             *
             * @param attr a <code>String</code> value
             */
        public AttributeDefinedConstraint ( String attr )
            {
                this.attr = attr;
                    //throw new NullPointerException("Not yet implemented");
            }

            /**
             * Describe <code>match</code> method here.
             *
             * @param r a <code>Resource</code> value
             * @return a <code>boolean</code> value
             */
        public boolean match( Resource r )
            {
                if(r.getAttribute(attr)==null){
                    return(false);
                }
                else{
                    return(true);
                }
            }
        
        public String getAttribute()
            {
                return(attr);
            }

        private String attr;
    }

    public class SimpleRegexpConstraint
        extends SelectionConstraint
    {
            /**
             * Creates a new <code>SimpleRegexpConstraint</code> instance.
             *
             * @param attr a <code>String</code> value
             * @param regexp a <code>String</code> value
             */
        public SimpleRegexpConstraint ( String attr, String regexp )
            {
                this.attr = attr;
                this.regexp = regexp;
                    //throw new NullPointerException("Not yet implemented");
            }
  
            /**
             * Describe <code>match</code> method here.
             *
             * @param r a <code>Resource</code> value
             * @return a <code>boolean</code> value
             */
        public boolean match ( Resource r )
            {
                String value;     
                System.out.println(r.getAttribute(attr));
            
                if((value=r.getAttribute(attr))==null){
                    return(false);
                }
                else{
                    System.out.println(regexp);
                    System.out.println(value.matches(regexp));
                    return(value.matches(regexp));
                }
            
            }
    
        public String getAttribute()
            {
                return (attr);
            }
        
        public String getRegexp()
            {
                return (regexp);
            }

        private String attr;
        private String regexp;
    }

        /**
         * Describe <code>select</code> method here.
         *
         * @param filter a <code>SelectionConstraint</code> value
         * @param maxResults an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
	 * @deprecated Use the alterative select(SelectionConstraint, NameSpace, int) form.
         */
    public ResourceName[] select( SelectionConstraint filter, int maxResults );

    public ResourceName[] select( SelectionConstraint filter, NameSpace ns, int maxResults);
    
       /**
         * Describe <code>select</code> method here.
         *
         * @param filter a <code>SelectionConstraint</code> value
         * @param maxResults an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
	 * @deprecated Use the alterative select(SelectionConstraint, NameSpace, int) form.
         */
    public ResourceName[] select(SelectionConstraint sc, NameSpace[] ns, int maxResults);
    
        /**
         * Describe <code>selectByName</code> method here.
         *
         * @param n a <code>String</code> value
         * @param ns a <code>NameSpace</code> value
         * @param maxResults an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] selectByName(String n, NameSpace ns, int maxResults);

        /**
         * Describe <code>selectByType</code> method here.
         *
         * @param t a <code>String</code> value
         * @param ns a <code>NameSpace</code> value
         * @param maxResults an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] selectByType(String t, NameSpace ns, int maxResults);
    
        /**
         * Describe <code>getAttribute</code> method here.
         *
         * @param r a <code>ResourceName</code> value
         * @param a a <code>String</code> value
         * @return a <code>String</code> value
         */
    public String getAttribute(ResourceName r, String a);
    
        /**
         * Describe <code>setAttribute</code> method here.
         *
         * @param r a <code>ResourceName</code> value
         * @param a a <code>String</code> value
         * @param v a <code>String</code> value
         * @return a <code>String</code> value
         */
    public String setAttribute(ResourceName r, String a, String v);
    
        /**
         * Describe <code>getAttributes</code> method here.
         *
         * @param r a <code>ResourceName</code> value
         * @return a <code>String[]</code> value
         */
    public String[] getAttributes(ResourceName r);
    
        /**
         * Describe <code>addResource</code> method here.
         *
         * @param r a <code>ResourceName</code> value
         */
    public void addResource(ResourceName r);
    
        /**
         * Describe <code>removeResource</code> method here.
         *
         * @param r a <code>ResourceName</code> value
         */
    public void removeResource(ResourceName r);
}



