// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import java.io.*;
import org.isam.exehda.Exehda;
import org.isam.exehda.services.HttpService;
import org.isam.exehda.services.HttpService.ServiceRequest;
import org.isam.exehda.services.HttpService.ServiceResponse;
import org.isam.exehda.services.Logger.Sev;
import org.isam.util.xml.*;


// interface Gatekeeper
// {
//     void login();
//     void logout();
// }

public class GatekeeperService
    extends HttpService
{
    final static String SERVICE_NAME = "gatekeeper";
    final static int DEFAULT_PORT = 29901;

    final static String PROP_RESOURCE = "Resource";
    final static String PROP_CONTENT_LENGTH = "Content-Length";
    final static String PROP_USER = "Isam-User";
    
    public GatekeeperService()
        {
            super(Exehda.getServiceProperty(SERVICE_NAME, "port", DEFAULT_PORT));
        }

    protected void
    handleRequest( ServiceRequest req )
        {
            String resource = req.getProperty(PROP_RESOURCE);
            int idx1 = resource.indexOf('/');
            int idx2 = resource.lastIndexOf('/');

            if ( idx1 == idx2 ) {
                    // TODO: malformed request
            }

            String service = resource.substring(idx1+1,idx2);
            String operation = resource.substring(idx2+1);

            if ( SERVICE_NAME.equals(service) ) {
                if ( "login".equals(operation) ) {
                        // TODO: create session
                }
            }
            else {
                    // TODO: check session
                
                    // delegate op to proper service
                if ( "executor".equals(service) ) {
                    if ( "startApplication".equals(operation) ) {
                        Executor executor = (Executor) Exehda.getService("executor");
                        try {
                            DataInputStream in = new DataInputStream(
                                req.getClientSocket().getInputStream());

                            byte[] buff = new byte[
                                Integer.parseInt(
                                    req.getProperty(PROP_CONTENT_LENGTH))];

                            in.readFully(buff);
                            
                            XmlElement launchDesc = XmlElement.parse(
                                new InputStreamReader(
                                        new ByteArrayInputStream(buff)));
                            
                            executor.startApplication(launchDesc);

                            ServiceResponse resp =
                                new ServiceResponse( req,
                                                     ServiceResponse.CREATED);
                            resp.close();

                        }
                        catch (IOException ioe) {
                            ioe.printStackTrace();
                                // TODO: better error handling
                            try {
                                ServiceResponse resp =
                                    new ServiceResponse( req,
                                                         ServiceResponse.BAD_REQUEST);
                                resp.close();
                            }
                            catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }
            }
        }
}
