// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.CellInformationBase.NameComponent;
import org.isam.exehda.services.CellInformationBase.NameSpace;
import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.services.Worb;


import java.util.*;

public class CibImpl
    extends AbstractService
    implements CellInformationBase
{
    final static String PROP_CIB_BACKEND = "backEnd";
    final static String PROP_CIB_PASSWORD = "password";
    final static String PROP_CIB_CELLNAME = "cellName";
    final static String PROP_CIB_NEIGHBORHOOD = "neighborhood";
    
    private String cellName;
    
    public CibImpl()
        {
            String cibCellBackEnd = Exehda.getServiceProperty(CellInformationBase.SERVICE_NAME,
                                                          PROP_CIB_BACKEND,
                                                          "LdapBackEnd"); 
            
            
            cellName = Exehda.getServiceProperty(CellInformationBase.SERVICE_NAME,
                                                          PROP_CIB_CELLNAME,
                                                          "MyCell");
            
            String neighborhood = Exehda.getServiceProperty(CellInformationBase.SERVICE_NAME,
                                                          PROP_CIB_NEIGHBORHOOD,
                                                          "");
            
            if(cibCellBackEnd.equals("LdapBackEnd")){
                String cibCellPassword = Exehda.getServiceProperty(CellInformationBase.SERVICE_NAME,
                                                          PROP_CIB_PASSWORD,
                                                          "senha");
                backEnd = new LdapBackEnd(cibCellPassword,cellName);
            }
            else if(cibCellBackEnd.equals("HashtableBackEnd")){
                backEnd = new HashtableBackEnd();
            }
	    else {
	        throw new NullPointerException("Unsupported CIB back-end:"+cibCellBackEnd);
	    }
            
            createCellName(cellName);
            createNeighborhood(neighborhood);
            backEnd.addResource(new ResourceName("ugi:ksfdfds"));
            backEnd.setAttribute(new ResourceName("ugi:ksfdfds"),"teste","teste");
            NameSpace ns = new NameSpace("cell","MyCell",null);
            String att = this.getAttribute(new ResourceName("ugi","ksfdfds",new NameSpace("DEFAULT","DEFAULT",ns)), "teste");
            System.out.println("O Atributo:"+att);
    }
    
    private void createCellName(String name){
        ResourceName rn = new ResourceName("cell",name,new NameSpace("","",null));
        backEnd.addResource(rn);
    }
    
    private void createNeighborhood(String neighborhood){
        String[] neighbors = neighborhood.split(";");
        if((neighbors.length > 0)&&(!neighbors[0].equals(""))){
            for(int i = 0; i < neighbors.length; i++){
                createNeighbor(neighbors[i]);
            }
        }
    }
    
    private void createNeighbor(String neighbor){
        String cellName = "";
        
        Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);

        CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(neighbor, CellInformationBase.class);
        
        ResourceName[] rns = cibCell.selectByType("cell", null, 1);
        
        if(rns.length >0){
            cellName = rns[0].getCanonicalName().getValue();
            ResourceName rn = new ResourceName("neighbor",cellName,new NameSpace("","",null));
            backEnd.addResource(rn);
            backEnd.setAttribute(rn, "contactAddress", neighbor);
        }
    }
    
    
    private String findCellContactAddress(String cell){
        ResourceName[] currentCells;
        ResourceName[] newCells;
        List newCellList;
        Hashtable visited = new Hashtable();
        
        currentCells = backEnd.selectByType("neighbor",null,-1);
        
        Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
        
        while(true){
            newCellList = new LinkedList();
            for(int i = 0; i < currentCells.length; i++){
                System.out.println("Cell "+i+":"+currentCells[i].getCanonicalName().getValue());
                if(currentCells[i].getCanonicalName().getValue().equals(cell)){
                    System.out.println("got here******");
                    return(backEnd.getAttribute(currentCells[i],"contactAddress"));
                }
            }
            for(int i = 0; i < currentCells.length; i++){
                String contactAddress = (backEnd.getAttribute(currentCells[i],"contactAddress"));
                CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(contactAddress, CellInformationBase.class);
                newCells = cibCell.selectByType("neighbor",null,-1);
                for(int j = 0; j < newCells.length; i++){
                    if(!visited.containsKey(newCells[i].getCanonicalName().getValue())){
                        newCellList.add(newCells[i]);
                        visited.put(newCells[i].getCanonicalName().getValue(),"");
                    }
                }
            }
            
            if(newCellList.size() == 0){
                return(null);
            }
            currentCells = new ResourceName[newCellList.size()];
            Iterator it = newCellList.iterator();
            for(int i= 0;it.hasNext(); i++){
                currentCells[i] = (ResourceName)it.next();
            }
        }
        
    }
    
        /**
         * Finds the name of the Cell to which the ResourceName belongs
         *
         * @param  rn  the resource name 
         *         
         * @return a String with the name of the cell to which the given ResourceName
         * belongs to. Defaults to the current cell's name if the resource do not
         * explicitly defines a cell.
         */
    private String getCell(ResourceName rn){
        NameSpace ns = rn.getNameSpace();
        while(ns != null){
            if(ns.getName().getType().equals("cell")){
                return(ns.getName().getValue());
            }
            ns = ns.getParent();
        }
        return this.cellName;
        // return(null);
    }
    
      /**
    * Finds the name of the Cell to which the NameSpace belongs
    *
    * @param  rn  the name space
    *         
    * @return  a String with the name of the cell to which the given NameSpace belongs to. Null if it doesn't belong to any cell.
    */
    private String getCell(NameSpace ns){
        while(ns != null){
            if(ns.getName().getType().equals("cell")){
                return(ns.getName().getValue());
            }
            ns = ns.getParent();
        }
        return(null);
    }
    
    public void
    start()
        {
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            worb.exportService(this,
                               CellInformationBase.class,
                               CellInformationBase.SERVICE_NAME);
        }

    public void
    stop()
        {
                //
                // TODO: unexport service
                //
        }

    public ResourceName[] select(SelectionConstraint sc, int maxResults)
        {
            log(Sev.DEBUG, "select", "selecting resources that match constraints={0}", sc);
            return backEnd.select(sc, null, maxResults);
        }
    
    public ResourceName[] select(SelectionConstraint sc, NameSpace[] ns, int maxResults)
        {
            log(Sev.DEBUG, "select", "selecting resources that match constraints={0}", sc);
            int totalResults = 0;
            List resultList = new LinkedList();
            for(int i = 0; i < ns.length; i++){
                if((getCell(ns[i]).equals(cellName))||(getCell(ns[i]) == null)){
		   // FIX ME: should forward ns[i] to backend
                   ResourceName[] rs = backEnd.select(sc,null,maxResults);
                   resultList.add(rs);
                   totalResults += rs.length;
                }
                else{
                    Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
                    String contactAddress = findCellContactAddress(getCell(ns[i]));
                    if(contactAddress == null){
                        return(null);
                    }
                    CellInformationBase cibCell = (CellInformationBase)
                    worb.lookupService(contactAddress, CellInformationBase.class);
		    // FIX ME: should forward ns[i] to remote cell
                    ResourceName[] rs = cibCell.select(sc,maxResults);
                    resultList.add(rs);
                    totalResults += rs.length;
                }
            }
            ResourceName[] results;
            if((totalResults > maxResults)&&(maxResults != -1)){
                totalResults = maxResults;
            }
            results = new ResourceName[totalResults];
            
            for(int i = 0; i < totalResults;){
                Iterator it = resultList.iterator();
                ResourceName[] rs = (ResourceName[])it.next();
                for(int j = 0;((j<rs.length)&&(i<totalResults));i++){
                    results[i] = rs[j];
                }
            }
         
            return (results);
        }

    public ResourceName[] select(SelectionConstraint sc, NameSpace ns, int maxResults) 
        {
	    return this.select(sc, new NameSpace[]{ns}, maxResults);
        }
    
    
    public ResourceName[] selectByName(String n, NameSpace ns, int maxResults)
        {
            log(Sev.DEBUG, "selectByName", "selecting resources with name={0} in name space={1}", n, ns);
            if((getCell(ns).equals(cellName))||(getCell(ns) == null)){
                return backEnd.selectByName(n,ns,maxResults);
            }
            else{
                Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
                String contactAddress = findCellContactAddress(getCell(ns));
                if(contactAddress == null){
                    return(null);
                }
                CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(contactAddress, CellInformationBase.class);
                return(cibCell.selectByName(n,ns,maxResults));
            }
            
            
        }
    
    public ResourceName[] selectByType(String t, NameSpace ns, int maxResults)
        {
            log(Sev.DEBUG, "selectByType", "selecting resource of type={0} in name space={1}", t, ns);
            if((getCell(ns).equals(cellName))||(getCell(ns) == null)){
                return backEnd.selectByType(t,ns,maxResults);
            }
            else{
                Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
                String contactAddress = findCellContactAddress(getCell(ns));
                if(contactAddress == null){
                    return(null);
                }
                CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(contactAddress, CellInformationBase.class);
                return(cibCell.selectByType(t,ns,maxResults));
            }
        }
    
    public String getAttribute(ResourceName r, String a)
        {
            log(Sev.DEBUG, "getAttribute", "attr={0}, resource is {1}", a, r);
            //System.out.println("get cell:"+getCell(r));
            if((getCell(r).equals(cellName))||(getCell(r) == null)){
                return backEnd.getAttribute(r,a);
            }
            else{
                //System.out.println("Entrou aqui");
                Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
                String contactAddress = findCellContactAddress(getCell(r));
                //System.out.println("O contactAddress:"+contactAddress);
                if(contactAddress == null){
                    return(null);
                }
                //System.out.println("Chegou aqui");
                CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(contactAddress, CellInformationBase.class);
                return(cibCell.getAttribute(r,a));
            }
        }
    
    public String setAttribute(ResourceName r, String a, String v)
        {
            log(Sev.DEBUG, "setAttribute", "attr={0}, resource is {1}", a, r);
            if((getCell(r).equals(cellName))||(getCell(r) == null)){
                return backEnd.setAttribute(r,a,v);
            }
            else{
                Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
                String contactAddress = findCellContactAddress(getCell(r));
                if(contactAddress == null){
                    return(null);
                }
                CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(contactAddress, CellInformationBase.class);
                return(cibCell.setAttribute(r,a,v));
            }
        }
    
    public String[] getAttributes(ResourceName r)
        {
            //System.out.println("get cell:"+getCell(r));
            if((getCell(r).equals(cellName))||(getCell(r) == null)){
                return backEnd.getAttributes(r);
            }
            else{
                //System.out.println("Entrou aqui");
                String contactAddress = findCellContactAddress(getCell(r));
                //System.out.println("O contactAddress:"+contactAddress);
                if(contactAddress == null){
                    return(null);
                }
                //System.out.println("Chegou aqui");
                Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
                CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(contactAddress, CellInformationBase.class);
                return(cibCell.getAttributes(r));
            }
        }
    
    public void addResource(ResourceName r)
        {
            log(Sev.DEBUG, "addResource", "Adding resource {0}", r);
            if((getCell(r).equals(cellName))||(getCell(r) == null)){
                backEnd.addResource(r);
            }
            else{
                String contactAddress = findCellContactAddress(getCell(r));
                if(contactAddress == null){
                    return;
                }
                Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
                CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(contactAddress, CellInformationBase.class);
                cibCell.addResource(r);
            }
        }
    
    public void removeResource(ResourceName r)
        {
            if((getCell(r).equals(cellName))||(getCell(r) == null)){
                backEnd.removeResource(r);
            }
            else{
                String contactAddress = findCellContactAddress(getCell(r));
                if(contactAddress == null){
                    return;
                }
                Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
                CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(contactAddress, CellInformationBase.class);
                cibCell.removeResource(r);
            }
        }

    
    private CibBackEnd backEnd;

        ////////////////////////// Old methods ////////////////////////////////
//     public void
//     addHost(HostId h, String contactAddr)
//         {
//             log( Sev.DEBUG, "addHost", "entering h={0}, ca={1}", h, contactAddr);
//             String key= h.toString() + "#contactAddress";

//             attrs.put(h.toString(), "online");
//             attrs.put(key, contactAddr);
//         }
    
//     public void
//     removeHost(HostId h)
//         {
//             attrs.put(h.toString(), "online");
//         }
    
//     public String
//     getHostAttribute(HostId h, String attrName)
//         {
//             log( Sev.DEBUG, "getHostAttribute", "entering h={0}, ca={1}", h, attrName);
//             String key= h.toString() + "#"+ attrName;
            
//             return (String) attrs.get(key);
//         }
    
//     public void
//     setHostAttribute(HostId h, String attrName, String attrValue)
//         {
//             String key= h.toString() + "#"+ attrName;
            
//             attrs.put(key, attrValue);
//         }

//     public void
//     addApplication(ApplicationId app, String codeBase)
//         {
//             log( Sev.DEBUG, "addApplication", "entering app={0}, codebase={1}", app, codeBase);
//             String key = "APP::"+app+"#"+"codeBase";
            
//             attrs.put(key, codeBase);
//         }
    
//     public String
//     getAppAttribute(ApplicationId app, String attrName)
//         {
//             log( Sev.DEBUG, "getAppAttribute", "entering app={0}, attrName={1}", app, attrName);
//             String key = "APP::"+app+"#"+attrName;

//             return (String) attrs.get(key);
//         }

//     private Hashtable attrs;
}
