/*
 * Ldap.java
 *
 * Created on September 3, 2004, 3:28 PM
 */

package org.isam.exehda.services.cib;

import java.util.*;
import javax.naming.*;
import javax.naming.directory.*;
import javax.naming.ldap.*;




/**
 *
 * @author  frainer
 */
public class Ldap {
    
    
    String serverAddress;
    String user;
    String password;
    DirContext dir;
    
    /** Creates a new instance of Ldap */
    public Ldap(String serverAddress, String user, String password) {
        this.serverAddress = serverAddress;
        this.password = password;
        this.user = user;
    }
    
    /**
    * sets up the connection to the ldap server, using the server address, user
    * and password defined in the constructor 
    *      
    * @return void 
    */
    
    public void connect()  {
        // Set up the environment for creating the initial context
        try{
            Hashtable env = new Hashtable();
            System.out.println(password);
            env.put( Context.INITIAL_CONTEXT_FACTORY,
                         "com.sun.jndi.ldap.LdapCtxFactory" );
            env.put( Context.PROVIDER_URL, "ldap://localhost/dc=CELL");//+ serverAddress + ":389/"+"dc=CELL" );
            env.put( Context.SECURITY_PRINCIPAL, user);
            env.put( Context.SECURITY_CREDENTIALS, password);

            // Create the initial context
            dir = new InitialDirContext(env);
        }catch(javax.naming.NamingException e){e.printStackTrace();}
    }
    
    public void createCell() {
        try{
            Hashtable env = new Hashtable();
            System.out.println(password);
            env.put( Context.INITIAL_CONTEXT_FACTORY,
                         "com.sun.jndi.ldap.LdapCtxFactory" );
            env.put( Context.PROVIDER_URL, "ldap://localhost/");//+ serverAddress + ":389/"+"dc=CELL" );
            env.put( Context.SECURITY_PRINCIPAL, user);
            env.put( Context.SECURITY_CREDENTIALS, password);

            // Create the initial context
            dir = new InitialDirContext(env);
            
            this.createIfNotExists("dc=CELL");
        }catch(javax.naming.NamingException e){e.printStackTrace();}
    }
    
    
    /**
    * creates a new resource in the ldap server, if it doesn't exist already
    *
    * @param  node  the string indicating the name and where (in ldap format) the resource is to be created
    *         
    * @return  void    
    */
    public void createIfNotExists(String node){
        try{
                 Attributes myAttrs = new BasicAttributes(true);
                 //um atributo qualquer
                 myAttrs.put("objectclass","isamClass");
                 if(node.indexOf(",")==-1){
                    myAttrs.put("dn",node);
                 }else{
                    myAttrs.put("dn",node.substring(0,node.indexOf(",")));
                 }
                 myAttrs.put("fullDn",node);
                 dir.createSubcontext(node,myAttrs);
          
         } catch ( NameNotFoundException nnfe ) {
                        createIfNotExists(node.substring(node.indexOf(",")+1));
                        createIfNotExists(node);
                        //eat crow
                        //System.err.println( "value has already been bound!" );        
                 
         } catch ( NameAlreadyBoundException nabe ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        //System.err.println( e );
                        e.printStackTrace();
         }         
    }
    
    
    
    /**
    * retrieves from the ldap server the object in the given node
    * 
    *
    * @param  node  the name and place (in ldap format) of the resource that contains the object
    *         
    * @return  the current object in the given resource    
    */
    public Object getObject(String node){
        
        try{
            Object result = dir.lookup(node);
        
            return(result);    
        }catch ( NamingException ne ) {
                        ne.printStackTrace();    
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        e.printStackTrace();
        }  
        return(null);
    }
    
     /**
    * Saves an object in the given ldap resource
    * 
    *
    * @param  node  the name and place (in ldap format) of the resource that is to contain the object
    * @param  o  the object to be saved 
    *       
    * @return  void  
    */
    public void setObject(String node,Object o){
        try{
            dir.bind(node,o);
             
        }catch ( NamingException ne ) {
                        ne.printStackTrace();
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        System.err.println( e );
        }  
        
    }
    
    /**
    * retrieves from the ldap server the value of a given attribute
    * in a given resource
    *
    * @param  node  the name and place (in ldap format) of the resource that contains the attribute
    *         attr the name of the attribute
    *         
    * @return  the current value of the given attribute in the given resource    
    */
    public String getAttribute(String node, String attr){
        try{
                 Attributes result = dir.getAttributes(node,new String[]{attr});
                 return((String)result.get(attr).get());
                 
         } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
	 	e.printStackTrace();
         }  
        return(null);
    }
    
    /**
    * retrieves from the ldap server the value of a given attribute
    * in a given resource, takes into account isamType
    *
    * @param  node  the name and place (in ldap format) of the resource that contains the attribute
    *         attr the name of the attribute
    *         
    * @return  the current value of the given attribute in the given resource    
    */
    public String getAttributeIsam(String node, String attr){
      
        try{
            Attributes result = dir.getAttributes(node,new String[]{"isamType"});
            if(result.get("isamType") != null){
                Enumeration e = result.get("isamType").getAll();
                while(e.hasMoreElements()){
                    String element = (String)e.nextElement();
                    String type = element.substring(0,element.indexOf(":"));
                    String value = element.substring(element.indexOf(":")+1);
                    //System.out.println("");
                    if(type.equals(attr)){
                        return(value);
                    }
                }
            }
            else{
                return(null);
            }
            return(null);//((String)result.get(attr).get());
         } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        e.printStackTrace();
         }  
        return(null);
    }
    
    
    /**
    * retrieves from the lda server all the names of all the attributes
    * of a given resource 
    *
    * @param  node  the name and place (in ldap format) of the resource
    *         
    * @return  an array of String with the names of the attributes in the given resource    
    */
    public String[] getAttributes(String node){
        try{
                 Attributes attributes = dir.getAttributes(node);
                 String[] results = new String[attributes.size()];
                 NamingEnumeration all = attributes.getAll();
                 for(int i=0;all.hasMore();i++){
                     results[i] = (String)(((Attribute)all.next()).get());
                 }
                 return(results);
                 
         } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        e.printStackTrace();
         }  
        return(new String[0]);
    }
    
    
    /**
    * retrieves from the ldap server all the names of all the attributes
    * of a given resource. Takes into account resources with type
    * isamType 
    *
    * @param  node  the name and place (in ldap format) of the resource
    *         
    * @return  an array of String with the names of the attributes in the given resource    
    */
    public String[] getAttributesIsam(String node){
        try{
                 Attributes attributes = dir.getAttributes(node);
                 List preliminarResults = new LinkedList();
                 //String[] results = new String[attributes.size()];
                 NamingEnumeration all = attributes.getAll();
                 for(int i=0;all.hasMore();i++){
                     Attribute atrib = (Attribute)all.next(); 
                     String type = atrib.getID();
                     if(type.equals("isamType")){
                         Enumeration e = atrib.getAll();
                         while(e.hasMoreElements()){
                            String value = (String)e.nextElement();
                            System.out.println("\n#############Ldap1 : "+value+"\n############\n");
                            value = value.substring(0,value.indexOf(":"));
                            System.out.println("\n#############Ldap2 : "+value+"\n############\n");
                            preliminarResults.add(value);//.substring(value.indexOf(":")+1));
                         }
                     }
                     //preliminarResults.add((String)(atrib.get()));
                 }
                 
                 int i = 0;
                 String[] results = new String[preliminarResults.size()];
                 Iterator it = preliminarResults.iterator();
                 for(i=0;it.hasNext();i++){
                    results[i] = (String)it.next();
                 }
                 return(results);
                 
         } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        e.printStackTrace();
         }  
        return(new String[0]);
    }
    
    /**
    * removes a given resource from the ldap server
    *
    * @param  node the name and place (in ldap format) of the resource that is to be removed
    *         
    * @return  void    
    */
     public void remove(String node){
        try{
                 dir.destroySubcontext(node);
         } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        e.printStackTrace();
         }  
    }
     
     
     /**
    * sets the value a given attribute in a given resorce in tha ldap server
    *
    * @param  node the name and place (in ldap format) of the resource that contains the attribute
    *         attr the name of the attribute
    *         value the value that the attribute is to be set to   
    *        
    * @return  void    
    */
    
     public void setAttribute(String node, String attr, String value){
         try{
            Attributes attributes = dir.getAttributes(node);
            Attributes myAttrs = new BasicAttributes(true);
            Attribute attrib = new BasicAttribute(attr);
            attrib.add(value);
            Enumeration e = attributes.getAll();
            while(e.hasMoreElements()){
                System.out.println("Entrou aqui");
                Attribute a = (Attribute)e.nextElement();
                if(a.getID().equals(attr)){
                    Enumeration e2 = a.getAll();
                    while(e2.hasMoreElements()){
                        attrib.add(((String)e2.nextElement()));
                    }
                }
            }
            myAttrs.put(attrib);
            dir.modifyAttributes(node,DirContext.REPLACE_ATTRIBUTE,myAttrs);
         } catch ( NameAlreadyBoundException nabe ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        //System.err.println( e );
                        e.printStackTrace();
         }
     }
     
    /**
    * finds the resources in the ldap server that have a given name and that are
    * inside a given namespace
    *
    * @param  name the name that the returned resources must have
    *         nameSpace the name space inside of which the returned resources must reside. (in ldap format)  
    *         
    * @return  a list of resources that have the given name and are inside the given name space
    */ 
     
    public List findByName(String name, String nameSpace){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search(nameSpace, 
    "(dn=*="+name+"*)",      // Filter expression                // Filter arguments
    ctls);	
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes().get("fullDn").get());
            }
            return(result);
        } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        //System.err.println( e );
                        e.printStackTrace();
        }    
        return(new ArrayList());
    }
    
    /**
    * finds the resources in the ldap server that have a given name and that are
    * inside a given namespace. Takes into account the possibility of resources with type
    * isamType
    *
    * @param  name the name that the returned resources must have
    *         nameSpace the name space inside of which the returned resources must reside. (in ldap format)  
    *         
    * @return  a list of resources that have the given name and are inside the given name space
    */ 
     
    public List findByNameIsam(String name, String nameSpace){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search(nameSpace, 
    "(|(dn=*="+name+"*)(dn=isamResource=*:"+name+"))",      // Filter expression                // Filter arguments
    ctls);	
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes().get("fullDn").get());
            }
            return(result);
        } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        //System.err.println( e );
                        e.printStackTrace();
        }    
        return(new ArrayList());
    }
    
    /**
    * finds the resources in the ldap server that have a given type and that are
    * inside a given namespace
    *
    * @param  type the type that the returned resources must have
    *         nameSpace the name space inside of which the returned resources must reside. (in ldap format)  
    *         
    * @return  a list of resources that have the given type and are inside the given name space    
    */ 
    public List findByType(String type, String nameSpace){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search(nameSpace, 
    "(dn="+type+"=*)",      // Filter expression                // Filter arguments
    ctls);				    // Default search controls
            
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes().get("fullDn").get());
            }
            return(result);
        } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        e.printStackTrace();
        }    
        return(new ArrayList());
    }
    
    /**
    * finds the resources in the ldap server that have a given type and that are
    * inside a given namespace. Takes into account the possibility
    * of resources with type isamType
    *
    * @param  type the type that the returned resources must have
    *         nameSpace the name space inside of which the returned resources must reside. (in ldap format)  
    *         
    * @return  a list of resources that have the given type and are inside the given name space    
    */ 
    public List findByTypeIsam(String type, String nameSpace){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search(nameSpace, 
    "(|(dn="+type+"=*)(dn=isamResource="+type+":*))",      // Filter expression                // Filter arguments
    ctls);				    // Default search controls
            
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes().get("fullDn").get());
            }
            return(result);
        } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        e.printStackTrace();
        }    
        return(new ArrayList());
    }
    
    /**
    * finds resources in the ldap server using the given ldap search string
    *
    * @param  searchString a ldap search string
    *         
    * @return  a list of resources that were found using the given ldap search string
    */ 
    public List findBySearchString(String searchString){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search("", 
    searchString,      // Filter expression                // Filter arguments
    ctls);				    // Default search controls
            
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes().get("fullDn").get());
            }
            return(result);
        } catch ( NamingException ne ) {
                        ne.printStackTrace();
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        e.printStackTrace();
        }    
        return(new ArrayList());
    }
    
    
    /**
    * finds resources in the ldap server using the given ldap search string
    *
    * @param  searchString a ldap search string
    *         
    * @return  a list of all the attributes of the resources that were found using the given ldap search string
    */ 
    public List findBySearchStringAndReturnAllAttributes(String searchString){
        try{
            //System.out.println(searchString);
            SearchControls ctls = new SearchControls();
          
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search("", 
    searchString,      // Filter expression                // Filter arguments
    ctls);				    // Default search controls
            
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes());
            }
            return(result);
        } catch ( NamingException ne ) {
                        ne.printStackTrace();
        } catch ( Exception e ) {
                        e.printStackTrace();
        }    
        return(new ArrayList());
    }
    
    /**
    * finds resources in the ldap server using the given ldap search string
    *
    * @param  searchString a ldap search string
    *         where the context under which the search is to be performed  
    *         
    * @return  a list of all the attributes of the resources that were found using the given ldap search string
    */ 
    public List findBySearchStringAndReturnAllAttributes(String searchString, String where){
        try{
            //System.out.println(searchString);
            SearchControls ctls = new SearchControls();
          
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search(where, 
    searchString,      // Filter expression                // Filter arguments
    ctls);				    // Default search controls
            
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes());
            }
            return(result);
        } catch ( NamingException ne ) {
                        ne.printStackTrace();
        } catch ( Exception e ) {
                        e.printStackTrace();
        }    
        return(new ArrayList());
    }
    
    /**
    * disconnects from the ldap server. A new connect will be necessary
    * before acessing the server again.
    *       
    * @return  void
    */ 
    
    public void disconnect(){
        try{
            dir.close();
        } catch (NamingException ne){
            ne.printStackTrace();
        }
    }
    
}
