/*
 * LdapBackEnd.java
 *
 * Created on September 3, 2004, 10:43 AM
 */

package org.isam.exehda.services.cib;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.CellInformationBase.NameComponent;
import org.isam.exehda.services.CellInformationBase.NameSpace;
import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.services.CellInformationBase.Resource;
import org.isam.exehda.services.CellInformationBase.SelectionConstraint;
import org.isam.exehda.services.CellInformationBase.CompositeSelectionConstraint;
import org.isam.exehda.services.CellInformationBase.SimpleRegexpConstraint;
import org.isam.exehda.services.CellInformationBase.AttributeDefinedConstraint;
import org.isam.exehda.services.Worb;

import java.util.*;
import javax.naming.directory.*;

/**
 *
 * @author  frainer
 */



/**
 * Ldab Install and Configuration Instructions:
 * <ol>
 *
 * <li> Install (depends on which OS is being used, in the case of Debian an apt-get
 * install slapd will probably work).
 *
 * <li> edit slapd.conf (/etc/ldap/slapd.conf, generally)
 *
 *   <ul>
 *   <li> include the line:
 *        <pre>include         /etc/ldap/schema/java.schema</pre>
 *        below the other lines similar to this one (close to the beginning of the file)
 *
 *   <li> Change the line:
 *        <pre>schemacheck     on</pre>
 *        to:
 *        <pre>schemacheck     off</pre>
 *
 *   <li> Change the line:
 *        <pre>suffix          "alguma-coisa"</pre>
 *        to:
 *        <pre>suffix          "dc=CELL"</pre>
 *
 *   <li> Change the line:
 *        <pre>rootdn "alguma coisa" </pre>
 *        to:
 *        <pre>rootdn "cn=Manager,dc=CELL"</pre>
 *
 *   <li> Change the line:
 *        <pre>rootpw "uma-senha"</pre>
 *        to:
 *        <pre>rootpw "a-senha-que-voce-quer"</pre>
 *   </ul>
 *
 * <li> Edit the file core.schema (/etc/ldap/schema/core.schema, generally), adding the
 * following :
 *    <pre>
 *    attributetype ( 2.5.4.70 NAME 'dn'
 *    DESC 'DN'
 *    SUP name )
 *
 *    attributetype ( 2.5.4.71 NAME 'fullDn'
 *    DESC 'DN full'
 *    SUP name )
 *
 *    attributetype ( 2.5.4.72 NAME 'a'
 *    DESC 'for tests'
 *    SUP name )
 *
 *    attributetype ( 2.5.4.75 NAME 'isamType'
 *    DESC 'for isam'
 *    SUP name )
 *
 *    attributetype ( 2.5.4.79 NAME 'isamResource'
 *    DESC 'for isam'
 *    SUP name )
 *    </pre>
 *  
 *    <p> After the following entry:
 *    <pre>
 *    attributetype ( 2.5.4.41 NAME 'name' 
 *    EQUALITY caseIgnoreMatch
 *    SUBSTR caseIgnoreSubstringsMatch
 *    SYNTAX 1.3.6.1.4.1.1466.115.121.1.15{32768} )
 *    </pre>
 *
 *    <p> If the above entry does not exist, the new lines can be placed at the beginning of
 *    the file.
 *
 * <li> Restart ldap (/etc/init.d/slapd restart, probably)
 *
 * <li> execute org.isam.exehda.services.cib.CibInstaller
 *
 * <li> Done
 * </ol>
 */
 

public class LdapBackEnd implements CibBackEnd{
    
    Ldap ldap;
    Hashtable typeConversion;
    Hashtable typeBackConversion;
    Hashtable attributeConversion;
    Hashtable attributeBackConversion;
    String cellName;
    
    /** Creates a new instance of LdapBackEnd */
    public LdapBackEnd() {
        super();
        ldap = new Ldap("ldap://localhost", "cn=Manager,dc=CELL","senha");
        ldap.connect();
        Properties resourceProperties = (Properties)ldap.getObject("cn=ResourceMap");
        Properties attributeProperties = (Properties)ldap.getObject("cn=AttributeMap");
        prepareMapping(resourceProperties,attributeProperties);
        ldap.disconnect();
    }
    
     /** Creates a new instance of LdapBackEnd */
    public LdapBackEnd(String password) {
        super();
        ldap = new Ldap("ldap://localhost", "cn=Manager,dc=CELL",password);
        ldap.connect();
        Properties resourceProperties = (Properties)ldap.getObject("cn=ResourceMap");
        Properties attributeProperties = (Properties)ldap.getObject("cn=AttributeMap");
        prepareMapping(resourceProperties,attributeProperties);
        ldap.disconnect();
    }
    
     /** Creates a new instance of LdapBackEnd */
    public LdapBackEnd(String password, String cellName) {
        super();
        ldap = new Ldap("ldap://localhost", "cn=Manager,dc=CELL",password);
        ldap.connect();
        Properties resourceProperties = (Properties)ldap.getObject("cn=ResourceMap");
        Properties attributeProperties = (Properties)ldap.getObject("cn=AttributeMap");
        prepareMapping(resourceProperties,attributeProperties);
        ldap.disconnect();
        this.cellName = cellName;
    }
    
    
    /**
    * prepares the conversion maps that are to be used when writing
    * resources to Ldap and when reading resources from ldap.
    *
    * @param  resourceProperties the mapping for resource names
    * @param  attributeProperties the mapping for attribute types  
    *        
    * @return      void
    */
    public synchronized void prepareMapping(Properties resourceProperties, Properties attributeProperties){
        
        typeConversion = new Hashtable();
        typeBackConversion = new Hashtable();
        attributeConversion = new Hashtable();
        attributeBackConversion = new Hashtable();
        
        Enumeration e = resourceProperties.propertyNames();
        while(e.hasMoreElements()){
            String type = (String)e.nextElement();
            typeConversion.put(type,resourceProperties.getProperty(type));
            typeBackConversion.put(resourceProperties.getProperty(type),type);
        }
        
        e = attributeProperties.propertyNames();
        while(e.hasMoreElements()){
            String type = (String)e.nextElement();
            attributeConversion.put(type,attributeProperties.getProperty(type));
            attributeBackConversion.put(attributeProperties.getProperty(type),type);
        }
    }
    
    /**
    * adds a new resource to the cib, if it doesn't exist already.
    *
    * @param  r  The ResourceName that is to be added
    *        
    * @return      void
    */
    public synchronized void addResource(ResourceName r) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        //System.out.println("+++++++++++++++++++++"+r);
        //System.out.println("+++++++++++++++++++++"+resourceName);
        ldap.createIfNotExists(resourceName);
        ldap.disconnect();    
    }
    
    
     /**
    * Returns the value of the especified attribute in the
    * especified resource
    *
    * @param  r  The resource
    *         a  The name of the attribute  
    *        
    * @return the value of the attribute     
    */
    public synchronized String getAttribute(ResourceName r, String a) {
        ldap.connect();
        String  result;
        String resourceName = convertToLdap(r);
        //System.out.println("Rname:"+resourceName);
        String type = a;
        if(attributeConversion.containsKey(a)){
            a = (String)attributeConversion.get(a);
            result = ldap.getAttribute(resourceName,a);
        }
        else{
            //System.out.println("here");
            result = ldap.getAttributeIsam(resourceName,a);
            //System.out.println("here2"+result);
        }
       
        ldap.disconnect(); 
        return(result);
    }
    
     /**
    * returns all the names of the attribues of a given resource 
    *
    * @param  r  The name of the resource 
    *        
    * @return      an array of string cointaining all the names of the attributes of the resource
    */
    public synchronized String[] getAttributes(ResourceName r) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        String[] result = ldap.getAttributesIsam(resourceName);
        ldap.disconnect();
        return(result);
    }
    
     /**
    * removes a resource from the cib
    *
    * @param  r  The name of the resource that will be removed
    *        
    * @return      void
    */
    public synchronized void removeResource(ResourceName r) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        ldap.remove(resourceName);
        ldap.disconnect();
    }

    
    public ResourceName[] select(SelectionConstraint sc, NameSpace[] ns, int maxResults){
        throw new RuntimeException("No implemented");
    }
    
     /**
    * Searches the cib for resources that match a certain SelectionConstraint
    *
    * @param  filter  The SelectionConstraint that the resources to be returned must match
    *         maxResults The maximum number of results that are to be returned        
    *
    * @return      An array containing the ResourceNames of the cib entries that matched the SelectionConstraint
    */
    public synchronized ResourceName[] select(SelectionConstraint filter, int maxResults) {
        String searchString = "";
        searchString = makeSearchString(filter);
        
        //System.out.println(searchString);
        
        ldap.connect();
        List resultList = ldap.findBySearchStringAndReturnAllAttributes(searchString);
        ldap.disconnect();
        
        List trueResults = new LinkedList();
        Iterator it = resultList.iterator();
        while(it.hasNext()){
            ResourceImpl resource = makeResourceFromAttributes((Attributes)it.next());
            if(filter.match(resource)){
                trueResults.add(resource);
            }
        }
        
        resultList = trueResults;
        ResourceName[] results; 
        if((resultList.size()<maxResults)||(maxResults == -1)){
            results = new ResourceName[resultList.size()];
            maxResults = resultList.size();
        }
        else{
            results = new ResourceName[maxResults];
        }
        
        it = resultList.iterator();
        for(int i=0; ((i<maxResults)&&(it.hasNext())); i++){
            results[i] = ((ResourceImpl)it.next()).getName();
        }
        return(results);
        
    }
    
     /**
    * Searches the cib for resources that match a certain SelectionConstraint
    *
    * @param  filter  The SelectionConstraint that the resources to be returned must match
    *         maxResults The maximum number of results that are to be returned        
    *
    * @return      An array containing the ResourceNames of the cib entries that matched the SelectionConstraint
    */
    public synchronized ResourceName[] select(SelectionConstraint filter, NameSpace ns, int maxResults) {
        String searchString = "";
        searchString = makeSearchString(filter);
        String where = "";
        where = convertToLdap(ns);
        //System.out.println(searchString);
        System.out.println(where);
        
        ldap.connect();
        List resultList = ldap.findBySearchStringAndReturnAllAttributes(searchString,where);
        ldap.disconnect();
        
        List trueResults = new LinkedList();
        Iterator it = resultList.iterator();
        while(it.hasNext()){
            ResourceImpl resource = makeResourceFromAttributes((Attributes)it.next());
            if(filter.match(resource)){
                trueResults.add(resource);
            }
        }
        
        resultList = trueResults;
        ResourceName[] results; 
        if((resultList.size()<maxResults)||(maxResults == -1)){
            results = new ResourceName[resultList.size()];
            maxResults = resultList.size();
        }
        else{
            results = new ResourceName[maxResults];
        }
        
        it = resultList.iterator();
        for(int i=0; ((i<maxResults)&&(it.hasNext())); i++){
            results[i] = ((ResourceImpl)it.next()).getName();
        }
        return(results);
        
    }
    
     /**
    * Generates a search string that the ldap server can understand from a 
    * given SelectionConstraint 
    *
    * @param  filter  The filter to be transformed into a string
    *        
    * @return      a search string that can be understood by a ldap server
    */
    private String makeSearchString(SelectionConstraint filter){
        String result = "";
        if(filter instanceof SimpleRegexpConstraint){
            SimpleRegexpConstraint simple = (SimpleRegexpConstraint)filter;
            String regexp = convertRegexpToLdap(simple.getRegexp());
            String attribute =  simple.getAttribute();
            if(attributeConversion.containsKey(attribute)){
                result = "("+attributeConversion.get(attribute)+"="+regexp+")";
            }
            else{
                result = "(isamType="+attribute+":"+regexp+")";
            }
        }
        else if(filter instanceof AttributeDefinedConstraint){
            AttributeDefinedConstraint attribute = (AttributeDefinedConstraint)filter;
            String attr =  attribute.getAttribute();
            if(attributeConversion.containsKey(attr)){
                result = "("+attributeConversion.get(attr)+"=*)";  
            }
            else{
                result = "(isamType="+attr+":*)";
            }
            
        }
        else if(filter instanceof CompositeSelectionConstraint){
            CompositeSelectionConstraint composite = (CompositeSelectionConstraint) filter;
            result = "(";
            if(composite.getMode()==CompositeSelectionConstraint.M_AND){
                result += "&";
            }
            else if(composite.getMode()==CompositeSelectionConstraint.M_OR){
                result += "|";
            }
            else if(composite.getMode()==CompositeSelectionConstraint.M_XOR){
                result += "&";
                result += "(";
                result += "|";
                Enumeration constraints = composite.constraints();
            
                while(constraints.hasMoreElements()){
                    result += makeSearchString(((SelectionConstraint)constraints.nextElement()));
                }
                result += ")";
                
                result += "(";
                result += "!";
                result += "(";
                result += "&";
                
            }
            Enumeration constraints = composite.constraints();
            
            while(constraints.hasMoreElements()){
                result += makeSearchString(((SelectionConstraint)constraints.nextElement()));
            }
            
            result += ")";
            
            if(composite.getMode()==CompositeSelectionConstraint.M_XOR){
                result += ")"; 
                result += ")";
            }
            
        }
        return(result);
    }
    
    /**
    * Searches the cib for resources that hava a certain name and that are inside
    * a given namespace
    *
    * @param  n  a string representing the name that the returned resources must have
    *         ns Tha NameSpace to which the resource must belong in order to be returned
    *         maxResults The maximum number of results that are to be returned        
    *
    * @return      An array containing the ResourceNames of the cib entries that had the given name
    */
    public synchronized ResourceName[] selectByName(String n, NameSpace ns, int maxResults) {
        //ResourceName[] results = new ResourceName[maxResults];
        ldap.connect();
        String nameSpace = convertToLdap(ns);
        List resultList = ldap.findByNameIsam(n,nameSpace);
        List finalResultList = new LinkedList();
        ldap.disconnect();
        
        ResourceName[] results; 
       
        Iterator it = resultList.iterator();
        while(it.hasNext()){
            ResourceName resource = convertFromLdap((String)it.next());
            if(resource.getCanonicalName().getValue().equals(n)){
                finalResultList.add(resource);
            }
        }
        
        if((finalResultList.size()<maxResults)||(maxResults == -1)){
            results = new ResourceName[finalResultList.size()];
            maxResults = finalResultList.size();
        }
        else{
            results = new ResourceName[maxResults];
        }
        
        it = finalResultList.iterator();
        for(int i=0; ((i<maxResults)&&(it.hasNext())); i++){
           results[i] = (ResourceName)it.next();
        }
        return(results);
    }
    
    /**
    * Searches the cib for resources that hava a certain type and that are inside
    * a given namespace
    *
    * @param  t  a string representing the name of the type that the returned resources must have
    *         ns Tha NameSpace to which the resource must belong in order to be returned
    *         maxResults The maximum number of results that are to be returned        
    *
    * @return      An array containing the ResourceNames of the cib entries that had the given type
    */
    public synchronized ResourceName[] selectByType(String t, NameSpace ns, int maxResults) {
        //ResourceName[] results = new ResourceName[maxResults];
        ldap.connect();
        String nameSpace = convertToLdap(ns);
        List resultList = ldap.findByTypeIsam(t,nameSpace);
        List finalResultList = new LinkedList();
        ldap.disconnect();
        
        Iterator it = resultList.iterator();
        while(it.hasNext()){
            ResourceName resource = convertFromLdap((String)it.next());
            //System.out.println("*******************************"+resource.getCanonicalName().getType());
            if(resource.getCanonicalName().getType().equals(t)){
                //System.out.println("*******************************"+t);
                finalResultList.add(resource);
            }
        }
        
        //System.out.println("++++++++++++++++++++++++++++++"+finalResultList.size());
        //finalResultList = resultList;
        
        ResourceName[] results; 
        if((finalResultList.size()<maxResults)||(maxResults == -1)){
            //System.out.println("++++++++++++++++++++++++++++++"+results.length);
            results = new ResourceName[finalResultList.size()];
            //System.out.println("++++++++++++++++++++++++++++++"+results.length);
            maxResults = finalResultList.size();
        }
        else{
            results = new ResourceName[maxResults];
        }
        
        it = finalResultList.iterator();
        for(int i=0; ((i<maxResults)&&(it.hasNext())); i++){
            results[i] = (ResourceName)it.next();
        }
        //System.out.println("++++++++++++++++++++++++++++++"+results.length);
        return(results);
    }
    
     /**
    * Sets the value of a given attribute in a given resource
    *
    * @param  r  the resource which "owns" the attribute that is to be set
    *         a the name of the attribute that is to be set
    *         v the value to which the attribute is to set        
    *
    * @return  a string containing tha value to which the attribute was set    
    */
    public synchronized String setAttribute(ResourceName r, String a, String v) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        
        String type = a;
        if(attributeConversion.containsKey(a)){
            a = (String)attributeConversion.get(a); 
        }
        else{
            v = a+":"+v;
            a = "isamType";
        }
        
        ldap.setAttribute(resourceName,a,v);
        ldap.disconnect();
        return(v);
    }
   
    
     /**
    * Finds the name of the Cell to which the NameSpace belongs
    *
    * @param  rn  the name space
    *         
    * @return  a String with the name of the cell to which the given NameSpace belongs to. Null if it doesn't belong to any cell.
    */
    private String getCell(NameSpace ns){
        while(ns != null){
            if(ns.getName().getType().equals("cell")){
                return(ns.getName().getValue());
            }
            ns = ns.getParent();
        }
        return(null);
    }
    
     /**
    * Finds the name of the Cell to which the ResourceName belongs
    *
    * @param  rn  the resource name 
    *         
    * @return  a String with the name of the cell to which the given ResourceName belongs to. Null if it doesn't belong to any cell.
    */
    private String getCell(ResourceName rn){
        NameSpace ns = rn.getNameSpace();
        while(ns != null){
            if(ns.getName().getType().equals("cell")){
                return(ns.getName().getValue());
            }
            ns = ns.getParent();
        }
        return(null);
    }
    
    
     /**
    * Adds the appropiate cell NameSpace to the root of the given NameSpace
    *
    * @param  ns  the NameSpace that is to be modified
    *         
    * @return  a new version of the given NameSpace with the proper cell NameSpace at the root
    */
    private NameSpace setCellParent(NameSpace ns){
        if(ns == null){
            return(new NameSpace("cell",cellName,new NameSpace("","",null)));
        }
        else{
            return(new NameSpace(ns.getName().getType(),ns.getName().getValue(),setCellParent(ns.getParent())));
        }
    }
    
     /**
    * Adds the appropiate cell NameSpace to the root of the given ResourceName
    *
    * @param  rn  the resource name that is to be modified
    *         
    * @return  a new version of the given ResourceName with the proper cell NameSpace at the root
    */
    private ResourceName setCellParent(ResourceName rn){
        return(new ResourceName(rn.getCanonicalName().getType(),rn.getCanonicalName().getValue(),setCellParent(rn.getNameSpace())));
    }
    
    /**
    * converts a given ResourceName into a string representation that can be
    * understood by a ldap server
    *
    * @param  r  the resource name that is to be converted
    *         
    * @return  a string representation of the given ResourceName that can be understood by a ldap server    
    */
    private String convertToLdap(ResourceName r){
        if(r==null){
            return("");
        }
        
        if(getCell(r)==null){
           r = setCellParent(r);
           //System.out.println(r);
        }
        //System.out.println(r.getDN());
        String type = r.getCanonicalName().getType();
        String name = r.getCanonicalName().getValue();
        String dn = "";
        if(typeConversion.containsKey(type)){
             dn = (String)typeConversion.get(type)+"="+name;//r.getDN().replaceAll(":","=").replaceAll(type,(String)typeConversion.get(type)).replaceAll("\\[","").replaceAll("\\]","");
        }
        else{
            dn = "isamResource="+type+":"+name;//r.getDN().replaceAll(":","=").replaceAll(type,"isamType").replaceAll(name,type+":"+name).replaceAll("\\[","").replaceAll("\\]","");
        }
        //dn = dn.substring(0,dn.lastIndexOf(","));
        if(convertToLdap(r.getNameSpace()) != ""){
            return(dn+","+convertToLdap(r.getNameSpace()));
        }else{
            return(dn);
        }
    }
    
    /**
    * converts a given NameSpace into a string representation that can be
    * understood by a ldap server
    *
    * @param  ns  the NameSpace that is to be converted
    *         
    * @return  a string representation of the given NameSpace that can be understood by a ldap server    
    */
    public String convertToLdap(NameSpace ns){
        if((ns==null)||(ns.getName()==null)||(ns.getName().getType().equals(""))){
            //ns = CellInformationBase.NameSpace.DEFAULT_NS;
            return("");
        }
        
        if(getCell(ns)==null){
           ns = setCellParent(ns);
           //System.out.println(r);
        }
        
        //System.out.println(r.getDN());
        String type = ns.getName().getType();
        String name = ns.getName().getValue();
        String dn = "";
        if(typeConversion.containsKey(type)){
             dn = (String)typeConversion.get(type)+"="+name;//r.getDN().replaceAll(":","=").replaceAll(type,(String)typeConversion.get(type)).replaceAll("\\[","").replaceAll("\\]","");
        }
        else{
            dn = "isamResource="+type+":"+name;//r.getDN().replaceAll(":","=").replaceAll(type,"isamType").replaceAll(name,type+":"+name).replaceAll("\\[","").replaceAll("\\]","");
        }
        //dn = dn.substring(0,dn.lastIndexOf(","));
        if(convertToLdap(ns.getParent()) != ""){
            return(dn+","+convertToLdap(ns.getParent()));
        }else{
            return(dn);
        }
        
        /*String dn = ns.toString().replaceAll("\\[","").replaceAll(":","=").replaceAll("\\]","");
        int p = 0;
        while(dn.indexOf(",",p)!=-1){
            String type = dn.substring(p,dn.indexOf("=",p));
            String name = dn.substring(dn.indexOf("=",p)+1,dn.indexOf(",",p)-1);
            if(typeConversion.containsKey(type)){
                dn = dn.replaceFirst(type,(String)typeConversion.get(type));
            }
            else{
                dn = dn.replaceFirst(type,"isamType").replaceFirst(name,type+":"+name);
            }
            p = dn.indexOf(",",p)+1;
        }
        if(dn.indexOf("=") != -1){
            String type = dn.substring(p,dn.indexOf("=",p));
            String name = dn.substring(dn.indexOf("=",p)+1);
            if(typeConversion.containsKey(type)){
                dn = dn.replaceFirst(type,(String)typeConversion.get(type));
            }
            else{
                dn = dn.replaceFirst(type,"isamType").replaceFirst(name,type+":"+name);
            }
        //String dn = ns.toString().replace('[',' ').replace(':','=').replace(']',' ');
        }
        return(dn);*/
        
    }
    
    /**
    * converts a string with the name of an ldap resource, in the format supported
    * by ldap servers, into a ResourceName 
    *
    * @param  dn  A string with the name of a resource in the ldap server
    *         
    * @return  a ResourceName that is points to the same resource as the given String 
    */
    private ResourceName convertFromLdap(String dn){
        NameSpace ns = null;
        ResourceName rn = null;
        
        while(dn.indexOf(",")!=-1){
            String aux = dn.substring(dn.lastIndexOf(",")+1);
            String type = aux.substring(0,aux.indexOf("="));
            String value = aux.substring(aux.indexOf("=")+1);
            //System.out.println("Tipo -- "+type);
            if(type.equals("isamResource")){
                type = value.substring(0,value.indexOf(":"));
                value = value.substring(value.indexOf(":")+1);
            }
            if(typeBackConversion.containsKey(type)){
                type = (String)typeBackConversion.get(type);
            }
            //System.out.println("Tipo -- "+type);
            NameSpace auxNS = new NameSpace(type, value,ns);
            ns = auxNS;
            dn = dn.substring(0,dn.lastIndexOf(","));
        }
       
        String type = dn.substring(0,dn.indexOf("="));
        String value = dn.substring(dn.indexOf("=")+1);
        if(type.equals("isamResource")){
            type = value.substring(0,value.indexOf(":"));
            value = value.substring(value.indexOf(":")+1);
        }
        if(typeBackConversion.containsKey(type)){
                type = (String)typeBackConversion.get(type);
        }
        if(ns == null){
            rn = new ResourceName(type,value,new NameSpace("","",null));//type+":"+value);
        }
        else{
            rn = new ResourceName(type,value,ns);
        }
        
        return(rn);
    }
    
    
    /**
    * converts a set of attributes into a resource
    *
    * @param  attributes  The attributes to be converted into a resource
    *         
    * @return  a ResourceImpl that contains all the given attributes
    */
    private ResourceImpl makeResourceFromAttributes(Attributes attributes){
        try{
            ResourceName rn = convertFromLdap((String)attributes.get("fullDN").get());
            ResourceImpl resource = new ResourceImpl(rn);
            Enumeration attrs = attributes.getAll();
            while(attrs.hasMoreElements()){
                Attribute attr = (Attribute)attrs.nextElement();
                String type = (String)attr.getID();
                if(attributeBackConversion.containsKey(type)){
                    type = (String)attributeBackConversion.get(type);
                }
                else if(type.equals("isamType")){
                    Enumeration e = attr.getAll();
                    while(e.hasMoreElements()){
                        String value = (String)e.nextElement();
                        resource.setAttribute(value.substring(0,value.indexOf(":")),value.substring(value.indexOf(":")+1));
                    }
                }
                resource.setAttribute(type,(String)attr.get());
            }   
            return(resource);
        }catch(javax.naming.NamingException ne){ne.printStackTrace();}
        return(null);
    }
    
    /**
    * converts a full regular expression into something tha ldap can understand. 
    * Some of the meaning of original regular expression may be lost. 
    *
    * @param  regexp  A regular expression string
    *         
    * @return  a new String with a regular expression that ldap can understand
    */
    private String convertRegexpToLdap(String regexp){
        String ldapRegexp = "";
        ldapRegexp = regexp.replaceAll("\\.","*");
        ldapRegexp = ldapRegexp.replaceAll("\\[.*\\]", "*");
        ldapRegexp = ldapRegexp.replaceAll("\\?","*");
        ldapRegexp = ldapRegexp.replaceAll("\\+","*");
        ldapRegexp = ldapRegexp.replaceAll("\\{.*\\}","*");
        ldapRegexp = ldapRegexp.replaceAll("\\(.*\\)","*");
        ldapRegexp = ldapRegexp.replace('\\','*');
        //System.out.println(ldapRegexp);
        return(ldapRegexp);
    }
    
}
