// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.worb;

import org.isam.perv.IsamClassLoader;
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.HttpService;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.Executor;
import org.isam.exehda.services.Logger.Sev;
import org.isam.util.codegen.Types;
import org.isam.util.codegen.proxy.InvocationHandler;
import org.isam.util.codegen.proxy.Proxy;
import java.util.Hashtable;
import java.lang.reflect.Method;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.MessageDigest;
import java.security.DigestOutputStream;
import java.net.URL;
import java.net.URLConnection;

/**
 * Webservices-like ORB implementation
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-03-10 07:24:09 -0300 (Thu, 10 Mar 2005) $ $Revision: 619 $
 */
public class WorbImpl
    extends HttpService
    implements Worb
{
    static final String SERVICE_NAME = "worb";
    static final int    DEFAULT_PORT = 1980;
    static final String REQ_PROP_RESOURCE = "Resource";
    static final String REQ_PROP_OPNUM = "Isam-Worb-Opnum";


    private final URL baseURL;
    private Hashtable servicesByName;
    private Hashtable actionsByThread;
    

    public WorbImpl() throws Exception
        {
            super(Exehda.getServiceProperty(SERVICE_NAME, "port", DEFAULT_PORT));
            
            baseURL = new URL("http",
                              "localhost",
                              Exehda.getServiceProperty(SERVICE_NAME, "port", DEFAULT_PORT),
                              "");
            
            this.servicesByName = new Hashtable();
            this.actionsByThread = new Hashtable();
        }
    
    public void
    exportService(Object obj, Class iface, String name)
        {
            log(Sev.DEBUG, "exportService", "entering for \"{0}\"", name);

            if ( iface.isAssignableFrom(obj.getClass()) ) {
                servicesByName.put(name, new WSServerRef(obj, iface));
            }
            else {
                throw new ClassCastException("Object does not support the provided interface");
            }
            
            log(Sev.DEBUG, "exportService", "leaving");
        }

    public void
    unexportService(Object obj, String name)
        {
            throw new NullPointerException("Not yet implemented.");
        }

    public void
    setExceptionHandler(Object proxy, WorbExceptionHandler ehandler)
        {
            throw new NullPointerException("Not yet implemented.");
        }

    public HostId
    getClientHost()
        {
            InvokeAction ca = currentAction();

            if ( ca == null ) return null;

            ServiceRequest req = ca.getServiceRequest();

            try {
                return HostId.getByName(
                    req.getClientSocket().getInetAddress().getHostAddress());
            }
            catch (java.net.UnknownHostException e) {
                log(Sev.DEBUG, "getClientHost", e);
                return null;
            }
        }

    public Object
    lookupService(String service, Class iface)
        {
            try {
                URL u = new URL(baseURL, service);
                String serviceURI = (
                    new URL("http", u.getHost(),DEFAULT_PORT, u.getFile())
                    ).toExternalForm();

                return Proxy.getInstance(iface, new WSClientRef(serviceURI, iface));
            }
            catch (Exception e) {
                log(Sev.DEBUG, "lookupService", e.toString());
                return null;
            }
        }

    protected void
    handleRequest( ServiceRequest req )
        {
            String resource = req.getProperty(REQ_PROP_RESOURCE);
            Long opnum = Long.valueOf(req.getProperty(REQ_PROP_OPNUM));

                // parse /A/B/C/method[/]
//             int idxSlash1 = resource.indexOf("/",1);
//             int idxSlash2 = resource.indexOf("/",idxSlash1+1);

//             if ( idxSlash2 == -1 ) idxSlash2 = resource.length();

                // TODO: better malformed URL handling
            int idxSlash2 = resource.endsWith("/") ? resource.length()-1 : resource.length();
            int idxSlash1 = resource.lastIndexOf("/",idxSlash2-1);

            if ( idxSlash1 < 2 ) {
                    // TODO: better malformed URL handling
                log(Sev.ERROR, "handleRequest", "Malformed URL {0}", resource);
            }
            
            String svc = resource.substring(1, idxSlash1);
            String mtd = resource.substring(idxSlash1+1, idxSlash2);

            log(Sev.DEBUG, "handleRequest", "Incoming call to {0}.{1}(...)", svc, mtd);

            WebServiceRef wsref = (WebServiceRef) servicesByName.get(svc);

                // TODO: check is wsref exists (is not null)

            try {

                BufferedInputStream rawin = new BufferedInputStream(
                    req.getClientSocket().getInputStream(), 1024);
                rawin.mark(512);
                
                ObjectInputStream in = new ObjectInputStream(rawin);

                ApplicationId aid = (ApplicationId) in.readObject();

                rawin.reset();

                InvokeAction action = new InvokeAction(wsref, opnum, rawin, req);
                
                ((Executor) Exehda.getService(Executor.SERVICE_NAME)).runAction(aid, action);
                
                byte[] resultData = action.getResult();
                
                ServiceResponse resp = new ServiceResponse(req, ServiceResponse.OK);
                resp.putHeader("Content-Length", Long.toString(resultData.length));
                resp.putData(resultData, 0, resultData.length);
                resp.close();
            }
            catch (Exception e) {
                log(Sev.INFO, "handleRequest", e);
//                System.exit(-1);
            }
        }

    private void
    bindAction(InvokeAction a)
        {
            actionsByThread.put(Thread.currentThread(), a);
        }

    private InvokeAction
    currentAction()
        {
            return (InvokeAction) actionsByThread.get(Thread.currentThread());
        }

    private void
    unbindAction(InvokeAction a)
        {
            actionsByThread.remove(Thread.currentThread());
        }

    class InvokeAction implements Runnable
    {
        private WebServiceRef wsref;
        private InputStream rawin;
        private Long opnum;
        private byte[] resultData;
        private Exception e;
        private ServiceRequest req;

        public InvokeAction(WebServiceRef wsref, Long opnum, InputStream rawin, ServiceRequest req)
            {
                this.wsref = wsref;
                this.opnum = opnum;
                this.rawin = rawin;
                this.e = null;
                this.req = req;
            }

        public ServiceRequest
        getServiceRequest()
            {
                return this.req;
            }
        
        public void run()
            {
                try {
                    WObjectInputStream in = new WObjectInputStream(rawin);

                        // skip application id
                    in.readObject();
                
                    Object[] params = (Object[]) in.readObject();

                    if (params == null) params = new Object[0];

                    Object result = null;
                    Exception exception = null;

                    try {
                            // enable getClientHost
                        WorbImpl.this.bindAction(this);

                            // invoke the object
                        result = wsref.invoke(opnum, null, params);
                        
                            // disable getClientHost
                        WorbImpl.this.unbindAction(this);
                    }
                    catch (Exception e) {
                            // propagate any exceptions to the client through the resultData
                        exception = e;
                        log(Sev.DEBUG, "InvokeAction.run", "Caugth exception");
                        log(Sev.DEBUG, "InvokeAction.run", e);
                    }
                    finally {
                        ByteArrayOutputStream buf = new ByteArrayOutputStream(8192);
                        ObjectOutputStream out = new ObjectOutputStream(buf);

                        out.writeObject(exception);
                        out.writeObject(result);
                        out.close();

                        this.resultData = buf.toByteArray();
                    }
                }
                catch (Exception e) {
                    this.e = e;
                    log(Sev.INFO, "InvokeAction.run", e);
                }
            }

        public byte[]
        getResult() throws Exception
            {
                if (e != null) {
                    throw e;
                }
                else {
                    return resultData;
                }
            }
    }
}



abstract class WebServiceRef
{
    protected Class serviceInterface;

    protected WebServiceRef(Class serviceIface)
        {
            this.serviceInterface = serviceIface;
        }

    public abstract Object
    invoke( Long opnum, Method m, Object[] params ) throws Exception;

    protected long
    computeMethodHash(Method m)
        throws java.security.NoSuchAlgorithmException, java.io.IOException
        {
            String methodName = m.getName() + Types.getDescriptor(m);
            
            ByteArrayOutputStream bs = new ByteArrayOutputStream(512);

            MessageDigest md = MessageDigest.getInstance("SHA");

            DataOutputStream ds =
                new DataOutputStream(
                    new DigestOutputStream(bs, md));

            ds.writeUTF(methodName);
            ds.flush();

            byte[] sha = md.digest();

            long hash = 0;
            
            for (int i=0,s=0; i<8; i++,s+=8) {
                hash += ((long) (sha[i] & 0xff)) << s;
            }
            return hash;
        }
}

class WSServerRef
    extends WebServiceRef
{
    private final Object serviceImpl;
    private Hashtable methodsByHash;
    
    public WSServerRef( Object impl, Class iface)
        {
            super(iface);
            serviceImpl = impl;
            methodsByHash = null;
        }
    
    public Object
    invoke( Long opnum, Method m, Object[] params )
        throws Exception
        {
            m = loadMethod(opnum);

            return m.invoke(serviceImpl, params);
        }
    
    
    private final synchronized Method
    loadMethod(Long opnum)
        throws java.security.NoSuchAlgorithmException, java.io.IOException
        {
                // delay method loading until the first invocation
            if (methodsByHash == null ) {
                methodsByHash = new Hashtable();

                Method[] methods = serviceInterface.getMethods();
                for ( int m=0; m < methods.length; m++ ) {
                        // register the given method
                    methodsByHash.put(
                        new Long(computeMethodHash(methods[m])),
                        methods[m]);
                }
            }
            return (Method) methodsByHash.get(opnum);
        }
}


class WSClientRef
    extends WebServiceRef
    implements InvocationHandler
{
    private final String    serviceBaseURL;
    private transient Hashtable hashByMethods;
    
    WSClientRef(String serviceBaseURL, Class serviceIface )
        {
            super(serviceIface);
            this.serviceBaseURL = serviceBaseURL;
            this.hashByMethods = null;
        }

    public Object
    invoke( Long opnum, Method m, Object[] args )
        throws Exception
        {
            Object remResult = null;
            Exception remException = null;

                // lookup method hash
            opnum = getMethodHash(m);

            try {
                URLConnection conn = (new URL(serviceBaseURL+'/'+m.getName())).openConnection();
                conn.setDoOutput(true);
                conn.setRequestProperty("Isam-Worb-Opnum",opnum.toString());

                ObjectOutputStream out = new ObjectOutputStream(
                    new BufferedOutputStream(conn.getOutputStream()));

                ApplicationId aid =
                    ((Executor) Exehda.getService("executor")).currentApplication();
                
                out.writeObject(aid);
                out.writeObject(args);
                out.close();

                WObjectInputStream in = new WObjectInputStream(
                    new BufferedInputStream(conn.getInputStream()));

                remException = (Exception) in.readObject();
                remResult = in.readObject();
            }
            catch (Exception e) {
                //
                // TODO: delegate to exception handler
                //
                throw (Exception) e.fillInStackTrace();
            }

            if ( remException == null ) {
                return remResult;
            }
            else {
                throw (Exception) remException.fillInStackTrace();
            }
        }

    public Object
    invoke( Proxy p, Method m, Object[] args ) throws Exception
        {
            return invoke( (Long) null, m, args);
        }

    private final synchronized Long
    getMethodHash(Method m)
        throws java.security.NoSuchAlgorithmException, java.io.IOException
        {
            if ( hashByMethods == null ) {
                Method[] mtds = serviceInterface.getDeclaredMethods();
                
                hashByMethods = new Hashtable((int) mtds.length);

                for ( int i=0; i< mtds.length; i++ ) {
                    hashByMethods.put(
                        mtds[i],
                        new Long(computeMethodHash(mtds[i])));
                }
            }

            return (Long) hashByMethods.get(m);
        }
}


class WObjectInputStream
    extends java.io.ObjectInputStream
{
    public WObjectInputStream(java.io.InputStream in) throws java.io.IOException
        {
            super(in);
        }
  
    protected Class 
    resolveClass(java.io.ObjectStreamClass v) throws java.io.IOException, ClassNotFoundException
        {
            return IsamClassLoader.getInstance().loadClass(v.getName());
        }
}

