// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos;

import org.isam.perv.IsamClassLoader;
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.ObjectSeed;
import org.isam.util.xml.*;
import java.io.*;
import java.net.UnknownHostException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Hashtable;
import java.util.Vector;

public class ObjectSeedImpl
    extends AbstractService
    implements ObjectSeed
{
    private final static String SERVICE_NAME = "objectseed";
    private final static boolean NDEBUG = false;
    private final Hashtable remoteObjects = new Hashtable();

    
    public void
    start()
        {
                // empty
        }
    
    public void
    stop() 
        {
                // empty
        }

    
    public ObjectHandle
    newInstance( Class cls, Object[] params, Activator a )
        {
            log(Sev.DEBUG, "createObject", "entering");
            try {
                Object obj = (params == null)
                    ? cls.newInstance()
                    : findConstructor(cls, params).newInstance(params);

                return activateObject(obj, a);

//                 return obj;
            }
            catch (Exception e) {
                log(Sev.ERROR, "newInstance", e);
                return null;
            }
            finally {
                log(Sev.DEBUG, "newInstance", "leaving");
            }
        }
    
    private final Constructor
    findConstructor( Class cls, Object[] params )
        {
            Class paramTypes[] = new Class [params.length];
            for (int i=0; i<params.length; i++){
                paramTypes[i] = params[i].getClass();
            }
                // find the matching constructor for the given parameter list
            Constructor[] candidate = cls.getConstructors();
            Constructor   matched    = null;
            for (int c=0; c<candidate.length && (matched== null); c++) {
                
                Class[] formalParamTypes = candidate[c].getParameterTypes();
                    // check against number of parameters
                if (formalParamTypes.length == paramTypes.length ) {
                    
                    matched = candidate[c];
                    
                        // check against parameter types
                    for (int p=0; p<formalParamTypes.length; p++){
                            /// TO DO: match primitive types
                            // if parameter matching fails clear matched
                        if (!formalParamTypes[p].isAssignableFrom(
                                paramTypes[p])) {
                            matched = null;;
                        }
                    }
                }
            }

            return matched;
        }

    private final ObjectHandle
    activateObject( Object obj, Activator activator )
        throws Exception
        {
            log(Sev.DEBUG, "activateObject", "entering");
            
//             Port port = TNL.allocPort();
//             log(Sev.DEBUG, "activateObject", "After allocPort()");
            
            if ( obj instanceof Activatable ) {
                ((Activatable) obj).activate();
                activator = null;
                log(Sev.DEBUG, "activateObject", "Activating Activatable object...");
                
            }
            else if ( activator != null ) {
                activator.activate(obj);
                log(Sev.DEBUG, "activateObject",
                    "Activating object with custom provided activator");

            }
//             else if ( obj instanceof ActiveObject ) {
//                     // active object activation
//                 log(Sev.DEBUG, "activateObject", "Activating active  object...");
//                 ActiveObject ao = (ActiveObject) obj;
//                Remote stub = java.rmi.server.UnicastRemoteObject.exportObject(ao);
// //                 Remote stub = primos.rmi.Orb.exportObjectTNL(ao, port);
//                 ao.activate();
//                 return new ObjectHandle (
//                     new ObjectAddress( port.getAddress()), stub);     
//             }
            else if ( obj instanceof java.rmi.Remote ) {
                    // remote object activation
                log(Sev.DEBUG, "activateObject", "Activating remote object...");
                java.rmi.Remote r = (java.rmi.Remote) obj;
                    // TODO: use lease references
                remoteObjects.put(obj,obj);
                Remote stub = java.rmi.server.UnicastRemoteObject.exportObject(r);
//                 Remote stub = primos.rmi.Orb.exportObjectTNL(r, port);

                return new ObjectHandle(stub);
                // return new ObjectHandle (
//                     new ObjectAddress( port.getAddress()), stub);
                
            }
            else if ( obj instanceof java.lang.Thread ) {
                    // thread activation
                log(Sev.DEBUG, "activateObject", "Activating thread object...");
                Thread t = (Thread) obj;
                t.start();
            }
            else {
                    // ordinary java object
                log(Sev.DEBUG, "activateObject", "Activating ordinary object...");
            }

//             managedObjects.put( port, obj, activator );

            log(Sev.DEBUG, "activateObject", "leaving");

            return new ObjectHandle(null);
//             return new ObjectHandle( new ObjectAddress(port.getAddress()) );
        }
}



// class RMIActivator
//     implements Activator
// {
//     public static Activator getInstance();
//     public void activate( Object obj ) throws Exception;
//     public void deactivate( Object obj ) throws Exception;
// }

// class ActivatableActivator
//     implements Activator
// {
//     public static Activator getInstance();
//     public void activate( Object obj ) throws Exception;
//     public void deactivate( Object obj ) throws Exception;
// }

// class ThreadActivator
//     implements Activator
// {
//     public static Activator getInstance();
//     public void activate( Object obj ) throws Exception;
//     public void deactivate( Object obj ) throws Exception;
// }

// class NullActivator
//     implements Activator
// {
//     public static Activator getInstance();
//     public void activate( Object obj ) throws Exception;
//     public void deactivate( Object obj ) throws Exception;
// }
