// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


//
// $Id: TipsHeuristic.java 626 2005-03-15 08:18:17Z rreal $
//


package org.isam.exehda.services.tips;

import org.isam.exehda.Exehda;
import org.isam.exehda.services.Collector;
import org.isam.exehda.services.Collector.Sensor;
import org.isam.exehda.services.Collector.MonitoringData;
import org.isam.exehda.services.Collector.ConsumerId;
import org.isam.exehda.services.Collector.MonitoringConsumer;
import org.isam.exehda.services.Scheduler;

public class TipsHeuristic
    implements MonitoringConsumer
{

    private Collector collector;
    private ConsumerId id;
    private Sensor[] sensors;

    private DataManager dataM;
    private UtilityGen ug;
    private BayesNetwork bn;
    private Learner l;
    private TipsCellProxy tcproxy;

    String cellProxy;
    
    public TipsHeuristic(DataManager dataM, UtilityGen ug, BayesNetwork bn, Learner l)
        {
            System.out.println("Starting TipsHeuristic (CVS $Revision: 626 $) ...");
            collector = (Collector) Exehda.getService(Collector.SERVICE_NAME);

            this.id = collector.addConsumer(this);
            this.dataM = dataM;
            this.ug = ug;
            this.bn = bn;
            this.l = l;

            cellProxy = Exehda.getServiceProperty(Scheduler.SERVICE_NAME,"cellproxy","localhost");
            tcproxy = new TipsCellProxy(cellProxy);
            
            initSensors();
        }

    private void initSensors()
        {
//            this.sensors = collector.getSensors();

            collector.setSensorEnabled(id,collector.getSensor("CPU_OCCUP_IDLE"),true);
            
//            for ( int i=0; i< sensors.length; i++ ) {
//                collector.setSensorEnabled(id,this.sensors[i],true);
//            }            
        }


    public void dumpData(MonitoringData[] data) 
        {
            for (int i=0; i<data.length; i++) {
                if ( data[i] != null ) {
                    System.out.print(data[i].getSensor().getSimpleName());
                    System.out.print(" \t= ");
                    System.out.println(data[i].getString());
                }
            }
        }
    

    public void update( long timeStamp, MonitoringData[] data )
        {
//            System.out.println("\nTipsHeuristic update timestamp: "+timeStamp);

            if (data[0] == null) return;
            
            
            int idx = 0;
            while ( !"CPU_OCCUP_IDLE".equals(data[idx].getSensor().getSimpleName()) ) {
//                System.out.println("TipsHeuristic: " + idx);
                idx++;
            }
            
            
            String sensorName = data[idx].getSensor().getSimpleName();
            Double sensorValue = new Double(data[idx].getDouble());

            
//            System.out.println("sensorValue = " + sensorValue + "  data[0]=" + data[0].getString());
            

            dataM.setSensorValue(sensorName,sensorValue);
//            System.out.println("TipsHeuristic.update --- dataM.setSensorValue(" + sensorName + "," + sensorValue +");");
            
            l.setInfo(sensorName);
            bn.eval(sensorName);
            
//            dumpData(data);
            
        }
}
