// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

//
// $Id: TipsNodeScheduler.java 626 2005-03-15 08:18:17Z rreal $
//

package org.isam.exehda.services.tips;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ObjectId;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.Scheduler;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.ResourceBroker.ResourceConstraint;
import org.isam.exehda.services.Collector;



import java.lang.reflect.*;
import java.util.*;


public class TipsNodeScheduler
    extends AbstractService
    implements Scheduler
{    

    private Scheduler tipsCell;
    final static String PROP_SCHEDULER_URI = "contactAddress";    
    private TipsHeuristic tipsHeuristic;
    private TipsMonitor monitor;

    private DataManager dataM;
    private Learner l;
    private BayesNetwork bn;
    private UtilityGen ug;
    
    
/**
         * Describe <code>chooseCreationHost</code> method here.
         *
         * @param c a <code>ResourceConstraints</code> value
         * @return a <code>HostId</code> value
         */
    public HostId chooseCreationHost( ResourceConstraint c ) 
        {
            return(tipsCell.chooseCreationHost(c));
        }
    

        /**
         * Describe <code>chooseMigrationHost</code> method here.
         *
         * @param c a <code>ResourceConstraints</code> value
         * @return a <code>HostId</code> value
         */
    public HostId chooseMigrationHost( ResourceConstraint c )
        {
            return(tipsCell.chooseMigrationHost(c));
        }


        /**
         * Describe <code>reschedule</code> method here.
         *
         * @param o an <code>ObjectHandle</code> value
         * @param c a <code>ResourceConstraints</code> value
         * @param auto a <code>boolean</code> value
         */
    public void reschedule( ObjectId o, ResourceConstraint c, boolean auto )
        {
                //repassar o retorno do TipsCellScheduler
        }


    public void stop() 
        {}

    public void start()
        throws Exception
        {
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);

            String tipsCellUri = Exehda.getServiceProperty(TipsCellScheduler.SERVICE_NAME,
                                                          PROP_SCHEDULER_URI,
                                                          "//h3p:1980/scheduler");

            System.out.println("TipsCellUri = " + tipsCellUri);
                
            tipsCell = (Scheduler) worb.lookupService(tipsCellUri, Scheduler.class);
            
                // Starting the data manager
            Class dataMClass =  Class.forName ("org.isam.exehda.services.tips.DataManagerImpl");
            DataManager dataM = (DataManager) dataMClass.newInstance();
            
                // Starting the utility generator
            Class ugClass = Class.forName(
                Exehda.getServiceProperty(SERVICE_NAME,
                                          "utilitygen",
                                          "org.isam.exehda.services.tips.UtilityGenImpl2"));
            
            Constructor ugConst = ugClass.getDeclaredConstructor(new Class[] {DataManager.class} );
            ug = (UtilityGen) ugConst.newInstance(new Object[] {dataM});            

                // Starting the BayesNetwork
            Class bnClass = Class.forName(
                Exehda.getServiceProperty(SERVICE_NAME,
                                          "bayesnetwork",
                                          "org.isam.exehda.services.tips.BayesNetworkImpl"));
            System.out.println("Loading " + (bnClass.getInterfaces()[0]).toString());
            
            Constructor bnConst = bnClass.getDeclaredConstructor(
                new Class[] {DataManager.class, UtilityGen.class} );
            bn = (BayesNetwork) bnConst.newInstance(new Object[] {dataM,ug});

                // Starting the Learner
            Class lClass = Class.forName(
                Exehda.getServiceProperty(SERVICE_NAME,
                                          "learner",
                                          "org.isam.exehda.services.tips.LearnerImpl"));
            Constructor lConst = lClass.getDeclaredConstructor(
                new Class[] { DataManager.class, BayesNetwork.class} );
            l = (Learner) lConst.newInstance(new Object[] {dataM,bn});


            monitor = new TipsMonitor(dataM);


            Collector collector = (Collector) Exehda.getService(Collector.SERVICE_NAME);
            collector.addMonitor(monitor);  
            
            TipsHeuristic tipsHeuristic;
            tipsHeuristic = new TipsHeuristic(dataM,ug,bn,l);
            
        }
    
}
