// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda;

/**
 * Matches resources that define a given attribute, i.e. the attributes's value is
 * non-null.
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-02-24 02:29:43 -0300 (Thu, 24 Feb 2005) $ $Revision: 584 $
 */
public class AttributeDefinedConstraint
    extends AttributeConstraint
{
        /**
         * Creates a new <code>AttributeDefinedConstraint</code> instance.
         *
         * @param att the name of the attribute to be checked
         */
    public AttributeDefinedConstraint(String att)
        {
            super(att);
        }


        /**
         * Checks whether the attribute is defined (i.e. non-null) for the given
         * resource.
         *
         * @param r a <code>ResourceInfo</code> value
         * @return ResourceConstraint.OK if it is defined, ResourceConstraint.BROKEN
         * otherwise.
         */
    public int
    check( ResourceInfo r )
        {
            return ( r.getAttribute(this.attrName) == null ) ?
                ResourceConstraint.BROKEN : ResourceConstraint.OK;
        }

}
