// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda;


/**
 * System-wide unique cell identifier.
 *
 * All cell identifiers are root ID's, so unicity depends on the local id. Thus, the cell
 * name assigment policy must guarantee that assigned names for each cell in the system
 * are unique.
 *
 * A sugested scheme that would satisfy such condition is to derive the cell name from
 * the organization's Internet domain name.
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-04-14 11:09:14 -0300 (Thu, 14 Apr 2005) $ $Revision: 763 $
 */
public final class CellId
    extends HierarchicalId
{    
        /**
         * {@inheritDoc}
         *
         * @return true if both objects are instances of CellId and their 'cellName'
         * attributes also match; false otherwise.
         */
    public boolean
    equals( Object o )
        {
            if ( o != null && o instanceof CellId ) {
                return cellName.equals(((CellId) o).cellName);
            }
            return false;
        }

        /**
         * {@inheritDoc}
         *
         * @return the hash code computed from the cell's name
         */
    public int
    hashCode()
        {
            return cellName.hashCode();
        }

        /**
         * Returns the cell's name.
         *
         * @return a string value representing the cell's name.
         */
    public String
    getName()
        {
            return cellName;
        }
    
        /**
         * Returns a String representation of the <code>CellId</code> type (actually, the
         * value "cell").
         *
         * @return the string "cell".
         */
    protected final String
    getType()
        {
            return "cell";
        }

    
        /**
         * Returns the local id, which in the case of this class corresponds to the
         * cell's name.
         *
         * @return the cell name
         */
    protected final Object
    getLocalId()
        {
            return cellName;
        }

    
        /**
         * Creates a new <code>CellId</code> instance.
         *
         * @param cellName the name of the cell whose id is being created.
         */
    CellId( String cellName )
        {
                // all CellId are supposed to be root IDs
            super(null);
            
            this.cellName = cellName;
        }

        /** the cell's name (i.e. it's local id) */
    private String cellName;
}

