// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import org.isam.exehda.ApplicationId;
import org.isam.exehda.HostId;

/**
 * Webservices-like ORB
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-04-13 15:57:49 -0300 (Wed, 13 Apr 2005) $ $Revision: 758 $
 */
public interface Worb
{
    public static final String SERVICE_NAME = "worb";

    public static class WorbException
//        extends XXX
    {}

    public static interface WorbProxy
    {
            /**
             * Describe <code>getServiceContactAddress</code> method here.
             *
             * @return a <code>String</code> value
             */
        public String getServiceContactAddress();
        
            /**
             * Describe <code>setServiceContactAddress</code> method here.
             *
             * @param ca a <code>String</code> value
             */
        public void setServiceContactAddress( String ca );
        
            /**
             * Describe <code>setExceptionHandler</code> method here.
             *
             * @param h a <code>WorbExceptionHandler</code> value
             */
        public void setExceptionHandler( WorbExceptionHandler h );
    }

    public static interface WorbExceptionHandler
    {
            /**
             * Describe <code>handleException</code> method here.
             *
             * @param proxy a <code>WorbProxy</code> value
             * @param e a <code>WorbException</code> value
             * @return a <code>Throwable</code> value
             */
        public Throwable handleException ( WorbProxy proxy, WorbException e );
    }

    public static abstract class AutoRetryHandler
        implements WorbExceptionHandler
    {
//         public AutoRetryHandler ( int ntimes, long delayMillis ){
//         }
//         public Throwable handleException ( WorbProxy proxy, WorbException e ){
//         }
    }
    
        /**
         * Exports the given service provider as implementing the given interface with
         * the defined service name.  service name.
         *
         * @param o the service provider
         * @param iface which interface is to be exported
         * @param name the service name
         * @return a URI suitable to be used with lookupService()
         */
    public String exportService ( Object o, Class iface, String name );

        /**
         * Unexports a service previously exported with exportService().
         *
         * @param o an <code>Object</code> value
         * @param name a <code>String</code> value
         */
    public void unexportService ( Object o, String name );
    
        /**
         * Returns a proxy to contact the service described by the given serviceURI which
         * implements the given interface.
         *
         * <p>The format accepted is
         *
         *       <code>worb:[<i>hostspec</i>]/<i>servicename</i></code>,
         *
         *  where <i>hostspec</i> would be:
         *  <ul>
         *
         *  <li> <i>the empty string</i>, in which case the id of the local host is
         *       assumed.
         *
         *  <li> <code>//hostid:<i>id</i>.<i>cellname</i></code>, menaning the host with
         *       host id <i>id</i>, an integer in hexadecimal notation, located at the cell
         *       with name <i>cellname</i> (See HostId.toExternalForm());
         *
         *  <li> <code>//hostid</code>, in which case the local cell is assumed
         *
         *  <li> <code>//@cellname</code>, meaning the celluar instance of the given
         *       service at the specified cell.
         *
         *  </ul>
         *
         * @param serviceURI a <code>String</code> value
         * @param iface a <code>Class</code> value
         * @return a proxy to contact the implements the given interface and that would
         * be used to access the given service.
         *
         * @todo This documentation is potencially out-dated.
         */
    public Object lookupService ( String serviceURI, Class iface );
    
        /**
         * Checks which host is the source of the current method call.
         *
         * @return a <code>HostId</code> describing the host that issued the call being
         * served by the current thread or null if the current method call is not getting
         * through WORB (e.g., because it is not a remote invocation).
         */
    public HostId getClientHost();
    
        /**
         * Describe <code>setExceptionHandler</code> method here.
         *
         * @param proxy an <code>Object</code> value
         * @param handler a <code>WorbExceptionHandler</code> value
         */
    public void setExceptionHandler ( Object proxy, WorbExceptionHandler handler );
    
}
