// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib.backend.ldap;


import org.isam.exehda.Exehda;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.CompositeResourceConstraint;
import org.isam.exehda.CompositeResourceConstraint.CRCElement;
import org.isam.exehda.AttributeSimpleMatchConstraint;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;



/**
 * A type handler that is actually built up of other type handler.
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-05-04 10:42:48 -0300 (Wed, 04 May 2005) $ $Revision: 850 $
 */
class CompositeLRTH
    extends LdapResourceTypeHandler
{
    public CompositeLRTH()
        {
                //
                // TO DO: proper super-class initialization
                //
            super(null, null);
            
            this.defaultPrefixHandler = null;
            this.defaultTypeHandler = null;
            this.handlersByCibType = new Hashtable();
            this.handlersByPrefix = new Hashtable();
            this.registeredPrefixes = new Vector();
            this.registeredCibTypes = new Vector();
        }
    
    public void
    addHandler( LdapResourceTypeHandler h )
        {
                //
                // FIX ME: perhaps should force default handler of types and prefixes to
                // be the same
                //
            
                // register handler for cib types
            String[] cts = h.getHandledCibTypes();
            if ( cts == null ) {
                    // register default handler
                registerTypeHandler(null, h);
            }
            else {
                    // register handler for specific types
                for ( int i=0; i< cts.length; i++ ) {
                    registerTypeHandler(cts[i], h);
                }
            }

                //
                // TO DO: register handler for dn prefixes
                //
            String[] dps = h.getHandledDnPrefixes();
            if ( dps == null ) {
                    // register default handler
                registerPrefixHandler(null, h);
            }
            else {
                    // register handler for specific prefixes
                for ( int i=0; i< dps.length; i++ ) {
                    registerPrefixHandler(dps[i], h);
                }
            }
        }

    public void
    deleteNode( LdapSession s, ResourceName rn )
        throws NamingException
        {
            LdapResourceTypeHandler h = lookupRNHandler(rn);
            h.deleteNode(s, rn);
        }

    public String
    createNode( LdapSession s, ResourceName rn )
        throws NamingException
        {
            LdapResourceTypeHandler h = lookupRNHandler(rn);
            return h.createNode(s, rn);
        }

    public String
    getNodeAttribute( LdapSession s, ResourceName rn, String a )
        throws NamingException
        {
            LdapResourceTypeHandler h = lookupRNHandler(rn);
            return h.getNodeAttribute(s, rn, a);
        }

    public String[]
    listNodeAttributes( LdapSession s, ResourceName rn )
        throws NamingException
        {
            LdapResourceTypeHandler h = lookupRNHandler(rn);
            return h.listNodeAttributes(s, rn);
        }

    public void
    setNodeAttribute( LdapSession s, ResourceName rn, String a , String v)
        throws NamingException
        {
            LdapResourceTypeHandler h = lookupRNHandler(rn);
            h.setNodeAttribute(s, rn, a, v);
        }

    public NameComponent
    decodeNameComponent( String dnc )
        {
            LdapResourceTypeHandler h = lookupDNHandler(dnc);
            return h.decodeNameComponent(dnc);
        }

    public NameSpace
    decodeNameSpace( String dn )
        {
            LdapResourceTypeHandler h = lookupDNHandler(dn);
            return h.decodeNameSpace(dn);
        }

    public ResourceName
    decodeResourceName( String dn )
        {
            LdapResourceTypeHandler h = lookupDNHandler(dn);
            return h.decodeResourceName(dn);
        }

    public DnBuffer
    encode( NameComponent nc, DnBuffer dnb )
        {
            LdapResourceTypeHandler h = lookupNCHandler(nc);
            return h.encode(nc, dnb);
        }

    public DnBuffer
    encode( ResourceName rn, DnBuffer dnb )
        {
            LdapResourceTypeHandler h = lookupRNHandler(rn);
            return h.encode(rn, dnb);
        }

    public DnBuffer
    encode( NameSpace ns, DnBuffer dnb ) 
        {
            LdapResourceTypeHandler h = lookupNCHandler(ns.getCommonName());
            return h.encode(ns, dnb);
        }

    public String
    toSearchFilter(ResourceConstraint rc)
        {
            String[] matchedTypes = checkResourceTypeMatch(rc);

                // if some resource type has been specified somewhere in the constraint
            if ( matchedTypes != null ) { 
                if ( matchedTypes.length == 1 ) {
                    LdapResourceTypeHandler h = (LdapResourceTypeHandler)
                        handlersByCibType.get(matchedTypes[0]);

                    return h.toSearchFilter(rc);
                }
                else {
                    StringBuffer filter = new StringBuffer("(|");
                    
                    for (int i=0; i<matchedTypes.length; i++) {                        
                        LdapResourceTypeHandler h = (LdapResourceTypeHandler)
                            handlersByCibType.get(matchedTypes[i]);
                        
                        filter.append(h.toSearchFilter(rc));
                    }
                    filter.append(")");

                    return filter.toString();
                }
            }
            else {
                return getDefaultSearchFilter();
            }
        }
    
    public String
    getDefaultSearchFilter()
        {
                //
                // TO DO: should return an OR of the internal handlers default filters.
                //
            throw new NullPointerException("Not yet implemented.");
        }

    public LdapResourceInfo
    toLdapResourceInfo( String dn, Attributes atts )
        {
            return lookupDNHandler(dn).toLdapResourceInfo(dn, atts);
        }

        /////////////////////////////////////////////////
        // PRIVATE STUFF
        /////////////////////////////////////////////////
    private final void
    registerTypeHandler( String type, LdapResourceTypeHandler h)
        {
            if ( type == null ) {
                this.defaultTypeHandler = h;
                log(Sev.DEBUG, "registerTypeHandler()",
                    "Registered default type handler "+h);
            }
            else {
                    // check if a handler have been already registered for that type
                Object oldh = handlersByCibType.put(type, h);
                if ( oldh != null ) {
                    log(Sev.WARNING, "registerTypeHandler()",
                        "overwritting handler for type <"+type+">");
                }
                else {
                    registeredCibTypes.add(type);
                    log(Sev.DEBUG, "registerTypeHandler()",
                        "Registered handler for type <"+type+">");
                }
            }
        }
    
    private final void
    registerPrefixHandler( String prefix, LdapResourceTypeHandler h)
        {
            if ( prefix == null ) {
                this.defaultPrefixHandler = h;
                log(Sev.DEBUG, "registerTypeHandler()",
                    "Registered default prefix handler "+h);
            }
            else {
                Object oldh = handlersByPrefix.put(prefix, h);
                if ( oldh != null ) {
                    log(Sev.WARNING, "registerPrefixHandler()",
                        "overwritting handler for prefix <"+prefix+">");
                }
                else {
                        //
                        // FIX ME: ignoring prefix precedence; need to keep registered
                        // types sorted so that more specific handlers are tryied prior
                        // to more general ones
                        //
                    registeredPrefixes.add(prefix);


                    log(Sev.DEBUG, "registerPrefixHandler()",
                        "Registered handler for prefix <"+prefix+">");
                }
            }
        }

    private final LdapResourceTypeHandler
    lookupDNHandler( String dn )
        {
            log(Sev.TRACE, "lookupDNHandler()", "dn="+dn);
            
            for ( int i=0; i< registeredPrefixes.size(); i++) {
                String prefix = (String) registeredPrefixes.elementAt(i);
                
                log(Sev.TRACE, "lookupDNHandler()", "checking prefix="+prefix);
            
                if ( dn.startsWith(prefix) ) {
                    log(Sev.TRACE, "lookupDNHandler()", "match! prefix="+prefix);
                    return (LdapResourceTypeHandler) handlersByPrefix.get(prefix);
                }
            }
            
            log(Sev.TRACE, "lookupDNHandler()", "prefix matching failed");
            return defaultPrefixHandler;
        }

    private final LdapResourceTypeHandler
    lookupHandler( String cibType )
        {
            Object h = handlersByCibType.get(cibType);

            return ( h == null ) ? defaultTypeHandler : (LdapResourceTypeHandler) h;
        }
    
    private final LdapResourceTypeHandler
    lookupNCHandler( NameComponent nc )
        {
            Object h = handlersByCibType.get(nc.getType());

            return ( h == null ) ? defaultTypeHandler : (LdapResourceTypeHandler) h;
        }

    private final LdapResourceTypeHandler
    lookupRNHandler( ResourceName rn )
        {
            return lookupNCHandler(rn.getCommonName());
        }

    private final void
    log(int sev, String mtd, String msg)
        {
            Logger logger = (Logger) Exehda.getService(Logger.SERVICE_NAME);

            logger.log(sev, getClass().toString(), mtd, msg, null);
        }

    
        /**
         * Returns a non-null array of CIB types if a constraint on the resource type has
         * been specified in the given ResourceConstraint object;
         *
         * @param rc a <code>ResourceConstraint</code> value
         * @return a <code>String[]</code> value
         */
    private final String[]
    checkResourceTypeMatch( ResourceConstraint rc )
        {
                //
                // TO DO: review recursion
                //
            
                // if its a composite constraint
            if ( rc instanceof CompositeResourceConstraint ) {
                    // check composite elements individually
                Enumeration ec = ((CompositeResourceConstraint) rc).constraints();
                while ( ec.hasMoreElements() ) {

                    String[] matchedTypes = checkResourceTypeMatch(
                        ((CRCElement) ec.nextElement()).getConstraint());
                        
                        //
                        // XXX: assuming an AND, so stopping at the first type match definition
                        //
                    if ( matchedTypes != null) return matchedTypes;
                }
            }
            else if ( rc instanceof AttributeSimpleMatchConstraint ) {

                AttributeSimpleMatchConstraint arc =
                    (AttributeSimpleMatchConstraint) rc;
                
                if ( CIB_ATT_RES_TYPE.equals(arc.getAttributeName()) ) {
                        // explode matching exp into actual handled types
                    
                        //
                        // XXX: requires JDK 1.4
                        //
                    return  arc.getMatchingExp().split("\\|");
                }
            }
            
            return null;
        }
    
//     private Hashtable rnToDnCache;
//     private Hashtable dnToHandlerCache;
    private LdapResourceTypeHandler defaultTypeHandler;
    private LdapResourceTypeHandler defaultPrefixHandler;
    private Hashtable handlersByCibType;
    private Hashtable handlersByPrefix;
    private Vector registeredPrefixes;
    private Vector registeredCibTypes;
}
