// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib.backend.ldap;


import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.ResourceConstraint.ResourceInfo;
import org.isam.exehda.CompositeResourceConstraint;
import org.isam.exehda.AttributeConstraint;
import org.isam.exehda.AttributeSimpleMatchConstraint;
import org.isam.exehda.AttributeDefinedConstraint;
import org.isam.exehda.services.cib.backend.CibBackEnd;
import org.isam.exehda.services.cib.backend.CibBackEndException;
import org.isam.exehda.services.cib.backend.CibBackEndConf;



import javax.naming.*;
import javax.naming.directory.*;
import javax.naming.ldap.*;


import java.util.Vector;
import java.util.Hashtable;
import java.util.Properties;
import java.util.List;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Enumeration;

/**
 *
 * @todo This documentation is out-dated.
 *
 *
 * Ldab Install and Configuration Instructions:
 * <ol>
 *
 * <li> Install (depends on which OS is being used, in the case of Debian an apt-get
 * install slapd will probably work).
 *
 * <li> edit slapd.conf (/etc/ldap/slapd.conf, generally)
 *
 *   <ul>
 *   <li> include the line:
 *        <pre>include         /etc/ldap/schema/java.schema</pre>
 *        below the other lines similar to this one (close to the beginning of the file)
 *
 *   <li> Change the line:
 *        <pre>schemacheck     on</pre>
 *        to:
 *        <pre>schemacheck     off</pre>
 *
 *   <li> Change the line:
 *        <pre>suffix          "alguma-coisa"</pre>
 *        to:
 *        <pre>suffix          "dc=CELL"</pre>
 *
 *   <li> Change the line:
 *        <pre>rootdn "alguma coisa" </pre>
 *        to:
 *        <pre>rootdn "cn=Manager,dc=CELL"</pre>
 *
 *   <li> Change the line:
 *        <pre>rootpw "uma-senha"</pre>
 *        to:
 *        <pre>rootpw "a-senha-que-voce-quer"</pre>
 *   </ul>
 *
 * <li> Edit the file core.schema (/etc/ldap/schema/core.schema, generally), adding the
 * following :
 *    <pre>
 *    attributetype ( 2.5.4.70 NAME 'dn'
 *    DESC 'DN'
 *    SUP name )
 *
 *    attributetype ( 2.5.4.71 NAME 'fullDn'
 *    DESC 'DN full'
 *    SUP name )
 *
 *    attributetype ( 2.5.4.72 NAME 'a'
 *    DESC 'for tests'
 *    SUP name )
 *
 *    attributetype ( 2.5.4.75 NAME 'isamType'
 *    DESC 'for isam'
 *    SUP name )
 *
 *    attributetype ( 2.5.4.79 NAME 'isamResource'
 *    DESC 'for isam'
 *    SUP name )
 *    </pre>
 *  
 *    <p> After the following entry:
 *    <pre>
 *    attributetype ( 2.5.4.41 NAME 'name' 
 *    EQUALITY caseIgnoreMatch
 *    SUBSTR caseIgnoreSubstringsMatch
 *    SYNTAX 1.3.6.1.4.1.1466.115.121.1.15{32768} )
 *    </pre>
 *
 *    <p> If the above entry does not exist, the new lines can be placed at the beginning
 *    of the file.
 *
 * <li> Restart ldap (/etc/init.d/slapd restart, probably)
 *
 * <li> execute org.isam.exehda.services.cib.CibInstaller
 *
 * <li> Done
 * </ol>
 *
 *
 * @author frainer. Last modified by $Author: lucc $
 * @version $Date: 2005-05-04 11:41:34 -0300 (Wed, 04 May 2005) $ $Revision: 855 $
 */
public class LdapBackEnd
    implements CibBackEnd
{
    final static String LDAP_ATT_ISAM_RSRC = "isamResource";
    final static String LDAP_ATT_ISAM_ATTR = "isamType";

    final static String CIB_NS_TYPE_CELL = "cell";
    
//     static class RInfo
//         extends Hashtable
//         implements ResourceInfo
//     {
//         ResourceName rn;
        
//             /** Creates a new instance of ResourceImpl */
//         public RInfo(ResourceName rn)
//             {
//                 this.rn = rn;
//             }
    
//         public String getAttribute(String n)
//             {
//                 return (String) get(n);
//             }
    
//         public void setAttribute(String n, String v)
//             {
//                 put(n,v);
//             }
    
//         public ResourceName getName()
//             {
//                 return this.rn;
//             }   
//     }

    public LdapBackEnd()
        {
            this.sessionFactory = null;
        }

    
    public void
    init( CibBackEndConf conf )
        throws CibBackEndException
        {
                //
                // FIX ME: property names in-line; default values
                //
            String serverURI = conf.getProperty("ldap.server",
                                                "ldap://localhost");
            
            this. rootDn = conf.getProperty("ldap.rootDn",
                                             "dc=CELL");
            
            String user = conf.getProperty("ldap.user",
                                           "cn=Manager,dc=CELL");
            
            String passwd = conf.getProperty("ldap.passwd", "*****");

                // create session factory
            this.sessionFactory = new LdapSessionFactory(serverURI, rootDn, user, passwd);

            try {
                    // check access to the LDAP server
                LdapSession sTest = sessionFactory.createSession();
                sessionFactory.releaseSession(sTest);
            }
            catch (Exception e) {
                throw new CibBackEndException("LdapBackend sanity check failed", e);
            }

                // init root LdapResourceTypeHandler
            initRootResourceTypeHandler();
        }

    
        /**
         * Adds a new resource to the cib, if it doesn't already exist .
         *
         * @param  r  The ResourceName that is to be added
         */
    public synchronized void
    addResource(ResourceName r)
        throws CibBackEndException
        {
            try {
                LdapSession ldap = sessionFactory.createSession();
            
                rootHandler.createNode(ldap, r);
            
                sessionFactory.releaseSession(ldap);
            }
            catch (NamingException ne) {
                throw new CibBackEndException("addResource() failed", ne);
            }
        }
    
    
        /**
         * Returns the value of the especified attribute in the especified resource.
         *
         * @param r The resource
         * @param a The name of the attribute  
         *        
         * @return the value of the attribute     
         */
    public synchronized String
    getAttribute( ResourceName r, String a )
        throws CibBackEndException
        {
            try {
                LdapSession ldap = sessionFactory.createSession();

                String result = rootHandler.getNodeAttribute(ldap, r, a);
       
                sessionFactory.releaseSession(ldap);
            
                return result;
            }
            catch ( NamingException ne ) {
                throw new CibBackEndException("getAttributed() failed", ne);
            }
        }
    
        /**
         * Returns all the names of the attribues of a given resource 
         *
         * @param r The name of the resource 
         *        
         * @return an array of string cointaining all the names of the attributes of the
         * resource
         */
    public synchronized String[]
    getAttributes ( ResourceName r )
        throws CibBackEndException
        {
            try {
                LdapSession ldap = sessionFactory.createSession();

                String[] result = rootHandler.listNodeAttributes(ldap, r);
            
                sessionFactory.releaseSession(ldap);
            
                return result;
            }
            catch ( NamingException ne ) {
                throw new CibBackEndException("getAttributes() failed", ne);
            }
        }
    
        /**
         * Removes a resource from the cib.
         *
         * @param r The name of the resource that will be removed
         */
    public synchronized void
    removeResource( ResourceName r )
        throws CibBackEndException
        {
            try {
                LdapSession ldap = sessionFactory.createSession();

                rootHandler.deleteNode(ldap, r);
            
                sessionFactory.releaseSession(ldap);
            }
            catch ( NamingException ne ) {
                throw new CibBackEndException("removeResources() failed", ne);
            }
        }
    
        /**
         * Searches the cib for resources that match a certain ResourceConstraint
         *
         * @param rc The constraints the returned resources must satisfy
         * @param ns Scope of the search
         * @param maxResults The maximum number of results that are to be returned
         *
         * @return An array containing the ResourceNames of the cib entries that matched
         * the ResourceConstraint
         */
    public synchronized ResourceName[]
    select( ResourceConstraint rc, NameSpace ns, int maxResults )
        throws CibBackEndException
        {            
            try {
                String filter = rootHandler.toSearchFilter(rc);

                System.out.println("Search filter: "+filter);
                                   
                    //
                    // TO DO: perhaps reuse DnBuffer
                    //
                String scope = rootHandler.encode(ns, null).toString();

                LdapSession ldap = sessionFactory.createSession();
                
                    //
                    // XXX: the '+' bellow allow use to access operation attributes too; e.g.
                    // modification time
                    //
                NamingEnumeration ne = ldap.search(scope, filter, new String[] { "*", "+" } );

                Vector tmp = new Vector( maxResults>0 ? maxResults : 20 );
            
                while ( ne.hasMore() ) {
                    SearchResult sr = (SearchResult) ne.next();
                    Attributes atts = sr.getAttributes();
                    
                        // re-build full dn
                    String dn = sr.getName();
                    if ( "".equals(dn) ) {
                        dn = scope;
                    }
                    else {
                        dn += "," + scope;
                    }

                        // second stage filtering
                    LdapResourceInfo ri = rootHandler.toLdapResourceInfo(dn, atts);

                    System.out.println("select(): dn="+dn+", rn="+ri.getName());
                    
                    if ( rc.check(ri) == ResourceConstraint.OK ) {
                        tmp.add(ri.getName());
                    }
                }
                                               
                sessionFactory.releaseSession(ldap);

                    // limit answer size to maxResults
                maxResults = ( maxResults < 0 ) ? tmp.size()
                    : ( maxResults < tmp.size() ? maxResults : tmp.size() );

                ResourceName[] results = new ResourceName[maxResults];
                for (int i=0; i<results.length; i++) {
                    results[i] = (ResourceName) tmp.elementAt(i);
                }

                return results;
            }
            catch (NamingException ne) {
                throw new CibBackEndException("select() failed", ne);
            }
    }

    
        /**
         * Sets the value of a given attribute in a given resource
         *
         * @param r the resource which "owns" the attribute that is to be set
         * @param a the name of the attribute that is to be set
         * @param v the value to which the attribute is to set        
         *
         * @return a string containing tha value to which the attribute was set    
         */
    public synchronized String
    setAttribute(ResourceName r, String a, String v)
        throws CibBackEndException
        {
            try {
            LdapSession ldap = sessionFactory.createSession();
            
            rootHandler.setNodeAttribute(ldap, r, a, v);

                //
                // FIX ME: do we realy need to return the new value??
                //
            v = rootHandler.getNodeAttribute(ldap, r, a);
            
            sessionFactory.releaseSession(ldap);
            
            return v;
            
        }
            catch ( NamingException ne ) {
                throw new CibBackEndException("setAttribute() failed", ne);
            }
        }

        ///////////////////////////////////////////////////////////////////////
        // PRIVATE STUFF
        ///////////////////////////////////////////////////////////////////////

    
        /**
         * Install the root resource type handler.
         *
         * <p>In future the root handler will be installed in the LDAP itseft
         * (i.e. serialized to an LDAP entry) and loaded upon backend
         * initialization. This way it would be very straitforward to support different
         * mapping strategies for each ldap repository. Each repository <quote>will
         * know</quote> about its own strategy.
         *
         * <p>Currently, the root handler creation is hard-coded in the ldap backend. So,
         * only one mapping strategy is supported by now.
         *
         */
    private void
    initRootResourceTypeHandler()
        {
                //
                // TO DO: read the root handler from ldap
                //
//                 // load resource type handle table from the server
//             LdapSession session = sessionFactory.createSession();
//             ...
//             sessionFactory.releaseSession(session);
            
            CompositeLRTH rh = new CompositeLRTH();
            rh.addHandler(new DefaultLRTH());
            rh.addHandler(new CellLRTH());
            rh.addHandler(new HostLRTH());
            rh.addHandler(new ServiceLRTH());
            rh.addHandler(new ApplicationLRTH());

            this.rootHandler = rh;
            LdapResourceTypeHandler.setRootHandler(rh);
        }

//         /**
//          * Generates a search string that the ldap server can understand from a 
//          * given ResourceConstraint 
//          *
//          * @param filter The filter to be transformed into a string
//          *        
//          * @return a search string that can be understood by a ldap server
//          */
//     private void
//     toLdapQuery(ResourceConstraint filter, StringBuffer buff)
//         {
//                 // open expression
//             buff.append('(');

//             if ( filter instanceof CompositeResourceConstraint ) {
//                     // split the composite
//                 CompositeResourceConstraint crc = (CompositeResourceConstraint) filter;

//                     //
//                     // XXX: assumes the composition operation is always a logical AND
//                     //
//                 buff.append('&');

//                 Enumeration ce = crc.constraints();
//                 while ( ce.hasMoreElements() ) {
//                     toLdapQuery((ResourceConstraint) ce.nextElement(), buff);
//                 }
//             }
//             else if ( filter instanceof AttributeConstraint ) {
//                 String att = ((AttributeConstraint) filter).getAttributeName();

//                     // either match regexp or match anything
//                 String regexp = ( filter instanceof AttributeSimpleMatchConstraint )
//                     ? ((AttributeSimpleMatchConstraint) filter).getMatchingExp()
//                     : "*";
                
//                     // if its a mapped attribute type
//                 if ( attributeConversion.containsKey(att) ) {
//                         // add the attribute after converting it
//                     buff.append(attributeConversion.get(att)).append('=');
//                 }
//                 else {
//                         // otherwise, encode the attribute using the default isam scheme
//                     buff.append(LDAP_ATT_ISAM_ATTR).append('=').append(att).append(':');
//                 }
//                     // finally, append the regular expression to be matched
//                 buff.append(regexp);
//             }
//                 // close expression
//             buff.append(')');
//         }
    
    

    
        /**
         * Finds the name of the Cell to which the NameSpace belongs
         *
         * @param rn the name space
         *         
         * @return a String with the name of the cell to which the given NameSpace
         * belongs to. Null if it doesn't belong to any cell.
         */
    private String
    getCell(NameSpace ns)
        {
            while ( ns != null ) {
                if ( CIB_NS_TYPE_CELL.equals(ns.getType()) ) {
                    return ns.getSimpleName();
                }
                ns = ns.getParent();
            }
            return null;
        }
    
        /**
         * Finds the name of the Cell to which the ResourceName belongs
         *
         * @param  rn  the resource name 
         *         
         * @return a String with the name of the cell to which the given ResourceName
         * belongs to. Null if it doesn't belong to any cell.
         */
    private String
    getCell(ResourceName rn)
        {
            return getCell(rn.getNameSpace());
        }
    
    
//         /**
//          * Adds the appropiate cell NameSpace to the root of the given NameSpace
//          *
//          * @param ns the NameSpace that is to be modified
//          *         
//          * @return a new version of the given NameSpace with the proper cell NameSpace at
//          * the root
//          */
//     private NameSpace
//     setCellParent(NameSpace ns)
//         {
//             if ( ns == null ) {
//                 return new NameSpace("cell",
//                                      cellName,
//                                      new NameSpace("","",null));
//             }
//             else {
//                 return new NameSpace(ns.getType(),
//                                      ns.getSimpleName(),
//                                      setCellParent(ns.getParent()));
//             }
//         }
    
//         /**
//          * Adds the appropiate cell NameSpace to the root of the given ResourceName
//          *
//          * @param rn the resource name that is to be modified
//          *         
//          * @return a new version of the given ResourceName with the proper cell NameSpace
//          * at the root
//          */
//     private ResourceName
//     setCellParent(ResourceName rn)
//         {
//             return new ResourceName(rn.getType(),
//                                     rn.getSimpleName(),
//                                     setCellParent(rn.getNameSpace()));
//         }
    
//         /**
//          * converts a given ResourceName into a DN string representation that can be
//          * understood by a ldap server
//          *
//          * @param r the resource name that is to be converted
//          *         
//          * @return a string representation of the given ResourceName that can be
//          * understood by a ldap server
//          */
//     private final String
//     encodeResourceName(ResourceName rn)
//         {
// //             if ( rn == null ) {
// //                 return("");
// //             }

// //                 // fill default namespace
// //             if ( getCell(rn) == null ){
// //                 rn = setCellParent(r);
// //             }
             
// //             String rt = rn.getType();
// //             String ri = rn.getSimpleName();
            
// //             String dn = ( typeConversion.containsKey(rt) )
// //                 ? dn = (String) typeConversion.get(rt)+"="+ri
// //                 : LDAP_ATT_ISAM_RSRC+"="+rt+":"+ri;
            

// //             String nsdn = encondeNameSpace(rn.getNameSpace());

// //             return ( "".equals(nsdn) ) ? dn+","+nsdn : dn;

//             throw new NullPointerException("Not yet implemented.");
//         }
    
//         /**
//          * converts a given NameSpace into a DN string representation that can be
//          * understood by a ldap server
//          *
//          * @param ns the NameSpace that is to be converted
//          *         
//          * @return a string representation of the given NameSpace that can be understood
//          * by a ldap server
//          */
//     public String
//     encondeNameSpace( NameSpace ns ) {
// //         if ( ns == null 
// //              || ns.getName() == null
// //              || "".equals(ns.getType()) ) {
// //                 //ns = CellInformationBase.NameSpace.DEFAULT_NS;
// //             return("");
// //         }
        
// //         if ( getCell(ns)==null ) {
// //             ns = setCellParent(ns);
// //                 //System.out.println(r);
// //         }

// //             //
// //             // <======== STOPED HERE ---------------------------------------------------------------------------------------------
// //             //

// //             //System.out.println(r.getDN());
// //         String type = ns.getType();
// //         String name = ns.getSimpleName();
// //         String dn = "";
        
// //         if(typeConversion.containsKey(type)){
// //             dn = (String)typeConversion.get(type)+"="+name;
// //         }
// //         else{
// //             dn = "isamResource="+type+":"+name;//r.getDN().replaceAll(":","=").replaceAll(type,"isamType").replaceAll(name,type+":"+name).replaceAll("\\[","").replaceAll("\\]","");
// //         }
// //             //dn = dn.substring(0,dn.lastIndexOf(","));
// //         if(convertToLdap(ns.getParent()) != ""){
// //             return(dn+","+convertToLdap(ns.getParent()));
// //         }else{
// //             return(dn);
// //         }
        
// //             /*String dn = ns.toString().replaceAll("\\[","").replaceAll(":","=").replaceAll("\\]","");
// //               int p = 0;
// //               while(dn.indexOf(",",p)!=-1){
// //               String type = dn.substring(p,dn.indexOf("=",p));
// //               String name = dn.substring(dn.indexOf("=",p)+1,dn.indexOf(",",p)-1);
// //               if(typeConversion.containsKey(type)){
// //               dn = dn.replaceFirst(type,(String)typeConversion.get(type));
// //               }
// //               else{
// //               dn = dn.replaceFirst(type,"isamType").replaceFirst(name,type+":"+name);
// //               }
// //               p = dn.indexOf(",",p)+1;
// //               }
// //               if(dn.indexOf("=") != -1){
// //               String type = dn.substring(p,dn.indexOf("=",p));
// //               String name = dn.substring(dn.indexOf("=",p)+1);
// //               if(typeConversion.containsKey(type)){
// //               dn = dn.replaceFirst(type,(String)typeConversion.get(type));
// //               }
// //               else{
// //               dn = dn.replaceFirst(type,"isamType").replaceFirst(name,type+":"+name);
// //               }
// //                   //String dn = ns.toString().replace('[',' ').replace(':','=').replace(']',' ');
// //                   }
// //                   return(dn);*/

        
//         throw new NullPointerException("Not yet implemented.");
//     }
    
//         /**
//          * converts a string with the name of an ldap resource, in the format supported
//          * by ldap servers, into a ResourceName 
//          *
//          * @param  dn  A string with the name of a resource in the ldap server
//          *         
//          * @return  a ResourceName that is points to the same resource as the given String 
//          */
//     private ResourceName convertFromLdap(String dn){
//         NameSpace ns = null;
//         ResourceName rn = null;
        
//         while(dn.indexOf(",")!=-1){
//             String aux = dn.substring(dn.lastIndexOf(",")+1);
//             String type = aux.substring(0,aux.indexOf("="));
//             String value = aux.substring(aux.indexOf("=")+1);
//                 //System.out.println("Tipo -- "+type);
//             if(type.equals("isamResource")){
//                 type = value.substring(0,value.indexOf(":"));
//                 value = value.substring(value.indexOf(":")+1);
//             }
//             if(typeBackConversion.containsKey(type)){
//                 type = (String)typeBackConversion.get(type);
//             }
//                 //System.out.println("Tipo -- "+type);
//             NameSpace auxNS = new NameSpace(type, value,ns);
//             ns = auxNS;
//             dn = dn.substring(0,dn.lastIndexOf(","));
//         }
       
//         String type = dn.substring(0,dn.indexOf("="));
//         String value = dn.substring(dn.indexOf("=")+1);
//         if(type.equals("isamResource")){
//             type = value.substring(0,value.indexOf(":"));
//             value = value.substring(value.indexOf(":")+1);
//         }
//         if(typeBackConversion.containsKey(type)){
//             type = (String)typeBackConversion.get(type);
//         }
//         if(ns == null){
//             rn = new ResourceName(type,value,new NameSpace("","",null));//type+":"+value);
//         }
//         else{
//             rn = new ResourceName(type,value,ns);
//         }
        
//         return(rn);
//     }
    
    
//         /**
//          * converts a set of attributes into a resource
//          *
//          * @param  attributes  The attributes to be converted into a resource
//          *         
//          * @return  a ResourceImpl that contains all the given attributes
//          */
//     private RInfo
//     makeResourceFromAttributes(Attributes attributes)
//         {
//             try{
//                 ResourceName rn = convertFromLdap((String)attributes.get("fullDN").get());
//                 RInfo resource = new RInfo(rn);
//                 Enumeration attrs = attributes.getAll();
//                 while(attrs.hasMoreElements()){
//                     Attribute attr = (Attribute)attrs.nextElement();
//                     String type = (String)attr.getID();
//                     if(attributeBackConversion.containsKey(type)){
//                         type = (String)attributeBackConversion.get(type);
//                     }
//                     else if(type.equals("isamType")){
//                         Enumeration e = attr.getAll();
//                         while(e.hasMoreElements()){
//                             String value = (String)e.nextElement();
//                             resource.setAttribute(value.substring(0,value.indexOf(":")),value.substring(value.indexOf(":")+1));
//                         }
//                     }
//                     resource.setAttribute(type,(String)attr.get());
//                 }   
//                 return(resource);
//             }catch(javax.naming.NamingException ne){ne.printStackTrace();}
//             return(null);
//         }
    
//         /**
//          * converts a full regular expression into something tha ldap can understand. 
//          * Some of the meaning of original regular expression may be lost. 
//          *
//          * @param  regexp  A regular expression string
//          *         
//          * @return  a new String with a regular expression that ldap can understand
//          */
//     private String convertRegexpToLdap(String regexp)
//         {
//             String ldapRegexp = "";
//             ldapRegexp = regexp.replaceAll("\\.","*");
//             ldapRegexp = ldapRegexp.replaceAll("\\[.*\\]", "*");
//             ldapRegexp = ldapRegexp.replaceAll("\\?","*");
//             ldapRegexp = ldapRegexp.replaceAll("\\+","*");
//             ldapRegexp = ldapRegexp.replaceAll("\\{.*\\}","*");
//             ldapRegexp = ldapRegexp.replaceAll("\\(.*\\)","*");
//             ldapRegexp = ldapRegexp.replace('\\','*');
//                 //System.out.println(ldapRegexp);
//             return(ldapRegexp);
//         }

//     private final ResourceName
//     filterLdapResults(ResourceConstraint rc, List ldapResults, int maxResults)
//         {
//             maxResults = ( maxResults == -1 ) ?
//                 ldapResults.size() : Math.min(ldapResults.size(), maxResults);
        
// //             List trueResults = new LinkedList();
// //             Iterator it = ldapResults.iterator();
// //             while ( it.hasNext() ) {
// //                 RInfo ri = makeResourceFromAttributes((Attributes)it.next());
            
// //                 if ( filter.check(ri) == ResourceConstraint.OK ) {
// //                     trueResults.add(ri);
// //                 }
// //             }

// //                 //
// //                 // <======== STOPED HERE ---------------------------------------------------------------------------------------------
// //                 //
// //                 // TODO: re-factor trueResults filtering into a separated method
        
// //             ldapResults = trueResults;
// //             ResourceName[] results; 
// //             if((ldapResults.size()<maxResults)||(maxResults == -1)){
// //                 results = new ResourceName[ldapResults.size()];
// //                 maxResults = ldapResults.size();
// //             }
// //             else{
// //                 results = new ResourceName[maxResults];
// //             }
        
// //             it = ldapResults.iterator();
// //             for(int i=0; ((i<maxResults)&&(it.hasNext())); i++){
// //                 results[i] = ((RInfo)it.next()).getName();
// //             }
// //             return(results);

//             return null;
//         }


    private LdapResourceTypeHandler rootHandler;
    private LdapSessionFactory sessionFactory;
    private Hashtable typeConversion;
    private Hashtable typeBackConversion;
    private Hashtable attributeConversion;
    private Hashtable attributeBackConversion;
    private String rootDn;
    
}
