// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.ctxm;

import org.isam.exehda.services.ContextManager.Context;
import org.isam.exehda.services.ContextManager.ContextState;


/**
 * Event that describes a change in the state of a context element.
 *
 * @todo RT issue: what about event time-stamps and deadline?
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2004-12-31 19:21:29 -0200 (Fri, 31 Dec 2004) $ $Revision: 552 $
 */
class ContextEvent
{
        /**
         * Creates a new <code>ContextEvent</code> instance.
         *
         * Either <code>prev</code> or <code>curr</code> may be null but not both.
         *
         * @param prev the previous state of the context element.
         * @param curr the current (new) state of the context element.
         * @param timeStamp the time-stamp for the event in milliseconds since 1970.
         */
    public ContextEvent( ContextState prev, ContextState curr, long timeStamp )
        {
            if ( prev == null && curr == null ) {
                throw new IllegalArgumentException(
                    "Either previous or current state must be a non-null value");
            }

            if ( prev != null && curr != null ) {
                if ( !prev.getType().equals(curr.getType()) ) {
                    throw new IllegalArgumentException(
                        "Incompatible context elements");
                }
            }
            
            this.prev = prev;
            this.curr = curr;
            this.timeStamp = timeStamp;
        }
    
        /**
         * Returns the context element to which this context change event is related.
         *
         * @return a <code>Context</code> value
         */
    public Context
    getContextType()
        {
            return ( prev == null ) ? curr.getType() : prev.getType();
        }

        /**
         * Returns the previous state of the context element.
         *
         * @return the previous state or null if it is undefined/unknown.
         */
    public ContextState
    getPreviousState()
        {
            return this.prev;
        }

        /**
         * Returns the current (new) state of the context element.
         *
         * @return the current state or null if it is undefined/unknown.
         */
    public ContextState
    getState()
        {
            return this.curr;
        }

        /**
         * The time-stamp describing the moment when this event was generated.
         *
         * @return a <code>long</code> value
         */
    public long
    getTimeStamp()
        {
            return this.timeStamp;
        }

    private ContextState prev;
    private ContextState curr;
    private long timeStamp;
}

