// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.ctxm.filter.basic;

import org.isam.exehda.services.ctxm.ContextFilter;
import org.isam.exehda.services.ctxm.ContextFilterFactory;
import java.lang.reflect.Constructor;

/**
 * Factory for context chains that are able to filter data from a single raw sensor.
 *
 * <p>Accepted context definition idiom:
 <code><pre>
 &lt;CONTEXT n="ctxname"&gt;
 &lt;STATES&gt;
 &lt;STATE n="s1"/&gt;
 &lt;STATE n="s2"/&gt;
 &lt;STATE n="s3"/&gt;
 &lt;STATE n="unknow"/&gt;
 &lt;STATES&gt;

 &lt;INDEX&gt;
 &lt;SENSOR n="sname"/&gt;
 &lt;/INDEX&gt;

 &lt;RANGES&gt;
 &lt;RANGE ub="1.0" state="s1"/&gt;
 &lt;RANGE olb="1.0" ub="2.0" state="s2"/&gt;
 &lt;RANGE olb="2.0" oub="3.0" state="s3"/&gt;
 &lt;DEFAULT state="unknown"/&gt;
 &lt;/RANGES&gt;
     
 &lt;/CONEXT&gt;
 </pre></code>
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-06-16 16:39:15 -0300 (Thu, 16 Jun 2005) $ $Revision: 920 $
 */
public class BasicContextFilterFactory
    implements ContextFilterFactory
{
    final static String FILTER_PREFIX = "org.isam.exehda.services.ctxm.filter.basic.SingleSensor";
    final static String FILTER_SUFFIX = "Filter";
    final static Class[] CONSTRUCTOR_PARAMS = new Class[] { String.class };

       /**
         * Create a context filter able to produce the context state
         * information acordingly to the given filter definition.
         *
         * @todo Throw InvallidContextDefinitionException
         * @todo Document supported filter definition sintaxes/idioms
         *
         * @param fdef a filter definition
         * @return a <code>ContextFilter</code> value
         */
    public ContextFilter
    createFilter ( String fdef )
        {  
                //
                // TO DO: read attribute 'syntax' from fdef
                //
            final String syntax = "NumericRangeMapping";

            try {
                    // dinamically load the filter corresponding to the selected syntax
                Constructor c = Class.forName(FILTER_PREFIX + syntax + FILTER_SUFFIX)
                    .getDeclaredConstructor(CONSTRUCTOR_PARAMS);
            
                return (ContextFilter) c.newInstance(new Object[] {fdef});
            }
            catch (Exception e) {
                throw new IllegalArgumentException(
                    "Could not create the context filter for \n"
                    +fdef
                    +"\n due to "+e);
            }
        }
}
