// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.ox;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ObjectId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.OXManager;
import org.isam.exehda.services.OXManager.OXAttributeOps;
import org.isam.exehda.services.OXManager.OXHandle;
import org.isam.exehda.services.OXManager.MasterOXHandle;
import org.isam.exehda.services.OXManager.HomeOXHandle;
import org.isam.exehda.services.OXManager.ProxyOXHandle;
import org.isam.exehda.services.OXManager.IllegalOXHandleException;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.Worb;
import java.util.Hashtable;



/**
 * OX Manager node instance.
 *
 * @author frainer, last modified by $Author: lucc $
 * @version $Date: 2005-04-08 15:24:11 -0300 (Fri, 08 Apr 2005) $ $Revision: 734 $
 */
public class OXManagerClientImpl
    extends AbstractService
    implements OXManager
{    
    final static String PROP_OX_URI = "contactAddress";

    final static HostId LOCAL_HOST = HostId.getLocalHost();

    
    public void
    start() throws Exception
        {
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);

            String oxmCellUri = Exehda.getServiceProperty(OXManager.SERVICE_NAME,
                                                         PROP_OX_URI,
                                                         "//localhost:1980/ox");
            this.oxmCell = (OXManager)
                worb.lookupService(oxmCellUri, OXManager.class);
        }

    public void
    stop()
        {
                //
                // TODO: unexport service
                //
        }

    
    
        /**
         * Creates a new instance of OXManagerClientImpl
         */
    public OXManagerClientImpl()
        {
            this(null);
        }
    
        /**
         * Creates a new instance of OXManagerClientImpl
         */
    public OXManagerClientImpl(OXManager oxmCell)
        {
            this.handlesById = new Hashtable();
            this.grantedHandles = new Hashtable();
            this.pendingHomeHandles = new Hashtable();
            this.timeBetweenUpdates = 1000;
            this.oxmCell = oxmCell;
        }
    
        /**
         * Create handle is used to create a new OXHandle for the OX with the given
         * ObjectId. Wheter a new MasterOXHanlde or a new ProxyOXhandle will be created
         * depends on wheter an OXHandle had already been created for the given OX or not
         *
         * @param oid a <code>ObjectId</code> representing the OX to which the OXHandle
         * to be created will refer
         * @return a new <code>OXHandle</code>
         */
    public OXHandle
    createHandle(org.isam.exehda.ObjectId oid)
        {
                // if this node is the home node for the given ox
            if ( LOCAL_HOST.equals(oid.getHost()) ) {
                    // create the master handle (if needed)
                createMasterHandle(oid);
            }
                //  always return a proxy; no cheking is done if the remote ox actually
                //  exists
            return new ProxyOXHandle(oid);
        
                 
//                 // if master handle already exists
//             if ( handlesById.containsKey(oid) ) {
//                     // return a proxy for it
//                 return new ProxyOXHandle(oid);
//             }
//             else {
//                 if ( oid.getHost().equals(HostId.getLocalHost())
//                      && !grantedHandles.containsKey(oid) ) {
//                         //System.out.println("**************adding:"+oid);
//                     MasterOXHandle handle = new MasterOXHandle(oid);
//                     handlesById.put(oid,handle);
//                         //oxmCell.createHandle(oid);
//                     pendingHomeHandles.put(oid,"");
//                     return new ProxyOXHandle(oid,timeBetweenUpdates);
//                 }
//                 else {
//                     return new ProxyOXHandle(oid,timeBetweenUpdates);
//                 }
//             }
//             }
//             return null;
        }
    
        /**
         * Grant ownership invalidates the ownership of the given OXHandle, allowing a
         * new node to request the ownership of this OXHanlde. While the OXHandle
         * ownership is undefined all calls to that handle get queed until a new
         * ownership is requested.  To be used when an OXhandle migrates from one node to
         * another.
         *
         * @param ox a <code>OXHandle</code> whose ownership is being granted
         * @return a <code>OXhandle</code> suitable to be used in a subsequent call to
         * requestOwnership()
         */
    public OXHandle
    grantOwnership(OXHandle ox)
        throws IllegalOXHandleException // HandleNotFoundException, NotMasterHandleException
        {
            ObjectId oid = ox.getId();
                // if the ownership of the given ox belongs to this manager
            if ( handlesById.containsKey(oid) ) {
                    // commit pending home handle if needed
                if ( pendingHomeHandles.containsKey(oid) ) {
                    oxmCell.createHandle(oid);
                    pendingHomeHandles.remove(oid);
                }
                    // stops managing this ox
                MasterOXHandle masterHandle = (MasterOXHandle) handlesById.remove(oid);
                grantedHandles.put(oid,oid);
                    // forwards ownership to the cellular oxmanager
                return oxmCell.grantOwnership(masterHandle);
            }
            else {
                throw new IllegalOXHandleException("Trying to grant onwership on un-owned ox:"+oid); 
            }
        }
    
        /**
         * Release handle takes away the given handle from the handle list.
         * Only to be uswed when the OX is no longer necessary.
         *
         * @param ox a <code>OXHandle</code> that is to be realeased
         * @return void
         */
    public void
    releaseHandle(OXHandle ox)
        {
                //
                // TO DO: should delete the master handle
                //
            throw new NullPointerException("Not yet implemented.");
        }
    
        /**
         * Requests the MasterHandle to be moved towards the calling host.
         *
         * <p>Given that the master handle is not currently owned by other manager (i.e. its
         * stored in the base of the home cell) it will be moved through the
         * requestOwnership() calling chain and eventually it will reach the calling
         * node. Although, some manager in the calling path would choose not forwarding
         * the master handle and keep it with itself.
         *
         * @param ox a <code>OXHandle</code>, probably a proxy, describing the ox whose
         * master handle ownership is beign requested
         
         * @return a <code>OXHandle</code> value, eventually the master handle
         */
    public OXHandle
    requestOwnership(OXHandle oxh)
        throws IllegalOXHandleException // HandleNotFoundException, WrongHandleException, NotMasterHandleException, WrongOwnershipCodeException
        {
            ObjectId oid = oxh.getId();
            
                // if the ox's master handle is not currently owned by this manager
            if ( ! handlesById.containsKey(oid) ) {
                
                    // FIX ME: possible dead lock if ox is a proxy handle
                setHandleOwnership(oxh);
                
                    // forwards the request to the cellular ox manager instance
                OXHandle noxh = oxmCell.requestOwnership(oxh);
                    // if the master handle has been returned
                if ( noxh instanceof MasterOXHandle ) {
                        // register handle ownership
                    handlesById.put(oid, noxh);
                    
                        // XXX: always returns a proxy. really need this?
                    return new ProxyOXHandle(oid);
                }
                else {
                    return noxh;
                }

                    // TO DO: should some how invalidate the original handle oxh
//                 oxh.invalidate();
            }
            else {
                    // nothing to do: already the owner; return the very same handle
                return oxh;
            }
            
//             try {                
//                 ox.setOwner( HostId.getLocalHost() );
                
//                 MasterOXHandle  result = (MasterOXHandle)oxmCell.requestOwnership(ox);
//                 handlesById.put(result.getId(), result);
                
//                 ox = new InvalidOXHandle();
//                 return(new ProxyOXHandle(result.getId(),timeBetweenUpdates));
//             }
//             catch (java.net.UnknownHostException uhe) {
//                 uhe.printStackTrace();
//             }
            
//             return null;
        }
    
    
        /**
         * OXhandle updates the attributes of the given ox. The attributes to be updated
         * are defined in a String array. What type of update is to be done (read or write)
         * is defined in an byte array.
         *
         * @param ox a <code>OXHanlde</code> whose attributes are to be updated
         * @param att an <code>String[]</code> defining what attributes are to be updated
         * @return a <code>OXHandle</code> that is the OXHandle passed as parameter with
         * it's attributes updated
         */
    public OXHandle
    updateAttributes(OXHandle ox, String[] att, byte[] op)
        throws IllegalOXHandleException // HandleNotFoundException
        {
            if (!handlesById.containsKey(ox.getId())){
                return(oxmCell.updateAttributes(ox, att, op));
            }
            else {
                    //System.out.println("In here");
                MasterOXHandle mHandle = (MasterOXHandle)handlesById.get(ox.getId());
                for (int i=0; i<att.length; i++) {
                    if ( op[i] == OXAttributeOps.WRITE ) {
                        mHandle.setAttribute(att[i], ox.getAttribute(att[i]));
                    }
                    else if ( op[i] == OXAttributeOps.READ ) {
                        ox.setAttribute(att[i], mHandle.getAttribute(att[i]));
                    }
                }
                
                OXHandle aux = ox;
                    //ox = new InvalidOXHandle();
                return(aux);
            }
        }


        //////////////////////////////////////////////////////////////
        // Private stuff
        //////////////////////////////////////////////////////////////
    private final void
    setHandleOwnership(OXHandle oxh)
        {
                // FIX ME: there is no setOwner() method
//             oxh.setOwner(HostId.getLocalHost());
        }

    
        /**
         * Creates the Master OXHandle for the given ox if it was not yet created
         *
         * @param oid an <code>ObjectId</code> value
         */
    private final void
    createMasterHandle(ObjectId oid)
        {
                // if the master handle was not yet created
            if ( ! (handlesById.containsKey(oid) || grantedHandles.containsKey(oid)) ) {
                    // create the master handle
                MasterOXHandle mh = new MasterOXHandle(oid);
                handlesById.put(oid, mh);
                    // schedule creation of the home handle
                pendingHomeHandles.put(oid,oid);
            }
        }

    
        /**
         * Reference to the cellular instance (this cell) of OXManager
         */
    private OXManager oxmCell;
        /**
         * Map from ox ID to active master handles
         */
    private Hashtable handlesById;
        /**
         * Set of ox ID whose homeHandle creation is pending (delayed)
         */
    private Hashtable pendingHomeHandles;
        /**
         * Set of existing but unmanaged master handles. The ownership of this handles
         * have been granted to another oxmanager.
         */
    private Hashtable grantedHandles;
    
    private long timeBetweenUpdates;
}
