// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.worb;

import org.isam.perv.IsamClassLoader;
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.HttpService;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.Executor;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;
import org.isam.util.codegen.proxy.Proxy;
import java.util.Hashtable;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.net.Socket;


/**
 * Webservices-like ORB implementation
 *
 * TO DO: implement new protocol version
 *
 * TO DO: source clean-up, reduce hardcoded inline constants
 *
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-05-04 10:22:02 -0300 (Wed, 04 May 2005) $ $Revision: 842 $
 */
public class WorbImpl
    extends HttpService
    implements Worb, WorbProtocolConstants
{
    static final String PROP_WORB_PORT = "port";
    static final int    DFLT_WORB_PORT = 1980;
    
    static final String CIB_ATT_CONTACT_ADDR = "contactAddress";

    static final String URI_PROTO_TCP = "tcp://";
    static final String URI_PROTO_WORB = "worb://";


    public WorbImpl() throws Exception
        {            
            this.servicesByName = new Hashtable();
            this.actionsByThread = new Hashtable();
            this.registeredToCib = false;
        }

    public void
    start()
        throws java.io.IOException
        {
                // get parameters from profile
            int pPort = Exehda.getServiceProperty(Worb.SERVICE_NAME,
                                                  PROP_WORB_PORT,
                                                  DFLT_WORB_PORT);
                // inform superclass the port the server socket it will be listening
            setPort(pPort);            
                // setup server socket to be used in accepting connections
            setServerSocket(new WorbServerSocket(pPort));
                // start listening for incoming requests
            super.start();
        }


    public String
    exportService(Object obj, Class iface, String name)
        {
            log(Sev.TRACE, "exportService", "entering for \"{0}\"", name);

            try {
                    // check if the given provider really implements the given interface
                if ( iface.isAssignableFrom(obj.getClass()) ) {

                        //
                        // TO DO: check if service already registered for the given name
                        //
                        // TO DO: disjunt service namespaces for each application
                        //
                
                        // register the new service
                    servicesByName.put(name, new WSServerRef(obj, iface));

                        //
                        // XXX: return service URI
                        //
                    String uri = "worb://"+HostId.getLocalHost().toExternalForm()+"/"+name;
                                        
                    log(Sev.INFO, "exportService", "exported \"{0}\" iface={1}", uri, iface);

                        //
                        // FIX ME: wait for cib exporting in order to avoid dead lock, cib needs
                        // WORB and worb needs the CIB. But, what if the CIB never exports itself?
                        //
//            if ( CellInformationBase.SERVICE_NAME.equals(name)) {
                    registerWorbResource();
//            }

                    return uri;
                }
                else {
                    throw new ClassCastException("Object does not support the provided interface");
                }
            }
            finally {
                log(Sev.TRACE, "exportService", "leaving");
            }


        }

    public void
    unexportService(Object obj, String name)
        {
                //
                // TO DO: add an implementation
                //
            throw new NullPointerException("Not yet implemented.");
        }

    public void
    setExceptionHandler(Object proxy, WorbExceptionHandler ehandler)
        {
                //
                // TO DO: add an implementation
                //
            throw new NullPointerException("Not yet implemented.");
        }

    public HostId
    getClientHost()
        {
            InvokeAction ca = currentAction();

            if ( ca == null ) return null;

            ServiceRequest req = ca.getServiceRequest();

            Socket sock = req.getClientSocket();

                // if remote host information is available
            if ( sock instanceof WorbSocket ) {
                    // return it
                return ((WorbSocket) sock).getRemoteHost();
            }
            else {
                    // otherwise, (information not available)

                    //
                    // FIX ME: perhaps we should do a try on HostId.getHostByName()
                    // before givin up an returning null
                    //
                
                return null;
            }
        }
    
        /**
         * Returns a proxy for accessing the identified service through the given
         * interface.
         *
         * @param service a <code>String</code> value
         * @param iface a <code>Class</code> value
         * @return an <code>Object</code> value
         */
    public Object
    lookupService(String service, Class iface)
        {            
            try {
                    // resolve abstract references to real (physical) addresses
                String serviceURL = toWorbURL(service);

                return Proxy.getInstance(iface, new WSClientRef(serviceURL, iface));
            }
            catch (Exception e) {
                log(Sev.DEBUG, "lookupService", e);
                return null;
            }
        }

        /////////////////////////////////////////////////////////////////////
        // PROTECTED STUFF
        /////////////////////////////////////////////////////////////////////

    
        /**
         * Implementation required by the super-class.
         *
         * @todo Upgrade to new Worb protocol
         * 
         * @param req a <code>ServiceRequest</code> value
         */
    protected void
    handleRequest( ServiceRequest req )
        {
            String resource = req.getProperty(WORB_REQ_PROP_RESOURCE);
            Long opnum = Long.valueOf(req.getProperty(WORB_REQ_PROP_OPNUM));

                //
                // TO DO: support Worb protocol version 2 (session id, resume, etc.)
                //

                // parse /A/B/C/method[/]

                //
                // TO DO: better check for malformed URLs
                //
            int idxSlash2 = resource.endsWith("/")
                ? resource.length()-1 : resource.length();
            int idxSlash1 = resource.lastIndexOf("/",idxSlash2-1);

                // check for malformed URL
            if ( idxSlash1 < 2 ) {
                    //
                    // TO DO: better malformed URL handling
                    //
                log(Sev.ERROR, "handleRequest",
                    "Malformed URL {0}", resource);

                try {
                    ServiceResponse resp = new ServiceResponse(
                        req, ServiceResponse.BAD_REQUEST);
                    resp.close();
                }
                catch ( Exception e ) {
                    log(Sev.WARNING, "handleRequest", e);
                }
            }
            else {
            
                String svc = resource.substring(1, idxSlash1);
                String mtd = resource.substring(idxSlash1+1, idxSlash2);

                log(Sev.DEBUG, "handleRequest",
                    "Incoming call to {0}.{1}(...)", svc, mtd);

                WorbServiceRef wsref = (WorbServiceRef) servicesByName.get(svc);

                    //
                    // TO DO: check is wsref exists (is not null)
                    //

                try {

                    BufferedInputStream rawin = new BufferedInputStream(
                        req.getClientSocket().getInputStream(), 1024);
                    rawin.mark(512);
                
                    ObjectInputStream in = new ObjectInputStream(rawin);

                    ApplicationId aid = (ApplicationId) in.readObject();

                    rawin.reset();

                    InvokeAction action = new InvokeAction(wsref, opnum, rawin, req);
                
                    getExecutor().runAction(aid, action);
                
                    byte[] resultData = action.getResult();
                
                    ServiceResponse resp = new ServiceResponse(req, ServiceResponse.OK);
                    resp.putHeader("Content-Length", Long.toString(resultData.length));
                    resp.putData(resultData, 0, resultData.length);
                    resp.close();
                }
                catch (Exception e) {
                    log(Sev.INFO, "handleRequest", e);

                    try {
                        ServiceResponse resp = new ServiceResponse(
                            req, ServiceResponse.INTERNAL_ERROR);
                        resp.close();
                    }
                    catch ( Exception e2 ) {
                        log(Sev.WARNING, "handleRequest", e2);
                    }
                }
            }
        }




        ////////////////////////////////////////////////////////////
        // PRIVATE STUFF
        ////////////////////////////////////////////////////////////

        /**
         * Register the WORB service as a resource in the cell information base.
         *
         */
    private final synchronized void
    registerWorbResource()
        {
            if ( registeredToCib ) return;

            final ResourceName worbRN = new ResourceName(
                "service", Worb.SERVICE_NAME, HostId.getLocalHost().toNameSpace());

            final String contactAddress = URI_PROTO_TCP
                + HostId.getLocalHost().getInetAddress().getHostAddress()
                +":"+getPort();

            log(Sev.DEBUG, "registerWorbService()",
                "Registering service WORB in the CIB with contactAddress={0}",contactAddress);


            (new Thread() {
                    
                    ResourceName wrn;
                    String wca;

                    {
                        this.wrn = worbRN;
                        this.wca = contactAddress;
                    }
                    
                    public void run()
                        {
                            try {
                                    //
                                    // XXX: let the CIB complete its initialization before
                                    // try to register the worb resource
                                    //
                                try { Thread.sleep(5000); }
                                catch (InterruptedException ie) {}
                            
                                CellInformationBase cib = (CellInformationBase)
                                    Exehda.getService(CellInformationBase.SERVICE_NAME);

                                cib.addResource(wrn);
                                cib.setAttribute(wrn, CIB_ATT_CONTACT_ADDR, wca);
                            }
                            catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                }).start();


            registeredToCib = true;
        }
    
    private void
    bindAction(InvokeAction a)
        {
            actionsByThread.put(Thread.currentThread(), a);
        }

    private InvokeAction
    currentAction()
        {
            return (InvokeAction) actionsByThread.get(Thread.currentThread());
        }

    private void
    unbindAction(InvokeAction a)
        {
            actionsByThread.remove(Thread.currentThread());
        }

        /**
         * Converts an abstract service URI in the form
         * worb://hostid@<cellname>/<service> to an URL in the form
         * worb://ip:port/<service>. For that end, the CIB would be accessed to determine
         * the contact address of the given host.
         *
         * @param uri a <code>String</code> value
         * @return a "worb:tcp://ip:port/<service>" URI
         */
    private String
    toWorbURL( String uri )
        {
            if ( uri.startsWith("worb:") ) {
                uri = uri.substring(5);

                    // check if transport already defined
                if ( uri.startsWith("tcp://") ) {
                        // direct internet addressing scheme

                    log(Sev.TRACE, "toWorbURL()",
                        "internet addressing scheme url={0}", uri);

                    return "worb:"+uri;
                }
                    // no transport defined
                else if ( uri.startsWith("//hostid:") ) {
                        // assume default host id addressing scheme, need to be
                        // translated to internet

                    HostId hid = HostId.parseId(uri.substring(2, uri.indexOf('/', 3)));
                    String service = uri.substring(uri.indexOf('/', 3));

                        //
                        // FIX ME: this code to work already need a worb connection to the
                        // cib, since it would be in another node. So, we need a fall back
                        // addressing scheme that do not depends on the CIB attribute
                        // resolution.
                        //
                        // Need to support at least directly ip addressing.
                        //
                        //
                
                    ResourceName wrn = new ResourceName(
                        "service", Worb.SERVICE_NAME, hid.toNameSpace());

                    String ca = getCib().getAttribute(wrn, CIB_ATT_CONTACT_ADDR);

//                     if ( ca == null ) {
//                             //
//                             // TO DO: raise exception
//                             //
//                     }

                    return "worb:"+ca+service;
                }
                else {
                        // unknown addressing scheme
                    throw new IllegalArgumentException("unknown addressing scheme url="+ uri);
                }
            }
            else {
                    //
                    // TO DO: perhaps default to localhost and use just the service name
                    //
                
                throw new IllegalArgumentException("Unsuported protocol: "+uri);
            }
        }

        /**
         * Shortcut for accessing the Cell Information Base service.
         *
         * @return a <code>CellInformationBase</code> value
         */
    private final CellInformationBase
    getCib()
        {
            return (CellInformationBase) Exehda.getService(
                CellInformationBase.SERVICE_NAME);
        }
    
        /**
         * Shortcut for accessing the Executor service.
         *
         * @return an <code>Executor</code> value
         */
    private final Executor
    getExecutor()
        {
            return (Executor) Exehda.getService(Executor.SERVICE_NAME);
        }
    

    private Hashtable servicesByName;
    private Hashtable actionsByThread;
    private boolean registeredToCib;


        ////////////////////////////////////////////////////////////
        // INNER CLASSES
        ////////////////////////////////////////////////////////////

    
        /**
         * Utility class to handle an incoming method call in the scope of an specific
         * application through the Executor.runAction() API.
         *
         */
    class InvokeAction implements Runnable
    {
        private WorbServiceRef wsref;
        private InputStream rawin;
        private Long opnum;
        private byte[] resultData;
        private Exception e;
        private ServiceRequest req;

        public InvokeAction(WorbServiceRef wsref, Long opnum, InputStream rawin, ServiceRequest req)
            {
                this.wsref = wsref;
                this.opnum = opnum;
                this.rawin = rawin;
                this.e = null;
                this.req = req;
            }

        public ServiceRequest
        getServiceRequest()
            {
                return this.req;
            }
        
        public void run()
            {
                try {
                    WorbObjectInputStream in = new WorbObjectInputStream(rawin);

                        // skip application id
                    in.readObject();
                
                    Object[] params = (Object[]) in.readObject();
                    if (params == null) params = new Object[0];

                    Object result = null;
                    Exception exception = null;

                    try {
                            // enable getClientHost
                        WorbImpl.this.bindAction(this);
                            // invoke the object
                        result = wsref.invoke(opnum, null, params);
                            // disable getClientHost
                        WorbImpl.this.unbindAction(this);
                    }
                    catch (Exception e) {
                            // propagate any raised exception to the client through the
                            // resultData
                        exception = e;
                        log(Sev.DEBUG, "InvokeAction.run", "Caugth exception");
                        log(Sev.DEBUG, "InvokeAction.run", e);
                    }
                    finally {
                        ByteArrayOutputStream buf = new ByteArrayOutputStream(8192);
                        ObjectOutputStream out = new ObjectOutputStream(buf);

                        out.writeObject(exception);
                        out.writeObject(result);
                        out.close();

                        this.resultData = buf.toByteArray();
                    }
                }
                catch (Exception e) {
                    this.e = e;
                    log(Sev.INFO, "InvokeAction.run", e);
                }
            }

        public byte[]
        getResult() throws Exception
            {
                if (e != null) {
                    throw e;
                }
                else {
                    return resultData;
                }
            }
    }
}


