
#ifdef HAVE_CONFIG_H
#  include <acconfig.h>
#endif

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif

#include <NativeMonitor.h>

#include <MonitorImpl.hh>


static primos::MonitorImpl* impl = NULL;

static jclass CLS_JAVA_SENSOR;
static jmethodID MTD_JAVA_SENSOR;
static jfieldID FLD_JAVA_SENSOR_KEY;
static jfieldID FLD_JAVA_SENSOR_INTVAL;
static jfieldID FLD_JAVA_SENSOR_FLOATVAL;
static jfieldID FLD_JAVA_SENSOR_STRINGVAL;
 
JNIEXPORT jobjectArray JNICALL
Java_org_isam_exehda_services_primos_NativeMonitor_getNativeSensors
  (JNIEnv *env, jobject thisObj)
{
    if ( impl == NULL ) {
        impl = primos::MonitorImpl::newInstance();
        impl->quantumExpired();
        sleep(1);
        impl->quantumExpired();

        CLS_JAVA_SENSOR =
            env->FindClass("org/isam/exehda/services/primos/NativeMonitor$NativeSensor");

        MTD_JAVA_SENSOR =
            env->GetMethodID(CLS_JAVA_SENSOR, "<init>", "(JILjava/lang/String;)V");

        FLD_JAVA_SENSOR_INTVAL =
            env->GetFieldID(CLS_JAVA_SENSOR, "intVal", "I");

        FLD_JAVA_SENSOR_FLOATVAL =
            env->GetFieldID(CLS_JAVA_SENSOR, "floatVal", "F");

        FLD_JAVA_SENSOR_STRINGVAL =
            env->GetFieldID(CLS_JAVA_SENSOR, "stringVal", "Ljava/lang/String;");

        FLD_JAVA_SENSOR_KEY =
            env->GetFieldID(CLS_JAVA_SENSOR, "key", "J");
    }

    const primos::SensorSet& nativeSensors = impl->availableSensors();
    
    jobjectArray javaSensors = env->NewObjectArray( nativeSensors.size(),
                                                    CLS_JAVA_SENSOR,
                                                    NULL);

    primos::SensorSet::iterator s;
    int i;
    
    for (i=0,s=nativeSensors.begin(); s!=nativeSensors.end(); s++,i++) {

        jlong sKey = reinterpret_cast<jlong>((*s));
        jint sType = (*s)->Type();
        jstring sName = env->NewStringUTF((*s)->Name().c_str());

        jobject rs = env->NewObject(CLS_JAVA_SENSOR,
                                    MTD_JAVA_SENSOR,
                                    sKey,
                                    sType,
                                    sName);

        env->SetObjectArrayElement(javaSensors, i, rs);
    }

    return javaSensors;
}


JNIEXPORT void JNICALL
Java_org_isam_exehda_services_primos_NativeMonitor_updateRawSensorData
  (JNIEnv *env, jobject thisObj, jobjectArray javaSensors)
{
    impl->quantumExpired();
    
    int nsensors = env->GetArrayLength(javaSensors);
    
    for ( int i = 0; i < nsensors; i++ ) {
        
        jobject rs = env->GetObjectArrayElement(javaSensors, i);

        jlong sKey = env->GetLongField(rs, FLD_JAVA_SENSOR_KEY);
            
        primos::Sensor *s = reinterpret_cast<primos::Sensor*>(sKey);

        const primos::SensorData* data = s->probe();
        
        if ( data->isInteger() ) {
            env->SetIntField(rs,
                             FLD_JAVA_SENSOR_INTVAL,
                             (jint) data->intValue());
        }
        else if ( data->isString() ) {
            env->SetObjectField(rs,
                                FLD_JAVA_SENSOR_STRINGVAL,
                                env->NewStringUTF(data->stringValue().c_str()));
        }
        else if ( data->isFloat() ) {
            env->SetFloatField(rs,
                               FLD_JAVA_SENSOR_FLOATVAL,
                               (jfloat) data->floatValue());
        }
    }
    
    return;
}
