// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


//
// $Id: LearnerImpl.java 611 2005-03-07 10:10:30Z rreal $
//


package org.isam.exehda.services.tips;

import java.util.*;

public class LearnerImpl
    extends Thread
    implements Learner
{
        /** Associations */
    private DataManager dataM;
    private Double constancy;

    private int threashold;
    private int count;
    private final int period = 10000;

    private BayesNetwork bn;
    String sensorName;
    
    
    public LearnerImpl(DataManager dataM, BayesNetwork bn) 
        {
            this.dataM = dataM;
            threashold = 10;
            count = 0;
            this.bn = bn;
            setDaemon(true);
            sensorName = new String("CPU_OCCUP_IDLE");
            
            start();
        }


    public void waitHist() 
        {

            Vector hist=null;
            Vector sv=null;
            Vector tstamp;
            boolean ok=false;
            
            while(!ok) {
                try {
//                    System.out.println("Going to sleep");
                    Thread.sleep(100);
                } catch (Exception e) {}

                hist = (Vector) dataM.getSensorHistory(sensorName);
                
                if (hist != null) {
                    try {
                        tstamp = (Vector)hist.get(0);
                        sv = (Vector) hist.get(1);
                        if (sv != null) {
                            ok=true;
                        }
                    } catch(NullPointerException e) {}
                }
                
//                System.out.println("LearnerImpl: hist = " + hist + "sv = " + sv);
            }


        }
    
    
        /** 
         * Operation
         *
         *  
         */
    public void
    run() 
        {

            int i;

            double lastValue;
            double newValue;
            double constancy;
            Vector sv;
            Vector tstamp;
            Vector hist=null;
            long tdelta=0;
            
            
            System.out.println("Starting Learner (CVS $Revision: 611 $) ...");
            
            while(true) {
//                System.out.println("Learner looping ...");

                if (hist == null) {
                    waitHist();
                }
                
                hist = (Vector) dataM.getSensorHistory(sensorName);
                

                tstamp = (Vector)hist.get(0);
                sv = (Vector) hist.get(1);
                
                if ( tstamp.size() >= 1) {
                    tdelta = System.currentTimeMillis() - 
                        ((Long) tstamp.elementAt(tstamp.size()-1)).longValue();
                } else {
                    tdelta = 0;
                }
                

//                System.out.println("TDELTA = " + tdelta);
                
                if ( sv.size() >= 2 ) {
                    newValue = ((Double)sv.elementAt(sv.size()-1)).doubleValue();
                    lastValue = ((Double) sv.elementAt(sv.size()-2)).doubleValue();                        
                    constancy = ((Double)dataM.getConstancy(sensorName)).doubleValue();
                    
                } else if ( sv.size() == 1) {
                    newValue = ((Double)sv.elementAt(sv.size()-1)).doubleValue();
                    lastValue = newValue;
                    if ( dataM.hasConstancy(sensorName)) {
                        constancy = ((Double)dataM.getConstancy(sensorName)).doubleValue();
                    } else {
//                            dataM.setConstancy(sensorName, new Double(0));
                        bn.eval(sensorName);
                        constancy=0;
                    }
                        
                } else {
                    newValue = 0;
                    lastValue = newValue;
//                        dataM.setConstancy(sensorName, new Double(0));
                    constancy=0;
                }
                
                if ( (tdelta <= period) &&
                     (newValue >= (lastValue + threashold) ||
                      newValue <= (lastValue - threashold)) ) {
                        // subtract 0.1 from constancy
//                        System.out.println("Constancy ---");
                    constancy -= 0.1;
                    if (constancy < 0)
                        constancy = 0;
                    dataM.setConstancy(sensorName, new Double(constancy));
                    bn.eval(sensorName);
                    count=0;
                } else {
                    count++;
                }
                if (count == 5) {
                        // add 0.1 to constancy
//                        System.out.println("Constancy +++");
                    constancy += 0.1;
                    if (constancy > 1)
                        constancy=1;
                    dataM.setConstancy(sensorName, new Double(constancy));
                    bn.eval(sensorName);
                    count =0;
                }
                try {
                    Thread.sleep(period);
                } catch (Exception e) {}
            }
        }
        
    
    public void setInfo(String sensorName)
        {
            if (!dataM.hasConstancy(sensorName)) {
                dataM.setConstancy(sensorName, new Double(0.5));
            }
        }
}

