// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceConstraint;


/**
 * Cell's resource broker and allocation manager. Specficially, it is responsible for
 * implementing resource access control policies in inter-cell operations.
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-04-06 10:34:36 -0300 (Wed, 06 Apr 2005) $ $Revision: 715 $
 */
public interface ResourceBroker
{
    public static final String SERVICE_NAME = "resourcebroker";
    
        /**
         * Locate (but do not allocate) a set o resources that match the specified criteria.
         *
         * @param query a <code>ResourceConstraint[]</code> value
         * @param scope an <code>int</code> value
         * @param num an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] findResource( ResourceConstraint query, int scope, int num );
    
        /**
         * Allocate a set of resources that match the given criteria..
         *
         * @param req a <code>ResourceConstraint[]</code> value
         * @param scope an <code>int</code> value
         * @param num an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] allocResource( ResourceConstraint req, int scope, int num );
}

