// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib.backend.ldap;

import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.directory.Attribute;

//
// TO DO: should ensure resources of type 'user' are always in a namespace of type
// 'cell'
//

class UserLRTH
    extends BaseLRTH
{
        private static final int ATT_OBJCLASS = 0;
    private static final int ATT_USERID = 1;
    
    private static final String[] DFLT_ATTR;
    private static final String[] DFLT_ATTR_VAL;

    private static final String[] HANDLED_CIB_TYPES = { "user" }; 
    private static final String[] HANDLED_DN_PREFIXES = { "xUserID"};

    static {
        
        DFLT_ATTR = new String[] {
            "objectClass",     // ATT_OBJCLASS
            "xUserID"          // ATT_USERID
        };
        
        DFLT_ATTR_VAL = new String[] {
            "exehdaUser",      // ATT_OBJCLASS
            null               // ATT_USERID
        };
    }

    public UserLRTH()
        {
            super(HANDLED_CIB_TYPES, HANDLED_DN_PREFIXES);
        }

    public String
    createNode( LdapSession s, ResourceName rn )
        throws NamingException
        {
            if ( !"user".equals(rn.getType()) ) {
                throw new IllegalArgumentException(
                    "Invalid resource type: "+rn.getType());
            }
                // map the given resource name to an ldap DN
            String dn = encode(rn, null).toString();
                // fill in mandatory attributes
            String[] avs = (String[]) DFLT_ATTR_VAL.clone();
                // overwrite default values with user-provided values
            avs[ATT_USERID] = rn.getSimpleName();
                // commit to LDAP
            internalCreateNode(s, rn, dn, DFLT_ATTR, avs);

            return dn;
        }

    public NameComponent
    decodeNameComponent( String dnc )
        {
            System.out.println(dnc);
            
            int idx = dnc.indexOf('=');
            if ( idx <= 0 || idx == (dnc.length()-1) ) {
                throw new IllegalArgumentException("DN component:"+dnc);
            }

                //
                // FIX ME: not checking if dnc starts with "xUserID="
                //
            
            String t = "user";
            String i = dnc.substring(idx+1);

            return new NameComponent(t,i);
        }

    public String
    getDefaultSearchFilter()
        {
            return "(objectClass=exehdaUser)";
        }

    public LdapResourceInfo
    toLdapResourceInfo(String dn, Attributes atts)
        {
            ResourceName rn = decodeResourceName(dn);            
                
            return new LdapResourceInfo(rn, atts) {
                    public String getAttribute(String a) {
                        try {
                            final String LDAP_ATT_ID = "xUserID";
                            final String LDAP_ATT_XATTR = "xAttr";
                            
                            if ( CIB_ATT_RES_TYPE.equals(a) ) {
                                return "user";
                            }
                            else if ( CIB_ATT_RES_INSTANCE.equals(a)) {
                                return (String) atts.get(LDAP_ATT_ID).get();
                            }
                            else {
                                if ( this.cache == null ) {
                                    this.cache = new java.util.Hashtable();
                                        
                                        // parse xAttrs only once
                                    Attribute att = atts.get(LDAP_ATT_XATTR);
                                    for ( int i=0; i<att.size(); i++ ) {
                                        String s = (String) att.get(i);
                                        int p = s.indexOf(':');
                                        cache.put(s.substring(0,p),
                                                  s.substring(p+1));
                                    }
                                }
                                return (String) cache.get(a);
                            }
                        }
                        catch ( NamingException ne ) {
                                //
                                // TO DO: use logger
                                //
                            ne.printStackTrace();
                            return null;
                        }
                    }
                };
        }
    
        /////////////////////////////////////////////////////////
        // PROTECTED STUFF
        /////////////////////////////////////////////////////////
    
    protected String
    encodeAttrName( String ct, String ca )
        {
            if ( CIB_ATT_RES_TYPE.equals(ca) ) {
                return "objectClass";
            }
            else if ( CIB_ATT_RES_INSTANCE.equals(ca) ) {
                return "xUserID";
            }
            else {
                return "xAttr";
            }
        }

    protected String
    encodeAttrValue( String ct, String ca, String cav )
        {
            if ( CIB_ATT_RES_TYPE.equals(ca) ) {
                return "exehdaUser";
            }
            else if ( CIB_ATT_RES_INSTANCE.equals(ca) ) {
                return cav;
            }
            else {
                return ca+":"+cav;
            }
        }
    
    protected String
    decodeAttrValue( String ca, String la, String lav )
        {
            if ( "xAttr".equals(la) ) {
                return lav.substring(lav.indexOf(':')+1);
            }
            else if ( CIB_ATT_RES_INSTANCE.equals(ca)
                      && "xUserID".equals(la)) {
                return lav;
            }
            else if ( CIB_ATT_RES_TYPE.equals(ca) ) {
                return "user";
            }
            else {
                throw new NullPointerException(
                    "Unknown encoding: ca="+ca+" la="+ca+" lav="+lav);
            }
        }
}
