
package org.isam.exehda.services.gw;

import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.net.*;
import java.util.Iterator;
import org.isam.exehda.services.Service;


public class SocksGateway
    implements Service, Runnable
{
        // server port
    final static int SOCKS_SERVER_PORT = 29904;
    
    private Thread serverThread = null;
    private ServerSocketChannel serverChannel = null;
    private InetSocketAddress socksServerAddr = null;
    
    public void
    start()
        {
            if (serverThread == null) {
                serverThread = new Thread(this);
                serverThread.start();
            }
            else {
//                 System.err.println("Service already running");
            }
        }

    public void
    stop()
        {
            try {
                if (serverThread != null) {
                    serverThread.interrupt();
                    serverChannel.close();
                    serverThread = null;
                }
                else {
//                     System.out.println("Error: service not running");
                }
            }
            catch (Exception e) {
                System.out.println(e);
            }
        }

    public void
    run()
        {
            try {
                ServerSocketChannel serverChannel = ServerSocketChannel.open();
                serverChannel.socket().bind(
                    new InetSocketAddress( SOCKS_SERVER_PORT ));
                
                socksServerAddr = (InetSocketAddress)
                    serverChannel.socket().getLocalSocketAddress();
                
                while ( !Thread.interrupted() ) {
                    try {
                        SocketChannel clientChannel = serverChannel.accept();
                        
                        (new SocksConnectionHandler(clientChannel)).start();
                        
                    } catch ( Exception e ) {
                        e.printStackTrace();
                    }
                }
            } catch ( IOException ioe ) {
                ioe.printStackTrace();
                System.exit(-1);
            }
        }

    class SocksConnectionHandler
        extends java.lang.Thread
    {
            // protocol versions
        final static int SOCKS_V5                     = 0x05;
            // authentication methods
        final static int SOCKS_AUTH_NONE              = 0x00;
        final static int SOCKS_AUTH_GSSAPI            = 0x01;
        final static int SOCKS_AUTH_USRPWD            = 0x02;
        final static int SOCKS_AUTH_IANA              = 0x03;
        final static int SOCKS_AUTH_PRIV              = 0x80;
        final static int SOCKS_AUTH_UNKNOWN           = 0xFF;
            // reques commands
        final static int SOCKS_CMD_CONNECT            = 0x01;
        final static int SOCKS_CMD_BIND               = 0x02;
        final static int SOCKS_CMD_UDPASSOC           = 0x03;
            // address types
        final static int SOCKS_ADDR_IPV4              = 0x01;
        final static int SOCKS_ADDR_DOMAINNAME        = 0x03;
        final static int SOCKS_ADDR_IPV6              = 0x04;
            // reply codes
        final static int SOCKS_REPLY_SUCCESS          = 0x00;
        final static int SOCKS_REPLY_SERVER_FAILURE   = 0x01;
        final static int SOCKS_REPLY_CONN_NOT_ALLOWED = 0x02;
        final static int SOCKS_REPLY_NET_UNREACH      = 0x03;
        final static int SOCKS_REPLY_HOST_UNREACH     = 0x04;
        final static int SOCKS_REPLY_CONN_REFUSED     = 0x05;
        final static int SOCKS_REPLY_TTL_EXPIRED      = 0x06;
        final static int SOCKS_REPLY_CMD_NOT_SUP      = 0x07;
        final static int SOCKS_REPLY_ADDR_TYPE_NOT_SUP= 0x08;
    
    


    
        private SocketChannel localChannel;
    
        public SocksConnectionHandler( SocketChannel s ) 
            {
                localChannel = s;
            }

        public void
        run() 
            {
                try {
                    if ( doAuthentication(localChannel) ) {
                            // read request details from localConnection                    
                        InputStream in = localChannel.socket().getInputStream();
                            // read version (0x05)
                        int version  = in.read();
                            // read cmd: connect (0x01) | bind (0x02) | udp associate (0x03)
                        int cmd = in.read();
                            // read reserved (0x00)
                        int reserved = in.read();
                            // read destination address and port
                        InetSocketAddress target = readDestinAddress(localChannel);
                    
                        switch (cmd) {
                            case SOCKS_CMD_CONNECT:
                                if ( checkConnectionAllowed(target) ) {
                                    handleConnect( target );
                                } else {
                                    doLocalReply( SOCKS_REPLY_CONN_NOT_ALLOWED );    
                                }
                                break;
                            case SOCKS_CMD_BIND:
                                handleBind( target );
                                break;
                            case SOCKS_CMD_UDPASSOC:
                                handleUdpAssociate( target );
                                break;
                            case -1:
                                    // end of stream
                                localChannel.close();
                                return;
                            default:
                                System.err.println("Command not supported:" + cmd
                                                   +", req version was "+ version);
                                doLocalReply( SOCKS_REPLY_CMD_NOT_SUP );
                        }
                    } else {
                            // authentication failed
                        doLocalReply( SOCKS_REPLY_SERVER_FAILURE );
                    }
                        // delay 1 second before closing the connection
                    sleep(1000);
                    
                } catch (IOException ioe ) {
                    ioe.printStackTrace();
                } catch ( InterruptedException ie ){
                        // empty
                }
                finally {
                    try {
                        localChannel.close();
                        System.out.println("Gataway thread done.");
                    } catch ( Exception e ) {
                            // empty
                    }
                }
            }

        private boolean
        doAuthentication( SocketChannel ch )
            throws IOException, InterruptedException
            {
                final byte[] AUTH_NONE_OK = { 0x05, 0x00 };
                final byte[] AUTH_NO_ACCEPTBL_MTDS = { 0x05,  (byte) 0xFF };
                    
                    // read 'protocol version' from localConnection
                    // read 'n auth method' from localConnection
                InputStream in = localChannel.socket().getInputStream();

            
                int version = in.read();
                int nmethods= in.read();

                if ( version == SOCKS_V5 ) {
                
                    for ( int i=0; i<nmethods; i++ ) {
                        int method = in.read();
                        switch ( method ) {
                            case SOCKS_AUTH_NONE:
                            {
                                    // consume remaingin methods
                                while ( ++i < nmethods ) {
                                    in.read();
                                }
                                    // flag success
                                return  2 == ch.write( ByteBuffer.wrap(AUTH_NONE_OK));
                            }
                            case SOCKS_AUTH_GSSAPI:
                            case SOCKS_AUTH_USRPWD:
                            case SOCKS_AUTH_IANA:
                            case SOCKS_AUTH_PRIV:
                            default:
                        }
                    }
                }
                ch.write( ByteBuffer.wrap( AUTH_NO_ACCEPTBL_MTDS ));
                return false;
            }

        private InetSocketAddress
        readDestinAddress( SocketChannel ch )
            throws IOException
            {
                InputStream in = ch.socket().getInputStream();
                int addrType = in.read();

                    // read address type : ipv4 (0x01) | domainname (0x03) | ipv6 (0x04)
                switch ( addrType ) {
                    case SOCKS_ADDR_IPV4: 
                    {   
                        ByteBuffer addrBuff = ByteBuffer.allocate(6);
                        while ( addrBuff.remaining() > 0 ) {
                            if ( ch.read(addrBuff) < 0 ) {
                                return null;
                            }
                        }
                        addrBuff.order(ByteOrder.BIG_ENDIAN);
                        addrBuff.flip();
                    
                            // read destination address
                        byte[] ipv4Addr = new byte[4];
                        addrBuff.get(ipv4Addr);
                            // read destination port ( 2 octets, network octet order )                    
                        int port = addrBuff.getShort();
                    
                        return new InetSocketAddress(
                            InetAddress.getByAddress(ipv4Addr), port);
                    }
                    case SOCKS_ADDR_DOMAINNAME:
                    case SOCKS_ADDR_IPV6:
                    default:
                }
            
                return null;
            }


        private boolean
        checkConnectionAllowed( InetSocketAddress target )
            throws UnknownHostException
            {
                    // deny recursive connection requests
                InetAddress ta = target.getAddress();
                if ( socksServerAddr.equals( target ) ) {
                    return false;
                } else if ( ta.isLoopbackAddress() || InetAddress.getLocalHost().equals(ta)) {
                    return target.getPort() != socksServerAddr.getPort();
                }

                return true;
            }
    
        private ByteBuffer
        makeReply( int code, InetSocketAddress bindAddr )
            {            
                byte[] addrBytes = ( bindAddr != null ) ?
                    addrBytes = bindAddr.getAddress().getAddress() : new byte[] {0, 0, 0, 0};
            
                int  port = ( bindAddr != null ) ?
                    bindAddr.getPort() : 0 ;

                ByteBuffer msg =
                    ByteBuffer.allocate( 6+addrBytes.length ).order(ByteOrder.BIG_ENDIAN);
            
                    // version 0x05
                    // reply code: succeeeded (0x00) | general SOCKS server failure (0x01)
                    //             connection not allowed (0x02) | network  unreachable (0x03)
                    //             host unreachable (0x04) | connection refused (0x05)
                    //             ttl expired (0x06) | command not supported (0x07)
                    //             address type not supported (0x08)
                    //             In case od error the server should terminate the connection
                    //             shortly (within 10s) after sending the reply
                    // reserved (0x00)
                msg.put((byte) SOCKS_V5).put((byte) code).put((byte) 0x00);
            
                    // address type: ipv4 (0x01) | domainname (0x03) | ipv6 (0x04)
                msg.put(addrBytes.length > 4  ? (byte) SOCKS_ADDR_IPV6 : (byte) SOCKS_ADDR_IPV4 );
                    // server bound address
                    // server bound port ( 2 octets, network octet order )
                msg.put(addrBytes).putShort( (short) port );

                    // make the buffer readable and return
                return (ByteBuffer) msg.flip();
            }

        private void
        doLocalReply( int replyCode )
            throws IOException
            {
                doLocalReply( replyCode, null );
            }
    
    
        private void
        doLocalReply( int replyCode, InetSocketAddress bindAddr)
            throws IOException
            {
                ByteBuffer replyMsg = makeReply( replyCode, bindAddr );

                while ( localChannel.write(replyMsg) > 0 );
            }
    
        private void
        handleConnect( InetSocketAddress target )
            throws IOException
            {
                try {
                    System.out.println("Forwarding (local)"+localChannel.socket().getRemoteSocketAddress()
                                       +" <-> (remote)"+target);
                    SocketChannel remoteChannel = SocketChannel.open(target);

                    doLocalReply( SOCKS_REPLY_SUCCESS,
                                  (InetSocketAddress) remoteChannel.socket().getLocalSocketAddress());
                        // put channels in non blocking mode
                    localChannel.configureBlocking(false);
                    remoteChannel.configureBlocking(false);
                        // create selector
                    Selector bridgeSelector = Selector.open();
                    SelectionKey localChannelKey = localChannel.register( bridgeSelector,
                                                                          SelectionKey.OP_READ);
                    SelectionKey remoteChannelKey = remoteChannel.register( bridgeSelector,
                                                                            SelectionKey.OP_READ);
                        // create buffers
                    ByteBuffer bridgeBuffer = ByteBuffer.allocate(4096);

                    System.out.println("Waiting for data...");
                
                        // wait for I/O, loop forwarding data between connections
                    while ( bridgeSelector.select() > 0  ) {
                    
                        System.out.println("Incoming data");
                    
                        Iterator readyKeys = bridgeSelector.selectedKeys().iterator();
                            // walk through  input-ready channels 
                        while ( readyKeys.hasNext() ) {
                            SelectionKey sk = (SelectionKey) readyKeys.next();
                            SocketChannel readyChannel = (SocketChannel) sk.channel();
                                // flag the oposite channel as output channel
                            SocketChannel outputChannel =
                                ( readyChannel == localChannel ) ? remoteChannel : localChannel ;
                                // prepare the buffer for writing
                            bridgeBuffer.clear();
                            if ( readyChannel.read(bridgeBuffer) == -1 ) {
                                    // reached eos, close connections
                                outputChannel.close();
                                readyChannel.close();
                                break;
                            } else {                            
                                    // prepare the buffer for reading
                                bridgeBuffer.flip();
                                    // forward the data ready channel to the output channel
                                while ( bridgeBuffer.remaining() > 0 ) {
                                    outputChannel.write( bridgeBuffer );
                                }
                            }
                        }
                    }
                    if ( localChannel.isOpen() ) localChannel.close();
                    if ( remoteChannel.isOpen() ) remoteChannel.close();
                
                    System.out.println("Select < 0 ");
                
                } catch (java.net.ConnectException ce ) {
                    doLocalReply( SOCKS_REPLY_CONN_REFUSED );
                }
            }

        private void
        handleBind( InetSocketAddress target )
            throws IOException
            {
                localChannel.write( makeReply(SOCKS_REPLY_CMD_NOT_SUP, null) );
            }

        private void
        handleUdpAssociate( InetSocketAddress target )
            throws IOException
            {
                localChannel.write( makeReply(SOCKS_REPLY_CMD_NOT_SUP, null) );
            }
        
    }
}




