// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos.exec;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ObjectId;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.services.Executor;
import org.isam.exehda.services.Executor.SchedulingHeuristic;
import org.isam.exehda.services.Scheduler;
import java.util.Vector;


/**
 * This scheduling heuristic uses the scheduler service for selecting the target host for
 * remote instantiation and migration operations if such service is available in the
 * system. Otherwise, it falls back to the semantic defined by
 * <code>HostIdSchedHeuristic</code>.
 *
 * @see HostIdSchedHeuristic
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-05-04 10:11:00 -0300 (Wed, 04 May 2005) $ $Revision: 837 $
 */
class SysSchedulerSchedHeuristic
    extends HostIdSchedHeuristic
{
    
    public SysSchedulerSchedHeuristic()
        {
            this.schedulerService = null;
        }
    
    public HostId
    chooseCreationHost(String clsName,
                       Object[] params,
                       Object hint,
                       Vector avoidedHosts)
        {
                //
                // TO DO: pack everything in ResourceConstraints
                // 
            if ( schedulerService != null && hint instanceof ResourceConstraint ) {
                return schedulerService.chooseCreationHost(
                    (ResourceConstraint) hint);
            }
            else {
                return super.chooseCreationHost(
                    clsName, params, hint, avoidedHosts);
            }
        }

    public HostId
    chooseMigrationHost(ObjectId ox,
                        Object hint,
                        Vector avoidedHosts)
        {
                //
                // TO DO: pack everything in ResourceConstraints
                // 
            if ( schedulerService != null && hint instanceof ResourceConstraint ) {
                return schedulerService.chooseMigrationHost(
                    (ResourceConstraint) hint);
            }
            else {
                return super.chooseMigrationHost(
                    ox , hint, avoidedHosts);
            }
        }

    private final synchronized Scheduler
    getScheduler()
        {
            try {
                this.schedulerService = (Scheduler)
                    Exehda.getService(Scheduler.SERVICE_NAME);
            }
            catch ( Exception e ) {
                this.schedulerService = null;
            }

            return this.schedulerService;
        }

    private Scheduler schedulerService;
}
