// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

//
// $Id: TipsCellScheduler.java 840 2005-05-04 13:16:01Z lucc $
//

package org.isam.exehda.services.tips;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ObjectId;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.services.Scheduler;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.Logger.Sev;

import java.lang.reflect.*;
import java.util.*;

public class TipsCellScheduler
    extends AbstractService
    implements Scheduler,CollectUtility
{
    static private final String ATT_CONTACT_ADDR = "contactAddress";

    private Hashtable hostTable;
    private Hashtable quarentine;
    private TDecMaker decisionM;
    
        /**
         * Describe <code>chooseCreationHost</code> method here.
         *
         * @param c a <code>ResourceConstraints</code> value
         * @return a <code>HostId</code> value
         */
    public HostId chooseCreationHost( ResourceConstraint c ) 
        {
            HostId hi=null;

            
            hi = getBestHostid();

            log(Sev.DEBUG, "chooseCreationHost()", "BestHostid {0}", hi);
            
            return(hi);
        }
    

        /**
         * Describe <code>chooseMigrationHost</code> method here.
         *
         * @param c a <code>ResourceConstraints</code> value
         * @return a <code>HostId</code> value
         */
    public HostId chooseMigrationHost( ResourceConstraint c )
        {
            HostId hi;
            
            hi = getBestHostid();
            
            return(hi);
        }


        /**
         * Describe <code>reschedule</code> method here.
         *
         * @param o an <code>ObjectHandle</code> value
         * @param c a <code>ResourceConstraints</code> value
         * @param auto a <code>boolean</code> value
         */
    public void reschedule( ObjectId o, ResourceConstraint c, boolean auto )
        {
                // fazer a chamada do TiPS mesmo aqui
        }


    public void stop() 
        {}

    public void start()
        throws java.rmi.RemoteException, ClassNotFoundException, InstantiationException,
        IllegalAccessException, InvocationTargetException, NoSuchMethodException
        {
                // Starting the data manager
            hostTable = new Hashtable();
            quarentine = new Hashtable();

                // exporting the service
            Worb worb = (Worb)
                Exehda.getService(Worb.SERVICE_NAME);
            
            CellInformationBase cib = (CellInformationBase)
                Exehda.getService(CellInformationBase.SERVICE_NAME);
            
            NameSpace nsLocalCell = HostId.getLocalHost().getCell().toNameSpace();

            String ca = null;
            ResourceName rn = null;
            
                // exporta o servio escalonador e registra na CIB
            ca = worb.exportService(this, Scheduler.class, Scheduler.SERVICE_NAME);
            rn = new ResourceName( "service", Scheduler.SERVICE_NAME, nsLocalCell);
            cib.addResource(rn);
            cib.setAttribute(rn, ATT_CONTACT_ADDR, ca);

                // export o servio "collect utility" e o registra na CIB
            ca = worb.exportService(this, CollectUtility.class, CollectUtility.SERVICE_NAME);
            rn = new ResourceName( "service", CollectUtility.SERVICE_NAME, nsLocalCell);
            cib.addResource(rn);
            cib.setAttribute(rn, ATT_CONTACT_ADDR, ca);
        }


    public boolean inQuarentine(HostId h) {
        if (quarentine.containsKey(h)) {
                // if host is for more than one minute in quarentine, frees it
            if ((System.currentTimeMillis()) > ((Long)quarentine.get(h)).longValue()+60*1000) {
                quarentine.remove(h);
                return (false);
            } else {
                return (true);
            }
        }
        return(false);
    }
    
    
    public void updateUtility(long timeStamp, HostId src, double utility) 
        {
            if (!inQuarentine(src)) {                
                hostTable.put(src,new Double(utility));
            }

            log( Sev.DEBUG, "updateUtility()",
                 "timestamp = {0}, host = {1}, utility = {2}",
                 Long.toString(timeStamp), src, Double.toString(utility));
        }

    
    public HostId getBestHostid()
        {
            log( Sev.DEBUG, "getBestHostid()", "########## UOUOUOUOUOUO ################");
    
            HostId hi = null;

            double maxut = -200;
            HostId bestHostid = null;
            
            Set hostidSet = hostTable.keySet();            
            
            Iterator hostIt = hostidSet.iterator();
            while(hostIt.hasNext()) {
                hi = (HostId) hostIt.next();
                if ( maxut <  ((Double)hostTable.get(hi)).doubleValue()) {
                    maxut = ((Double)hostTable.get(hi)).doubleValue();
                }                
            }

                // code for randomly getting the best node
            hostIt = hostidSet.iterator();
            double hostVal;
            Vector alternatives = new Vector();
            while(hostIt.hasNext()) {
                hi = (HostId) hostIt.next();
                hostVal = ((Double)hostTable.get(hi)).doubleValue();
                if ( hostVal > (maxut - 0.5) ) {
                    alternatives.add(hi);
                }                
            }

            if (alternatives.size() > 0) 
                bestHostid = (HostId) alternatives.get( (new Random()).nextInt(alternatives.size()) );
            else
                bestHostid = null;
            
            log( Sev.DEBUG, "getBestHostid()",
                 "Choice = {0}, utility = {2}", bestHostid, Double.toString(maxut));

            hostTable.put(bestHostid,new Double(-100));
            quarentine.put(bestHostid,new Long(System.currentTimeMillis()));
            
            return(bestHostid);
        }

}
