// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


//
// $Id: TipsHeuristic.java 839 2005-05-04 13:15:35Z lucc $
//


package org.isam.exehda.services.tips;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.Collector;
import org.isam.exehda.services.Collector.Sensor;
import org.isam.exehda.services.Collector.MonitoringData;
import org.isam.exehda.services.Collector.ConsumerId;
import org.isam.exehda.services.Collector.MonitoringConsumer;
import org.isam.exehda.services.Scheduler;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;

public class TipsHeuristic
    implements MonitoringConsumer
{   
    private static final String SENSOR_NAME= "CPU_OCCUP_IDLE";

    private Collector collector;
    private ConsumerId id;
    private Sensor[] sensors;

    private DataManager dataM;
    private UtilityGen ug;
    private BayesNetwork bn;
    private Learner l;
    private TipsCellProxy tcproxy;
    
    public TipsHeuristic(DataManager dataM, UtilityGen ug, BayesNetwork bn, Learner l)
        {
            log( Sev.INFO, "<init>", "Starting tips heuristic (CVS $Revision: 839 $)");
            
            collector = (Collector) Exehda.getService(Collector.SERVICE_NAME);

            this.id = collector.addConsumer(this);
            this.dataM = dataM;
            this.ug = ug;
            this.bn = bn;
            this.l = l;

                //
                // XXX: forwarder of sensor data to cellular tips; workaround for
                // limitation of current Collector implementation.
                //
            startCellProxy();
            
            initSensors();
        }    

    public void update( long timeStamp, MonitoringData[] data )
        {
            for ( int i=0; (i<data.length) && (data[i]!=null); i++) {
                
                String sensorName = data[i].getSensor().getSimpleName();
                
                if ( SENSOR_NAME.equals(sensorName) ) {
                    Double sensorValue = new Double(data[i].getDouble());

                    dataM.setSensorValue(sensorName,sensorValue);
            
                    l.setInfo(sensorName);
                    bn.eval(sensorName);
                }
            }
            
//            dumpData(data);
            
        }

        ////////////////////////////////////////////////////////////
        // PRIVATE STUFF
        ////////////////////////////////////////////////////////////

    private final void initSensors()
        {
            Sensor targetSensor = collector.getSensor(SENSOR_NAME);
            collector.setSensorEnabled(id, targetSensor, true);
        }


    private final void dumpData(MonitoringData[] data) 
        {
            for (int i=0; i<data.length; i++) {
                if ( data[i] != null ) {
                    System.out.print(data[i].getSensor().getSimpleName());
                    System.out.print(" \t= ");
                    System.out.println(data[i].getString());
                }
            }
        }

    private final void log(int sev, String mtd, String msg)
        {
            Logger logger = (Logger) Exehda.getService(Logger.SERVICE_NAME);
            logger.log(sev, this.getClass().getName(), mtd, msg, null);
        }

    
    private final void startCellProxy()
        {
                // get contact address from CIB
            CellInformationBase cib = (CellInformationBase)
                Exehda.getService(CellInformationBase.SERVICE_NAME);

            ResourceName rnCollectUtility = new ResourceName(
                "service", CollectUtility.SERVICE_NAME,
                HostId.getLocalHost().getCell().toNameSpace());
            
            String caCollectUtility = cib.getAttribute(
                rnCollectUtility, CollectUtility.ATT_CONTACT_ADDRESS);
            
            this.tcproxy = new TipsCellProxy(caCollectUtility);
        }
}
