
package org.isam.exehda.services.avu;

import java.io.*;
import org.isam.exehda.Exehda;
import org.isam.exehda.services.HttpService;
import org.isam.exehda.services.Logger.Sev;

/**
 * This class implements ISAM AVU service, the user virtual environment of the ISAM platform.
 *
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-05-12 15:03:22 -0300 (Thu, 12 May 2005) $ $Revision: 866 $
 */
public class AvuImpl
    extends HttpService
{
    static final String SERVICE_NAME = "avu";
    static final String DEFAULT_DOCROOT = "./";
    static final int    DEFAULT_PORT = 1977;
    
    static final String PROP_AVU_OPERATION = "Isam-Avu-Operation";
    static final String PROP_AVU_RESOURCE  = "Resource";
    static final String PROP_AVU_USER      = "Isam-User";
    static final String CONTENT_LENGTH     = "Content-Length";

    static final int    AVU_OP_READ        = 0;
    static final int    AVU_OP_WRITE       = 1;

    static final int    AVU_CELL_INDEX        = 1;
    static final int    AVU_USER_INDEX        = 0;

    private String docRoot;

    
    public void
    start() 
        throws IOException
        {
            final String METHOD_NAME = "start()";
                        
            docRoot = Exehda.getServiceProperty(
                SERVICE_NAME, "docroot", DEFAULT_DOCROOT);            
            log(Sev.INFO, METHOD_NAME, "Set up file system docroot to \"{0}\"", docRoot);

            System.getProperties().put("isam.avuHost","localhost");

            int port = Exehda.getServiceProperty(
                SERVICE_NAME, "port", DEFAULT_PORT);

            setPort(port);
            super.start();
            log(Sev.INFO, METHOD_NAME, "Listening at port "+port);
        }

    protected void
    handleRequest( ServiceRequest req )
        throws IOException
        {
            switch ( getAvuOperation(req) ) {
                case AVU_OP_READ:
                    handleAvuRead(req);
                    break;
                case AVU_OP_WRITE:
                    handleAvuWrite(req);
                    break;
                default:
                    log( Sev.ERROR, "handleRequest()", "Opps! Unknown AVU operation.");
            }
                // op read
                // check if is a valid file name( do not start with .. for example)
                // open local file
                // output HTTP response header
                // output file contents
        }

    protected boolean
    checkAccess( String fileName, String userName, int op )
        {
                // TODO: check if its a valid filename
            
                // grant read access to all users
            if ( op == AVU_OP_READ ) {
                return true;
            }
            else {
                    // grant all the rest to
                return true;
            }
        }

    protected int
    getAvuOperation( ServiceRequest req )
        {
                // defaults to 'read'
            int op = AVU_OP_READ;
            String opString = req.getProperty(PROP_AVU_OPERATION);
            System.out.println("Operation == " + opString);
            if ( "write".equals(opString) ) {
                op = AVU_OP_WRITE;
            }

            return op;
        }

    protected String[]
    getUserCellPair( String userName)
        {
            String []userCellPair = new String[2];

            int atIndex = userName.indexOf("@");
                //TODO: Throw Exception
            userCellPair[AVU_USER_INDEX] = userName.substring(0,atIndex);
            userCellPair[AVU_CELL_INDEX] = userName.substring(atIndex+1);

            return(userCellPair);

        }

    protected void
    handleAvuRead( ServiceRequest req )
        throws IOException
        {
            final String METHOD_NAME = "handleAvuRead()";
            
            String fileName = req.getProperty(PROP_AVU_RESOURCE);
            String userName = req.getProperty(PROP_AVU_USER);

            String[] userCellPair = getUserCellPair(userName);

	   

            if ( checkAccess(fileName, userName, AVU_OP_READ) ) {

                String realPath = docRoot
                    + userCellPair[AVU_CELL_INDEX]
                    + "/"
                    + userCellPair[AVU_USER_INDEX]
                    + ""
                    + fileName;

                
                System.out.println("real path:"+realPath);
                
                File file = new File(realPath);

                if ( ! file.exists()  ) {
                    
                    log(Sev.ERROR, METHOD_NAME,
                        "File not found {0}, real path is {1}", fileName, realPath);
                    
                    ServiceResponse resp =
                    new ServiceResponse( req,
                                         ServiceResponse.BAD_REQUEST);
                    resp.close();
                }
                else {
                    log(Sev.INFO, METHOD_NAME,
                        "Serving {0}, real path is {1}", fileName, realPath);
                    
                    ServiceResponse resp =
                        new ServiceResponse(req,
                                            ServiceResponse.OK);

                    resp.putHeader("Content-Length", ""+file.length());

                    byte[] buff = new byte[4096];
                    InputStream ins =
                        new FileInputStream(file);

                    int nr;

                    while ( (nr = ins.read(buff)) != -1 ) {
                        resp.putData(buff, 0, nr);
                    }
                
                    resp.close();
                }
            }
            else {
                log(Sev.ERROR, METHOD_NAME,
                        "Access denied {0}", fileName);
                
                ServiceResponse resp =
                    new ServiceResponse( req,
                                         ServiceResponse.BAD_REQUEST);
                resp.close();
            }
        }

    protected void
    handleAvuWrite( ServiceRequest req )
        throws IOException{
	
        String fileName = req.getProperty(PROP_AVU_RESOURCE);
        String userName = req.getProperty(PROP_AVU_USER);
        int bytesToRead = Integer.parseInt(req.getProperty(CONTENT_LENGTH));
	
        String[] userCellPair = getUserCellPair(userName);
	
        if ( checkAccess(fileName, userName, AVU_OP_WRITE) ) {
	
            java.net.Socket clientSocket = req.getClientSocket();

            InputStream ins = clientSocket.getInputStream();
	    
            byte[] buff = new byte[4096];
            OutputStream outs =
                new FileOutputStream(docRoot
                                     + userCellPair[AVU_CELL_INDEX]
                                     + "/"
                                     + userCellPair[AVU_USER_INDEX]
                                     + ""
                                     + fileName);
	    
            int nr = 0;
            int bytesRead = 0;
	   
            while (bytesRead < bytesToRead) {
                nr = ins.read(buff);
                outs.write(buff, 0, nr);
                bytesRead += nr;
            }
	    
            outs.close();
                //ins.close();
	   
            ServiceResponse resp =
                new ServiceResponse(req,
                                    ServiceResponse.OK);
	   
	    
            resp.close();
        }
        else {
            ServiceResponse resp =
                new ServiceResponse( req,
                                     ServiceResponse.BAD_REQUEST);
            resp.close();
        }
    }
}
    
