// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib.backend.hashtable;


import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.services.cib.backend.CibBackEnd;
import org.isam.exehda.services.cib.backend.CibBackEndConf;
import org.isam.exehda.services.cib.backend.CibBackEndException;
import java.util.Hashtable;


public class HashtableBackEnd
    extends Hashtable
    implements CibBackEnd
{
    public HashtableBackEnd()
        {
            super();
        }

    public void
    init( CibBackEndConf conf )
        {
                // empty
        }


    public ResourceName[]
    select(ResourceConstraint sc, NameSpace ns, int maxResults)
        {
                // TODO: generic select
            throw new NullPointerException("Not yet implemented");
        }
       
    public ResourceName[]
    selectByName(String n, NameSpace ns, int maxResults)
        {
            java.util.Vector results = new java.util.Vector();
            java.util.Enumeration kenum = keys();

            while (kenum.hasMoreElements() && (maxResults != 0) ) {
                ResourceName rn = (ResourceName) kenum.nextElement();

                if ( rn.getSimpleName().equals(n) && isInNameSpace(rn,ns) ) {
                    results.add(rn);

                    if ( maxResults > 0 ) --maxResults;
                }
            }

            ResourceName[] tmp = new ResourceName[results.size()];
            results.copyInto(tmp);
            return tmp;
        }
    
    public ResourceName[]
    selectByType(String t, NameSpace ns, int maxResults)
        {
            java.util.Vector results = new java.util.Vector();
            java.util.Enumeration kenum = keys();

            while (kenum.hasMoreElements() && (maxResults != 0) ) {
                ResourceName rn = (ResourceName) kenum.nextElement();

                if ( rn.getType().equals(t) && isInNameSpace(rn,ns) ) {
                    results.add(rn);
                    if ( maxResults > 0 ) --maxResults;
                }
            }

            ResourceName[] tmp = new ResourceName[results.size()];
            results.copyInto(tmp);
            return tmp;
        }
    
    public String
    getAttribute(ResourceName r, String a)
        {
            HashtableResourceEntry entry = findEntry(r);

            if ( entry != null ) {
                return entry.getAttribute(a);
            }
            else {
                return null;
            }
        }
    
    public String
    setAttribute(ResourceName r, String a, String v)
        {
            HashtableResourceEntry entry = findEntry(r);

            if ( entry != null ) {
                return entry.setAttribute(a,v);
            }
            else {
                return null;
            }
        }
    
    public String[]
    getAttributes(ResourceName r)
        {
            HashtableResourceEntry entry = findEntry(r);

            if ( entry != null ) {
                return entry.getAttributes();
            }
            else {
                return null;
            }
        }
    
    public void
    addResource(ResourceName r)
        {            
            HashtableResourceEntry entry = findEntry(r);

            if ( entry == null ) {
                put(r, new HashtableResourceEntry());
            }
        }
    
    public void
    removeResource(ResourceName r)
        {
            remove(r);
        }

        /////////////////////////////////////////////////////////////
        // PRIVATE STUFF
        /////////////////////////////////////////////////////////////

    private final HashtableResourceEntry
    findEntry(ResourceName r)
        {
            return (HashtableResourceEntry) get(r);
        }

    private final boolean
    isInNameSpace(ResourceName r, NameSpace ns)
        {
            if ( ns == null ) {
                return true;
            }
            else {
                NameSpace rns = r.getNameSpace();

                if ( ns.equals(rns) || ns.contains(rns) ) return true;
            }
            return false;
        }
}
