// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.ctxm.filter.basic;


/**
 * Represents a range of numbers, which are represented as a pair of double values
 * (ranges's upper and lower bounds).
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-06-16 13:53:32 -0300 (Thu, 16 Jun 2005) $ $Revision: 916 $
 */
class NumericRange
{
    public NumericRange(double ub, boolean ubo, double lb, boolean lbo,
                       String ctxs)
        {            
            this.ub = (ubo) ?
                (RangeLimit) new OpenUB(ub) : (RangeLimit) new ClosedUB(ub);
            this.lb = (lbo) ?
                (RangeLimit) new OpenLB(lb) : (RangeLimit) new ClosedUB(lb);
            this.state = ctxs;
        }
    
    public final boolean contains(double v)
        {
            return ( ub.holds(v) && lb.holds(v) );
        }

    public final String getState()
        {
            return this.state;
        }

    public final String toString()
        {
            return "Range "+lb+", "+ub;
        }

        //////////////////////////////////////////////////////
        // PRIVATE STUFF
        //////////////////////////////////////////////////////
    
    private static abstract class RangeLimit
    {
        RangeLimit(double v)
            {
                this.val = v;
            }
    
        abstract boolean holds(double v);
    
        protected double val;
    }
    
    private static class OpenUB extends RangeLimit
    {
        OpenUB(double value)
            {
                super(value);
            }
        
        final boolean holds(double v)
            {
                return ( v < this.val );
            }

        public final String toString()
            {
                return Double.toString(val)+")";
            }
    }

    private static class ClosedUB extends RangeLimit
    {
        ClosedUB(double value)
            {
                super(value);
            }
        
        final boolean holds(double v)
            {
                return ( v <= this.val );
            }

        public final String toString()
            {
                return Double.toString(val)+"]";
            }
    }

    private static class OpenLB extends RangeLimit
    {
        OpenLB(double value)
            {
                super(value);
            }
        
        final boolean holds(double v)
            {
                return ( v > this.val );
            }

        public final String toString()
            {
                return "("+Double.toString(val);
            }
    }

    private static class ClosedLB extends RangeLimit
    {
        ClosedLB(double value)
            {
                super(value);
            }
        
        final boolean holds(double v)
            {
                return ( v >= this.val );
            }

        public final String toString()
            {
                return "["+Double.toString(val);
            }
    }

    private RangeLimit ub, lb;
    private String state;
}
