/*
 * Discoverer.java
 *
 * Created on February 14, 2005, 12:35 PM
 */

package org.isam.exehda.services.discoverer;

import org.isam.exehda.ResourceName;

/**
 *
 * @author  frainer
 */
public interface Discoverer {
    
    public static final String SERVICE_NAME = "discoverer";
    final static String PROP_DISCOVERER_URI = "contactAddress";
    
    /**
    * Registers a new resource that is equivalent to the one
    * described in the xml passed as a parameter.
    *
    * @param xmlDesc a <code>String</code> containing a xml document that describes the attributes of a resource
    * @return a new <code>ResourceName</code> that represents the resource added.
    */
    public ResourceDescriptor register(String xmlDesc);
    
    
    /**
    * Removes a previously added resource.
    *
    * @param r a <code>ResourceDescriptor</code> that represents the resource that is to be removed
    * @return void
    */
    public void remove(ResourceDescriptor r);
    
    /**
    * Renews a previously added resource.
    *
    * @param r a <code>ResourceDescriptor</code> that represents the resource that is to be renewed
    * @return void
    */
    public void renew(ResourceDescriptor r);
    
    
    /**
    * Finds a resource that matches the given query.
    *
    * @param xmlQuery a <code>String</code> containing a xml document that describes the query
    * @return a array of <code>ResourceDescriptor</code>, representing the matching resources found.
    */
    public ResourceDescriptor[] find(Query q);
    
}
