/*
 * DiscovererImpl.java
 *
 * Created on February 15, 2005, 1:15 PM
 */

package org.isam.exehda.services.discoverer;

import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.CellInformationBase;
/*
import org.isam.exehda.services.CellInformationBase.CompositeSelectionConstraint;
import org.isam.exehda.services.CellInformationBase.SimpleRegexpConstraint;
 **/
import org.isam.exehda.CompositeResourceConstraint;
import org.isam.exehda.AttributeSimpleMatchConstraint;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.Worb;
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.util.xml.*;
import java.util.*;
import java.io.*;

/**
 *
 * @author  frainer
 */
public class DiscovererImpl extends AbstractService implements Discoverer, Runnable {
    
    final static String PROP_DISCOVERER_CONTACT = "contactAddress";
    
    CellInformationBase cib;
    Hashtable resources;
    boolean running;
    
    public void start() throws java.lang.Exception {
        
        //find cib
        Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);

        String cibCellUri = Exehda.getServiceProperty(CellInformationBase.SERVICE_NAME,
                                                          Discoverer.PROP_DISCOVERER_URI,
                                                          "//localhost:1980/cib");

        cib = (CellInformationBase)
                Exehda.getService(CellInformationBase.SERVICE_NAME);
        
        resources = new Hashtable();
        running = true;
        //new Thread(this).start();
        
      
      //register the dicoverer service in the cib
      //put the contact address as one of the properties 
      String contact = Exehda.getServiceProperty(Discoverer.SERVICE_NAME,
                                                          PROP_DISCOVERER_CONTACT,
                                                          "//localhost/discoverer"); 
      
      
      ResourceName rn = new ResourceName("service","discoverer",HostId.getLocalHost().getCell().toNameSpace());
      System.out.println("gooo");
      cib.addResource(rn);
      System.out.println("gooo");
      cib.setAttribute(rn, "contactAddress", contact);
      System.out.println("gooo2");
      cib.setAttribute(rn, "service", "discoverer");
      System.out.println("gooo3");
       
      
      //export the discoverer
      worb.exportService(this,
                               Discoverer.class,
                               Discoverer.SERVICE_NAME);
      System.out.println("gooo4");
    }
    
    public void stop() throws java.lang.Exception {
        //stop the thread that checks and removes old resources
        running = false;
    }
    
    /** Creates a new instance of DiscovererImpl */
    public DiscovererImpl() {
    }
    
    /**
    * Finds a resource that matches the given query.
    *
    * @param q a <code>Query</code> containing the query
    * @return a array of <code>ResourceDescriptor</code>, representing the matching resources found.
    */
    public ResourceDescriptor[] find(Query q) {
        if(q.getScope() == 0){
            //if scope is zero search the cib for matching resources that are in the same computer node
            ResourceName[] rns = cib.select(q, HostId.getLocalHost().toNameSpace(), -1); 
            return(makeResourceDescriptors(rns));
        }
        else if(q.getScope() == 1){
            //not implemented
            
        }
        else if(q.getScope() == 2){
            //if scope is 2 search the cib for any matching resources in this cell
            ResourceName[] rns = cib.select(q, (NameSpace)null, -1); 
            return(makeResourceDescriptors(rns));
        }
        else if(q.getScope() > 2){
            //if scope is larger than 2 we must search in neighbor cells too.
            return(findInOtherCells(q.getScope()-2,q));
        }
     
        return(null);
    }
    
    
     /**
    * Finds resources in this and in other cells.
    *
    * @param cellScope a <code>int</code> that defines how far we should look for resources(1 looks for cells in the imediate neighborhood, 2 looks for cells in the neighborhood of the immediate neighborhood, etc).
    * @param q a <code>Query</code> defining what the types of resources that we are looking for
    * @return a <code>ResourceDescriptor</code> array with the found resources.
    */
    private ResourceDescriptor[] findInOtherCells(int cellScope, Query q){
        ResourceName[] currentCells;
        String[] currentCellsAddress;
        ResourceName[] newCells;
        List newCellList;
        List newCellAddressList;
        List found = new LinkedList();
        Hashtable visited = new Hashtable();
        int quantos = 0;
        
        //use current quary to create a new new query that is similar but has 2 as scope
        String xml = q.getXmlQuery();
        q = new Query(xml);
        q.setScope(2);
        
        //find the resources in the current cell and add them to found
        ResourceDescriptor[] rd = this.find(q);
        quantos += rd.length;
        found.add(rd);
        
        //find the neighbors of the current cell 
        currentCells = cib.selectByType("neighbor",HostId.getLocalHost().getCell().toNameSpace(),-1);
        currentCellsAddress = new String[currentCells.length];
        for(int i=0; i<currentCells.length; i++){
            currentCellsAddress[i] = cib.getAttribute(currentCells[i],"contactAddress");
        }
        
        Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
        
        //find resources in other cells. Each iteration looks at cells that are father away
        while(cellScope > 0){
            newCellList = new LinkedList();
            newCellAddressList = new LinkedList();
            //go through each one of the currently known cells
            for(int i = 0; i < currentCells.length; i++){
                    //mark this cell as visited
                    visited.put(currentCells[i].getCommonName().getInstance(),"");
                    String contact = currentCellsAddress[i];//cib.getAttribute(currentCells[i],"contactAddress");
                    //System.out.println("-*/-*/-*/-*/-*/==="+currentCellsAddress[i]);
                    //look up remote cib...
                    CellInformationBase cibCell = (CellInformationBase) worb.lookupService(contact, CellInformationBase.class);
                    //and tries to find the discoverer service of that cell
                    CompositeResourceConstraint csc = new CompositeResourceConstraint();
                    csc.addConstraint(new AttributeSimpleMatchConstraint("service","discoverer"),false);
                    csc.addConstraint(new AttributeSimpleMatchConstraint("TYPE","service"),false);
                    System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!Contacting:"+contact);
                    ResourceName[] drns = cibCell.select(csc, (NameSpace)null, -1);
                    //ResourceName drn = new ResourceName("service","discoverer",null);
                    
                    //if discoverer service was found
                    if(drns.length > 0){
                        //get the contact address of the discoverer...
                        contact = cibCell.getAttribute(drns[0], "contactAddress");
			System.out.println("!!!!!********!!!!!!!!"+contact
);
			Discoverer rDiscoverer = (Discoverer) worb.lookupService(contact, Discoverer.class);
                        //...and call the remote find 
                        ResourceDescriptor[] rds = rDiscoverer.find(q);
                        quantos += rds.length;
                        //add found resources to found list
                        found.add(rds);    
                    }
            }
            cellScope--;
            //if we have to look further
            if(cellScope>0){
                //go through each one of the cells that were already visited in the last iteration and get their neighbors
                for(int i = 0; i < currentCells.length; i++){
                    String contactAddress = (cib.getAttribute(currentCells[i],"contactAddress"));
                    CellInformationBase cibCell = (CellInformationBase)
                    //look up cell...
                    worb.lookupService(contactAddress, CellInformationBase.class);
                    //...and find it's neighbors
                    newCells = cibCell.selectByType("neighbor",null,-1);
                    for(int j = 0; j < newCells.length; j++){
                        
                        if(!visited.containsKey(newCells[i].getCommonName().getInstance())){
                            //if cell wasn't previously visited add it to the list of cells to be visited
                            newCellList.add(newCells[i]);
                            newCellAddressList.add(cibCell.getAttribute(newCells[i],"contactAddress"));
                            visited.put(newCells[i].getCommonName().getInstance(),"");
                        }
                       
                    }
                }
            
                //if no new cells were found, break
                if(newCellList.size() == 0){
                    break;
                }
                
                //transfer the cells to be visited to the array currentCells
                // and the addresses to the array currentCellsAddress
                currentCells = new ResourceName[newCellList.size()];
                currentCellsAddress = new String[newCellList.size()];
                Iterator it = newCellList.iterator();
                Iterator it2 = newCellAddressList.iterator();
                for(int i= 0;it.hasNext(); i++){
                    currentCells[i] = (ResourceName)it.next();
                    currentCellsAddress[i] = (String)it2.next();
                }
            }
        }
        
        
        //found all the resources we could. Now create one big array with all of them
        ResourceDescriptor[] results = new ResourceDescriptor[quantos]; 
        Iterator it = found.iterator();
        int i = 0;
	while(it.hasNext()){
            System.out.println("Once More Baby!!!!!!!!!");
            ResourceDescriptor[] rds = (ResourceDescriptor[])it.next();
	    System.out.println("In Herre, Itss ssosoosososo cold");
            for(int j=0; j < rds.length; j++,i++){
		System.out.println("In Herre, Itss ssosoosososo hot");
                results[i] = rds[j];
            }
	    
        }
        System.out.println("Problems in pAAAAAAAAAAAAAAradise?"+i+":"+quantos);
        return(results);
        
        
       
    }
    
     /**
    * Registers a new resource that is equivalent to the one
    * described in the xml passed as a parameter.
    *
    * @param xmlDesc a <code>String</code> containing a xml document that describes the attributes of a resource
    * @return a new <code>ResourceName</code> that represents the resource added.
    */
    public ResourceDescriptor register(String xmlDesc) {
       
        //make a MetaResourceDescriptor from the xml
        MetaResourceDescriptor desc = makeResourceFromXml(xmlDesc);
        if(desc != null){
            //add resources to our table of resources and call saveResource
            resources.put(desc.getResourceDescriptor().getName(), desc);
            saveResource(desc);
            return(desc.getResourceDescriptor());
        }
        return(null);
    }
    
    /**
    * Removes a previously added resource.
    *
    * @param r a <code>ResourceDescriptor</code> that represents the resource that is to be removed
    * @return void
    */
    public void remove(ResourceDescriptor r) {
        if(resources.containsKey(r.getName())){
            //if this resource has previously been added to this discoverer remove
            // it from the cib and from our list of resources
            cib.removeResource(r.getName());
            resources.remove(r.getName());
        }
    }
    
    
    /**
    * Renews a previously added resource.
    *
    * @param r a <code>ResourceDescriptor</code> that represents the resource that is to be renewed
    * @return void
    */
    public void renew(ResourceDescriptor r) {
        if(resources.containsKey(r.getName())){
            //if this resource was previously added, renew the addition, setting a new expiration time and updating
            // the entry in the cib
            MetaResourceDescriptor mrd = (MetaResourceDescriptor)resources.get(r.getName());
            r.getLease().setExpirationTime();
            mrd = new MetaResourceDescriptor(r,mrd.getAttributes());       
            saveResource(mrd);
        }
    }
    
    /**
    * Takes a xml description and creates a MetaResourceDescriptor
    *
    * @param xmlDesc a <code>String</code> containing a xml document that describes a resource
    * @return a MetaResourceDescriptor describing the resource
    */
    private MetaResourceDescriptor makeResourceFromXml(String xmlDesc){
        ResourceDescriptor rd;
        ResourceName rn;
        List attributes = new LinkedList();
        XmlElement xml;
        try{
            //parse the String and create a XmlElement
            xml = XmlElement.parse(new StringReader(xmlDesc));
            //the first is docroot, not what we want
            //ignore the root
            xml = (XmlElement)xml.getChildren().nextElement();
            //the first tag should have an attribute name and a attribute type. Retrieve them
            String name = xml.getAttribute("name", "default"); 
            String type = xml.getAttribute("type", "default");
            //create a ResourceName using the data obtained
            NameSpace ns = HostId.getLocalHost().toNameSpace();   
            rn = new ResourceName("name",name,ns);
            //create a new resource descriptor with the ResourceName created before
            rd = new ResourceDescriptor(rn,new ResourceLease(new Long(1)));
            rd.setAttribute("type",type);
            attributes.add("type");
            Enumeration enum = xml.getChildren();
            //the children of the Resource tag contain the attributes of the resource
            while(enum.hasMoreElements()){
                //get the new attribute and recovers it's name and value
                XmlElement attribute = (XmlElement)enum.nextElement();
                String n = attribute.getAttribute("name","null");
                String v = attribute.getAttribute("value","void");
                //add the attribute to the ResourceDescriptor
                rd.setAttribute(n,v);
                attributes.add(n);
            }
            //create a new MetaResourceDescriptor using the list of attributes and and the ResourceDescriptor previously created
            return(new MetaResourceDescriptor(rd,attributes));
        }catch(Exception e){e.printStackTrace();}
        return(null);
        
    }
    
    /**
    * Takes a MetaResourceDescriptor and save it's informations in the cib
    *
    * @param mrd a <code>MetaResourceDescriptor</code> 
    * @return void
    */
    private void saveResource(MetaResourceDescriptor mrd){
        //get the resource name from mrd and create an entry for it in the cib, if one does not already exists
        cib.addResource(mrd.getResourceDescriptor().getName());
        //go through the list of attributes and save them in the cib
        List attributes = mrd.getAttributes();
        Iterator it = attributes.iterator();
        while(it.hasNext()){
            String name = (String)it.next();
            cib.setAttribute(mrd.getResourceDescriptor().getName(), name, mrd.getResourceDescriptor().getAttribute(name, ""));
        }
    }
    
     /**
    * Removes all resources that have expired
    *
    * @return void
    */
    private void removeExpiredResources(){
        //go through every one of the resources
        Enumeration enum = resources.elements();
        while(enum.hasMoreElements()){
            MetaResourceDescriptor mrd = (MetaResourceDescriptor)enum.nextElement();
            //Check the resources lease to see if it has expired
            if(mrd.getResourceDescriptor().getLease().getExpirationTime() < System.currentTimeMillis()){
                //remove expired resource from cib and from local resource list 
                cib.removeResource(mrd.getResourceDescriptor().getName());
                resources.remove(mrd.getResourceDescriptor().getName());
            }
        }
    }
    
    
     /**
    * Takes an array of ResourceNames and creates an array
    * of resource descriptors.
    *
    * @param rns a array of <code>ResourceName</code>s
    * @return a array pf R<code>esourceDescriptor</code>s
    */
    private ResourceDescriptor[] makeResourceDescriptors(ResourceName[] rns){
        //every ResourceName becomes a ResourceDescritor. Create
        //ResourceDescriptor array of same length as rns.
        ResourceDescriptor[] rds = new ResourceDescriptor[rns.length];
        for(int i = 0; i < rns.length; i++){
            //create a new ResourceDescriptor for every member of rns
            if(resources.containsKey(rns[i])){
                //if we already have the resource locally use the lease already created
                System.out.println("TESTEONNNNNNNNNNNNNNNNNNNNNNNNNN="+rns[i]);
                rds[i] = new ResourceDescriptor(rns[i], ((MetaResourceDescriptor)resources.get(rns[i])).getResourceDescriptor().getLease());
            }
            else{
                System.out.println("TESTEONNNNNNNNNNNNNNNNNNNNNNNNNN2="+rns[i]);
                //create a new lease
                rds[i] = new ResourceDescriptor(rns[i], new ResourceLease(new Long(1)));
            }
        }
        return(rds);
    }
    
    /**
    * Run method for the thread that will exclude expired resources
    *
    * @return a array pf R<code>esourceDescriptor</code>s
    */
    public void run() {
        //while service hasn't been stoped
        while(running == true){
            try{
                //remove the resources that have expired already
                removeExpiredResources();
                //wait a little before removing again
                Thread.sleep(10000);
            }catch(java.lang.InterruptedException ie){
                ie.printStackTrace();
            }
        }
    }    
    
    
}
