/*
 * Query.java
 *
 * Created on February 15, 2005, 4:22 PM
 */

package org.isam.exehda.services.discoverer;

/*
import org.isam.exehda.services.CellInformationBase.SelectionConstraint;
import org.isam.exehda.services.CellInformationBase.CompositeSelectionConstraint;
import org.isam.exehda.services.CellInformationBase.SimpleRegexpConstraint;
 **/
import org.isam.exehda.CompositeResourceConstraint;
import org.isam.exehda.AttributeSimpleMatchConstraint;
import org.isam.util.xml.*;
import java.util.*;
import java.io.StringReader;

/**
 *
 * @author  frainer
 */
public class Query extends CompositeResourceConstraint{
    String xmlQuery;
    CompositeResourceConstraint constraint;
    int scope;
    
    
    /** Creates a new instance of Query */
    public Query(String xmlQuery) {
        //super();
        this.xmlQuery = xmlQuery;
        this.constraint = makeConstraintFromXml(xmlQuery);
        this.scope = getScopeFromXml(xmlQuery);
    }
    
     /**
    * Returns the scope of this query
    *
    * @return  a <code>int</code> representing the scope of this query
    */    
    public int getScope(){
        return(scope);
    }
    
     /**
    * Sets the scope of this query
    *
    * @param scope a <code>int</code> containing the desired scope
    * @return  void
    */   
    public void setScope(int scope){
        this.scope = scope;
    }
    
     /**
    * Returns the xml version of the query
    *
    * @return  a <code>String</code> representing the query in xml
    */    
    public String getXmlQuery(){
        return(xmlQuery);
    }
      
    
    /**
    * Checks whether the resource represented by the given ResourceInfo satisfies
    * this constraint.
    *
    * @param r a <code>ResourceInfo</code> value
    * @return OK is the resource satisfies the criteria, BROKEN if it does not or
    * UNDEFINED if the criteria could not be evaluated for this resource.
    */
    public int check(ResourceInfo r) {
        return(constraint.check(r));
    }
    
     /**
    * takes a xmlQuery and creates a CompositeSelectionConstraint out of it
    *
    * @param xmlQuery a <code>String</code> containing the xml version of the query
    * @return  a CompositeSelectionConstraint
    */ 
    private CompositeResourceConstraint makeConstraintFromXml(String xmlQuery){
        
        CompositeResourceConstraint constraint = new CompositeResourceConstraint();
        try{
            //parse the xml
            XmlElement xml = XmlElement.parse(new StringReader(xmlQuery));
            xml = (XmlElement)xml.getChildren().nextElement();
            //we are only interested in the children here
            Enumeration enum = xml.getChildren();
            while(enum.hasMoreElements()){
                //each children has a name and a value. We use those to create a 
                // SimpleRegexpConstraint
                XmlElement attribute = (XmlElement)enum.nextElement();
                String n = attribute.getAttribute("name","null");
                String v = attribute.getAttribute("value","void");
                constraint.addConstraint(new AttributeSimpleMatchConstraint(n,v),false);
            }
        }catch(Exception e){e.printStackTrace();}
        return(constraint);
       
    }
    
    /**
    * Takes a xml query and gets the scope of the query
    *
    * @param xmlQuery a <code>String</code> containing a xml query
    * @return  an <code>int</code> representing the scope of the query
    */ 
    private int getScopeFromXml(String xmlQuery){
        try{
            XmlElement xml = XmlElement.parse(new StringReader(xmlQuery));
            xml = (XmlElement)xml.getChildren().nextElement();
            String s = xml.getAttribute("scope","0");
            return(Integer.parseInt(s));
        }catch(Exception e){e.printStackTrace();}
        return(-1);
    }
    
    
     /**
    * Returns all the individual constraints in this Query
    *
    * @return  a <code>Enumeration</code>
    */ 
    public Enumeration constraints()
            {
                return (constraint.constraints());
            }
    
     /**
    * returns the mode in which the individual constraints are to
    * evaluated (and, or, etc). It will always be and.
    *
    * @return  an <code>int</code> representing the mode 
    */ /*
    public int getMode()
            {
                return (constraint.getMode());
            }*/
}
