/*
 * ResourceDescriptor.java
 *
 * Created on February 15, 2005, 10:08 AM
 */

package org.isam.exehda.services.discoverer;

//import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.ResourceName;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Worb;
import org.isam.exehda.Exehda;
import java.util.Hashtable;


/**
 *
 * @author  frainer
 */
public class ResourceDescriptor implements java.io.Serializable {
    
    private ResourceName rn;
    private ResourceLease rl;
    Hashtable attributes;
    
    /** Creates a new instance of ResourceDescriptor */
    public ResourceDescriptor(ResourceName r, ResourceLease lease) {
            rn = r;
            rl = lease;
            attributes = new Hashtable();
    }
    
    /**
    * Return the ResourceName associated with the resource
    *
    * @return a  <code>ResourceName</code> .
    */
    public ResourceName getName(){
        return(rn);
    }
    
    
    /**
    * Return the ResourceLease associated with the resource
    *
    * @return a  <code>ResourceLease</code> .
    */
    public ResourceLease getLease(){
        return(rl);
    }
    

    /**
    * Recovers the value of the given attribute from the resource. If that attribute
    * is not set in the resource the defaultValue is returned 
    *
    * @param a a <code>String</code> with the name of the attribute to be recovered 
    * @param defaultValue a <code>String</code> with the value to be returned if the attribute is not defined
    * @return  a <code>String</code> with the value of the attribute
    */    
    public String getAttribute(String a, String defaultValues){
       String attr = (String)attributes.get(a);
       if(attr == null){
           CellInformationBase cib;
           cib = (CellInformationBase)
                Exehda.getService(CellInformationBase.SERVICE_NAME);
           attr = cib.getAttribute(rn,a);
           if(attr == null){
               attr = defaultValues;
           }
           else{
               attributes.put(a,attr);
           }
       }
       return(attr);
    }
    
    
    /**
    * Sets the value of one of the resources attributes
    *
    * @param a a <code>String</code> with the name of the attribute to be set
    * @param v a <code>String</code> with the new value of the attribute
    */  
    public void setAttribute(String a, String v){
        attributes.put(a, v);
    }
    
}
