// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.oxm;


import org.isam.exehda.Exehda;
import org.isam.exehda.ObjectId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.OXManager;
import org.isam.exehda.services.OXManager.OXHandle;
import org.isam.exehda.services.OXManager.IllegalOXHandleException;
import org.isam.exehda.services.Executor;


/**
 * A wrapper for the real cellular manager implementation allowing it to be host a a
 * non-dedicated node (i.e. a node that whould also run applications).
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-04-25 02:36:26 -0300 (Mon, 25 Apr 2005) $ $Revision: 798 $
 */
public class DualOXManagerCell
    extends AbstractService
    implements OXManager
{
    public DualOXManagerCell()
        {
            this.impl = new OXManagerCell();
        }
    
    public void
    start()
        {
            this.impl.start();
        }

    public void
    stop()
        {
            this.impl.stop();
        }

    public OXHandle
    createHandle(ObjectId oid)
        {
            final ObjectId _id = oid;
            
            OXHandleAction a = new OXHandleAction(null) {
                    public OXHandle runAction() {
                        return impl.createHandle(_id);
                    }
                };

            return runAction(a);
        }

    public void
    releaseHandle( OXHandle oxh )
        {
            OXHandleAction a = new OXHandleAction(oxh) {
                    public OXHandle runAction() {
                        impl.releaseHandle(this.h);
                        return null;
                    }
                };
            
            runAction(a);
        }

    public OXHandle
    grantOwnership(OXHandle oxh)
        throws IllegalOXHandleException 
        {
            OXHandleAction a = new OXHandleAction(oxh) {
                    public OXHandle runAction() {
                        return impl.grantOwnership(this.h);
                    }
                };
            
            return runAction(a);
        }

    public OXHandle
    requestOwnership(OXHandle oxh)
        throws IllegalOXHandleException 
        {
            OXHandleAction a = new OXHandleAction(oxh) {
                    public OXHandle runAction() {
                        return impl.requestOwnership(this.h);
                    }
                };
            
            return runAction(a);
        }

    public OXHandle
    updateAttributes(OXHandle oxh, String[] atts, byte[] ops)
        throws IllegalOXHandleException
        {
            final String[] _atts = atts;
            final byte[] _ops = ops;
            
            OXHandleAction a = new OXHandleAction(oxh) {
                    public OXHandle runAction() {
                        return impl.updateAttributes(this.h, _atts, _ops);
                    }
                };
            
            return runAction(a);
        }

    
        ////////////////////////////////////////////////////////
        // PRIVATE STUFF
        ////////////////////////////////////////////////////////
    
    private final OXHandle
    runAction(OXHandleAction action)
        {
            Executor executor = (Executor) Exehda.getService(Executor.SERVICE_NAME);

            executor.runAction(executor.currentApplication(), action);

            action.checkException();

            return action.getResult();
        }

    
    private abstract class OXHandleAction
        implements Runnable
    {
        abstract public OXHandle runAction() throws RuntimeException;

        public OXHandleAction (OXHandle h)
            {
                this.h = h;
            }
        
        public void
        run()
            {
                try {
                    this.h = runAction();
                }
                catch (RuntimeException e) {
                    this.e = e;
                }
            }
    
        public void
        checkException() throws RuntimeException
            {
                if ( e != null ) throw e;
            }
    
        public OXHandle
        getResult()
            {
                return this.h;
            }

        protected OXHandle h = null;
        protected RuntimeException e = null;    
    }

        /**
         * Real manager implementation.
         *
         */
    final OXManagerCell impl;
}
