// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.worb;


import org.isam.exehda.Exehda;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.Executor;
import org.isam.util.codegen.proxy.InvocationHandler;
import org.isam.util.codegen.proxy.Proxy;
import java.util.Hashtable;
import java.lang.reflect.Method;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.security.NoSuchAlgorithmException;



class WSClientRef
    extends WorbServiceRef
    implements InvocationHandler //, WorbProtocolConstants
{    
    private final String    serviceBaseURL;
    private transient Hashtable hashByMethods;
    
    WSClientRef(String serviceBaseURL, Class serviceIface )
        {
            super(serviceIface);
            this.serviceBaseURL = serviceBaseURL;
            this.hashByMethods = null;
        }

    public Object
    invoke( Long opnum, Method m, Object[] args )
        throws Exception
        {
            Object remResult = null;
            Exception remException = null;

            try {                
//                 URLConnection conn = (new URL(serviceBaseURL+'/'+m.getName())).openConnection();
//                 conn.setDoOutput(true);
//                 conn.setRequestProperty(WORB_REQ_PROP_OPNUM,opnum.toString());

                URLConnection conn = createConnection(m);
                
                ObjectOutputStream out = new ObjectOutputStream(
                    new BufferedOutputStream(conn.getOutputStream()));

                out.writeObject(getCurrentApplication());
                out.writeObject(args);
                out.close();

                WorbObjectInputStream in = new WorbObjectInputStream(
                    new BufferedInputStream(conn.getInputStream()));

                remException = (Exception) in.readObject();
                remResult = in.readObject();
            }
            catch (Exception e) {
                    //
                    // TO DO: delegate to exception handler
                    //
                throw e;
//                throw (Exception) e.fillInStackTrace();
            }

            if ( remException == null ) {
                return remResult;
            }
            else {
                throw (Exception) remException.fillInStackTrace();
            }
        }

    public Object
    invoke( Proxy p, Method m, Object[] args )
        throws Exception
        {
            return invoke( (Long) null, m, args);
        }

        /////////////////////////////////////////////////////////
        // PRIVATE STUFF
        /////////////////////////////////////////////////////////

    
        /**
         * Lookup the of the given method
         *
         * @param m a <code>Method</code> value
         * @return a <code>Long</code> value
         * @exception NoSuchAlgorithmException if an error occurs
         * @exception IOException if an error occurs
         */
    private final synchronized Long
    getMethodHash(Method m)
        throws NoSuchAlgorithmException, IOException
        {
            if ( hashByMethods == null ) {
                Method[] mtds = serviceInterface.getDeclaredMethods();
                
                hashByMethods = new Hashtable((int) mtds.length);

                for ( int i=0; i< mtds.length; i++ ) {
                    hashByMethods.put(
                        mtds[i],
                        new Long(computeMethodHash(mtds[i])));
                }
            }

            return (Long) hashByMethods.get(m);
        }

    
        /**
         * Create a connection to the given method of the remote service.
         *
         * @param m a <code>Method</code> value
         * @return an <code>URLConnection</code> value
         */
    private final URLConnection
    createConnection( Method m )
        throws MalformedURLException, IOException, NoSuchAlgorithmException
        {
            URLConnection conn = (new URL(encodeMethod(m))).openConnection();

            conn.setDoOutput(true);
                // output method hash
            Long opnum = getMethodHash(m);
            conn.setRequestProperty(WorbProtocolConstants.WORB_REQ_PROP_OPNUM,
                                    opnum.toString());

            return conn;
        }

    
        /**
         * Builds a URL-formated string representation of the given method reference.
         *
         * @param m a <code>Method</code> value
         * @return a <code>String</code> value
         */
    private final String
    encodeMethod( Method m )
        {
            return serviceBaseURL+'/'+m.getName();
        }

    
        /**
         * Returns the ID of the current application
         *
         * @return an <code>ApplicationId</code> value
         */
    private final ApplicationId
    getCurrentApplication()
        {
            return ((Executor) Exehda.getService(Executor.SERVICE_NAME)).currentApplication();
        }
}
