// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.debug.ctxm;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.GridLayout;
import java.io.*;

import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.ContextManager;
import org.isam.exehda.services.ContextManager.Context;
import org.isam.exehda.services.ContextManager.ContextState;
import org.isam.exehda.services.ContextManager.ContextListener;

import org.isam.util.xml.XmlElement;

public class DebugCtxManagerImpl
    extends AbstractService
    implements ContextManager
{

    
    public DebugCtxManagerImpl()
        {
                // initilize other internal tables
            currStateByCtx = new Hashtable();
            clByCtx = new Hashtable();

            debugGuiPane = new Panel();
            debugGuiPane.setLayout(
                new GridLayout(0,1));
            
            debugGui = new Frame("Debug Context Manager");
            debugGui.setSize(200, 300);
            debugGui.add( "North", debugGuiPane);
            debugGui.add( "Center", new Panel());
        }

    public void start()
        {
        }

    public void stop()
        {
        }


        /**
         * Parses the given context definition creating the respective GUI element. Only
         * the states definition is considered.
         *
         * @param xmlDesc a <code>String</code> value
         * @return a <code>Context</code> value
         */
    public Context createContext( String xmlDesc )
        {
            try {
                XmlElement docRoot = XmlElement.parse(
                    new StringReader(xmlDesc));

                    // extract context element's name
                XmlElement ctxDef = (XmlElement)
                    XmlElement.selectByPath(docRoot,
                                            "/CONTEXT",
                                            null).elementAt(0);
                String ctxName = ctxDef.getAttribute("n", null);
                if ( ctxName == null ) {
                    throw new NullPointerException("Missing context element name");
                }

                    // extract context element's states
                Vector tmp = XmlElement.selectByPath(docRoot,
                                                           "/CONTEXT/STATES/STATE",
                                                           null);
                String[] ctxStates = new String[tmp.size()];
                for (int i=0; i<tmp.size(); i++) {
                    ctxStates[i] = ((XmlElement)
                                    tmp.elementAt(i)).getAttribute("n", null);
                    if ( ctxStates[i] == null ) {
                        throw new NullPointerException("Malformed ctx definition");
                    }
                }

                Context ctx = new Context(ctxName,
                                          null,
                                          xmlDesc,
                                          ctxStates);

                addToGui(ctx, ctxStates);
                
                currStateByCtx.put( ctx,
                                    ctx.getStateInstance(ctxStates[0]));
                
                return ctx;
            }
            catch (IOException ioe) {
                ioe.printStackTrace();
            }
            
            return null;
        }
    

    public void releaseContext( Context ctx )
        {
        }
    
    public void addContextListener( ContextListener l, Context ctx )
        {
            ContextListenerGroup lg = (ContextListenerGroup) clByCtx.get(ctx);
            if ( lg == null ) {
                lg = new ContextListenerGroup();
                clByCtx.put(ctx, lg);
            }
            lg.add(l);

            l.contextChanged((ContextState) null,
                             (ContextState) currStateByCtx.get(ctx));
        }
        
    public void removeContextListener( ContextListener l, Context ctx )
        {
        }

    public void fireContextEvent( Context ctx, ContextState newState )
        {
            ContextState prevState = (ContextState) currStateByCtx.get(ctx);

            if ( prevState == newState ) return;
            
            currStateByCtx.put( ctx, newState );

            ContextListener l = (ContextListener) clByCtx.get(ctx);

            if ( l != null ) {
                l.contextChanged(prevState, newState);
            }
        }

        /////////////////////////////////////////////////////////////////
        // Private Stuff
        /////////////////////////////////////////////////////////////////
    
    private void addToGui(Context ctx, String[] states)
        {
            debugGuiPane.add( new ContextForm( ctx, states, this ));
            debugGui.setVisible(true);    
        }

    
    private Hashtable currStateByCtx;
    private Hashtable clByCtx;
    private Frame     debugGui;
    private Panel     debugGuiPane;
}



class ContextListenerGroup
    implements ContextListener
{
    Vector list = new Vector();
    
    public void add( ContextListener l)
        {
            list.addElement(l);
        }
    
    public void remove( ContextListener l)
        {
            list.removeElement(l);
        }
    
    public void contextChanged( ContextState prev, ContextState curr)
        {
            Enumeration e = list.elements();
            while ( e.hasMoreElements()) {
                ContextListener l = (ContextListener) e.nextElement();
                l.contextChanged(prev,curr);
            }
        }
}
