package dimi.channel;

import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Collection;
import java.util.Iterator;

import dimi.exception.CantFindChannelException;
import dimi.Disseminator;
import dimi.message.Message;
import dimi.channel.DisseminationUnitChannel;
import dimi.channel.connection.DisseminationUnitConnection;
import dimi.ConsumerEntryPoint;
import dimi.Disseminator;
import dimi.UserEntryPoint;
import dimi.UserEntryPointImpl;
import dimi.DisseminationUnit;
import dimi.name.DimiName;
import dimi.channel.connection.*;
import dimi.channel.connection.exception.CantCreateConnectionException;

import java.rmi.RemoteException;

/**
 * Gerente de conexoes de um disseminador.
 * @author mcmoraes
 */
public class ConnectionManager {

    //atributos - INICIO
    protected DisseminationUnitChannel channel = null;
    protected Map inputConnections = null;
    protected Map outputConnections = null;
    //atributos - FIM
    
    //construtores - INICIO
    /**
     * Creates a new instance of Connections
     * @param channel o canal de disseminador que possui este gerente
     */
    public ConnectionManager(DisseminationUnitChannel channel) {
        this.channel = channel;
        this.inputConnections = new Hashtable();
        this.outputConnections = new Hashtable();
    }
    //construtores - FIM

    //getters e setters - INICIO
    /**
     * Getter for property channel.
     * @return Value of property channel.
     */
    public final DisseminationUnitChannel getChannel() {
        return channel;
    }
    //getters e setters - FIM
    
    /**
     * Obtem uma conexao de um canal.
     * @return a conexao procurada
     * @param disseminatorName o nome do disseminador apontado pela conexao
     */    
    public final synchronized DisseminationUnitConnection getConnection(DimiName disseminatorName){
        if(inputConnections.containsKey(disseminatorName)){
            return (DisseminationUnitConnection)inputConnections.get(disseminatorName);
        }
        else if(outputConnections.containsKey(disseminatorName)){
            return (DisseminationUnitConnection)outputConnections.get(disseminatorName);
        }
        return null;
    }
    
    protected final boolean isDisseminatorAllowedToConnect(Disseminator disseminator) throws RemoteException{
        return true;
    }
    
    /**
     * Cria uma nova conexao com um disseminador.
     * @return a nova conexao
     * @param disseminator o disseminador que sera conectado
     * @param input se a conexao e de entrada ou de saida
     * @throws CantEnterChannelException se nao for possivel entrar no canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public final synchronized DisseminationUnitConnection createConnection(Disseminator disseminator,boolean input) throws RemoteException,CantCreateConnectionException{
        if(getConnection(disseminator.getName())!=null){
            throw new CantCreateConnectionException("Connection already exists.");
        }
        DisseminationUnitConnection disseminationUnitConnection = null;
        if(isDisseminatorAllowedToConnect(disseminator)){
            disseminationUnitConnection = new DisseminationUnitConnection(this,disseminator,input);
            if(input){
                inputConnections.put(disseminator.getName(), disseminationUnitConnection);
            }
            else{
                outputConnections.put(disseminator.getName(), disseminationUnitConnection);
            }
        }
        else{
            throw new CantCreateConnectionException("Not allowed to connect.");
        }
        return disseminationUnitConnection;
    }

    /**
     * As conexoes de entrada deste manager.
     * @return as conexoes de entrada deste manager
     */    
    public final DisseminationUnitConnection[] getInputConnections(){
        return (DisseminationUnitConnection[])inputConnections.values().toArray();
    }

    /**
     * As conexoes de saida deste manager.
     * @return as conexoes de saida deste manager.
     */    
    public final DisseminationUnitConnection[] getOutputConnections(){
        return (DisseminationUnitConnection[])outputConnections.values().toArray();
    }

    /**
     * Destroi uma conexao
     * @param disseminatorName o nome do disseminador-alvo da conexao
     */    
    public final synchronized void destroyConnection(DimiName disseminatorName){
        DisseminationUnitConnection disseminationUnitConnection = null;
        if(outputConnections.containsKey(disseminatorName)){
            disseminationUnitConnection = (DisseminationUnitConnection)outputConnections.get(disseminatorName);
            outputConnections.remove(disseminatorName);
        }
        else if(inputConnections.containsKey(disseminatorName)){
            disseminationUnitConnection = (DisseminationUnitConnection)inputConnections.get(disseminatorName);
            inputConnections.remove(disseminatorName);
        }
        if(disseminationUnitConnection != null){
            try{
                disseminationUnitConnection.destroy();
            }catch(RemoteException re){}
        }
    }
            
    /**
     * Destroi todas as conexoes deste manager.
     */    
    public final synchronized void destroyAllConnections(){
        Iterator it = null;
        it = inputConnections.values().iterator();
        while(it.hasNext()){
            destroyConnection(((DisseminationUnitConnection)it.next()).getDisseminatorName());
        }
        it = outputConnections.values().iterator();
        while(it.hasNext()){
            destroyConnection(((DisseminationUnitConnection)it.next()).getDisseminatorName());
        }
    }

    public final synchronized void disseminate(Message message){
        Iterator it = outputConnections.values().iterator();
        while(it.hasNext()){
            DisseminationUnitConnection disseminationUnitConnection = (DisseminationUnitConnection)it.next();
            try{
System.out.println("connection target: " + disseminationUnitConnection.getDisseminator().getName());
                disseminationUnitConnection.getMessageBuffer().add(message);
                //disseminationUnitConnection.getDisseminator().disseminate(message);
            }
            catch(RemoteException re){
System.out.println("connection broken: " + disseminationUnitConnection.getDisseminatorName());
                destroyConnection(disseminationUnitConnection.getDisseminatorName());
System.out.println("connection removed: " + disseminationUnitConnection.getDisseminatorName());
            }
            /*
            catch(CantFindChannelException re){
System.out.println("connection broken: " + disseminationUnitConnection.getDisseminatorName());
                destroyConnection(disseminationUnitConnection.getDisseminatorName());
System.out.println("connection removed: " + disseminationUnitConnection.getDisseminatorName());
            }
             */
        }
    }

}
