package dimi.channel.connection;

import dimi.channel.EntryPointChannel;
import dimi.Disseminator;
import dimi.channel.Channel;
import dimi.message.Message;
import dimi.message.MessageHeader;
import dimi.name.DisseminationUnitName;

import java.net.URL;
import java.rmi.RemoteException;

/**
 * Uma conexao possuida por um canal de ponto de entrada de usuario.
 * @author mcmoraes
 */
public class EntryPointConnection extends Connection{
    
    private EntryPointChannel entryPointChannel = null;
    
    /**
     * Creates a new instance of Connection
     * @param input se a conexao e de entrada ou saida
     * @param entryPointChannel o canal que possui esta conexao
     * @param disseminator o disseminador para o qual esta conexao aponta
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public EntryPointConnection(EntryPointChannel entryPointChannel,Disseminator disseminator,boolean input) throws RemoteException{
        super(input,disseminator);
        this.entryPointChannel = entryPointChannel;
    }
    
    /**
     * Getter for property entryPointChannel.
     * @return Value of property entryPointChannel.
     */
    public EntryPointChannel getEntryPointChannel() {
        return entryPointChannel;
    }
    
    /**
     * Monta o cabecalho da mensagem de saida do canal.
     * @return o cabecalho da mensagem de saida do canal
     */    
    protected MessageHeader mountLeaveMessageHeader(){
        MessageHeader messageHeader = new MessageHeader();
        try{
            messageHeader.setChannelName(entryPointChannel.getNames()[Channel.ROOT_NAME]);
            messageHeader.setOriginName(entryPointChannel.getUserEntryPointImpl().getName());
        }
        catch(RemoteException re){re.printStackTrace();}
        return messageHeader;
    }
    
}
