package dimi.gui.simple;

import java.util.Timer;
import java.util.TimerTask;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Collection;

import java.rmi.RemoteException;

import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.io.IOException;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import dimi.DNodeImpl;
import dimi.channel.DisseminationUnitChannel;
import dimi.channel.Channel;
import dimi.name.ChannelName;
import dimi.name.BadformedDimiNameException;

/**
 * Um frame para a visualizacao de uma unidade de disseminacao.
 * @author mauricio
 */
public class DNodeGUI extends javax.swing.JFrame {

    private int DISSEMINATION_UNIT_TAB_INDEX = 0;
    private int CHANNELS_TAB_INDEX = 1;
    private int OUTPUT_TAB_INDEX = 2;
    private int OUTPUT_ERROR_TAB_INDEX = 3;
    
    private int NAME_PROPERTIES_LIST_INDEX = 0;
    private void putNameOnPropertiesList() throws RemoteException{
        propertiesListModel.add(NAME_PROPERTIES_LIST_INDEX, "Name: " + dnodeImpl.getName().toString());
    }
    private int FILTER_LIST_PROPERTIES_LIST_INDEX = 1;
    private void putFilterListOnPropertiesList() throws RemoteException{
        StringBuffer filterList = new StringBuffer();
        Iterator it = dnodeImpl.getConfiguration().getFilterList().iterator();
        while(it.hasNext()){
            filterList.append(it.next().toString());
            if(it.hasNext()){
                filterList.append(",");
            }
        }
        propertiesListModel.add(FILTER_LIST_PROPERTIES_LIST_INDEX, "Filter List: " + filterList.toString());
    }
    
    private Timer timer = new Timer();
    private Updater updater = new Updater();
    
    private DNodeImpl dnodeImpl = null;
    
    private DefaultListModel propertiesListModel = new DefaultListModel();
    private DefaultListModel channelsListModel = new DefaultListModel();
    private DefaultListModel messageBufferListModel = new DefaultListModel();
    
    /**
     * Creates new form Principal
     * @param dnodeImpl a unidade de disseminacao a ser visualizada
     */
    public DNodeGUI(DNodeImpl dnodeImpl) {
        this.dnodeImpl = dnodeImpl;
        initComponents();
        //devido a limitacao do ambiente swing do netbeans!?
        this.disseminationUnitTPane.setTitleAt(0, "Dissemination Unit");
        //adicao de tabs
        this.disseminationUnitTPane.addTab("Channels",channelsPanel);
        this.disseminationUnitTPane.addTab("Output",outputPanel);
        this.disseminationUnitTPane.addTab("Error Output",errorPanel);
        //ajuste dos modelos das listas
        propertiesList.setModel(propertiesListModel);
        channelsList.setModel(channelsListModel);
        //
        messageBufferList.setModel(messageBufferListModel);
        //ajuste de dimensoes
        setBounds(0, 0, DBaseGUI.screenSize.width/2, DBaseGUI.screenSize.height/2);
        //otimizacao do swing
        channelsDesktop.putClientProperty("JDesktopPane.dragMode", "outline");
        //adicao dos listeners para as listas
        channelsList.addListSelectionListener(new ChannelsListSelectionListener());
        //
        try{
            disseminationUnitNameLabel.setText(dnodeImpl.getName().toString());
            dbaseNameLabel.setText("@"+dnodeImpl.getDbase().getName().toString());
            putNameOnPropertiesList();
            putFilterListOnPropertiesList();
        }
        catch(RemoteException re){re.printStackTrace();}
        
        timer.schedule(updater,DBaseGUI.UPDATER_PERIOD, DBaseGUI.UPDATER_PERIOD);
        Thread outputReaderThread = new Thread(new OutputReader());
        outputReaderThread.start();
        Thread errorReaderThread = new Thread(new ErrorReader());
        errorReaderThread.start();

    }
    
    private Map channelsIFrameMap = new TreeMap();
    class ChannelsListSelectionListener implements ListSelectionListener{
        public void valueChanged(ListSelectionEvent evt) {
            if (evt.getValueIsAdjusting() == false) {
                if (channelsList.getSelectedIndex() != -1) {
                    disseminationUnitTPane.setSelectedIndex(CHANNELS_TAB_INDEX);
                    DisseminationUnitChannel channel = (DisseminationUnitChannel)channelsList.getSelectedValue();
                    ChannelIFrame frame = (ChannelIFrame)channelsIFrameMap.get(channel.toString());
                    if(frame == null || frame.isClosed()){
                        frame = new ChannelIFrame(channel);
                        frame.setBounds(0, 0, DBaseGUI.screenSize.width/2, DBaseGUI.screenSize.height/2);
                        channelsIFrameMap.put(channel.toString(),frame);
                        channelsDesktop.add(frame);
                    }
                    //:( - swing juca! - BEGIN
                    else{
                        frame.setVisible(false);
                    }
                    try {
                        frame.setSelected(true);
                    }
                    catch (java.beans.PropertyVetoException pve) {
                        pve.printStackTrace();
                    }
                    frame.setVisible(true);
                    //:( - swing juca! - END
                }
            }
        }
    }

    class Updater extends TimerTask{
        public void run(){
            DBaseGUI.updateDefaultListModel(channelsListModel,dnodeImpl.getChannelManager().getChannels());
            DBaseGUI.updateDefaultListModel(messageBufferListModel,dnodeImpl.getMessageBuffer().getMessages());
        }
    }

    private PipedOutputStream pipedOutputStream = null;
    private PipedInputStream pipedInputStream = null;
    class OutputReader implements Runnable{
        public void run(){
            while(true){
                try{
                    pipedOutputStream = new PipedOutputStream();
                    pipedInputStream = new PipedInputStream(pipedOutputStream);
                }catch(IOException ioe){
                    ioe.printStackTrace();
                }
                System.setOut(new PrintStream(pipedOutputStream,true));
                byte[] barray = new byte[1];
                try{
                    while(true){
                        pipedInputStream.read(barray);
                        outputTextArea.append(new String(barray));
                    }
                }
                catch(IOException ioe){}
            }
        }
    }

    private PipedOutputStream pipedErrorOutputStream = null;
    private PipedInputStream pipedErrorInputStream = null;
    class ErrorReader implements Runnable{
        public void run(){
            while(true){
                try{
                    pipedErrorOutputStream = new PipedOutputStream();
                    pipedErrorInputStream = new PipedInputStream(pipedErrorOutputStream);
                }catch(IOException ioe){
                    ioe.printStackTrace();
                }
                System.setErr(new PrintStream(pipedErrorOutputStream,true));
                byte[] barray = new byte[1];
                try{
                    while(true){
                        pipedErrorInputStream.read(barray);
                        errorTextArea.append(new String(barray));
                        disseminationUnitTPane.setSelectedIndex(OUTPUT_ERROR_TAB_INDEX);
                    }
                }
                catch(IOException ioe){}
            }
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        channelsPanel = new javax.swing.JPanel();
        channelsDesktop = new javax.swing.JDesktopPane();
        outputPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        outputTextArea = new javax.swing.JTextArea();
        errorPanel = new javax.swing.JPanel();
        jScrollPane2 = new javax.swing.JScrollPane();
        errorTextArea = new javax.swing.JTextArea();
        jPanel7 = new javax.swing.JPanel();
        disseminationUnitTPane = new javax.swing.JTabbedPane();
        jPanel8 = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        JPanel900 = new javax.swing.JPanel();
        JScrollPane900 = new javax.swing.JScrollPane();
        propertiesList = new javax.swing.JList();
        JLabel900 = new javax.swing.JLabel();
        JPanel901 = new javax.swing.JPanel();
        JScrollPane901 = new javax.swing.JScrollPane();
        channelsList = new javax.swing.JList();
        JLabel901 = new javax.swing.JLabel();
        jPanel4 = new javax.swing.JPanel();
        JScrollPane903 = new javax.swing.JScrollPane();
        messageBufferList = new javax.swing.JList();
        JLabel903 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        actionsPanel = new javax.swing.JPanel();
        jPanel6 = new javax.swing.JPanel();
        connectionButton = new javax.swing.JToggleButton();
        jPanel3 = new javax.swing.JPanel();
        disseminationUnitNameLabel = new javax.swing.JLabel();
        dbaseNameLabel = new javax.swing.JLabel();

        channelsPanel.setLayout(new java.awt.BorderLayout());

        channelsPanel.setBorder(new javax.swing.border.EtchedBorder());
        channelsPanel.add(channelsDesktop, java.awt.BorderLayout.CENTER);

        outputPanel.setLayout(new java.awt.BorderLayout());

        outputPanel.setBorder(new javax.swing.border.EtchedBorder());
        jScrollPane1.setViewportView(outputTextArea);

        outputPanel.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        errorPanel.setLayout(new java.awt.BorderLayout());

        errorPanel.setBorder(new javax.swing.border.EtchedBorder());
        jScrollPane2.setViewportView(errorTextArea);

        errorPanel.add(jScrollPane2, java.awt.BorderLayout.CENTER);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Dissemination Unit Graphical Interface");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jPanel7.setLayout(new java.awt.BorderLayout());

        disseminationUnitTPane.setBorder(new javax.swing.border.EtchedBorder());
        disseminationUnitTPane.setFont(new java.awt.Font("Serif", 3, 12));
        jPanel8.setLayout(new java.awt.BorderLayout());

        jPanel1.setLayout(new java.awt.GridLayout(1, 0));

        jPanel1.setBorder(new javax.swing.border.EtchedBorder());
        JPanel900.setLayout(new java.awt.BorderLayout());

        propertiesList.setFont(new java.awt.Font("Default", 0, 10));
        propertiesList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JScrollPane900.setViewportView(propertiesList);

        JPanel900.add(JScrollPane900, java.awt.BorderLayout.CENTER);

        JLabel900.setFont(new java.awt.Font("Serif", 2, 12));
        JLabel900.setText("Properties:");
        JPanel900.add(JLabel900, java.awt.BorderLayout.NORTH);

        jPanel1.add(JPanel900);

        JPanel901.setLayout(new java.awt.BorderLayout());

        channelsList.setFont(new java.awt.Font("Default", 0, 10));
        channelsList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JScrollPane901.setViewportView(channelsList);

        JPanel901.add(JScrollPane901, java.awt.BorderLayout.CENTER);

        JLabel901.setFont(new java.awt.Font("Serif", 2, 12));
        JLabel901.setText("Channels:");
        JPanel901.add(JLabel901, java.awt.BorderLayout.NORTH);

        jPanel1.add(JPanel901);

        jPanel4.setLayout(new java.awt.BorderLayout());

        messageBufferList.setFont(new java.awt.Font("Default", 0, 10));
        messageBufferList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        JScrollPane903.setViewportView(messageBufferList);

        jPanel4.add(JScrollPane903, java.awt.BorderLayout.CENTER);

        JLabel903.setFont(new java.awt.Font("Serif", 2, 12));
        JLabel903.setText("Buffer:");
        jPanel4.add(JLabel903, java.awt.BorderLayout.NORTH);

        jPanel1.add(jPanel4);

        jPanel8.add(jPanel1, java.awt.BorderLayout.CENTER);

        jPanel2.setLayout(new java.awt.BorderLayout());

        actionsPanel.setLayout(new java.awt.BorderLayout());

        jPanel6.setBorder(new javax.swing.border.EtchedBorder());
        connectionButton.setText("Connect/Disconnect");
        connectionButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                connectionButtonActionPerformed(evt);
            }
        });

        jPanel6.add(connectionButton);

        actionsPanel.add(jPanel6, java.awt.BorderLayout.SOUTH);

        jPanel2.add(actionsPanel, java.awt.BorderLayout.NORTH);

        jPanel8.add(jPanel2, java.awt.BorderLayout.SOUTH);

        disseminationUnitTPane.addTab("tab2", jPanel8);

        jPanel7.add(disseminationUnitTPane, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel7, java.awt.BorderLayout.CENTER);

        jPanel3.setLayout(new java.awt.BorderLayout());

        disseminationUnitNameLabel.setFont(new java.awt.Font("Dialog", 3, 12));
        disseminationUnitNameLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        disseminationUnitNameLabel.setText("DNode Name");
        jPanel3.add(disseminationUnitNameLabel, java.awt.BorderLayout.NORTH);

        dbaseNameLabel.setFont(new java.awt.Font("Dialog", 3, 12));
        dbaseNameLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        dbaseNameLabel.setText("@DBase Name");
        jPanel3.add(dbaseNameLabel, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel3, java.awt.BorderLayout.NORTH);

        pack();
    }//GEN-END:initComponents

    private void connectionButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_connectionButtonActionPerformed
        if(connectionButton.isSelected()){
            //System.out.println("pressionou");
        }
        else{
            //System.out.println("soltou");
        }
    }//GEN-LAST:event_connectionButtonActionPerformed

    private JFrame frame = new JFrame("DialogDemo");    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        updater.cancel();
        timer.cancel();
        Iterator it = null;
        it = channelsIFrameMap.values().iterator();
        while(it.hasNext()){
            ChannelIFrame frame = (ChannelIFrame)it.next();
            
        }
        System.exit(0);
    }//GEN-LAST:event_exitForm
        
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel JLabel900;
    private javax.swing.JLabel JLabel901;
    private javax.swing.JLabel JLabel903;
    private javax.swing.JPanel JPanel900;
    private javax.swing.JPanel JPanel901;
    private javax.swing.JScrollPane JScrollPane900;
    private javax.swing.JScrollPane JScrollPane901;
    private javax.swing.JScrollPane JScrollPane903;
    private javax.swing.JPanel actionsPanel;
    private javax.swing.JDesktopPane channelsDesktop;
    private javax.swing.JList channelsList;
    private javax.swing.JPanel channelsPanel;
    private javax.swing.JToggleButton connectionButton;
    private javax.swing.JLabel dbaseNameLabel;
    private javax.swing.JLabel disseminationUnitNameLabel;
    private javax.swing.JTabbedPane disseminationUnitTPane;
    private javax.swing.JPanel errorPanel;
    private javax.swing.JTextArea errorTextArea;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JList messageBufferList;
    private javax.swing.JPanel outputPanel;
    private javax.swing.JTextArea outputTextArea;
    private javax.swing.JList propertiesList;
    // End of variables declaration//GEN-END:variables
    
}
