package dimi.message.buffer;

import dimi.message.Message;
import java.util.Iterator;
import java.util.List;

/**
 * Um buffer de mensagens e necessario para que possa haver suporte
 * a desconexao planejada
 * @author mcmoraes
 */
public interface MessageBuffer {
    
    /**
     * Obtem a proxima mensagem do buffer.
     * @return a proxima mensagem do buffer
     */    
    public Message get();

   /**
     * A proxima mensagem a ser lida do buffer.
     * @return a mensagem retirada do buffer
     */  
    public Message remove();
    
    /**
     * Remove uma mensagem especifica do buffer.
     * @param message a mensagem a ser retirada
     * @return a mensagem retirada
     */
    public boolean remove(Message message);
    
    /**
     * Adiciona uma mensagem ao buffer. Se o buffer tiver tamanho limitado,
     * a adicao sera de acordo com a politica de bufferizacao, que pode ser
     * 'first drop' ou 'last drop'.
     * @param message a mensagem a ser adicionada
     */
    public void add(Message message);
    
    /**
     * O buffer esta vazio?
     * @return true, se sim; false, caso contrario
     */
    public boolean isEmpty();
    
    /**
     * Obtem um iterador para o buffer.
     * @return o iterador do buffer
     */
    public Iterator iterator();
    
    /**
     * Obtem o numero de mensagens que ha no buffer.
     * @return o numero de mensagens do buffer
     */
    public int numberOfMessages();
    
    public long size();
    
    /**
     * Obtem o mediador que realiza a comunicacao desse buffer com o buffer
     * do disseminador
     * @return o mediador desse buffer
     */    
    public MessageBufferMediator getMediator();
    
    public Message[] getMessages();
}
