package dimi.message.buffer;

import dimi.message.Message;
import dimi.message.buffer.MessageBuffer;

import java.util.Iterator;
import java.util.Vector;
import java.util.List;

/**
 * Implementacao da interface MessageBuffer. Um buffer de mensagens
 * em um disseminador e sempre do tipo FIFO.
 * @author mcmoraes
 */
public abstract class MessageBufferImpl implements MessageBuffer{
    
    /**
     * O vetor fifo.
     */    
    protected Vector buffer = new Vector();
        
    public final Message[] getMessages() {
        return (Message[])buffer.toArray();
    }

    /**
     * Obtem a proxima mensagem do buffer.
     * @return a proxima mensagem do buffer
     */    
    public final Message get() {
        return (Message)buffer.lastElement();
    }
    
    /**
     * A proxima mensagem a ser lida do buffer.
     * @return a mensagem retirada do buffer
     */    
    public Message remove(){
        return (Message)buffer.remove(buffer.size()-1);
    }
    
    /**
     * Remove uma mensagem especifica do buffer.
     * @param message a mensagem a ser retirada
     * @return a mensagem retirada
     */    
    public final boolean remove(Message message){
        return buffer.remove(message);
    }

    /**
     * Adiciona uma mensagem ao buffer. Se o buffer tiver tamanho limitado,
     * a adicao sera de acordo com a politica de bufferizacao, que pode ser
     * 'first drop' ou 'last drop'.
     * @param message a mensagem a ser adicionada
     */    
    public abstract void add(Message message);
    
    /**
     * O buffer esta vazio?
     * @return true, se sim; false, caso contrario
     */    
    public final boolean isEmpty() {
        return buffer.isEmpty();
    }
    
    /**
     * Obtem um iterador para o buffer.
     * @return o iterador do buffer
     */    
    public final Iterator iterator() {
        return buffer.iterator();
    }
    
    /**
     * Obtem o numero de mensagens que ha no buffer.
     * @return o numero de mensagens do buffer
     */    
    public final int numberOfMessages() {
        return buffer.size();
    }
    
    public final long size() {
        long result = 0;
        Iterator it = buffer.iterator();
        while(it.hasNext()){
            Message message = (Message)it.next();
            result += message.getHeader().getContentSize();
        }
        return result;
    }

    public abstract MessageBufferMediator getMediator();
    
}
