/*
 * DiscovererProxy.java
 *
 * Created on July 6, 2004, 4:28 PM
 */

package org.isam.exehda.services.perdis;


import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.DiscovererService;
import org.isam.exehda.services.ContextManager;
import org.isam.exehda.services.ContextManager.Context;
import org.isam.exehda.services.ContextManager.ContextListener;
import org.isam.exehda.services.ContextManager.ContextState;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.services.CellInformationBase.NameSpace;
import org.isam.exehda.services.Worb;
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.util.xml.XmlElement;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.Enumeration;

/**
 *
 * @author  egon
 */
public class DiscovererProxy extends AbstractService implements DiscovererService, ContextListener {
    
    private CellInformationBase cellInformationBase;
    private AdvertiseManagerProxy advertiseManagerProxy;
    private ResourceCataloguer resourceCataloguer;
    private LeaseManager leaseManager;
    private ResourceMatcher resourceMatcher;
    private DispatcherThread dispatcherThread;
    private PreferenceManager preferenceManager;
    private NameSpace nameSpace;
    
    private String unicastAddress;
    private Integer unicastPort;
    private String multicastProxyAddress;
    private Integer multicastProxyPort;
    private String multicastBaseAddress;
    private Integer multicastBasePort;
    private Hashtable discovererBaseHst;
    private Object lock;
    private Object lockLease;

    private String ctx_connectivity;
    private Object lockCtxConn;
    
    /** 
     * Creates a new instance of DiscovererProxy 
     */
    public DiscovererProxy() {
        
    }
    
    /**
     * start
     */
    public void start() throws Exception {

        //Get configuration properties
        this.unicastAddress = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "unicastAddress", null);
        this.unicastPort = new Integer(Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "unicastPort", null));
        this.multicastProxyAddress = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "multicastProxyAddress", null);
        this.multicastProxyPort = new Integer(Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "multicastProxyPort", null));
        this.multicastBaseAddress = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "multicastBaseAddress", null);
        this.multicastBasePort = new Integer(Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "multicastBasePort", null));

        this.discovererBaseHst = new Hashtable();
        this.lock = new Object();
        this.lockLease = new Object();
        this.lockCtxConn = new Object();
        
        this.getInformedBases();
        
        //this.nameSpace = "service:" + DiscovererService.SERVICE_NAME + ",host:" + HostId.getLocalHost();
        this.nameSpace = new NameSpace("host", Integer.toString(HostId.getLocalHost().getId()), NameSpace.DEFAULT_NS);
        
        //Get CIB reference
        this.cellInformationBase = (CellInformationBase) Exehda.getService(CellInformationBase.SERVICE_NAME);
        
        //Create advertise listener
        this.advertiseManagerProxy = new AdvertiseManagerProxy(this.unicastAddress, this.unicastPort, this.multicastProxyAddress, this.multicastProxyPort, this.multicastBaseAddress, this.multicastBasePort, this.discovererBaseHst, this.lock);
        
        //Configure context manager for connectivity
        Context contextConnectivity = this.configureContextManagerForConnectivity();
        
        //Create dispatcher thread
        this.dispatcherThread = new DispatcherThread(discovererBaseHst, contextConnectivity);
        this.dispatcherThread.start();
        
        //Create lease manager
        this.leaseManager = new LeaseManager(this.cellInformationBase, this.nameSpace, this.lockLease);
        this.leaseManager.start();
        
        //Create resource cataloguer
        this.resourceCataloguer = new ResourceCataloguer(this.cellInformationBase, this.nameSpace, this.leaseManager, this.lockLease);
        
        //Create resource matcher
        this.resourceMatcher = new ResourceMatcher(this.cellInformationBase, this.nameSpace);
          
        //Create preference manager
        this.preferenceManager = new PreferenceManager();
        
        log(Sev.DEBUG, "start", "DiscovererProxy started!!!!");
    }

    /**
     * stop
     */
    public void stop() throws Exception {
    }

    /**
     * registerResource
     */
    public ResourceName registerResource(XmlElement xmlDescriptor) throws Exception {

        ResourceName rn = this.resourceCataloguer.catalogue(xmlDescriptor, true);
        this.dispatcherThread.enqueueOnRegisterResource(xmlDescriptor);
        
        return rn;
    }

    /**
     * removeResource
     */
    public void removeResource(ResourceName rn) {
                
        this.cellInformationBase.removeResource(rn);
        this.dispatcherThread.enqueueOnRemoveResource(rn);        
    }

    /**
     * renewResource
     */
    public void renewResource(ResourceName rn, XmlElement xmlDescriptor, boolean isModified) throws Exception {

        this.resourceCataloguer.renew(rn, xmlDescriptor, isModified);
        if (this.ctx_connectivity.equalsIgnoreCase("online")) {
            Enumeration enumeration = discovererBaseHst.elements();
            while (enumeration.hasMoreElements()){
                DiscovererService discovererTmp = (DiscovererService) enumeration.nextElement();
                discovererTmp.renewResource(rn, xmlDescriptor, isModified);
            }
        }
    }
    
    /**
     * requestResource
     */
    public XmlElement requestResource(XmlElement xmlRequest) throws Exception {
        
        log(Sev.DEBUG, "requestResource", "##### ctx_connectivity = " + this.ctx_connectivity);
        
        //If it is not online
        if (!this.ctx_connectivity.equalsIgnoreCase("online")) {
        
            //Get Timeout specified by the user
            long requestTimeout = this.getRequestTimeout(xmlRequest);
            log(Sev.DEBUG, "requestResource", "##### requestTimeout = " + requestTimeout);
            
            //Wait for connection restablishment
            synchronized(this.lockCtxConn) {
                if (requestTimeout > 0) {
                    this.lockCtxConn.wait(requestTimeout);
                }
            }
            
            //If it remains not online, return only local results
            if (!this.ctx_connectivity.equalsIgnoreCase("online")) {
                log(Sev.DEBUG, "requestResource", "##### FAZENDO PESQUISA LOCAL #####");
                return this.resourceMatcher.match(xmlRequest);    
            }
        }
        
        //If it reaches here, it is online and a complete search must be done
        Enumeration enumeration = discovererBaseHst.elements();
        if (enumeration.hasMoreElements()) {
            DiscovererService discovererTmp = (DiscovererService) enumeration.nextElement();
            log(Sev.DEBUG, "requestResource", "##### FAZENDO PESQUISA REMOTA #####");
            return discovererTmp.requestResource(xmlRequest);
        } else {
            throw new Exception("There is no Discoverer Base found!");
        }
    }
    
    /**
     * getRequestTimeout
     */
    private long getRequestTimeout(XmlElement xmlRequest) {
    
        Vector vec = XmlElement.selectByPath(xmlRequest, "/QUERY/TIMEOUT", null);
        XmlElement xmlTimeout = (XmlElement) vec.firstElement();
        long requestTimeout = new Long((String) xmlTimeout.getContents()).longValue();
        
        return requestTimeout;
    }
    
    /**
     * getInformedBases
     */
    private void getInformedBases() {
    
        //Read the parameter about informed discoverer bases from the profile
        String informedBases = Exehda.getServiceProperty(DiscovererService.SERVICE_NAME, "informedBases", null);
        if ((informedBases == null) || informedBases.equals("")) {
            return;
        }
        
        //Lookup the informed discoverer bases and set them on the hashtable
        Worb worb = (Worb) Exehda.getService("worb");
        StringTokenizer st = new StringTokenizer(informedBases, ";");        
        while(st.hasMoreTokens()) {
        
            String token = st.nextToken();
            synchronized(this.lock) {

                if (this.discovererBaseHst.get(token) == null) {
                    
                    try {
                        DiscovererService discovererTmp = (DiscovererService) worb.lookupService(token, DiscovererService.class);
                        this.discovererBaseHst.put(token, discovererTmp);
                    } catch(Exception e) {
                        log(Sev.ERROR, "getInformedBases", "Failed trying to lookup informed contact address: " + token);
                        e.printStackTrace();
                    }
                }
            }
        }
    }
    
    /**
     * configureContextManagerForConnectivity
     */
    private Context configureContextManagerForConnectivity() {
    
        log(Sev.DEBUG, "configureContextManagerForConnectivity", "##### ENTROU #####");
        
        //Create the context and add a listener
        ContextManager contextManager = (ContextManager) Exehda.getService(ContextManager.SERVICE_NAME);
        Context context = contextManager.createContext(ContextDefs.CONNECTIVITY);
        contextManager.addContextListener(this, context);
        
        //Wait for the context be initialized
        while (this.ctx_connectivity == null) {
            synchronized(this) {
                try{
                    this.wait(1000);
                } catch(Exception e) {
                    e.printStackTrace();
                }
            }
        }
        
        return context;
    }
    
    /**
     * contextChanged
     */
    public void contextChanged(ContextState prevContext, ContextState currContext) {
        
        log(Sev.DEBUG, "contextChanged", "##### currContext = " + currContext.getValue());
    
        if (currContext.getType().getName().equalsIgnoreCase("connectivity")) {
            if (currContext.getValue().equalsIgnoreCase("online")) {
                
                //Re-advertise the discoverer instance
                this.reconnectDiscoverer();
                
                //Wake up any requester
                synchronized(this.lockCtxConn) {
                    this.lockCtxConn.notifyAll();
                }
            }
            
            //Always set the new context
            this.ctx_connectivity = currContext.getValue();
        }
    }
    
    /**
     * reconnectDiscoverer
     */
    private void reconnectDiscoverer() {
    
        log(Sev.DEBUG, "reconnectDiscoverer", "##### ENTROU #####");
        
        //Clear hashtable of known discoverer bases
        synchronized(this.lock) {
            this.discovererBaseHst.clear();
        }
              
        //Re-advertise the discoverer instance
        try {            
            this.advertiseManagerProxy.advertise();
        } catch(IOException ioe) {
            ioe.printStackTrace();
            System.exit(1);
	}
        
        //Read again the informed discoverer bases from the profile
        this.getInformedBases();
    }
    
    /**
     * setOrderBy
     */
    public void setOrderBy(String type, String attribute, String order) {
        
        this.preferenceManager.setOrderBy(type, attribute, order);
    }
    
    /**
     * setDefaultSelection
     */
    public void setDefaultSelection(XmlElement xmlResults, String defaultName) {
        
        this.preferenceManager.setDefaultSelection(xmlResults, defaultName);
    }
}