/*
 * LeaseManager.java
 *
 * Created on 25 de Maio de 2004, 17:32
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.services.CellInformationBase.NameSpace;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.Logger;
import org.isam.exehda.Exehda;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Date;


/**
 *
 * @author  egon
 */
public class LeaseManager extends Thread {
    
    public static final double TOLERANCE_FACTOR = 1;
    public static final long WAIT_TIME = 10000;

    private CellInformationBase cib;
    private NameSpace nameSpace;
    private Object lockLease;
    private Hashtable leasetable;
    private Logger logger;
    
    
    /**
     * Creates a new instance of LeaseManager 
     */
    public LeaseManager(CellInformationBase cib, NameSpace nameSpace, Object lockLease) {
        
        this.cib = cib;
        this.nameSpace = nameSpace;
        this.lockLease = lockLease;
        this.leasetable = new Hashtable();
        this.logger =((Logger) Exehda.getService(Logger.SERVICE_NAME));
    }
    
    /**
     * run
     */
    public void run() {
        
        while(true) {
            
            synchronized(this.lockLease) {
                
                Enumeration leaseKeys = this.leasetable.keys();
                long currentTime = System.currentTimeMillis();
                
                while (leaseKeys.hasMoreElements()){
                    ResourceName rnTmp = (ResourceName) leaseKeys.nextElement();
                    Long expTmp = (Long) this.leasetable.get(rnTmp);
                    if (expTmp.longValue() < currentTime){
                       this.cib.removeResource(rnTmp);
                       this.leasetable.remove(rnTmp);
                    }
                }
            }

            try {
                System.gc();
                this.sleep(this.WAIT_TIME);
                System.gc();
            } catch(java.lang.InterruptedException ie){
                ie.printStackTrace();
                System.exit(1);
            }
        }
    }
    
    /**
     * updateLease
     */
    public void updateLease(ResourceName rn, long lease){
        
        synchronized(this.lockLease){

            Long newLease = new Long(System.currentTimeMillis() + lease + ((long)(lease*TOLERANCE_FACTOR)));
            this.leasetable.put(rn, newLease);
        }
    }   
}