/*
 * ResourceCataloguer.java
 *
 * Created on July 6, 2004, 2:53 PM
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.services.CellInformationBase.NameSpace;
import org.isam.util.xml.XmlElement;
import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import java.net.UnknownHostException;
import java.io.StringReader;
import java.io.IOException;
import java.util.Vector;

/**
 *
 * @author  egon
 */
public class ResourceCataloguer {
    
    private CellInformationBase cellInformationBase;
    private Logger logger;
    private NameSpace nameSpace;
    private LeaseManager leaseManager;
    private Object lockLease;
    private int resourceCounter;
    private long lease;
    
    
    /** 
     * Creates a new instance of ResourceCataloguer 
     */
    public ResourceCataloguer(CellInformationBase cellInformationBase, NameSpace nameSpace, LeaseManager leaseManager, Object lockLease) {
        
        this.logger = ((Logger) Exehda.getService(Logger.SERVICE_NAME));
        this.cellInformationBase = cellInformationBase;
        this.nameSpace = nameSpace;
        this.leaseManager = leaseManager;
        this.lockLease = lockLease;
        this.resourceCounter = 0;
    }
    
    /**
     * catalogue
     */
    public ResourceName catalogue(XmlElement xmlReceived, boolean createResourceName) throws IOException, UnknownHostException {
        
        //Separate the main tags of the xml
        Vector vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/PROPERTIES/PROPERTY", null);
        int vecSize = vec.size();
        XmlElement[] xmlRecProp = new XmlElement[vecSize];
        
        for (int i = 0; i < vecSize; i++) {
            xmlRecProp[i] = (XmlElement) vec.elementAt(i);
        }
        
        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/NAME", null);
        XmlElement xmlName = (XmlElement) vec.firstElement();
        
        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/DESCRIPTION", null);
        XmlElement xmlDescript = (XmlElement) vec.firstElement();

        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/TYPE", null);
        XmlElement xmlType = (XmlElement) vec.firstElement();
        
        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/URL", null);
        XmlElement xmlURL = (XmlElement) vec.firstElement();

        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/LEASE", null);
        XmlElement xmlLease = (XmlElement) vec.firstElement();
        Long aux = new Long (xmlLease.getContents().toString());
        this.lease=aux.longValue();
        

        //Build resource name
        String typeStr = xmlType.getContents().toString();
        
        String instanceStr = null;
        if (createResourceName) {
            instanceStr = ((xmlName.getContents() != null) ? xmlName.getContents().toString() : "") + ":" + this.getResourceCounter() + ":" + HostId.getLocalHost().getId();
            xmlName.clearContents();
            xmlName.addChild(XmlElement.createPCData(instanceStr));
        } else {
            instanceStr = xmlName.getContents().toString();
        }
        
        ResourceName rn = new ResourceName(typeStr, instanceStr, this.nameSpace);        
        this.logger.log(Logger.Sev.DEBUG, "ResourceCataloguer", "catalogue", "###### Recurso catalogado (rn) = "+rn.getDN(), null);
        
         
        //Add the ResourceName component to the CIB component and set its atributes
        this.cellInformationBase.addResource(rn);

        for (int i = 0; i < vecSize; i++) {
            this.cellInformationBase.setAttribute(rn, xmlRecProp[i].getAttribute("name","0"),xmlRecProp[i].getAttribute("value","0"));
        }

        this.cellInformationBase.setAttribute(rn, xmlName.getTag(), xmlName.getContents().toString());
        this.cellInformationBase.setAttribute(rn, xmlDescript.getTag(), xmlDescript.getContents().toString());
        this.cellInformationBase.setAttribute(rn, xmlType.getTag(), xmlType.getContents().toString());
        this.cellInformationBase.setAttribute(rn, xmlURL.getTag(), xmlURL.getContents().toString());

        this.leaseManager.updateLease(rn, this.lease);
        return rn;
    }
    
    /**
     * renew
     */
    public void renew(ResourceName rnGeneric, XmlElement xmlDescriptor, boolean isModified) throws java.io.IOException{
        
        synchronized(this.lockLease){
            
            ResourceName [] resNames = this.cellInformationBase.selectByName(rnGeneric.getSimpleName(), this.nameSpace, -1);
            if (resNames.length == 0){
                
                try {
                    this.catalogue(xmlDescriptor, false);
                } catch (java.io.IOException ioe) {
                    ioe.printStackTrace();
                    throw new IOException("I/O Error!!!");
                }
            } else {
                
                if (isModified){
                    this.update(resNames[0], xmlDescriptor);
                }
                this.leaseManager.updateLease(resNames[0], this.lease);
            }
        }
    }
    
    /**
     * getResourceCounter
     */
    private synchronized long getResourceCounter() {
        
        this.resourceCounter++;
        return this.resourceCounter;
    }

    /**
     * update
     */
    private void update(ResourceName rn, XmlElement xmlReceived){

        Vector vec;
        
        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/DESCRIPTION", null);
        XmlElement xmlDescript = (XmlElement) vec.firstElement();
        String description = xmlDescript.getContents().toString();
        String attribDescription = this.cellInformationBase.getAttribute(rn, "DESCRIPTION");

        if (!(attribDescription.equals(description))){
            this.cellInformationBase.setAttribute(rn, "DESCRIPTION",description);
        }
        
        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/TYPE", null);
        XmlElement xmlType = (XmlElement) vec.firstElement();
        String type = xmlType.getContents().toString();
        String attribType = this.cellInformationBase.getAttribute(rn, "TYPE");

        if (!(attribType.equals(type))){
            this.cellInformationBase.setAttribute(rn, "TYPE",type);
        }
        
        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/URL", null);
        XmlElement xmlURL = (XmlElement) vec.firstElement();
        String url = xmlURL.getContents().toString();
        String attribURL = this.cellInformationBase.getAttribute(rn,"URL");

        if (!(attribURL.equals(url))){
            this.cellInformationBase.setAttribute(rn, "URL",url);
        }
        
        vec = XmlElement.selectByPath(xmlReceived, "/RESOURCE/PROPERTIES/PROPERTY", null);
        int vecSize = vec.size();
        XmlElement[] xmlRecProp = new XmlElement[vecSize];

        for (int i = 0; i < vecSize; i++) {
            xmlRecProp[i] = (XmlElement) vec.elementAt(i);
            this.cellInformationBase.setAttribute(rn, xmlRecProp[i].getTag(),xmlRecProp[i].getContents().toString());            
        }
    }
}