// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos;

import org.isam.exehda.services.Collector.MonitoringConsumer;
import org.isam.exehda.services.Collector.MonitoringData;
import org.isam.exehda.services.Collector.ConsumerId;
import org.isam.exehda.services.Collector.Sensor;
import java.util.Vector;

    
/**
 * Facility for manipulating consumers internally in the collector implementation.
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-03-01 04:08:13 -0300 (Tue, 01 Mar 2005) $ $Revision: 604 $
 */
class MetaConsumer
{
    public MetaConsumer(ConsumerId id, MonitoringConsumer cb)
        {
            this.id = id; 
            this.callBack = cb;
            this.enabledSensors = new Vector();
            this.data = new MonitoringData[5];
        }

    public ConsumerId getId()
        {
            return this.id;
        }

    public void
    addEnabledSensor(MetaSensor s)
        {
            Object sname = s.getPublicSensor().getName();

            if ( ! enabledSensors.contains(sname) ) {
                enabledSensors.addElement(sname);
                s.addConsumer(this.id);
                s.setEnabled(true);
            }
        }

    public void
    removeEnabledSensor(MetaSensor s)
        {
            if ( enabledSensors.removeElement(s.getPublicSensor().getName()) ) {
                s.removeConsumer(this.id);
            }
        }

    public void
    update( long timeStamp, MonitoringData[] newData)
        {
            if ( callBack != null && enabledSensors.size() > 0 ) {
                    // grow data on-demand
                if ( data.length < enabledSensors.size() ) {
                    data = new MonitoringData[enabledSensors.size()+5];
                }
                    // filter newData selecting only sensores that are enabled for this consumer
                int k=0;
                for (int i=0; i<newData.length && newData[i] != null; i++) {
                        //
                        // TODO: optimze search, perhaps use a hashtable
                        //
                    if ( enabledSensors.contains(newData[i].getSensor()) ) {
                        data[k++] = newData[i];
                    }
                }
                
                    // avoid delivering an empty data set
                if ( k == 0 ) {
                    return;
                }
                else {
                        // tag end-of-data
                    if ( k < data.length ) data[k] = null;
                
                        // for security reasons, use a copy of new data when delivering
                    callBack.update(timeStamp, data);
                }
            }
            else {
                    // TO DO: use multicast channel
            }
        }

    private MonitoringData[] data;
    private ConsumerId id;
    private MonitoringConsumer callBack;
    private Vector enabledSensors;
}
