// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos.exec;

import org.isam.perv.IsamClassLoader;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.Executor.SchedulingHeuristic;
import org.isam.exehda.services.Executor.MalformedAppLaunchingDescriptorException;
import org.isam.util.xml.XmlElement;

import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Vector;


/**
 * The local, per-node, realization of a distributed aplication: a group of local threads
 * sharing the same execution context (classloader, permissions etc.)
 *
 
------------------------ 
<?xml version="1.0" encoding="UTF-8"?>

<isamapp spec="1.0" href="helloworld.isam">
    <info>
        <title>Hello World</title>
	<vendor>ISAM team</vendor>
	<description>This is ISAM HelloWorld demo application</description>
	<icon href="helloworld.png" />
    </info>
    
    <code>
        <main class="HelloWorld"/>
	    <jar href="bda://helloworld.jar"/>
    </code>

    <parameters>
        <param>args0</param>
        <param>args1</param>
        <param>args2</param>
    </parameters>

    ----- experimental stuff -----
    <runtime>
        <scheduling heuristic="org.isam.exehda.services.tips.TipsSchedHeuristic"/>
    </runtime>
    ------------------------------
    
</isamapp>
--------------------------

 * @author last modified by $Author: lucc $
 * @version $Date: 2005-03-30 12:52:08 -0300 (Wed, 30 Mar 2005) $ $Revision: 666 $
 */
class Application
    extends ThreadGroup
    implements Runnable
{
    public Application( ApplicationId appID, String ldesc, boolean slave)
        throws Exception
        {
            this( appID,
                  XmlElement.parse(new java.io.StringReader(ldesc)),
                  slave );
        }
    
    public Application( ApplicationId appID, XmlElement ldesc, boolean slave)
        throws Exception
        {
            super("Application "+appID.getInstanceId());

            parseLaunchingDescriptor(ldesc);

                // sanity check
            if ( this.mainClassName == null ) {
                throw new MalformedAppLaunchingDescriptorException(
                    "Missing main class name");
            }

            if ( this.codeBaseURL == null ) {
                throw new MalformedAppLaunchingDescriptorException(
                    "Missing app code base URL");
            }

            if ( this.args == null ) {
                this.args = new String[0];
            }

            this.launchingDescriptor = ldesc.toString();
            this.appClassLoader = IsamClassLoader.newInstance(this.codeBaseURL, this);
            this.appId = appID;
            this.heuristic = null;
            this.mainThread = null;
            this.isSlave = slave;
        }

    public void
    start()
        {
            if (mainThread == null && ! isSlave) {
                mainThread = new Thread(this, this, "main");
                mainThread.setDaemon(false);
                mainThread.start();
            }
        }
    
    public void
    run()
        {
            try {
                
                Class mainClass = appClassLoader.loadClass(mainClassName);
                Class[] mainArgTypes = new Class[] { String[].class };

                Method mainMethod = mainClass.getMethod("main", mainArgTypes);
                
                mainMethod.invoke(null, new Object[] { args });
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }

    public ApplicationId
    getId()
        {
            return this.appId;
        }

    public String
    getCodeBase()
        {
            return this.codeBaseURL;
        }

    public String
    getLaunchingDescriptor()
        {
            return this.launchingDescriptor;
        }
    
    public void
    setHeuristic( SchedulingHeuristic h)
        {
            if ( heuristic == null ) {
                heuristic = h;
            }
        }

    public SchedulingHeuristic
    getHeuristic()
        {
            return heuristic;
        }

    public void
    runAction( Runnable action )
        {
            try {
                Thread t = new Thread(this, action, "actionThread");
                t.start();
                t.join();
            }
            catch (InterruptedException ie)
            {
                    // empty
            }
        }

        //////////////////////////////////////////////////////
        // PRIVATE STUFF
        //////////////////////////////////////////////////////
    
    private static final String LD_ELEM_CODE = "code";
    private static final String LD_ELEM_MAIN = "main";
    private static final String LD_ATT_MAIN_CLASS = "class";
    private static final String LD_ELEM_CODEBASE = "jar";
    private static final String LD_ATT_CODEBASE_URL = "href";
    private static final String LD_ELEM_PARAMETERS = "parameters";
    private static final String LD_ELEM_PARAM = "param";
    private static final String LD_ELEM_OWNER = "owner";
    private static final String LD_ATT_OWNER_ID = "id";
    
    private final void
    parseLaunchingDescriptor( XmlElement ldesc)
        throws Exception
        {
            Enumeration docEnum = ldesc.getChildren();

            docEnum = ((XmlElement) docEnum.nextElement()).getChildren();

            
            while ( docEnum.hasMoreElements() ) {
                XmlElement e = (XmlElement) docEnum.nextElement();
                String tag = e.getTag();

                    //
                    // <code>...</code>
                    //
                if ( LD_ELEM_CODE.equals(tag) ) {
                    Enumeration codeEnum = e.getChildren();
                    while ( codeEnum.hasMoreElements() ) {
                        XmlElement ce = (XmlElement) codeEnum.nextElement();
                        if ( LD_ELEM_MAIN.equals(ce.getTag()) ) {
                            this.mainClassName = ce.getAttribute(LD_ATT_MAIN_CLASS,
                                                                 null);
                        }
                        else if ( LD_ELEM_CODEBASE.equals(ce.getTag()) ) {
                            this.codeBaseURL = ce.getAttribute(LD_ATT_CODEBASE_URL,
                                                               null);
                        }
                        else {
                            throw new MalformedAppLaunchingDescriptorException(
                                "Unexpected tag in launching descriptor:"+ ce.getTag());
                        }
                    }
                }
                    //
                    // <parameters>...</parameters>
                    //
                else if ( LD_ELEM_PARAMETERS.equals(tag) ) {
                    Enumeration paramEnum = e.getChildren();
                    Vector v = new Vector();

                    while ( paramEnum.hasMoreElements() ) {
                        XmlElement p = (XmlElement) paramEnum.nextElement();


                        if ( LD_ELEM_PARAM.equals(p.getTag()) ) {
                            v.addElement((String) p.getContents());                    
                        }
                        else {
                            throw new MalformedAppLaunchingDescriptorException(
                                "Unexpected tag in launching descriptor:"+ p.getTag());
                        }
                    }
                        // fill up args
                    this.args = new String[v.size()];
                    for ( int i=0; i<args.length; i++ ) {
                        args[i] = (String) v.elementAt(i);
                    }
                }
                    //
                    // <owner>...</owner>
                    //
                else if ( LD_ELEM_OWNER.equals(tag) ) {
                    ownerId = e.getAttribute(LD_ATT_OWNER_ID,
                                             "no-owner");
                }
                else {
//                     throw new MalformedAppLaunchingDescriptorException(
//                         "Unexpected tag in launching descriptor:"+ e.getTag());
                }
            }
        }

    private ApplicationId appId;
    private String ownerId;
    private ClassLoader appClassLoader;
    private String codeBaseURL;
    private String mainClassName;
    private String[] args;
    private SchedulingHeuristic heuristic;
    private Thread mainThread;
    private boolean isSlave;
    private String launchingDescriptor;
}

