// 
// PRIMOS - Primitives for Object Scheduling 
// 
// Copyright (C) 2002 Luciano Cavalheiro da Silva  <lucc@inf.ufrgs.br> 
//                    Claudio Fernando Resin Geyer <geyer@inf.ufrgs.br> 
// 
// This file is part of PRIMOS. 
// 
// PRIMOS is free software; you can redistribute it and/or 
// modify it under the terms of the GNU General Public License 
// as published by the Free Software Foundation; either version 2 
// of the License, or any later version. 
// 
// PRIMOS is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty of 
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details. 
// 
// You should have received a copy of the GNU General Public License 
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
// 

/**
* @file  Sensor.hh 
* @brief Abstract class Sensor
*
* @namespace primos::kernel::
* @cvsid $Id: Sensor.hh 170 2004-03-19 20:13:46Z lucc $
*/

#ifndef primos_Sensor_hh
#define primos_Sensor_hh

#ifdef HAVE_CONFIG_H
#  include <acconfig.h>
#endif

#ifdef HAVE_STRING
#  include <string>
#endif

#ifdef HAVE_IOSFWD
#  include <iosfwd>
#endif

#ifdef HAVE_SET
#  include <set>
#endif

#include "SensorParameter.hh"
#include "SensorData.hh"

namespace primos
{
        /**
         * @class Sensor 
         * @brief 
         *
         */
    class Sensor
    {
            // Operations /////////////////////////////////////
    public:
            /**
             * @brief 
             *
             * @param name
             * @param type
             */            
        Sensor( std::string name, SensorData::DataTypes type )
                : name(name),type(type)
            {}
            
            /**
             * @brief 
             *
             */
        virtual 
        ~Sensor()
            {}

            /**
             * @brief 
             *
             */
        virtual const std::string&
        Name()
            {
                return name;
            }
            
            /**
             * @brief 
             *
             */
        virtual SensorData::DataTypes
        Type()
            {
                return type;
            }

            /**
             * @brief 
             *
             */
        virtual const SensorParameterSet&
        Parameters() const = 0;

            /**
             * @brief 
             *
             */
        virtual bool
        isEnabled() const = 0;

            /**
             * @brief 
             *
             */
        virtual bool
        isStatic() const = 0;

            /**
             * @brief 
             *
             */
        virtual bool
        isNative() const = 0;

            /**
             * @brief 
             *
             */
        virtual const SensorData*
        probe() = 0;

            /**
             * @brief 
             *
             * @param enable
             */
        virtual void
        setEnabled( bool enable ) = 0;

            /**
             * @brief 
             *
             * @param name
             * @param value
             */
        virtual void
        setParameter( const std::string name, const std::string value ) = 0;

            /**
             * @brief 
             *
             * @param name
             */
        virtual std::string
        getParameter( const std::string name ) const = 0;

            /**
             * @brief 
             *
             * @param out
             */
        virtual void
        toString( std::ostream& out ) const = 0;

    protected:
            /**
             * @brief 
             *
             */
        Sensor()
                : name(""),type(SensorData::INTEGER_DATA)
            {}


            // Attributes /////////////////////////////////////
    protected:
        std::string name;
        SensorData::DataTypes type;

            // Associations ///////////////////////////////////

    };// Sensor

    typedef std::set<Sensor*> SensorSet;
    
}//namespace primos::kernel



inline std::ostream&
operator <<( std::ostream& out, primos::Sensor& s)
{
    s.toString(out);
    return out;
}


#endif
