// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos.objseed;

import org.isam.exehda.services.ObjectSeed.Activator;


class DefaultActivatorFactory
    implements ActivatorFactory
{
    
        /**
         * Detects a suitable Activator for the given Object, selecting it
         * based on the object type if no custom activator is provided by the caller.
         *
         * <p>Currently, the activator automatic detection knows about objects of type
         * <code>java.lang.Thread</code> and <code>java.rmi.Remote</code>.
         *
         * @param obj an <code>Object</code> value
         * @param customActivator an <code>Activator</code> value
         * @return an <code>Activator</code> value
         */
    public Activator
    createActivator( Object obj, Activator customActivator )
        {
            if ( customActivator == null ) {
                if ( obj instanceof Thread ) {
                    customActivator = new ThreadActivator();
                }
                else if ( obj instanceof java.rmi.Remote ) {
                    customActivator = new RMIActivator();
                }
            }

            return customActivator;
        }
}
