// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

//
// $Id: TipsNodeScheduler.java 841 2005-05-04 13:18:23Z lucc $
//

package org.isam.exehda.services.tips;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ObjectId;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.ResourceConstraint;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.Scheduler;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Collector;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.Logger.Sev;



import java.lang.reflect.*;
import java.util.*;


public class TipsNodeScheduler
    extends AbstractService
    implements Scheduler
{
    final static String ATT_SCHEDULER_CA = "contactAddress";    

    private Scheduler tipsCell = null;
    private TipsHeuristic tipsHeuristic;
    private TipsMonitor monitor;

    private DataManager dataM;
    private Learner l;
    private BayesNetwork bn;
    private UtilityGen ug;
    
    
        /**
         * Describe <code>chooseCreationHost</code> method here.
         *
         * @param c a <code>ResourceConstraints</code> value
         * @return a <code>HostId</code> value
         */
    public HostId chooseCreationHost( ResourceConstraint c ) 
        {
            return getCellScheduler().chooseCreationHost(c);
        }
    

        /**
         * Describe <code>chooseMigrationHost</code> method here.
         *
         * @param c a <code>ResourceConstraints</code> value
         * @return a <code>HostId</code> value
         */
    public HostId chooseMigrationHost( ResourceConstraint c )
        {
            return getCellScheduler().chooseMigrationHost(c);
        }


        /**
         * Describe <code>reschedule</code> method here.
         *
         * @param o an <code>ObjectHandle</code> value
         * @param c a <code>ResourceConstraints</code> value
         * @param auto a <code>boolean</code> value
         */
    public void reschedule( ObjectId o, ResourceConstraint c, boolean auto )
        {
                //repassar o retorno do TipsCellScheduler
            throw new NullPointerException("Not yet implemented.");
        }


    public void stop() 
        {}

    public void start()
        throws Exception
        {
            try {
                // Starting the data manager
            Class dataMClass =  Class.forName ("org.isam.exehda.services.tips.DataManagerImpl");
            DataManager dataM = (DataManager) dataMClass.newInstance();
            
                // Starting the utility generator
            Class ugClass = Class.forName(
                Exehda.getServiceProperty(SERVICE_NAME,
                                          "utilitygen",
                                          "org.isam.exehda.services.tips.UtilityGenImpl2"));
            
            Constructor ugConst = ugClass.getDeclaredConstructor(new Class[] {DataManager.class} );
            ug = (UtilityGen) ugConst.newInstance(new Object[] {dataM});            

                // Starting the BayesNetwork
            Class bnClass = Class.forName(
                Exehda.getServiceProperty(SERVICE_NAME,
                                          "bayesnetwork",
                                          "org.isam.exehda.services.tips.BayesNetworkImpl"));
            System.out.println("Loading " + (bnClass.getInterfaces()[0]).toString());
            
            Constructor bnConst = bnClass.getDeclaredConstructor(
                new Class[] {DataManager.class, UtilityGen.class} );
            bn = (BayesNetwork) bnConst.newInstance(new Object[] {dataM,ug});

                // Starting the Learner
            Class lClass = Class.forName(
                Exehda.getServiceProperty(SERVICE_NAME,
                                          "learner",
                                          "org.isam.exehda.services.tips.LearnerImpl"));
            Constructor lConst = lClass.getDeclaredConstructor(
                new Class[] { DataManager.class, BayesNetwork.class} );
            l = (Learner) lConst.newInstance(new Object[] {dataM,bn});


            log(Sev.DEBUG, "start()", "before new TipsMonitor()");
            
            monitor = new TipsMonitor(dataM);

            log(Sev.DEBUG, "start()", "after new TipsMonitor()");
            
            Collector collector = (Collector) Exehda.getService(Collector.SERVICE_NAME);
            collector.addMonitor(monitor);  
            
            TipsHeuristic tipsHeuristic;
            tipsHeuristic = new TipsHeuristic(dataM,ug,bn,l);
            }
            catch (Exception e) {
                e.printStackTrace();
                throw e;
            }
            
        }

    private final synchronized Scheduler
    getCellScheduler()
        {
            if ( this.tipsCell == null ) {
                    // get contact address from CIB
                CellInformationBase cib = (CellInformationBase)
                    Exehda.getService(CellInformationBase.SERVICE_NAME);

                ResourceName rnCellScheduler = new ResourceName(
                    "service", Scheduler.SERVICE_NAME,
                    HostId.getLocalHost().getCell().toNameSpace());
            
                String ca = cib.getAttribute( rnCellScheduler, ATT_SCHEDULER_CA);

                Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            
                this.tipsCell = (Scheduler) worb.lookupService(ca, Scheduler.class);
            }

            return tipsCell;
        }
    
}
