
package exehda.mw;

import java.util.Random;
import java.net.InetAddress;
import org.isam.exehda.services.Worb;
import or.isam.exehda.Exehda;

/**
 * 
 * 
 */
public class JobWorker
    extends Thread
{
    private static      long id = 0;
    private JobMaster   jobMaster;
    private JobCriteria criteria;

        /** 
         * Constructor
         * 
         * @param masterContactAddress contact address for the master object
         * @param jobCriteria criteria to be used in job size dynamic adaptation
         * 
         * @return 
         */
    public 
    JobWorker(String masterContactAddress, String jobCriteria)
        {
           
	    Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);

	    

	    this.jobMaster = (JobMaster)
                worb.lookupService(masterContactAddress, JobMaster.class);

            this.criteria = (TimeGoalCriteria) Class.forName(jobCriteria).newInstance();

            try {
                InetAddress localhost = InetAddress.getLocalHost();
                this.setName(localhost.toString() + "#" + id++);
            }
            catch (java.net.UnknownHostException uhe){
                this.setName("Unknown Host ("+System.getProperty("os.name")+")" + "#" + id++);
            }
        }

    public static JobWorker
    getCurrentWorker()
        {
            return (JobWorker) Thread.currentThread();
        }
    
        /** 
         * Worker entry point
         * 
         */
    public void
    run()
        {
            System.out.println("Worker <"+this.getName()+"> running...");
            try {
                Object jobParams = null;
                long elapsed_cpu = 0;
                long elapsed_init= 0;
                long elapsed_net = 0;
            
                while (true) {
                
                        /*
                         * Get a Job from the server
                         */
                    elapsed_net = -System.currentTimeMillis();
                    Job job = jobMaster.getJob(jobParams);
                    elapsed_net += System.currentTimeMillis();
                
                    if (job == null) {
                        System.out.println("Worker: Master.getJob() returned null");
                        break;
                    }

                        /*
                         * Let the job to set up the environtment without affecting
                         * the mesuared computing time.
                         */
                    elapsed_init = -System.currentTimeMillis();
                    job.init();
                    elapsed_init += System.currentTimeMillis();
                
                        /*
                         * While there are pending iterations, executes the Job
                         */
                    while ( job.hasNext() ) {
                        elapsed_cpu = -System.currentTimeMillis();
                        job.run();
                        elapsed_cpu += System.currentTimeMillis();
                        job.next();
                    }
                    job.setElapsedTime(elapsed_cpu);
                    job.setWorkerName(this.getName());                
                
                        /*
                         * Publish the results;
                         */
                    elapsed_net -= System.currentTimeMillis();
                    jobMaster.publishResult(job);
                    elapsed_net += System.currentTimeMillis();

                        /*
                         * Guess the best job size for the next job
                         */
                    System.out.println("Elapsed cpu =" + elapsed_cpu);
                    
                    jobParams = criteria.guessNewJobSize(job, elapsed_cpu, job.getTimeGoal());
                }
                
            }
            catch( java.rmi.ConnectException ce ) {
                System.out.println(ce);
                ce.printStackTrace();
            }
            catch(Exception e) {
            
                e.printStackTrace();
            }
            System.out.println("Slave done.");
        }
}
