// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda;

import java.util.Properties;
import java.util.Enumeration;
import org.isam.perv.IsamClassLoader;
import org.isam.util.xml.*;

public class Exehda
{
    private static String CFG_URL_PROP = "config.url";
    private static ServiceManager serviceManager;
    private static ProfileManager profileManager;
    private static Properties exehdaProps;
    private static boolean    exehdaIsUp = false;
    private static String     bootProfile = null;

    static {
            // enable isam protocol handlers
        System.getProperties().put("java.protocol.handler.pkgs",
                                   "org.isam.perv.protocols");
            // init middleware props
        exehdaProps = new Properties();
        try {            
                // load middleware profiles
            String customURL = System.getProperty("exehda."+CFG_URL_PROP);
            java.net.URL configURL = null;
            if ( customURL == null) {
                
                ClassLoader cl = IsamClassLoader.getInstance();
                configURL = cl.getResource(
                    "org/isam/exehda/exehda-services.xml");
            }
            else {
                configURL = new java.net.URL(customURL);
            }
            
            Exehda.setProperty(CFG_URL_PROP, configURL.toString());

            XmlElement docroot = XmlElement.parse(
                new java.io.InputStreamReader(configURL.openStream()));

            XmlElement configDoc = (XmlElement)
                XmlElement.selectByPath(docroot, "/EXEHDA", null).elementAt(0);

            if ( ! "EXEHDA".equals(configDoc.getTag()) ) {
                throw new java.io.IOException("exehda-services.xml corrupted!");
            }
            
            profileManager = new ProfileManager();

            Enumeration profileEnum = configDoc.getChildren();
            while ( profileEnum.hasMoreElements() ) {
                profileManager.addProfileData((XmlElement)profileEnum.nextElement());
            }

                // init services manager
            serviceManager = new ServiceManager(profileManager);

                // import default middleware props from system
            Enumeration penum = System.getProperties().propertyNames();
            while ( penum.hasMoreElements() ) {
                String propName = (String) penum.nextElement();
                if ( propName.startsWith("exehda.")) {
                    exehdaProps.setProperty(
                        propName.substring(7),
                        System.getProperty(propName));
                }

                    //
                    // TO DO: remove exehda.* from System properties
                    // 
            }
        }
        catch (Exception e) {
            System.err.println("EXEHDA initialization failed.");
            e.printStackTrace();
            System.exit(-1);
        }
    }

    public static void
    bootstrap( String profileName )
        {
//             do {
                shutdown();
            
                System.out.println("Bootstraping EXEHDA services [profile:"
                                   + profileName
                                   +"]...");
            
                if ( serviceManager.bootstrapServices(profileName) ) {
                    System.out.println("EXEHDA is ready.");
                    exehdaIsUp = true;
                }
                else {
                    System.out.println("EXEHDA initialization failed! Exiting...");
                    System.exit(-1);
                }
//             }
//             while ( bootProfile != profileName );
        }

    public static void
    shutdown()
        {
            if ( exehdaIsUp ) {
                System.out.println("Shuting down EXEHDA services...");
                if ( serviceManager.shutdownServices() ) {
                        // force flushing of unused objects
                    System.gc();
                    System.out.println("...shutdown completed.");
                    exehdaIsUp = false;
                }
                else {
                    System.out.println("EXEHDA Shutdown failed! Aborting...");
                    System.exit(-1);
                }
            }
        }

    public static Object
    getService( String serviceName )
        {
            serviceManager.startService(serviceName);
            return serviceManager.getServiceProxy(serviceName);
        }

    public static String
    getProperty( String propName, String defaultValue )
        {
            String r = profileManager.getProperty(propName);

            if ( r != null ) {
                return r;
            }
            else {
                r = exehdaProps.getProperty(propName);
                return r == null ? defaultValue : r;
            }
        }

    public static String
    getServiceProperty( String serviceName, String propName, String defValue)
        {
            String r = profileManager.getServiceProperty(serviceName, propName);

            return r == null ? defValue : r;
        }

    public static int
    getServiceProperty( String serviceName, String propName, int defValue)
        {
            String r = profileManager.getServiceProperty(serviceName, propName);

            return r == null ? defValue : Integer.parseInt(r);
        }
    
    public static String
    getActiveProfile()
        {
            return serviceManager.getActiveProfile();
        }
    
    public static void
    setProperty( String propName, String propValue )
        {
            exehdaProps.put(propName, propValue);
        }

    public static void
    addProfile( XmlElement profileData )
        {
            profileManager.addProfileData(profileData);
        }

    public static void
    main( String[] args )
        {
            String optProfile = "node";

            try {
            for ( int idx = 0; idx < args.length; idx++ ) {
                String opt = args[idx];

                if ( "--profile".equals(opt) ) {
                    optProfile = args[++idx];
                }
            }
            }
            catch (ArrayIndexOutOfBoundsException e) {
                System.err.println("Missing parameter: " + e);
                System.exit(-1);   
            }

            bootstrap(optProfile);

            try {
//                 while (true) {
//                     switch (System.in.read()) {
//                         case -1: 
//                             optProfile.wait();
//                         case 'q':
//                             shutdown();
//                             Thread.sleep(2);
//                             System.exit(0);
//                             break;
//                     }
//                 }
                Object lock = new Object();

                synchronized (lock) {
                    lock.wait();
                }
            }
            catch (Exception e) {
                    // empty
            }
        }
}


