// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda;


import java.net.InetAddress;
import java.net.UnknownHostException;


public final class HostId
    implements java.io.Serializable
{
        /** simbolic host name if available*/
    String    hostName;
        /** unique system-wide host identifier (mandatory field) */
    int id;

    InetAddress inetAddr;

    public static HostId
    getByName( String host ) throws UnknownHostException
        {
            return new HostId(InetAddress.getByName(host));
        }

    public String
    getName()
        {
              return hostName;
        }

    public String
    toString()
        {
            return Integer.toHexString(id)+':'+hostName;
        }

    public int
    getId()
        {
            return id;
        }

    public InetAddress
    getInetAddress()
        {
            return inetAddr;
        }

    public static HostId
    getLocalHost() throws UnknownHostException
        {
            return new HostId(InetAddress.getLocalHost());
        }
    

    public boolean
    equals( Object obj ) 
        {
            if ( obj instanceof HostId ) {
                return ((HostId) obj).id == id;
            }
            else {
                return false;
            }
        }

    public int
    hashCode()
        {
            return id;
        }
    

    private
    HostId( InetAddress addr )
        {
            this.hostName = addr.getHostName();
            this.inetAddr = addr;
            byte[] b = addr.getAddress();

                // TODO: search CIB for the host id
            this.id = ((b[0] & 0xFF) << 24)
                | ((b[1] & 0xFF) << 16)
                | ((b[2] & 0xFF) << 8)
                | ( b[3] & 0xFF);
        }
}

