// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import org.isam.exehda.services.CellInformationBase.ResourceName;

public interface ResourceBroker
{
    public static final String SERVICE_NAME = "resourcebroker";


    public static abstract class ResourceConstraint
        implements java.io.Serializable
    {
        public static final int UNDEFINED = -1;
        public static final int OK = 0;
        public static final int BROKEN = 1;
        
            /**
             * Describe <code>check</code> method here.
             *
             * @param r a <code>ResourceInfo</code> value
             * @return an <code>int</code> value
             */
        public abstract int check ( ResourceInfo r );
            
            /**
             * Describe <code>select</code> method here.
             *
             * @param canditates a <code>ResourceInfo[]</code> value
             * @param undefinedIsOk a <code>boolean</code> value
             * @return a <code>ResourceInfo[]</code> value
             */
        public abstract ResourceInfo[] select ( ResourceInfo[] canditates, boolean undefinedIsOk );
    }

    public static interface ResourceInfo
    {
        
            /**
             * Describe <code>getAttribute</code> method here.
             *
             * @param attrName a <code>String</code> value
             * @return a <code>String</code> value
             */
        public String getAttribute( String attrName );
        
            /**
             * Describe <code>getIndex</code> method here.
             *
             * @param idx a <code>DynamicIndex</code> value
             * @return an <code>Object</code> value
             */
        public Object getIndex( DynamicIndex idx );
    }

    public static abstract class DynamicIndex
    {
            /**
             * Describe <code>getName</code> method here.
             *
             * @return a <code>String</code> value
             */
        public abstract String getName();
        
            /**
             * Describe <code>getDef</code> method here.
             *
             * @return a <code>String</code> value
             */
        public abstract String getDef();
    }

    public static abstract class CompositeResourceConstraint
        extends ResourceConstraint
    {
//         public void addConstraint ( ResourceConstraint c,  undefinedIsOk ){
//         }

//         public Enumeration constraints (  ){
//         }
    }
    
    public static abstract class StaticConstraint
        extends ResourceConstraint {}

    public static abstract class AttributeInRangeConstraint
        extends StaticConstraint {
//         public AttributeInRangeConstraint ( String attName, Number lb, Number up ){
//         }
    }

    public static abstract class PreferredSetConstraint
        extends StaticConstraint {
//         public PreferredSetConstraint ( ResourceName[] resources ){
//         }
    }

    public static abstract class ExclusiveAccessConstraint
        extends StaticConstraint {
//         public ExclusiveAccessConstraint ( long leaseTimeMinutes ){
//         }
    }

    public static abstract class AvoidedSetConstraint
        extends StaticConstraint {
//         public AvoidedSetConstraint ( ResourceName[] resources ){
//         }
    }

    public static abstract class AttributeMatchConstraint
        extends StaticConstraint
    {
//         public AttributeMatchConstraint ( String attName, String regexp ){
//         }
    }
    
    public static abstract class DynamicConstraint
        extends ResourceConstraint {}
    
        /**
         * Describe <code>findResource</code> method here.
         *
         * @param query a <code>ResourceConstraint[]</code> value
         * @param scope an <code>int</code> value
         * @param num an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] findResource( ResourceConstraint[] query, int scope, int num );
    
        /**
         * Describe <code>allocResource</code> method here.
         *
         * @param req a <code>ResourceConstraint[]</code> value
         * @param scope an <code>int</code> value
         * @param num an <code>int</code> value
         * @return a <code>ResourceName[]</code> value
         */
    public ResourceName[] allocResource( ResourceConstraint[] req, int scope, int num );
}

