// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services;

import org.isam.exehda.HostId;
import org.isam.exehda.ObjectId;
import org.isam.exehda.services.ResourceBroker.ResourceConstraint;
import org.isam.exehda.services.ResourceBroker.DynamicConstraint;
import org.isam.exehda.services.ResourceBroker.DynamicIndex;


public interface Scheduler
{
    public static final String SERVICE_NAME = "scheduler";

    public static abstract class SchedConstraint
        extends DynamicConstraint {}

    public static abstract class SchedIndex
        extends DynamicIndex {
//         public static ScheIndex createIndex ( String idx, String def ){
//         }
    }

    public static abstract class CompositeIndex
        extends SchedIndex {}

    public static abstract class SensorIndex
        extends SchedIndex {}

    public static abstract class MappedIndex
        extends SchedIndex {}

    public static abstract class DistanceToAnchorIndex
        extends SchedIndex {}

    public static abstract class RelationalConstraint
        extends SchedConstraint {
//         public RelationaConstraint ( SchedIndex idx ){
//         }
//         public NodeInfo[] selectNodes ( NodeInfo[] candidates ){
//         }
//         abstract protected int compare ( NodeInfo n1, NodeInfo n2 );
    }

    public static abstract class MinimizeIndexConstraint
        extends RelationalConstraint {   
//         public MinimizeIndexConstraint ( SchedIndex idx ){
//         }
//         public int compare ( NodeInfo n1, NodeInfo n2 ){
//         }
    }

    public static abstract class MaximizeIndexConstraint
        extends RelationalConstraint {
//         public MaximizeIndexConstraint ( ScheIndex idx ){
//         }
//         protected int compare ( NodeInfo n1, NodeInfo n2 ){
//         }
    }

    public static abstract class IndexValueInSetConstraint
        extends SchedConstraint {
//         public IndexValueInSet ( SchedIndex idx, Object[] values ){
//         }
    }

    public static abstract class IndexValueInRangeConstraint
        extends SchedConstraint {
//         public IndexInRangeConstraint ( SchedIndex idx, Number lb, Number ub ){
//         }
    }

    public static abstract class IndexEqualsConstraint
        extends SchedConstraint {
//         public IndexEqualsConstraint ( SchedIndex idx, Object val ){
//         }
    }

    public static abstract class MobilityConstraint
        extends SchedConstraint {}
    
        /**
         * Describe <code>chooseCreationHost</code> method here.
         *
         * @param c a <code>ResourceConstraints</code> value
         * @return a <code>HostId</code> value
         */
    public HostId chooseCreationHost( ResourceConstraint c );

        /**
         * Describe <code>chooseMigrationHost</code> method here.
         *
         * @param c a <code>ResourceConstraints</code> value
         * @return a <code>HostId</code> value
         */
    public HostId chooseMigrationHost( ResourceConstraint c );

        /**
         * Describe <code>reschedule</code> method here.
         *
         * @param o an <code>ObjectHandle</code> value
         * @param c a <code>ResourceConstraints</code> value
         * @param auto a <code>boolean</code> value
         */
    public void reschedule( ObjectId o, ResourceConstraint c, boolean auto );
}

