// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.bda;

import java.io.*;
import org.isam.exehda.Exehda;
import org.isam.exehda.services.HttpService;
import org.isam.exehda.services.Logger.Sev;


/**
 * This class implements ISAM BDA service, the pervasive code repository of the ISAM platform.
 *
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2004-04-29 11:20:47 -0300 (Thu, 29 Apr 2004) $ $Revision: 190 $
 */
public class BdaImpl
    extends HttpService
{
    static final String SERVICE_NAME = "bda";
    static final String DEFAULT_DOCROOT = "./";
    static final int    DEFAULT_PORT = 1978;
    
    static final String PROP_BDA_OPERATION = "Isam-Bda-Operation";
    static final String PROP_BDA_RESOURCE  = "Resource";
    static final String PROP_BDA_USER      = "Isam-User";

    static final int    BDA_OP_READ        = 0;
    static final int    BDA_OP_WRITE       = 1;


    final private String docRoot;

    public
    BdaImpl()
        {
            super(Exehda.getServiceProperty(SERVICE_NAME, "port", DEFAULT_PORT));
            
            docRoot = Exehda.getServiceProperty(SERVICE_NAME,
                                                "docroot",
                                                DEFAULT_DOCROOT);
        }

    public void
    start() 
        {
            super.start();
            System.getProperties().put("isam.bdaHost","localhost");
        }

    protected void
    handleRequest( ServiceRequest req )
        throws IOException
        {
            switch ( getBdaOperation(req) ) {
                case BDA_OP_READ:
                    handleBdaRead(req);
                    break;
                case BDA_OP_WRITE:
                    handleBdaWrite(req);
                    break;
                default:
                    log(Sev.WARNING, "handleRequest", "Unknown BDA operation.", null);
            }
                // op read
                // check if is a valid file name( do not start with .. for example)
                // open local file
                // output HTTP response header
                // output file contents
        }

    protected boolean
    checkAccess( String fileName, String userName, int op )
        {
                // TODO: check if its a valid filename
            
                // grant read access to all users
            if ( op == BDA_OP_READ ) {
                return true;
            }
            else {
                return false;
            }
        }

    protected int
    getBdaOperation( ServiceRequest req )
        {
                // defaults to 'read'
            int op = BDA_OP_READ;
            String opString = req.getProperty(PROP_BDA_OPERATION);
            if ( "write".equals(opString) ) {
                op = BDA_OP_WRITE;
            }

            return op;
        }

    protected void
    handleBdaRead( ServiceRequest req )
        throws IOException
        {
            String fileName = req.getProperty(PROP_BDA_RESOURCE);
            String userName = req.getProperty(PROP_BDA_USER);

            if ( checkAccess(fileName, userName, BDA_OP_READ) ) {
                File file = new File(docRoot+fileName);
                
                ServiceResponse resp =
                    new ServiceResponse(req,
                                        ServiceResponse.OK);
                resp.putHeader("Content-Length", ""+file.length());

                byte[] buff = new byte[4096];
                InputStream ins = new FileInputStream(file);

                int nr;

                while ( (nr = ins.read(buff)) != -1 ) {
                    resp.putData(buff, 0, nr);
                }
                
                resp.close();
            }
            else {
                ServiceResponse resp =
                    new ServiceResponse( req,
                                         ServiceResponse.BAD_REQUEST);
                resp.close();
            }
        }

    protected void
    handleBdaWrite( ServiceRequest req )
        {
        }
}
    
