// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.CellInformationBase.NameComponent;
import org.isam.exehda.services.CellInformationBase.NameSpace;
import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.services.Worb;


public class CibImpl
    extends AbstractService
    implements CellInformationBase
{
    public CibImpl()
        {
            backEnd = new LdapBackEnd();//HashtableBackEnd();
        }
    
    public void
    start()
        {
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            worb.exportService(this,
                               CellInformationBase.class,
                               CellInformationBase.SERVICE_NAME);
        }

    public void
    stop()
        {
                //
                // TODO: unexport service
                //
        }

    public ResourceName[] select(SelectionConstraint sc, int maxResults)
        {
            log(Sev.DEBUG, "select", "selecting resources that match constraints={0}", sc);
            return backEnd.select(sc,maxResults);
        }
    
    
    public ResourceName[] selectByName(String n, NameSpace ns, int maxResults)
        {
            log(Sev.DEBUG, "selectByName", "selecting resources with name={0} in name space={1}", n, ns);
            return backEnd.selectByName(n,ns,maxResults);
        }
    
    public ResourceName[] selectByType(String t, NameSpace ns, int maxResults)
        {
            log(Sev.DEBUG, "selectByType", "selecting resource of type={0} in name space={1}", t, ns);
            return backEnd.selectByType(t,ns,maxResults);
        }
    
    public String getAttribute(ResourceName r, String a)
        {
            log(Sev.DEBUG, "getAttribute", "attr={0}, resource is {1}", a, r);
            return backEnd.getAttribute(r,a);
        }
    
    public String setAttribute(ResourceName r, String a, String v)
        {
            log(Sev.DEBUG, "setAttribute", "attr={0}, resource is {1}", a, r);
            return backEnd.setAttribute(r,a,v);
        }
    
    public String[] getAttributes(ResourceName r)
        {
            return backEnd.getAttributes(r);
        }
    
    public void addResource(ResourceName r)
        {
            log(Sev.DEBUG, "addResource", "Adding resource {0}", r);
            backEnd.addResource(r);
        }
    
    public void removeResource(ResourceName r)
        {
            backEnd.removeResource(r);
        }

    
    private CellInformationBase backEnd;

        ////////////////////////// Old methods ////////////////////////////////
//     public void
//     addHost(HostId h, String contactAddr)
//         {
//             log( Sev.DEBUG, "addHost", "entering h={0}, ca={1}", h, contactAddr);
//             String key= h.toString() + "#contactAddress";

//             attrs.put(h.toString(), "online");
//             attrs.put(key, contactAddr);
//         }
    
//     public void
//     removeHost(HostId h)
//         {
//             attrs.put(h.toString(), "online");
//         }
    
//     public String
//     getHostAttribute(HostId h, String attrName)
//         {
//             log( Sev.DEBUG, "getHostAttribute", "entering h={0}, ca={1}", h, attrName);
//             String key= h.toString() + "#"+ attrName;
            
//             return (String) attrs.get(key);
//         }
    
//     public void
//     setHostAttribute(HostId h, String attrName, String attrValue)
//         {
//             String key= h.toString() + "#"+ attrName;
            
//             attrs.put(key, attrValue);
//         }

//     public void
//     addApplication(ApplicationId app, String codeBase)
//         {
//             log( Sev.DEBUG, "addApplication", "entering app={0}, codebase={1}", app, codeBase);
//             String key = "APP::"+app+"#"+"codeBase";
            
//             attrs.put(key, codeBase);
//         }
    
//     public String
//     getAppAttribute(ApplicationId app, String attrName)
//         {
//             log( Sev.DEBUG, "getAppAttribute", "entering app={0}, attrName={1}", app, attrName);
//             String key = "APP::"+app+"#"+attrName;

//             return (String) attrs.get(key);
//         }

//     private Hashtable attrs;
}
