// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cib;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.CellInformationBase.NameComponent;
import org.isam.exehda.services.CellInformationBase.NameSpace;
import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.services.CellInformationBase.SelectionConstraint;
import org.isam.exehda.services.Worb;
import java.util.Hashtable;


public class HashtableBackEnd
    extends Hashtable
    implements CellInformationBase
{
    public HashtableBackEnd()
        {
            super();
        }


    public ResourceName[] select(SelectionConstraint sc, int maxResults)
        {
                // TODO: generic select
            throw new NullPointerException("Not yet implemented");
        }
        
    public ResourceName[] selectByName(String n, NameSpace ns, int maxResults)
        {
            if ( ns != null ) {
                throw new NullPointerException("Not yet implemented");
            }

            java.util.Vector results = new java.util.Vector();
            java.util.Enumeration kenum = keys();

            while (kenum.hasMoreElements() && (maxResults != 0) ) {
                ResourceName rn = (ResourceName) kenum.nextElement();

                if ( rn.getSimpleName().equals(n) && isInNameSpace(rn,ns) ) {
                    results.add(rn);

                    if ( maxResults > 0 ) --maxResults;
                }
            }

            ResourceName[] tmp = new ResourceName[results.size()];
            results.copyInto(tmp);
            return tmp;
        }
    
    public ResourceName[] selectByType(String t, NameSpace ns, int maxResults)
        {
            if ( ns != null ) {
                throw new NullPointerException("Not yet implemented");
            }

            java.util.Vector results = new java.util.Vector();
            java.util.Enumeration kenum = keys();

            while (kenum.hasMoreElements() && (maxResults != 0) ) {
                ResourceName rn = (ResourceName) kenum.nextElement();

                if ( rn.getCanonicalName().getType().equals(t) && isInNameSpace(rn,ns) ) {
                    results.add(rn);
                    if ( maxResults > 0 ) --maxResults;
                }
            }

            ResourceName[] tmp = new ResourceName[results.size()];
            results.copyInto(tmp);
            return tmp;
        }
    
    public String getAttribute(ResourceName r, String a)
        {
            ResourceEntry entry = findEntry(r);

            if ( entry != null ) {
                return entry.getAttribute(a);
            }
            else {
                return null;
            }
        }
    
    public String setAttribute(ResourceName r, String a, String v)
        {
            ResourceEntry entry = findEntry(r);

            if ( entry != null ) {
                return entry.setAttribute(a,v);
            }
            else {
                return null;
            }
        }
    
    public String[] getAttributes(ResourceName r)
        {
            ResourceEntry entry = findEntry(r);

            if ( entry != null ) {
                return entry.getAttributes();
            }
            else {
                return null;
            }
        }
    
    public void addResource(ResourceName r)
        {            
            ResourceEntry entry = findEntry(r);

            if ( entry == null ) {
                put(r, new ResourceEntry());
            }
        }
    
    public void removeResource(ResourceName r)
        {
            remove(r);
        }

    private final ResourceEntry findEntry(ResourceName r)
        {
            return (ResourceEntry) get(r);
        }

    private final boolean isInNameSpace(ResourceName r, NameSpace ns)
        {
            if ( ns == null ) {
                return true;
            }
            else {
                NameSpace rns = r.getNameSpace();

                if ( ns.equals(rns) || ns.contains(rns) ) return true;
            }
            return false;
        }

    private class ResourceEntry
        extends Hashtable
    {
        public ResourceEntry()
            {
                super(7);
            }

        public String
        getAttribute(String a)
            {
                return (String) get(a);
            }

        public String
        setAttribute(String a, String v)
            {
                return (String) put(a,v);
            }

        public String[]
        getAttributes()
            {
                String[] attrs = new String[size()];
                java.util.Enumeration kenum = keys();

                for (int i=0; kenum.hasMoreElements(); i++) {
                    attrs[i] = (String) kenum.nextElement();
                }

                return attrs;
            }
    }
}
