/*
 * Ldap.java
 *
 * Created on September 3, 2004, 3:28 PM
 */

package org.isam.exehda.services.cib;

import java.util.*;
import javax.naming.*;
import javax.naming.directory.*;
import javax.naming.ldap.*;




/**
 *
 * @author  frainer
 */
public class Ldap {
    
    
    String serverAddress;
    String user;
    String password;
    DirContext dir;
    
    /** Creates a new instance of Ldap */
    public Ldap(String serverAddress, String user, String password) {
        this.serverAddress = serverAddress;
        this.password = password;
        this.user = user;
    }
    
    /**
    * sets up the connection to the ldap server, using the server address, user
    * and password defined in the constructor 
    *      
    * @return void 
    */
    
    public void connect()  {
        // Set up the environment for creating the initial context
        try{
            Hashtable env = new Hashtable();
            System.out.println(password);
            env.put( Context.INITIAL_CONTEXT_FACTORY,
                         "com.sun.jndi.ldap.LdapCtxFactory" );
            env.put( Context.PROVIDER_URL, "ldap://localhost/dc=CELL");//+ serverAddress + ":389/"+"dc=CELL" );
            env.put( Context.SECURITY_PRINCIPAL, user);
            env.put( Context.SECURITY_CREDENTIALS, password);

            // Create the initial context
            dir = new InitialDirContext(env);
        }catch(javax.naming.NamingException e){e.printStackTrace();}
    }
    
    
    /**
    * creates a new resource in the ldap server, if it doesn't exist already
    *
    * @param  node  the string indicating the name and where (in ldap format) the resource is to be created
    *         
    * @return  void    
    */
    public void createIfNotExists(String node){
        try{
                 Attributes myAttrs = new BasicAttributes(true);
                 //um atributo qualquer
                 myAttrs.put("objectclass","isamClass");
                 if(node.indexOf(",")==-1){
                    myAttrs.put("dn",node);
                 }else{
                    myAttrs.put("dn",node.substring(0,node.indexOf(",")));
                 }
                 myAttrs.put("fullDn",node);
                 dir.createSubcontext(node,myAttrs);
          
         } catch ( NameNotFoundException nnfe ) {
                        createIfNotExists(node.substring(node.indexOf(",")+1));
                        createIfNotExists(node);
                        //eat crow
                        //System.err.println( "value has already been bound!" );        
                 
         } catch ( NameAlreadyBoundException nabe ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        //System.err.println( e );
                        e.printStackTrace();
         }         
    }
    
    
    /**
    * retrieves from the ldap server the value of a given attribute
    * in a given resource
    *
    * @param  node  the name and place (in ldap format) of the resource that contains the attribute
    *         attr the name of the attribute
    *         
    * @return  the current value of the given attribute in the given resource    
    */
    public String getAttribute(String node, String attr){
        try{
                 Attributes result = dir.getAttributes(node,new String[]{attr});
                 return((String)result.get(attr).get());
                 
         } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        System.err.println( e );
         }  
        return(null);
    }
    
    
    /**
    * retrieves from the lda server all the names of all the attributes
    * of a given resource 
    *
    * @param  node  the name and place (in ldap format) of the resource
    *         
    * @return  an array of String with the names of the attributes in the given resource    
    */
    public String[] getAttributes(String node){
        try{
                 Attributes attributes = dir.getAttributes(node);
                 String[] results = new String[attributes.size()];
                 NamingEnumeration all = attributes.getAll();
                 for(int i=0;all.hasMore();i++){
                     results[i] = (String)(((Attribute)all.next()).get());
                 }
                 return(results);
                 
         } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        System.err.println( e );
         }  
        return(new String[0]);
    }
    
    
    /**
    * removes a given resource from the ldap server
    *
    * @param  node the name and place (in ldap format) of the resource that is to be removed
    *         
    * @return  void    
    */
     public void remove(String node){
        try{
                 dir.destroySubcontext(node);
         } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        System.err.println( e );
         }  
    }
     
     
     /**
    * sets the value a given attribute in a given resorce in tha ldap server
    *
    * @param  node the name and place (in ldap format) of the resource that contains the attribute
    *         attr the name of the attribute
    *         value the value that the attribute is to be set to   
    *        
    * @return  void    
    */
    
     public void setAttribute(String node, String attr, String value){
         try{
            Attributes myAttrs = new BasicAttributes(true);
            myAttrs.put(attr,value);
            dir.modifyAttributes(node,DirContext.REPLACE_ATTRIBUTE,myAttrs);
         } catch ( NameAlreadyBoundException nabe ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
         } catch ( Exception e ) {
                        //System.err.println( e );
                        e.printStackTrace();
         }
     }
     
    /**
    * finds the resources in the ldap server that have a given name and that are
    * inside a given namespace
    *
    * @param  name the name that the returned resources must have
    *         nameSpace the name space inside of which the returned resources must reside. (in ldap format)  
    *         
    * @return  a list of resources that have the given name and are inside the given name space
    */ 
     
    public List findByName(String name, String nameSpace){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search(nameSpace, 
    "(dn=*="+name+"*)",      // Filter expression                // Filter arguments
    ctls);	
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes().get("fullDn").get());
            }
            return(result);
        } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        //System.err.println( e );
                        e.printStackTrace();
        }    
        return(new ArrayList());
    }
    
    
    /**
    * finds the resources in the ldap server that have a given type and that are
    * inside a given namespace
    *
    * @param  type the type that the returned resources must have
    *         nameSpace the name space inside of which the returned resources must reside. (in ldap format)  
    *         
    * @return  a list of resources that have the given type and are inside the given name space    
    */ 
    public List findByType(String type, String nameSpace){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search(nameSpace, 
    "(dn="+type+"=*)",      // Filter expression                // Filter arguments
    ctls);				    // Default search controls
            
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes().get("fullDn").get());
            }
            return(result);
        } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        System.err.println( e );
        }    
        return(new ArrayList());
    }
    
    /**
    * finds resources in the ldap server using the given ldap search string
    *
    * @param  searchString a ldap search string
    *         
    * @return  a list of resources that were found using the given ldap search string
    */ 
    public List findBySearchString(String searchString){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search("", 
    searchString,      // Filter expression                // Filter arguments
    ctls);				    // Default search controls
            
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes().get("fullDn").get());
            }
            return(result);
        } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        System.err.println( e );
        }    
        return(new ArrayList());
    }
    
    
    /**
    * finds resources in the ldap server using the given ldap search string
    *
    * @param  searchString a ldap search string
    *         
    * @return  a list of all the attributes of the resources that were found using the given ldap search string
    */ 
    public List findBySearchStringAndReturnAllAttributes(String searchString){
        try{
            SearchControls ctls = new SearchControls();
            ctls.setSearchScope(SearchControls.SUBTREE_SCOPE);
            NamingEnumeration answer = dir.search("", 
    searchString,      // Filter expression                // Filter arguments
    ctls);				    // Default search controls
            
            List result = new LinkedList();
            while(answer.hasMore()){
                SearchResult sr = (SearchResult)answer.next();
                result.add(sr.getAttributes());
            }
            return(result);
        } catch ( NamingException ne ) {
                        //eat crow
                        //System.err.println( "value has already been bound!" );
        } catch ( Exception e ) {
                        System.err.println( e );
        }    
        return(new ArrayList());
    }
    
    /**
    * disconnects from the ldap server. A new connect will be necessary
    * before acessing the server again.
    *       
    * @return  void
    */ 
    
    public void disconnect(){
        try{
            dir.close();
        } catch (NamingException ne){
            ne.printStackTrace();
        }
    }
    
}
