/*
 * LdapBackEnd.java
 *
 * Created on September 3, 2004, 10:43 AM
 */

package org.isam.exehda.services.cib;

import org.isam.exehda.Exehda;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.services.CellInformationBase.NameComponent;
import org.isam.exehda.services.CellInformationBase.NameSpace;
import org.isam.exehda.services.CellInformationBase.ResourceName;
import org.isam.exehda.services.CellInformationBase.Resource;
import org.isam.exehda.services.CellInformationBase.SelectionConstraint;
import org.isam.exehda.services.Worb;
import org.isam.exehda.services.cib.*;
import java.util.*;
import javax.naming.directory.*;

/**
 *
 * @author  frainer
 */
public class LdapBackEnd implements CellInformationBase{
    
    Ldap ldap;
    Hashtable typeConversion;
    Hashtable attributeConversion;
    
    /** Creates a new instance of LdapBackEnd */
    public LdapBackEnd() {
        super();
        ldap = new Ldap("ldap://localhost", "cn=Manager,dc=CELL", "senha");
    }
    
    /**
    * adds a new resource to the cib, if it doesn't exist already.
    *
    * @param  r  The ResourceName that is to be added
    *        
    * @return      void
    */
    public void addResource(ResourceName r) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        System.out.println("+++++++++++++++++++++"+r);
        System.out.println("+++++++++++++++++++++"+resourceName);
        ldap.createIfNotExists(resourceName);
        ldap.disconnect();    
    }
    
    
     /**
    * Returns the value of the especified attribute in the
    * especified resource
    *
    * @param  r  The resource
    *         a  The name of the attribute  
    *        
    * @return the value of the attribute     
    */
    public String getAttribute(ResourceName r, String a) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        String result = ldap.getAttribute(resourceName,a);
        ldap.disconnect(); 
        return(result);
    }
    
     /**
    * returns all the names of the attribues of a given resource 
    *
    * @param  r  The name of the resource 
    *        
    * @return      an array of string cointaining all the names of the attributes of the resource
    */
    public String[] getAttributes(ResourceName r) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        String[] result = ldap.getAttributes(resourceName);
        ldap.disconnect();
        return(result);
    }
    
     /**
    * removes a resource from the cib
    *
    * @param  r  The name of the resource that will be removed
    *        
    * @return      void
    */
    public void removeResource(ResourceName r) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        ldap.remove(resourceName);
        ldap.disconnect();
    }
    
     /**
    * Searches the cib for resources that match a certain SelectionConstraint
    *
    * @param  filter  The SelectionConstraint that the resources to be returned must match
    *         maxResults The maximum number of results that are to be returned        
    *
    * @return      An array containing the ResourceNames of the cib entries that matched the SelectionConstraint
    */
    public ResourceName[] select(SelectionConstraint filter, int maxResults) {
        String searchString = "";
        searchString = makeSearchString(filter);
        
        //System.out.println(searchString);
        
        ldap.connect();
        List resultList = ldap.findBySearchStringAndReturnAllAttributes(searchString);
        ldap.disconnect();
        
        List trueResults = new LinkedList();
        Iterator it = resultList.iterator();
        while(it.hasNext()){
            ResourceImpl resource = makeResourceFromAttributes((Attributes)it.next());
            if(filter.match(resource)){
                trueResults.add(resource);
            }
        }
        
        resultList = trueResults;
        ResourceName[] results; 
        if((resultList.size()<maxResults)||(maxResults == -1)){
            results = new ResourceName[resultList.size()];
            maxResults = resultList.size();
        }
        else{
            results = new ResourceName[maxResults];
        }
        
        it = resultList.iterator();
        for(int i=0; ((i<maxResults)&&(it.hasNext())); i++){
            results[i] = ((ResourceImpl)it.next()).getName();
        }
        return(results);
        
    }
    
     /**
    * Generates a search string that the ldap server can understand from a 
    * given SelectionConstraint 
    *
    * @param  filter  The filter to be transformed into a string
    *        
    * @return      a search string that can be understood by a ldap server
    */
    private String makeSearchString(SelectionConstraint filter){
        String result = "";
        if(filter instanceof SimpleRegexpConstraint){
            SimpleRegexpConstraint simple = (SimpleRegexpConstraint)filter;
            String regexp = convertRegexpToLdap(simple.getRegexp());
            result = "("+simple.getAttribute()+"="+regexp+")";
        }
        else if(filter instanceof AttributeDefinedConstraint){
            AttributeDefinedConstraint attribute = (AttributeDefinedConstraint)filter;
            result = "("+attribute.getAttribute()+"=*)";
        }
        else if(filter instanceof CompositeSelectionConstraint){
            CompositeSelectionConstraint composite = (CompositeSelectionConstraint) filter;
            result = "(";
            if(composite.getMode()==CompositeSelectionConstraint.M_AND){
                result += "&";
            }
            else if(composite.getMode()==CompositeSelectionConstraint.M_OR){
                result += "|";
            }
            else if(composite.getMode()==CompositeSelectionConstraint.M_XOR){
                result += "&";
                result += "(";
                result += "|";
                Enumeration constraints = composite.constraints();
            
                while(constraints.hasMoreElements()){
                    result += makeSearchString(((SelectionConstraint)constraints.nextElement()));
                }
                result += ")";
                
                result += "(";
                result += "!";
                result += "(";
                result += "&";
                
            }
            Enumeration constraints = composite.constraints();
            
            while(constraints.hasMoreElements()){
                result += makeSearchString(((SelectionConstraint)constraints.nextElement()));
            }
            
            result += ")";
            
            if(composite.getMode()==CompositeSelectionConstraint.M_XOR){
                result += ")"; 
                result += ")";
            }
            
        }
        return(result);
    }
    
    /**
    * Searches the cib for resources that hava a certain name and that are inside
    * a given namespace
    *
    * @param  n  a string representing the name that the returned resources must have
    *         ns Tha NameSpace to which the resource must belong in order to be returned
    *         maxResults The maximum number of results that are to be returned        
    *
    * @return      An array containing the ResourceNames of the cib entries that had the given name
    */
    public ResourceName[] selectByName(String n, NameSpace ns, int maxResults) {
        //ResourceName[] results = new ResourceName[maxResults];
        ldap.connect();
        String nameSpace = convertToLdap(ns);
        List resultList = ldap.findByName(n,nameSpace);
        ldap.disconnect();
        
        ResourceName[] results; 
       
        if((resultList.size()<maxResults)||(maxResults == -1)){
            results = new ResourceName[resultList.size()];
            maxResults = resultList.size();
        }
        else{
            results = new ResourceName[maxResults];
        }
        
        Iterator it = resultList.iterator();
        for(int i=0; ((i<maxResults)&&(it.hasNext())); i++){
           ResourceName resource = convertFromLdap((String)it.next());
           results[i] = resource;
        }
        return(results);
    }
    
    /**
    * Searches the cib for resources that hava a certain type and that are inside
    * a given namespace
    *
    * @param  t  a string representing the name of the type that the returned resources must have
    *         ns Tha NameSpace to which the resource must belong in order to be returned
    *         maxResults The maximum number of results that are to be returned        
    *
    * @return      An array containing the ResourceNames of the cib entries that had the given type
    */
    public ResourceName[] selectByType(String t, NameSpace ns, int maxResults) {
        //ResourceName[] results = new ResourceName[maxResults];
        ldap.connect();
        String nameSpace = convertToLdap(ns);
        List resultList = ldap.findByType(t,nameSpace);
        ldap.disconnect();
        
        ResourceName[] results; 
        if((resultList.size()<maxResults)||(maxResults == -1)){
            results = new ResourceName[resultList.size()];
            maxResults = resultList.size();
        }
        else{
            results = new ResourceName[maxResults];
        }
        
        Iterator it = resultList.iterator();
        for(int i=0; ((i<maxResults)&&(it.hasNext())); i++){
            ResourceName resource = convertFromLdap((String)it.next());
            results[i] = resource;
        }
        return(results);
    }
    
     /**
    * Sets the value of a given attribute in a given resource
    *
    * @param  r  the resource which "owns" the attribute that is to be set
    *         a the name of the attribute that is to be set
    *         v the value to which the attribute is to set        
    *
    * @return  a string containing tha value to which the attribute was set    
    */
    public String setAttribute(ResourceName r, String a, String v) {
        ldap.connect();
        String resourceName = convertToLdap(r);
        ldap.setAttribute(resourceName,a,v);
        ldap.disconnect();
        return(v);
    }
    
    /**
    * converts a given ResourceName into a string representation that can be
    * understood by a ldap server
    *
    * @param  r  the resource name that is to be converted
    *         
    * @return  a string representation of the given ResourceName that can be understood by a ldap server    
    */
    private String convertToLdap(ResourceName r){
        if(r==null){
            return("");
        }
        System.out.println(r.getDN());
        String type = r.getCanonicalName().getType();
        String name = r.getCanonicalName().getValue();
        String dn = "";
        if(typeConversion.containsKey(type)){
             dn = r.getDN().replaceAll(type,(String)typeConversion.get(type)).replaceAll("\\[","").replaceAll(":","=").replaceAll("\\]","");
        }
        else{
            dn = r.getDN().replaceAll(type,"isamType").replaceAll(name,type+":"+name).replaceAll("\\[","").replaceAll(":","=").replaceAll("\\]","");
        }
        dn = dn.substring(0,dn.lastIndexOf(","));
        if(convertToLdap(r.getNameSpace()) != ""){
            return(dn+","+convertToLdap(r.getNameSpace()));
        }else{
            return(dn);
        }
    }
    
    /**
    * converts a given NameSpace into a string representation that can be
    * understood by a ldap server
    *
    * @param  ns  the NameSpace that is to be converted
    *         
    * @return  a string representation of the given NameSpace that can be understood by a ldap server    
    */
    public String convertToLdap(NameSpace ns){
        if(ns==null){
            //ns = CellInformationBase.NameSpace.DEFAULT_NS;
            return("");
        }
        
        String dn = ns.toString().replaceAll("\\[","").replaceAll(":","=").replaceAll("\\]","");
        int p = 0;
        while(dn.indexOf(",",p)!=-1){
            String type = dn.substring(p,dn.indexOf("=",p)-1);
            String name = dn.substring(dn.indexOf("=",p)+1,dn.indexOf(",",p)-1);
            if(typeConversion.containsKey(type)){
                dn = dn.replaceFirst(type,(String)typeConversion.get(type));
            }
            else{
                dn = dn.replaceFirst(type,"isamType").replaceFirst(name,type+":"+name);
            }
            p = dn.indexOf(",",p)+1;
        }
        //String dn = ns.toString().replace('[',' ').replace(':','=').replace(']',' ');
        return(dn);
    }
    
    /**
    * converts a string with the name of an ldap resource, in the format supported
    * by ldap servers, into a ResourceName 
    *
    * @param  dn  A string with the name of a resource in the ldap server
    *         
    * @return  a ResourceName that is points to the same resource as the given String 
    */
    private ResourceName convertFromLdap(String dn){
        NameSpace ns = null;
        ResourceName rn = null;
        
        while(dn.indexOf(",")!=-1){
            String aux = dn.substring(dn.lastIndexOf(",")+1);
            NameSpace auxNS = new NameSpace(aux.substring(0,aux.indexOf("=")), aux.substring(aux.indexOf("=")+1),ns);
            ns = auxNS;
            dn = dn.substring(0,dn.lastIndexOf(","));
        }
        if(ns == null){
            rn = new ResourceName(dn.replace('=',':'));
        }
        else{
            rn = new ResourceName(dn.replace('=',':'),ns);
        }
        
        return(rn);
    }
    
    
    /**
    * converts a set of attributes into a resource
    *
    * @param  attributes  The attributes to be converted into a resource
    *         
    * @return  a ResourceImpl that contains all the given attributes
    */
    private ResourceImpl makeResourceFromAttributes(Attributes attributes){
        try{
            ResourceName rn = convertFromLdap((String)attributes.get("fullDN").get());
            ResourceImpl resource = new ResourceImpl(rn);
            Enumeration attrs = attributes.getAll();
            while(attrs.hasMoreElements()){
                Attribute attr = (Attribute)attrs.nextElement();
                resource.setAttribute((String)attr.getID(),(String)attr.get());
            }   
            return(resource);
        }catch(javax.naming.NamingException ne){ne.printStackTrace();}
        return(null);
    }
    
    /**
    * converts a full regular expression into something tha ldap can understand. 
    * Some of the meaning of original regular expression may be lost. 
    *
    * @param  regexp  A regular expression string
    *         
    * @return  a new String with a regular expression that ldap can understand
    */
    private String convertRegexpToLdap(String regexp){
        String ldapRegexp = "";
        ldapRegexp = regexp.replaceAll("\\.","*");
        ldapRegexp = ldapRegexp.replaceAll("\\[.*\\]", "*");
        ldapRegexp = ldapRegexp.replaceAll("\\?","*");
        ldapRegexp = ldapRegexp.replaceAll("\\+","*");
        ldapRegexp = ldapRegexp.replaceAll("\\{.*\\}","*");
        ldapRegexp = ldapRegexp.replaceAll("\\(.*\\)","*");
        ldapRegexp = ldapRegexp.replace('\\','*');
        System.out.println(ldapRegexp);
        return(ldapRegexp);
    }
    
}
