package org.isam.exehda.services.dhc.client;

import org.isam.exehda.services.Service;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.dhc.*;
import org.isam.exehda.services.dhc.server.*;
import org.isam.exehda.*;
import org.isam.util.xml.*;
import java.net.*;
import java.rmi.*;
import java.io.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;


/*
 * KConfigMaker - Retrieves configurations from server
 */

public class DhcConfigMaker implements Service {

    int timeout;
    int waitTime;
    String primosdPath;
    UDPServer hostDaemon;
 

        /*creates new KConfigMakes with the given parameters*/
    public DhcConfigMaker(int timeout, String primosdPath) {
        this.timeout = timeout;
        this.primosdPath = primosdPath;
    }

        /*creates new KConfigMakes with default parameters*/
    public DhcConfigMaker() {
        this.timeout = 1000;
        this.primosdPath = "./";
    }
  
        /**start the configuration service in the node**/
    public void start() throws Exception {
        

	getConfiguration();

        File file = new File("DHC.Host");
            //if KConf.Host exists launch the process
        if (file.exists()) {
            BufferedReader br = new BufferedReader(new FileReader(file));
            String host = br.readLine();
            hostDaemon = new UDPServer(host);
            hostDaemon.start();
        }
    }

        /**stop the configuration service in the node**/
    public void stop() throws Exception {
        hostDaemon.makeStop();
        hostDaemon.interrupt();
    }


        /*Processes the given xml string and set the profiles*/

    public void processXML(String xmlString){
        try{
            Reader read = new InputStreamReader(new ByteArrayInputStream(xmlString.getBytes()));
            XmlElement element = XmlElement.parse(read);
	    
            java.util.Enumeration profileEnum = element.getChildren();
            while ( profileEnum.hasMoreElements() ) {
                Exehda.addProfile((XmlElement)profileEnum.nextElement());
            }
	    Exehda.bootstrap("default");
        }catch(Exception e){e.printStackTrace();}
    }
       
        /*looks for a server and tries to get the configuration from it*/
    public void getConfiguration(){
        UDPClient client = new UDPClient(timeout);
        String host = null;
        BufferedReader br =  new BufferedReader ( new InputStreamReader(System.in));
        HostQuerier querier = null;
        String hostName = "Unknown";
        String ipAddr = "0.0.0.0";
        String mac = "";

            //tries to find a configuration server
        while(host == null){
            host = client.queryHost();
	 
            if (host == null){
                try{
                    if(querier == null){
                        querier = new HostQuerier();
                        querier.start();
                    }
		  
                    host = querier.getHost();
    		                    }catch(Exception e){
                    e.printStackTrace();}
            }
        }
        if(querier != null){querier.makeStop();}
            //if a host was found...
        if(host != ""){
            try {

                System.out.println(InetAddress.getLocalHost().toString());
                InetAddress[] localhost = InetAddress.getAllByName(
                    InetAddress.getLocalHost().getHostName());

                for ( int i=0; i < localhost.length; i++ ) {
                    System.out.println(localhost[i].toString());
                    hostName = localhost[i].getHostName();
                    ipAddr = localhost[i].getHostAddress();
                    
                    if ( !localhost[i].isLoopbackAddress() ) break;
                }
            }
            catch (java.net.UnknownHostException uhe) {
                System.out.println("Could not find host. Exiting");
                System.exit(0);
            }
	    
	    
	    
            try {
                        //String look = "//"+host+"/Knoppix-ConfigServer";
                    //ConfigServer server = (ConfigServer) Naming.lookup(look);
                    //String propreties = server.getConfiguration(hostName,ipAddr,mac);
                String propreties = "";
                    /*make http request*/
                    //System.out.println("Host--"+host);
                java.net.URLConnection dhcConn =
                    new java.net.URL("http://"+
                                     host+":1976/").openConnection();
                dhcConn.setDoOutput(true);
                dhcConn.setRequestProperty("Isam-Dhc-Hostname",hostName);
                dhcConn.setRequestProperty("Isam-Dhc-IpAddr",ipAddr);
                dhcConn.setRequestProperty("Isam-Dhc-MacAddr",mac);

                    /*get response*/
                java.io.BufferedReader bread = new java.io.BufferedReader(new java.io.InputStreamReader(dhcConn.getInputStream()));
                String s;

                while((s=bread.readLine())!= null){
                    propreties+=s+"\n";
		    // System.out.println(s);
                }
                System.out.println(propreties);
                bread.close();

                    /*processa xml*/
                processXML(propreties);
                    /*spawn new listener*/
                BufferedWriter bw = new BufferedWriter(new FileWriter("DHC.Host"));
                bw.write(host,0,host.length());
                bw.newLine();
                bw.close();
            }catch(Exception e){
                e.printStackTrace();}
        }
        }
}
