package dimi;

import java.util.List;
import java.util.Iterator;
import java.rmi.RemoteException;

import dimi.exception.CantCreateChannelException;
import dimi.exception.CantFindChannelException;
import dimi.exception.CantFindDBaseException;
import dimi.exception.CantEnterChannelException;
import dimi.channel.connection.exception.CantCreateConnectionException;
import dimi.channel.exception.ChannelAlreadyExistsException;
import dimi.name.ChannelName;
import dimi.channel.NeighbourhoodManager;
import dimi.name.DisseminationUnitName;
import dimi.channel.ChannelData;
import dimi.channel.DisseminationUnitChannel;

/**
 * Gerente de canais de um disseminador.
 * @author mcmoraes
 */
public class DBaseChannelManager extends ChannelManager{
    
    /**
     * Creates a new instance of DisseminatorChannels
     * @param disseminationUnit a unidade disseminadora deste ChannelManager
     */
    public DBaseChannelManager(DisseminationUnitImpl disseminationUnitImpl){
        super(disseminationUnitImpl);
    }

    /**
     * Protocolo de inicio de consumo em um canal.
     * @param desiredPeerEntryPointsNumber o numero desejado de pais no canal
     * @param disseminator o disseminador que se tornara filho no canal
     * @param channelName o canal do qual se deseja consumir dados
     * @throws CantFindChannelException se nao for possivel encontrar o canal
     * @throws CantFindPeerException se nao for possivel encontrar o ponto de entrada no canal
     * @throws CantEnterChannelException se nao for possivel entrar no canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public ChannelData enterChannel(ChannelName channelName,int desiredPeerEntryPointsNumber,Disseminator disseminator) throws RemoteException,CantFindChannelException,CantFindDBaseException,CantEnterChannelException{
System.out.println("enterChannel - BEGIN");
        DisseminationUnitChannel disseminationUnitChannel = (DisseminationUnitChannel)getChannel(channelName);
        if(disseminationUnitChannel != null){
            try{
                disseminationUnitChannel.getConnectionManager().createConnection(disseminator,false);
            }catch(CantCreateConnectionException ccce){
                throw new CantEnterChannelException();
            }
            disseminationUnitChannel.setDesiredPeerEntryPointsNumber(desiredPeerEntryPointsNumber);
        }
        else{
            DisseminationUnitName disseminationUnitInitialEntryPointName = channelName.getDisseminationUnitName();
            if(!disseminationUnitInitialEntryPointName.contains(getDisseminationUnitName())){
                DBase dbaseInitialEntryPoint = disseminationUnitInitialEntryPointName.getDBase();
                ChannelData channelData = dbaseInitialEntryPoint.getChannelData(channelName);
                if(channelData != null){
                    NeighbourhoodManager neighbourhoodManager = new NeighbourhoodManager();
                    searchBestDBaseEntryPoint(channelName,dbaseInitialEntryPoint,neighbourhoodManager);
                    disseminationUnitChannel = enterChannel(channelData,neighbourhoodManager);
                    try{
                        disseminationUnitChannel.getConnectionManager().createConnection(disseminator,false);
                    }catch(CantCreateConnectionException ccce){
                        throw new CantEnterChannelException();
                    }
                    disseminationUnitChannel.setDesiredPeerEntryPointsNumber(desiredPeerEntryPointsNumber);
                }
                else{throw new CantFindChannelException();}
            }
            else{throw new CantFindChannelException();}
        }
System.out.println("enterChannel - END");
        if(disseminationUnitChannel == null){
            return null;
        }
        return disseminationUnitChannel.getChannelData();
    }
    
    private final DisseminationUnitChannel enterChannel(ChannelData channelData,NeighbourhoodManager neighbourhoodManager) throws CantEnterChannelException{
        DisseminationUnitChannel channel = null;
        boolean entered = false;
        DBase[] neighbours = neighbourhoodManager.getAllNeighbours();
        for(int i=0;i<neighbours.length;i++){
        Iterator it = neighbourhoodManager.getAllNeighbours().iterator();
        while( (it.hasNext()) && (!entered) ){
            DBase dbase = (DBase)it.next();
            try{
                dbase.connect(getDisseminationUnitImpl(),channelData.getChannelRootName());
                List partialAscendenceList = dbase.getPartialAscendenceList(channelData.getChannelRootName());
                try{
                    channel = this.createChannel(channelData,neighbourhoodManager);
                }
                catch(CantCreateChannelException ccce){}
                catch(ChannelAlreadyExistsException caee){
                    channel = getChannel(channelData.getChannelRootName());
                    entered = true;
                    break;
                }
                //channel.mountPartialAscendenceListFromParent(partialAscendenceList);
                channel.getConnectionManager().createConnection(dbase, true);
                entered = true;
            }
            catch(CantCreateConnectionException ccce){}
            catch(RemoteException re){}
        }
        if(!entered){
            throw new CantEnterChannelException();
        }
        return channel;
    }
    
    protected final void searchBestDBaseEntryPoint(ChannelName channelName,DBase dbaseEntryPoint,NeighbourhoodManager channelNeighbourhoodManager) throws RemoteException{
        NeighbourhoodManager neighbourhoodManager = new NeighbourhoodManager();
        neighbourhoodManager.addNeighbour(dbaseEntryPoint);
        neighbourhoodManager.addAllNeighbours(dbaseEntryPoint.getRelatives(channelName));
        channelNeighbourhoodManager.addAllPointers(neighbourhoodManager.getAllPointers());
        DBase closer = neighbourhoodManager.getTheCloserNeighbour();
        if(!closer.equals(dbaseEntryPoint)){
            searchBestDBaseEntryPoint(channelName,closer,channelNeighbourhoodManager);
        }
    }

}
