package dimi;

import java.util.Set;
import java.util.List;
import java.util.ArrayList;
import java.util.TreeSet;
import java.rmi.RemoteException;

import dimi.channel.Channel;
import dimi.configuration.DisseminationUnitConfiguration;
import dimi.exception.CantFindChannelException;
import dimi.exception.CantFindDBaseException;
import dimi.exception.CantEnterChannelException;
import dimi.channel.connection.exception.CantCreateConnectionException;
import dimi.name.ChannelName;
import dimi.name.DBaseName;
import dimi.name.DNodeName;
import dimi.name.BadformedDimiNameException;
import dimi.channel.ChannelData;
import dimi.channel.DisseminationUnitChannel;
import dimi.channel.Channel;
import dimi.channel.connection.DisseminationUnitConnection;
/**
 *
 * @author  mcmoraes
 */
public class DBaseImpl extends DisseminationUnitImpl implements DBase{

    //atributos - INICIO
    private long dnodeCounter = 0;
    //atributos - FIM
    
    //construtores - INICIO
    /** Creates a new instance of DBase */
    public DBaseImpl(DBaseName name,DisseminationUnitConfiguration disseminationConfiguration) throws RemoteException{
        super(name,disseminationConfiguration);
        setChannelManager(new DBaseChannelManager(this));
    }
    //construtores - FIM
    
    //interface DBase - INICIO
    /**
     *
     * @throws RemoteException
     * @return
     */    
    public synchronized DNodeName getDNodeName() throws RemoteException{
        DNodeName result = null;
        try{
            result = new DNodeName(getName().toString() + "/disseminator"+dnodeCounter);
        }catch(BadformedDimiNameException bfne){}
        return result;
    }

    /**
     * Obtem as unidades de disseminacao conhecidas em um canal.
     * @param channelName o nome do canal
     * @throws RemoteException if failed to export object
     * @return as unidades de disseminacao conhecidas em um canal
     */    
    public DBase[] getNeighbours(ChannelName channelName) throws RemoteException {
        return getChannelManager().getChannel(channelName).getNeighbourhoodManager().getAllNeighbours();
    }

    /**
     * Obtem a lista parcial de ascendencia em um canal.
     * @param channelName o canal desejado
     * @throws RemoteException if failed to export object
     * @return a lista parcial de ascendencia no canal especificado
     */    
    public List getPartialAscendenceList(ChannelName channelName) throws RemoteException{
        return getChannelManager().getChannel(channelName).getPartialAscendenceList();
    }
        
    /**
     * Obtem os filhos do disseminador em um canal.
     * @return os filhos do disseminador no canal
     * @param channelName o nome do canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public DBase[] getChildren(ChannelName channelName) throws RemoteException{
        List children = new ArrayList();
        Object[] connections = getChannelManager().getChannel(channelName).getConnectionManager().getOutputConnections();
        for(int i=0;i<connections.length;i++){
            DisseminationUnitConnection conn = (DisseminationUnitConnection)connections[i];
            if(conn instanceof DBase){
                children.add(conn.getDisseminator());
            }
        }
        return (DBase[])children.toArray();
    }
        
    /**
     * Obtem os pais do disseminador em um canal.
     * @return os pais do disseminador no canal
     * @param channelName o nome do canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public DBase[] getParents(ChannelName channelName) throws RemoteException{
        List parents = new ArrayList();
        Object[] connections = getChannelManager().getChannel(channelName).getConnectionManager().getInputConnections();
        for(int i=0;i<connections.length;i++){
            DisseminationUnitConnection conn = (DisseminationUnitConnection)connections[i];
            if(conn instanceof DBase){
                parents.add(conn.getDisseminator());
            }
        }
        return (DBase[])parents.toArray();
    }
    
    /**
     * Obtem os pais o os filhos da unidade de disseminacao em um canal.
     * @param channelName o nome do canal
     * @throws RemoteException if failed to export object
     * @return os pais e os filhos da unidade de disseminacao em um canal
     */
    public DBase[] getRelatives(ChannelName channelName) throws RemoteException {
        DBase[] parents = getParents(channelName);
        DBase[] children = getChildren(channelName);
        DBase[] relatives = new DBase[parents.length + children.length];
        for(int i=0;i<parents.length;i++){
            relatives[i] = parents[i];
        }
        int j = 0;
        for(int i=parents.length;i<relatives.length;i++){
            relatives[i] = children[j++];
        }
        return relatives;
    }
    
    /**
     * Torna-se filho da unidade de disseminacao em um canal.
     * @param disseminator o disseminador a tornar-se filho
     * @param channelName o nome do canal
     * @throws RemoteException if failed to export object
     * @throws CantEnterChannelException se nao foi possivel entrar no canal
     */
    public void connect(Disseminator disseminator,ChannelName channelName) throws RemoteException,CantEnterChannelException{
        try{
            getChannelManager().getChannel(channelName).getConnectionManager().createConnection(disseminator, false);
        }catch(CantCreateConnectionException ccce){
            throw new CantEnterChannelException();
        }
    }

    /**
     * Entra em um canal e assume um disseminador como filho nesse canal.
     * @param desiredPeerEntryPointsNumber o numero desejado de pais no canal
     * @param disseminator o disseminador que sera filho no canal
     * @param channelName o nome do canal
     * @throws CantFindDBaseException
     * @throws CantFindChannelException se nao foi possivel encontrar o canal
     * @throws CantEnterChannelException se nao foi possivel entrar no canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     * @return
     */
    public final ChannelData enterChannel(ChannelName channelName, int desiredPeerEntryPointsNumber,Disseminator disseminator) throws RemoteException, CantFindChannelException,CantFindDBaseException,CantEnterChannelException{
        return getChannelManager().enterChannel(channelName,desiredPeerEntryPointsNumber,disseminator);
    }
    //interface DBase - INICIO   

    //interface DisseminationUnit - INICIO
    /**
     *
     * @throws RemoteException
     */    
    public void destroy() throws RemoteException{
    }    
    //interface DisseminationUnit - FIM
    
}

