package dimi;

import java.rmi.RemoteException;

import dimi.configuration.ChannelConfiguration;
import dimi.exception.CantFindChannelException;
import dimi.exception.CantFindDBaseException;
import dimi.exception.CantEnterChannelException;
import dimi.exception.CantCreateChannelException;
import dimi.channel.exception.ChannelAlreadyExistsException;
import dimi.name.ChannelName;
import dimi.channel.NeighbourhoodManager;
import dimi.channel.DisseminationUnitChannel;
import dimi.channel.connection.exception.CantCreateConnectionException;
import dimi.channel.ChannelData;
/**
 * Gerente de canais de um disseminador.
 * @author mcmoraes
 */
public class DNodeChannelManager extends ChannelManager{

    /**
     * Creates a new instance of DisseminatorChannels
     * @param disseminationUnit a unidade disseminadora deste ChannelManager
     */
    public DNodeChannelManager(DisseminationUnitImpl disseminationUnitImpl){
        super(disseminationUnitImpl);
    }

    /**
     * Protocolo de inicio de consumo em um canal.
     * @param desiredPeerEntryPointsNumber o numero desejado de pais no canal
     * @param disseminator o disseminador que se tornara filho no canal
     * @param channelName o canal do qual se deseja consumir dados
     * @throws CantFindChannelException se nao for possivel encontrar o canal
     * @throws CantFindPeerException se nao for possivel encontrar o ponto de entrada no canal
     * @throws CantEnterChannelException se nao for possivel entrar no canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public synchronized ChannelData enterChannel(ChannelName channelName,int desiredPeerEntryPointsNumber,Disseminator disseminator) throws RemoteException,CantFindChannelException,CantFindDBaseException,CantEnterChannelException{
System.out.println("enterChannel - BEGIN");
        DisseminationUnitChannel disseminationUnitChannel = getChannel(channelName);
        if(disseminationUnitChannel != null){
            disseminationUnitChannel.addDisseminator(disseminator,desiredPeerEntryPointsNumber);
        }
        else{
            ChannelData channelData = ((DNode)getDisseminationUnitImpl()).getDbase().enterChannel(channelName,desiredPeerEntryPointsNumber,getDisseminationUnitImpl());
            if(channelData != null){
                try{
                    disseminationUnitChannel = createChannel(channelData,null);
                }
                catch(CantCreateChannelException ccce){}
                catch(ChannelAlreadyExistsException caee){
                    disseminationUnitChannel = getChannel(channelData.getChannelRootName());
                }
                try{
                    disseminationUnitChannel.getConnectionManager().createConnection(((DNode)getDisseminationUnitImpl()).getDbase(),false);
                }catch(CantCreateConnectionException ccce){
                    throw new CantEnterChannelException();
                }
                disseminationUnitChannel.setDesiredPeerEntryPointsNumber(desiredPeerEntryPointsNumber);
            }
        }
System.out.println("enterChannel - END");
        return null;
    }
    
    /**
     * Protocolo de criacao de um canal e inicio de producao no mesmo.
     * @return o ponto de entrada do produtor do novo canal
     * @param channelConfiguration a configuracao do novo canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     * @throws CantCreateChannelException se nao foi possivel criar o novo canal
     */
    public final synchronized ProducerEntryPointImpl createChannel(ChannelConfiguration channelConfiguration) throws CantCreateChannelException,RemoteException{
System.out.println("createChannel - BEGIN");
        DisseminationUnitChannel disseminationUnitChannel = null;
        try{
            disseminationUnitChannel = (DisseminationUnitChannel)createChannel(null,channelConfiguration,null);
        }
        catch(ChannelAlreadyExistsException caee){}
        ProducerEntryPointImpl producerEntryPoint = null;
        try{
            producerEntryPoint = new ProducerEntryPointImpl(disseminationUnitChannel);
        }
        catch(RemoteException re){throw new CantCreateChannelException(re);}
        try{
            disseminationUnitChannel.getConnectionManager().createConnection(producerEntryPoint,true);
        }
        catch(CantCreateConnectionException cece){}
        producerEntryPoint.activate();
System.out.println("createChannel - END");
        return producerEntryPoint;
    }

}
