package dimi;

import dimi.name.DimiName;
import dimi.Disseminator;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * Pontos de entradas (produtores e consumidores) e disseminadores
 * sao componentes dimi.
 * @author mcmoraes
 */
public abstract class DimiComponentImpl extends UnicastRemoteObject{

    private DimiName name = null;
    
    /**
     * Constroi um novo componente dimi.
     * @param name o nome do componente
     * @throws RemoteException if failed to export object
     */    
    protected DimiComponentImpl(DimiName name) throws RemoteException {
        super();
        this.name = name;
    }

    /**
     * Getter for property name.
     * @return Value of property name.
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public final DimiName getName() throws RemoteException {
        return name;
    }
    
    /**
     * Setter for property name.
     * @param name New value of property name.
     */
    public final void setName(DimiName name) {
        this.name = name;
    }
    
    /**
     * Verifica a se dois componentes dimi sao iguais. A igualdade de dois
     * componentes dimi e determinada pelo seu nome.
     * @param disseminator o disseminador a ser verificado
     * @return true, se forem iguais; false, caso contrario
     */    
    public final boolean equals(Object disseminator){
        if(disseminator == null ||!(disseminator instanceof Disseminator)){
            return false;
        }
        
        Disseminator other = (Disseminator)disseminator;
        
        boolean result = false;
        try{
            result = name.equals(other.getName());
        }
        catch(RemoteException re){}
        
        return result;
    }
    
    /**
     * Obtem a representacao em String deste componente DIMI.
     * @return a representacao em String deste componente DIMI
     */    
    public final String toString(){
        return name.toString();
    }
}
