package dimi.channel;

import java.rmi.RemoteException;

import dimi.configuration.ChannelConfiguration;
import dimi.name.ChannelName;
import dimi.message.Message;
import dimi.exception.CantFindChannelException;

/**
 * Um canal dimi.
 * @author mcmoraes
 */
public abstract class Channel {
    
    /**
     * Posicao do nome local no array names
     */    
    public static final int LOCAL_NAME = 0;
    /**
     * Posicao do nome de de raiz no array names.
     */    
    public static final int ROOT_NAME = 1;
    
    /**
     * O nome do canal.
     */    
    private ChannelName[] names;
    /**
     * A configuracao do canal.
     */    
    private ChannelConfiguration configuration = null;
    
    private boolean active = true;
    
    /**
     * Cria uma nova instancia de canal
     * @param names os nomes do canal. O primeiro e o nome local, o segundo o nome de raiz do canal.
     * @param configuration a configuracao do canal
     */    
    public Channel(ChannelName[] names,ChannelConfiguration configuration){
        this.names = names;
        this.configuration = configuration;
    }
    
    /**
     * Getter for property name.
     * @return Value of property name.
     */
    public ChannelName[] getNames() {
        return names;
    }   
 
    /**
     * Verifica a igualdade entre dois canais.
     * @param channel o canal a ser verificado
     * @return true, se iguais; false, caso contrario
     */    
    public boolean equals(Object channel){
        
        if(channel == null ||
            !(channel instanceof Channel)){
            return false;
        }
        
        Channel other = (Channel)channel;
        
        return names[ROOT_NAME].equals(other.getNames()[ROOT_NAME]);
        
    }

    /**
     * Getter for property configuration.
     * @return Value of property configuration.
     */
    public dimi.configuration.ChannelConfiguration getConfiguration() {
        return configuration;
    }
        
    /**
     * Destroi o canal.
     */    
    public void destroy(){
        active = false;
    }
    
    /**
     * Obtem a representacao em String deste canal.
     * @return a representacao em String deste canal
     */    
    public final String toString(){
        return names[Channel.ROOT_NAME].toString();        
    }
            
    /**
     * Getter for property active.
     * @return Value of property active.
     */
    public boolean isActive() {
        return active;
    }
    
    /**
     * Setter for property active.
     * @param active New value of property active.
     */
    public void setActive(boolean active) {
        this.active = active;
    }
    
    public abstract void disseminate(Message message) throws RemoteException,CantFindChannelException;
    
    public final ChannelData getChannelData(){
        return new ChannelData(names[Channel.ROOT_NAME],configuration);
    }
}
