package dimi.channel;

import dimi.configuration.ChannelConfiguration;
import dimi.ProducerEntryPointImpl;
import dimi.channel.ConnectionManager;
import dimi.ChannelManager;
import dimi.channel.connection.DisseminationUnitConnection;
import dimi.name.ChannelName;
import dimi.message.Message;
import dimi.message.MessageHeader;
import dimi.exception.CantFindChannelException;
import dimi.exception.CantEnterChannelException;
import dimi.channel.connection.exception.CantCreateConnectionException;
import dimi.Disseminator;

import java.util.Iterator;
import java.rmi.RemoteException;

import java.util.List;
import java.util.Vector;

import java.util.Timer;
import java.util.TimerTask;

/**
 * Um canal de disseminador. Se diferencia de um canal de ponto de entrada
 * de usuario pois pode possuir muitas conexoes.
 * @author mcmoraes
 */
public class DisseminationUnitChannel extends Channel{
    
    protected ConnectionManager connectionManager = null;
    protected ChannelManager channelManager = null;
    private TopologyManager topologyManager = null;
    private List partialAscendenceList = null;
    private NeighbourhoodManager neighbourhoodManager = null;
    private int desiredPeerEntryPointsNumber = 1;
    private Timer timer = null;

    
    /**
     * Creates a new instance of DisseminatorChannel
     * @param names os nomes do canal. Primeiro o nome local, depois o nome de raiz.
     * @param channelNeighbourhoodManager o NeighbourhooManager deste canal
     * @param configuration a configuracao do novo canal
     * @param channelManager o gerente de canais do novo canal
     */
    public DisseminationUnitChannel(ChannelName[] names, ChannelConfiguration configuration, ChannelManager channelManager,NeighbourhoodManager neighbourhoodManager) {
        super(names,configuration);
        this.connectionManager = new ConnectionManager(this);
        this.partialAscendenceList = new Vector();
        if(neighbourhoodManager != null){
            this.neighbourhoodManager = neighbourhoodManager;
        }
        else{
            this.neighbourhoodManager = new NeighbourhoodManager();
        }
        this.timer = new Timer();
        this.topologyManager = new TopologyManager();
        this.timer.schedule(topologyManager,2000,2000);

    }
   
    /**
     * Getter for property connections.
     * @return Value of property connections.
     */
    public ConnectionManager getConnectionManager() {
        return connectionManager;
    }    
    
    /**
     * Getter for property channels.
     * @return Value of property channels.
     */
    public ChannelManager getChannelManager() {
        return channelManager;
    }

    private class TopologyManager extends TimerTask{

        public void run() {

        }

    }
    
    /**
     * Getter for property partialAscendenceList.
     * @return Value of property partialAscendenceList.
     */
    public java.util.List getPartialAscendenceList() {
        return partialAscendenceList;
    }

    /**
     * Getter for property neighbourhoodManager.
     * @return Value of property neighbourhoodManager.
     */
    public NeighbourhoodManager getNeighbourhoodManager() {
        return neighbourhoodManager;
    }
    
    /**
     * Getter for property desiredPeerEntryPointsNumber.
     * @return Value of property desiredPeerEntryPointsNumber.
     */
    public int getDesiredPeerEntryPointsNumber() {
        return desiredPeerEntryPointsNumber;
    }

    /**
     * Setter for property desiredPeerEntryPointsNumber.
     * @param desiredPeerEntryPointsNumber New value of property desiredPeerEntryPointsNumber.
     */
    public void setDesiredPeerEntryPointsNumber(int desiredPeerEntryPointsNumber) {
        if(desiredPeerEntryPointsNumber>this.desiredPeerEntryPointsNumber){
            this.desiredPeerEntryPointsNumber = desiredPeerEntryPointsNumber;
        }
    }

   /**
     * Monta a lista parcial de ascendencia deste canal, a partir de outra
     * lista parcial de ascendencia.
     * @param partialAscendenceList a lista parcial de ascendencia a servir de base
     */    
    public void mountPartialAscendenceListFromParent(List partialAscendenceList) {
        try{
            partialAscendenceList.add(0,channelManager.getDisseminationUnitImpl().getName());
        }
        catch(RemoteException re){}
        this.partialAscendenceList = partialAscendenceList;
    }

    public void addDisseminator(Disseminator disseminator,int desiredEntryPointsNumber) throws CantEnterChannelException{
        try{
            connectionManager.createConnection(disseminator,false);
        }catch(RemoteException re){
            throw new CantEnterChannelException();
        }catch(CantCreateConnectionException ccce){
            throw new CantEnterChannelException();
        }
        setDesiredPeerEntryPointsNumber(desiredEntryPointsNumber);
    }

    /**
     * Destroi este canal.
     */    
    public void destroy(){
        super.destroy();
        connectionManager.destroyAllConnections();
        topologyManager.cancel();
        timer.cancel();
    }

    /**
     * Protocolo de disseminacao de dados em um canal
     * @param message a mensagem contendo os dados de um canal
     * @throws CantFindChannelException se nao for possivel encontrar o canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public final synchronized void disseminate(Message message) throws RemoteException{
        switch(message.getHeader().getType()){
            case (MessageHeader.DATA):{
                disseminateMessage(message);
                break;
            }
            case (MessageHeader.HEART_BEAT):{
                processHeartBeatMessage(message);
                break;
            }
            case (MessageHeader.DESTROY_CHANNEL):{
                processDestroyChannelMessage(message);
                break;
            }
            case (MessageHeader.LEAVE):{
                processLeaveMessage(message);
                break;
            }
        }
    }
    
    protected final synchronized void disseminateMessage(Message message){
        if(message.getHeader().getPath().contains(channelManager.getDisseminationUnitName())){
            DisseminationUnitConnection disseminationUnitConnection = connectionManager.getConnection(message.getHeader().getOriginName());
System.out.println("cicle found: " + disseminationUnitConnection.getDisseminatorName());
            connectionManager.destroyConnection(disseminationUnitConnection.getDisseminatorName());
System.out.println("connection removed: " + disseminationUnitConnection.getDisseminatorName());
        }
        else{
            message.getHeader().getPath().add(channelManager.getDisseminationUnitName());
            /*
            Iterator it = connectionManager.getOutputConnections().iterator();
            while(it.hasNext()){
                DisseminationUnitConnection disseminationUnitConnection = (DisseminationUnitConnection)it.next();
                try{
System.out.println("connection target: " + disseminationUnitConnection.getDisseminator().getName());
                    disseminationUnitConnection.getDisseminator().disseminate(message);
                }
                catch(RemoteException re){
System.out.println("connection broken: " + disseminationUnitConnection.getDisseminatorName());
                    connectionManager.destroyConnection(disseminationUnitConnection.getDisseminatorName());
System.out.println("connection removed: " + disseminationUnitConnection.getDisseminatorName());
                }
                catch(CantFindChannelException re){
System.out.println("connection broken: " + disseminationUnitConnection.getDisseminatorName());
                    connectionManager.destroyConnection(disseminationUnitConnection.getDisseminatorName());
System.out.println("connection removed: " + disseminationUnitConnection.getDisseminatorName());
                }
            }
             */
        }
    }    
    protected final synchronized void processHeartBeatMessage(Message message){
        disseminateMessage(message);
    }
    
    protected final synchronized void processDestroyChannelMessage(Message message){
        disseminateMessage(message);
        channelManager.destroyChannel(getNames()[Channel.ROOT_NAME]);
    }
    
    protected final synchronized void processLeaveMessage(Message message){
        connectionManager.destroyConnection(message.getHeader().getOriginName());
    }
    
}
