package dimi.channel;

import dimi.channel.connection.EntryPointConnection;
import java.net.URL;
import java.rmi.RemoteException;
import dimi.UserEntryPointImpl;
import dimi.Disseminator;
import dimi.ConsumerEntryPointImpl;
import dimi.ProducerEntryPointImpl;
import dimi.message.Message;
import dimi.exception.CantFindChannelException;

/**
 * Canal localizado no ponto de entrada do usuario. Difere de um canal
 * de disseminador por possuir apenas uma conexao.
 * @author mcmoraes
 */
public class EntryPointChannel extends Channel{
    
    private EntryPointConnection entryPointConnection = null;
    private UserEntryPointImpl userEntryPointImpl = null;
    
    /**
     * Creates a new instance of EntryPointChannel
     * @param disseminationUnitChannel o canal de unidade disseminadora correspondente a este canal
     * @param userEntryPointImpl o ponto de entrada de usuario que possui este canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public EntryPointChannel(DisseminationUnitChannel disseminationUnitChannel,UserEntryPointImpl userEntryPointImpl) throws RemoteException{
        super(disseminationUnitChannel.getNames(),disseminationUnitChannel.getConfiguration());
        this.userEntryPointImpl = userEntryPointImpl;
        Disseminator disseminator = disseminationUnitChannel.getChannelManager().getDisseminationUnitImpl();
        if(userEntryPointImpl instanceof ConsumerEntryPointImpl) {
            this.entryPointConnection = new EntryPointConnection(this,disseminator,false);
        }
        else if(userEntryPointImpl instanceof ProducerEntryPointImpl){
            this.entryPointConnection = new EntryPointConnection(this,disseminator,true);
        }

    }
    
    /**
     * Getter for property entryPointConnection.
     * @return Value of property entryPointConnection.
     */
    public EntryPointConnection getEntryPointConnection() {
        return entryPointConnection;
    }
    
    /**
     * Setter for property entryPointConnection.
     * @param entryPointConnection New value of property entryPointConnection.
     */
    public void setEntryPointConnection(EntryPointConnection entryPointConnection) {
        this.entryPointConnection = entryPointConnection;
    }
    
    /**
     * Getter for property userEntryPointImpl.
     * @return Value of property userEntryPointImpl.
     */
    public dimi.UserEntryPointImpl getUserEntryPointImpl() {
        return userEntryPointImpl;
    }
        
    /**
     * Destroi este canal
     */    
    public void destroy(){
        super.destroy();
        try{
            entryPointConnection.destroy();
        }
        catch(RemoteException re){re.printStackTrace();}
    }
    
    public void disseminate(Message message) throws RemoteException,CantFindChannelException{
        entryPointConnection.getDisseminator().disseminate(message);
    }
        
}
