package dimi.channel.connection;

import java.rmi.RemoteException;
import java.util.Date;

import dimi.name.DimiName;
//import dimi.name.DisseminationUnitName;
import dimi.message.Message;
import dimi.message.MessageHeader;
import dimi.message.buffer.MessageBuffer;
import dimi.message.buffer.ConnectionMessageBuffer;
import dimi.channel.ConnectionManager;
import dimi.channel.Channel;
import dimi.Disseminator;
import dimi.exception.CantFindChannelException;

/**
 * Representa uma conexao localizada em um disseminador.
 * @author mcmoraes
 */
public class DisseminationUnitConnection extends Connection {

    private ConnectionManager connectionManager = null;
    private boolean inDisconnectedOperation;
    private MessageBuffer messageBuffer = null;
    private DimiName disseminatorName = null;

    
    /**
     * Cria uma nova instancia de conexao de disseminador.
     * @param disseminator o disseminador para o qual o disseminador aponta
     * @param input se a conexao e de entrada ou saida
     * @param connectionManager o gerente de conexoes desta conexao
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public DisseminationUnitConnection(ConnectionManager connectionManager, Disseminator disseminator, boolean input) throws RemoteException{
        super(input,disseminator);
        this.connectionManager = connectionManager;
        this.disseminatorName = disseminator.getName();
        this.messageBuffer = new ConnectionMessageBuffer(this);
    }
    
    /**
     * Verifica a igualdade de duas conexoes. Duas conexoes sao iguais
     * se o canal a que pertencem e o disseminador para o qual apontam sao
     * os mesmos.
     * @param disseminatorConnection a conexao a ser verificada a igualdade
     * @return true, se forem iguais; false caso contrario
     */    
    public final boolean equals(Object disseminatorConnection){
        if(disseminatorConnection == null ||
            !(disseminatorConnection instanceof DisseminationUnitConnection)){
            return false;
        }
        DisseminationUnitConnection other = (DisseminationUnitConnection)disseminatorConnection;
        if( connectionManager.getChannel().getChannelManager().getDisseminationUnitImpl().equals(other.getConnectionManager().getChannel().getChannelManager().getDisseminationUnitImpl()) &&
            connectionManager.getChannel().equals(other.getConnectionManager().getChannel())){
            return true;
        }
        return false;
    }
    
    /**
     * Getter for property connectionManager.
     * @return Value of property connectionManager.
     */
    public final ConnectionManager getConnectionManager() {
        return connectionManager;
    }
        
    /**
     * Getter for property disseminatorName.
     * @return Value of property disseminatorName.
     */
    public final DimiName getDisseminatorName() {
        return disseminatorName;
    }

    /**
     * Monta o cabecalho da mensagem de saida do canal
     * @return o cabecalho da mensagem de saida do canal
     */    
    protected MessageHeader mountLeaveMessageHeader(){
        MessageHeader messageHeader = new MessageHeader();
        try{
            messageHeader.setChannelName(connectionManager.getChannel().getNames()[Channel.ROOT_NAME]);
            messageHeader.setOriginName(connectionManager.getChannel().getChannelManager().getDisseminationUnitImpl().getName());
        }
        catch(RemoteException re){re.printStackTrace();}
        return messageHeader;
    }
    
    /**
     * Getter for property inDisconnectedOperation.
     * @return Value of property inDisconnectedOperation.
     */
    public boolean isInDisconnectedOperation() {
        return inDisconnectedOperation;
    }
    
    /**
     * Setter for property inDisconnectedOperation.
     * @param inDisconnectedOperation New value of property inDisconnectedOperation.
     */
    public void setInDisconnectedOperation(boolean inDisconnectedOperation) {
        this.inDisconnectedOperation = inDisconnectedOperation;
    }
    
    /**
     * Getter for property messageBuffer.
     * @return Value of property messageBuffer.
     */
    public MessageBuffer getMessageBuffer() {
        return messageBuffer;
    }

}
