package dimi.channel.neighbour;

import dimi.DBase;
import dimi.name.DisseminationUnitName;

import java.rmi.RemoteException;
import java.util.Date;

/**
 * Um ponteiro para um vizinho em um canal
 * @author mcmoraes
 */
public class NeighbourPointer implements Comparable{
    
    private DBase target = null;
    private long distance = 0;
    private DisseminationUnitName targetName = null;
    private Date lastDistanceMeasure = null;
    
    /**
     * Creates a new instance of PeerPointer
     * @param target o vizinho a ser apontado
     * @param distance a distancia do vizinho
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public NeighbourPointer(DBase target, long distance) throws RemoteException{
        this.target = target;
        this.targetName = (DisseminationUnitName)target.getName();
        this.setDistance(distance);
    }
    
    /**
     * Compara-se com outro ponteiro atraves do compareTo dos nomes de seus target.
     * @param obj o ponteiro a ser comparado
     * @return a comparacao dos nomes dos targets
     */    
    public int compareTo(Object obj) {
        
        NeighbourPointer other = (NeighbourPointer)obj;
        
        return targetName.compareTo(other.getTargetName());

    }

    /**
     * Dois ponteiros sao iguais se apontarem para o mesmo target
     * @param obj o ponteiro a ser verificado
     * @return true, se forem iguais;
     * false, caso contrario
     */    
    public boolean equals(Object obj) {
        
        if(obj == null || !(obj instanceof NeighbourPointer)){
            return false;
        }
        
        NeighbourPointer other = (NeighbourPointer)obj;
        
        return targetName.equals(other.getTargetName());

    }

    /**
     * Getter for property distance.
     * @return Value of property distance.
     */
    public long getDistance() {
        return distance;
    }
    
    /**
     * Setter for property distance.
     * @param distance New value of property distance.
     */
    public void setDistance(long distance) {
        this.distance = distance;
        this.lastDistanceMeasure = new Date();
    }
    
    /**
     * Getter for property target.
     * @return Value of property target.
     */
    public DBase getTarget() {
        return target;
    }
    
    /**
     * Getter for property targetName.
     * @return Value of property targetName.
     */
    public DisseminationUnitName getTargetName() {
        return targetName;
    }    
        
    /**
     * Getter for property lastDistanceMeasure.
     * @return Value of property lastDistanceMeasure.
     */
    public java.util.Date getLastDistanceMeasure() {
        return lastDistanceMeasure;
    }
        
}
