package dimi.example;

import dimi.configuration.ChannelConfiguration;
import dimi.DisseminationService;
import dimi.exception.CantFindChannelException;
import dimi.message.MessageContent;
import dimi.message.Message;
import dimi.ProducerEntryPoint;

import java.net.InetAddress;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Properties;

public class ProducerExample {

    private ProducerEntryPoint producerEntryPoint = null;
    private ProducerEntryPointGUIExample producerEntryPointGUIExample = null;
    
    public ProducerExample(ProducerEntryPoint producerEntryPoint) throws RemoteException{
        this.producerEntryPoint = producerEntryPoint;
        producerEntryPointGUIExample = new ProducerEntryPointGUIExample(producerEntryPoint);
        producerEntryPointGUIExample.show();
    }
    
    public void go(){
        Timer timer = new Timer();
        timer.schedule(new MessageProduction(),1000,4000);
    }

    class MessageProduction extends TimerTask{

        public String getTestXML(){
            StringBuffer result = new StringBuffer();
            result.append("<root>").append("\n");
            result.append("<a").append(" attrib-a1=\"").append(Math.random()).append("\"/>").append("\n");
            result.append("<b").append(" attrib-b1=\"").append(Math.random()).append("\"").append(" attrib-b2=\"").append(Math.random()).append("\"").append(">").append("\n");
            result.append("<b1>").append(Math.random()).append("</b1>").append("\n");
            result.append("</b>").append("\n");
            result.append("<c>").append(Math.random()).append("</c>").append("\n");
            result.append("</root>").append("\n");
            return result.toString();
        }

        public void run(){
            //String data = "bla, bla, bla...";
            String data = getTestXML();
            byte[] bytes = data.getBytes();
            Message message = new Message(new MessageContent(bytes));
            try{
                producerEntryPoint.disseminate(message);
                producerEntryPointGUIExample.getTextArea().append(data);
            }
            catch(CantFindChannelException cfce){}
            catch(Exception e){e.printStackTrace();}
        }
        
    }   

    public static void main(String[] args) {
        
System.out.println("main - STARTING");
        try{
            DisseminationService service = (DisseminationService)Naming.lookup("//"+InetAddress.getLocalHost().getHostName()+"/disseminator"+args[1]);
System.out.println("service name: "+service.getServiceName());
            ChannelConfiguration cconf = new ChannelConfiguration(args[0]);
            ProducerEntryPoint producerEntryPoint = service.createChannel(cconf);
System.out.println("channel created: "+producerEntryPoint.getName());
            ProducerExample producer = new ProducerExample(producerEntryPoint);
            producer.go();
        }
        catch(Exception exc){
            exc.printStackTrace();
        }
    }

}

