package dimi.message;

import java.io.Serializable;

/**
 * Mensagem: unidade basica de informacao disseminada em dimi.
 * @author  mcmoraes
 */
public class Message implements Serializable{

    private MessageHeader header;
    private MessageContent content;
    
    /**
     * Creates a new instance of Message
     */    
    public Message(){
        this.header = new MessageHeader();
        this.content = new MessageContent(new byte[0]);
    }
    
    /**
     * Creates a new instance of Message
     * @param header o cabecalho da nova mensagem
     */
    public Message(MessageHeader header) {
        this.header = header;
        this.content = new MessageContent(new byte[0]);
    }

    /**
     * Creates a new instance of Message
     * @param content o conteudo da nova mensagem
     */
    public Message(MessageContent content) {
        this.header = new MessageHeader();
        this.content = content;
    }

    /**
     * Creates a new instance of Message
     * @param header o cabecalho da nova mensagem
     * @param content o conteudo da nova mensagem
     */
    public Message(MessageHeader header, MessageContent content) {
        this.header = header;
        this.content = content;
    }

    /**
     * Getter for property content.
     * @return Value of property content.
     */
    public MessageContent getContent() {
        return content;
    }
    
    /**
     * Setter for property content.
     * @param content New value of property content.
     */
    public void setContent(MessageContent content) {
        this.content = content;
    }
    
    /**
     * Getter for property header.
     * @return Value of property header.
     */
    public MessageHeader getHeader() {
        return header;
    }
    
    /**
     * Setter for property header.
     * @param header New value of property header.
     */
    public void setHeader(MessageHeader header) {
        this.header = header;
    }
    
    /**
     * Duas mensagens sao iguais se seus cabecalhos forem iguais.
     * @param message a mensagem a ser verificada
     * @return true, se sim; false, caso contrario
     */    
    public boolean equals(Object message){
        if(message == null || !(message instanceof Message)){
            return false;
        }
        MessageHeader messageHeader = ((Message)message).getHeader();
        return header.equals(messageHeader);
    }
        
}
