package dimi.message;

import java.util.Date;
import java.util.List;
import java.util.Vector;
import java.io.Serializable;

import dimi.name.ChannelName;
import dimi.name.DBaseName;
import dimi.name.DimiName;

/**
 * Cabecalho de uma mensagem.
 * @author  mcmoraes
 */
public class MessageHeader implements Serializable{
    
    /**
     * Identificador de mensagem de dados
     */    
    public static final int DATA = 1;
    /**
     * Identificador de mensagem de aviso de vida
     */    
    public static final int HEART_BEAT = 2;
    /**
     * Identificador de mensagem de destruicao de canal
     */    
    public static final int DESTROY_CHANNEL = 3;
    /**
     * Identificador de mensagem de saida de um canal
     */    
    public static final int LEAVE = 4;
    
    private ChannelName channelName;
    private DimiName originName;
    //private DisseminationUnitBaseName rootName;
    private long sequenceNumber = 0;
    private Date productionDateTime;
    private Date originDateTime;
    private List path;
    private int type;
    private int priority;
    private long contentSize;
    private boolean compressed;
    private boolean filtered;
    
    /** Creates a new instance of MessageHeader */
    public MessageHeader() {
        path = new Vector();
    }
    
    /**
     * Getter for property channelName.
     * @return Value of property channelName.
     */
    public ChannelName getChannelName() {
        return channelName;
    }
    
    /**
     * Setter for property channelName.
     * @param channelName New value of property channelName.
     */
    public void setChannelName(ChannelName channelName) {
        this.channelName = channelName;
    }
    
    /**
     * Getter for property originDateTime.
     * @return Value of property originDateTime.
     */
    public Date getOriginDateTime() {
        return originDateTime;
    }
    
    /**
     * Setter for property originDateTime.
     * @param originDateTime New value of property originDateTime.
     */
    public void setOriginDateTime(Date originDateTime) {
        this.originDateTime = originDateTime;
    }
    
    /**
     * Getter for property originName.
     * @return Value of property originName.
     */
    public DimiName getOriginName() {
        return originName;
    }
    
    /**
     * Setter for property originName.
     * @param originName New value of property originName.
     */
    public void setOriginName(DimiName originName) {
        this.originName = originName;
    }
    
    /**
     * Getter for property path.
     * @return Value of property path.
     */
    public List getPath() {
        return this.path;
    }
    
    /**
     * Setter for property path.
     * @param path New value of property path.
     */
    public void setPath(List path) {
        this.path = path;
    }
    
    /**
     * Getter for property priority.
     * @return Value of property priority.
     */
    public int getPriority() {
        return priority;
    }
    
    /**
     * Setter for property priority.
     * @param priority New value of property priority.
     */
    public void setPriority(int priority) {
        this.priority = priority;
    }
    
    /**
     * Getter for property productionDateTime.
     * @return Value of property productionDateTime.
     */
    public Date getProductionDateTime() {
        return productionDateTime;
    }
    
    /**
     * Setter for property productionDateTime.
     * @param productionDateTime New value of property productionDateTime.
     */
    public void setProductionDateTime(Date productionDateTime) {
        this.productionDateTime = productionDateTime;
    }
    
    /**
     * Getter for property sequenceNumber.
     * @return Value of property sequenceNumber.
     */
    public long getSequenceNumber() {
        return sequenceNumber;
    }
    
    /**
     * Setter for property sequenceNumber.
     * @param sequenceNumber New value of property sequenceNumber.
     */
    public void setSequenceNumber(long sequenceNumber) {
        this.sequenceNumber = sequenceNumber;
    }
    
    /**
     * Getter for property type.
     * @return Value of property type.
     */
    public int getType() {
        return type;
    }
    
    /**
     * Setter for property type.
     * @param type New value of property type.
     */
    public void setType(int type) {
        this.type = type;
    }
    
    /**
     * Getter for property contentSize.
     * @return Value of property contentSize.
     */
    public long getContentSize() {
        return contentSize;
    }
    
    /**
     * Setter for property contentSize.
     * @param contentSize New value of property contentSize.
     */
    public void setContentSize(long contentSize) {
        this.contentSize = contentSize;
    }
    
    /**
     * Getter for property filtered.
     * @return Value of property filtered.
     */
    public boolean isFiltered() {
        return filtered;
    }
    
    /**
     * Setter for property filtered.
     * @param filtered New value of property filtered.
     */
    public void setFiltered(boolean filtered) {
        this.filtered = filtered;
    }
    
    /**
     * Getter for property compressed.
     * @return Value of property compressed.
     */
    public boolean isCompressed() {
        return compressed;
    }
    
    /**
     * Setter for property compressed.
     * @param compressed New value of property compressed.
     */
    public void setCompressed(boolean compressed) {
        this.compressed = compressed;
    }
 
    /**
     * Dois cabecalhos de mensagens sao iguais se o canal a que eles pertencerem forem
     * iguais, os seus numeros de sequencias forem iguais.
     * @param messageHeader o cabecalho a ser comparado
     * @return true, se iguais; false, caso contrario.
     */    
    public boolean equal(Object messageHeader){
        if(messageHeader == null || 
            !(messageHeader instanceof MessageHeader)
            ){
            return false;
        }
        MessageHeader other = (MessageHeader)messageHeader;
        if(channelName.equals(other.getChannelName()) && 
            sequenceNumber == other.getSequenceNumber()
            ){
            return true;
        }
        return false;
    }
    
    /**
     * Representacao em String desse cabecalho de mesagem
     * @return representacao em String desse cabecalho de mensagem
     */    
    public String toString(){
        StringBuffer result = new StringBuffer();
        result.append("type:").append(" ");
        switch(type){
            case DATA:{result.append("DATA");break;}
            case HEART_BEAT:{result.append("HEART_BEAT");break;}
            case DESTROY_CHANNEL:{result.append("DESTROY_CHANNEL");break;}
            case LEAVE:{result.append("LEAVE");break;}
        }
        result.append(";").append(" ");
        result.append("sequence number:").append(" ").append(sequenceNumber).append(";").append(" ");
        result.append("channel name:").append(" ").append(channelName).append(";").append(" ");
        return result.toString();
    }    

    /**
     * Getter for property rootName.
     * @return Value of property rootName.
     */
    /*
    public DisseminationUnitBaseName getRootName() {
        return rootName;
    }
    */
    
    /**
     * Setter for property rootName.
     * @param rootName New value of property rootName.
     */
    /*
    public void setRootName(DisseminationUnitBaseName rootName) {
        this.rootName = rootName;
    }
    */
}
