package dimi.message.buffer;

import dimi.message.Message;
import dimi.channel.connection.DisseminationUnitConnection;
/**
 * Buffer de mensagens de uma conexao.
 * @author mcmoraes
 */
public class ConnectionMessageBuffer extends MessageBufferImpl{
    
    /**
     * O mediador desse buffer com o buffer do disseminador.
     */    
    protected ConnectionMessageBufferMediator mediator = null;
    /**
     * A conexao que possui este buffer.
     */    
    protected DisseminationUnitConnection connection = null;
    
    /**
     * Creates a new instance of ConnectionMessageBuffer
     * @param connection a conexao que deve ser dona do novo buffer
     */
    public ConnectionMessageBuffer(DisseminationUnitConnection connection) {
        this.mediator = (ConnectionMessageBufferMediator)connection.getConnectionManager().getChannel().getChannelManager().getDisseminationUnitImpl().getMessageBuffer().getMediator();
        this.connection = connection;
    }
    
    /**
     * Adiciona uma mensagem ao buffer.
     * @param message a mensagem a ser adicionada ao buffer
     */    
    public void add(Message message) {
        mediator.addToConnectionMessageBuffer(this, message);
    }
    
    /**
     * Getter for property connection.
     * @return Value of property connection.
     */
    public DisseminationUnitConnection getConnection() {
        return connection;
    }
    
    /**
     * Compara dois buffers de conexao. Dois buffers de conexao sao
     * iguais se as conexoes que os possuirem forem iguais.
     * @param connectionMessageBuffer o buffer de conexao a ser comparado
     * @return true, se forem iguais; false, caso contrario
     */    
    public boolean equals(Object connectionMessageBuffer){
        
        if(connectionMessageBuffer == null ||
            !(connectionMessageBuffer instanceof ConnectionMessageBuffer)){
            return false;
        }
        
        ConnectionMessageBuffer other = (ConnectionMessageBuffer)connectionMessageBuffer;
        
        return connection.equals(other.getConnection());
        
    }
        
    public final MessageBufferMediator getMediator() {
        return mediator;
    }

}
