package dimi.message.buffer;

import dimi.message.Message;
import dimi.name.ChannelName;

import java.util.Map;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

/**
 * Mediador da interacao entre os buffers do disseminador e das conexoes.
 * @author mcmoraes
 */
public class MessageBufferMediatorImpl implements ConnectionMessageBufferMediator, DisseminationUnitMessageBufferMediator{
    
    private DisseminationUnitMessageBuffer disseminationUnitMessageBuffer = null;
    
    //nome de canal --> mapa de mensagens do canal --> lista de buffers que contem a mensagen
    private Map channelMessageConnectionMessageBufferMap = null;
    
    /** Cria uma nova conexao de MessageBufferMediator*/
    public MessageBufferMediatorImpl() {
    }
    
    /**
     * Adiciona uma mensagem a um buffer de conexao e a coloca,
     * se necessario, no buffer do disseminador.
     * @param connectionMessageBuffer o buffer de conexao que deve armazenar a mensagem
     * @param message a mensagem
     */    
    public void addToConnectionMessageBuffer(ConnectionMessageBuffer connectionMessageBuffer,
                                            Message message){
        
        //adiciona a mensagem no buffer da conexao
        connectionMessageBuffer.buffer.add(message);
        //obtem o nome do canal
        ChannelName channelName = message.getHeader().getChannelName();
        //mensagem --> lista de buffers que contem a mensagem
        Map messageMap = (Map)channelMessageConnectionMessageBufferMap.get(channelName);
        //lista de buffers que contem a mensagem
        List messageBuffers = (List)messageMap.get(message);
        //se nao a mensagem nao esta no buffer do disseminador
        if(messageBuffers == null){
            //coloca a mensagem no buffer do disseminador
            disseminationUnitMessageBuffer.add(message);
            //inicia uma lista de buffers para a mensagem
            messageBuffers = new ArrayList();
            //associa a mensagem a lista de buffers da mensagen
            messageMap.put(message,messageBuffers);
        }
        //adiciona o buffer a lista
        messageBuffers.add(connectionMessageBuffer);
        
    }
    
    /**
     * Remove uma mensagem de um buffer de conexao e remove esta mensagem,
     * se necessario, do buffer do disseminador.
     * @param connectionMessageBuffer o buffer de conexao do qual a mensagem sera retirada
     * @param message a mensagem a ser retirada
     */    
    public void removeFromConnectionMessageBuffer(ConnectionMessageBuffer connectionMessageBuffer,
                                                Message message){
                                                    
        //remove a mensagem no buffer da conexao
        connectionMessageBuffer.buffer.remove(message);
        //obtem o nome do canal
        ChannelName channelName = message.getHeader().getChannelName();
        //mensagem --> lista de buffers que contem a mensagem
        Map messageMap = (Map)channelMessageConnectionMessageBufferMap.get(channelName);
        //lista de buffers que contem a mensagem
        List messageBuffers = (List)messageMap.get(message);
        //retira o buffer da lista
        messageBuffers.remove(connectionMessageBuffer);
        //se a lista ficou vazia, retira a mensagem do buffer do disseminador
        if(messageBuffers.isEmpty()){
            disseminationUnitMessageBuffer.remove(message);
        }
        
    }
    
    /**
     * !!!REVER!!! Remove uma mensagem de um buffer de disseminador e remove esta mensagem,
     * dos buffers de conexao na qual estiver armazenada. !!!REVER!!!
     * @param message a mensagem a ser retirada
     */    
    public void removeFromDisseminatorMessageBuffer(Message message) {
        
        //obtem o nome do canal
        ChannelName channelName = message.getHeader().getChannelName();
        //mensagem --> lista de buffers que contem a mensagem
        Map messageMap = (Map)channelMessageConnectionMessageBufferMap.get(channelName);
        //lista de buffers que contem a mensagem
        List messageBuffers = (List)messageMap.get(message);
        //retira a mensagem de todos os buffers de conexao em que ela existir
        Iterator it = messageBuffers.iterator();
        while(it.hasNext()){
            ConnectionMessageBuffer buffer = (ConnectionMessageBuffer)it.next();
            buffer.remove(message);
        }
        //retira a mensagem de 'mapa de mensagens do canal --> lista de buffers que contem a mensagen'
        messageMap.remove(message);
    }
        
}
