package dimi.name;

import java.io.Serializable;

/**
 * Um nome de um componente DIMI
 * @author mcmoraes
 */
public abstract class DimiName implements Serializable, Comparable{
    
    private String name = null;
    
    /**
     * Creates a new instance of DimiName
     * @param name o nome desejado para o componente
     * @throws BadformedDimiNameException se o nome nao estiver bem formado
     */    
    public DimiName(String name) throws BadformedDimiNameException{
        if(name.endsWith("/")){
            throw new BadformedDimiNameException();
        }
        this.name = name;
    }
    
    /**
     * Getter for property name.
     * @return Value of property name.
     */
    public java.lang.String getName() {
        return this.name;
    }
    
    /**
     * Setter for property name.
     * @param name New value of property name.
     */
    protected void setName(String name) {
        this.name = name;
    }
    
    /**
     * Dois nomes DIMI sao iguais se suas strings forem iguais
     * @param obj o nome DIMI a ser verificada a igualdade
     * @return true, se forem iguais;
     * false, caso contrario
     */    
    public final boolean equals(Object obj){
        if(obj == null || !(obj instanceof DimiName)){
            return false;
        }
        DimiName other = (DimiName)obj;
        return name.equals(other.getName());
    }
    
    /**
     * Compara dois nomes DIMI.
     * @param obj o nome DIMI a ser comparado
     * @return a comparacao das strings dos nomes DIMI
     */    
    public int compareTo(Object obj) {
        DimiName other = (DimiName)obj;
        return this.name.compareTo(other.getName());
    }

    /**
     * Obtem a representacao em String deste nome DIMI
     * @return a representacao deste nome DIMI
     */    
    public final String toString(){
        return name;
    }
    
    /**
     * O codigo hash deste nome DIMI. Este metodo e necessario para
     * o uso de hash tables
     * @return o codido hash deste nome DIMI
     */    
    public int hashCode(){
        return name.hashCode();
    }
    
}
