// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos;

import org.isam.exehda.services.primos.MetaMonitor.RawSensor;
import org.isam.exehda.services.primos.MetaMonitor.RawMonitor;
import org.isam.exehda.services.primos.MetaMonitor.RawSensorData;

public class NativeMonitor
    implements MetaMonitor.RawMonitor
{
    NativeSensor[] nativeSensors;
    
    static public class NativeSensor
        implements RawSensor, RawSensorData
    {
        long key;
        String name;
        Class type;
        boolean enabled;
        boolean staticSensor;
        int intVal;
        boolean booleanVal;
        float floatVal;
        String stringVal;

        
        public NativeSensor(long key, int type, String name)
            {
                this.key = key;
                this.name = name;

                switch (type) {
                    case 0:
                        this.type = Integer.TYPE;
                        break;
                    case 1:
                        this.type = Float.TYPE;
                        break;
                    case 2:
                        this.type = String.class;
                        break;
                }
            }

        public String getName()
            {
                return name;
            }

        public Class getType()
            {
                return type;
            }

        public String[] getParameterNames()
            {
                    // TO DO:
                throw new NullPointerException("Not yet implemented");
            }

        public Class[] getParameterTypes()
            {
                    // TO DO:
                throw new NullPointerException("Not yet implemented");
            }

        public boolean isEnabled()
            {
                return enabled;
            }
        
        public boolean isStatic()
            {
                return staticSensor;
            }
        
        public void setEnabled(boolean enable)
            {
                    // TO DO:
                throw new NullPointerException("Not yet implemented");
            }
        
        public void setParameter(String p, Object v)
            {
                    // TO DO:
                throw new NullPointerException("Not yet implemented");
            }

        public Object getParameter(String p)
            {
                    // TO DO:
                throw new NullPointerException("Not yet implemented");
            }
        
        public MetaMonitor.RawSensorData probe()
            {
                return this;
            }
        
        public String stringValue()
            {
                if ( Integer.TYPE == type ) {
                    return Integer.toString(intVal);
                }
                else if ( Float.TYPE == type ) {
                    return Float.toString(floatVal);
                }
                else {
                    return stringVal;
                }
            }
    }
    
    static {
        System.loadLibrary("PrimosNativeMonitor");
    }

    public NativeMonitor()
        {
            getSensors();
        }
    
    public void
    quantumExpired()
        {
//             System.out.println("Entering quantumExpired...");
            updateRawSensorData(nativeSensors);
//             System.out.println("Leaving quantumExpired.");
        }
    
    public RawSensor[]
    getSensors()
        {
            System.out.println("get Sensors");

            if ( nativeSensors == null ) {
                nativeSensors = getNativeSensors();
            }

            RawSensor[] s = new RawSensor[nativeSensors.length];

            System.arraycopy(nativeSensors, 0, s, 0, s.length);

            return s;
        }

    private native NativeSensor[] getNativeSensors();
    private native void updateRawSensorData(NativeSensor[] s);
}
