// Copyright 2002 Rodrigo Araujo Real
//		  Luciano Cavalheiro da Silva
//		  Iara Augustin
//		  Adenauer Correa Yamin
//                Gustavo Frainer
//		  Claudio Fernando Resin Geyer
//
// This file is part of TiPS.
//
// TiPS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TiPS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TiPS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: BayesNetworkImpl.java 83 2003-12-17 18:04:29Z rreal $
//

/**
 * @file   BayesNetworkImpl.java
 * @author Rodrigo Real
 * @date   Sat Aug  2 14:30:44 2003
 * 
 * @brief  
 * $Id: BayesNetworkImpl.java 83 2003-12-17 18:04:29Z rreal $
 * 
 */


package org.isam.exehda.services.tips;

import java.util.*;

public class BayesNetworkImpl
    implements BayesNetwork
{
    private final static int Free=0;
    private final static int Low=1;
    private final static int High=2;
    private final static int Over=3;

    private final static int Yes=1;
    private final static int No=0;
    
/** Attributes */
    private int sensorState;

    private double constancy[] = new double[2];
    private double workload[] = new double[4];

        /* [constancy][workload][eWorkload] */
    private double eWorkload[][][] = new double[2][4][4];
    private DataManager dataM;
    private UtilityGen ug;
    
    public BayesNetworkImpl(DataManager dataM, UtilityGen ug) 
        {
            System.out.println("Starting BayesNetwork (CVS $Revision: 83 $) ...");
            this.dataM = dataM;
            this.ug = ug;
            
            constancy[Yes] = 0.5;
            constancy[No] = 0.5;

            workload[Free] = 1;
            workload[Low] = 0;
            workload[High] = 0;
            workload[Over] = 0;

            eWorkload[Yes][Free][Free] = 0.8;
            eWorkload[Yes][Free][Low] = 0.2;
            eWorkload[Yes][Free][High] = 0;
            eWorkload[Yes][Free][Over] = 0;
            
            eWorkload[Yes][Low][Free] = 0;
            eWorkload[Yes][Low][Low] = 0.8;
            eWorkload[Yes][Low][High] = 0.2;
            eWorkload[Yes][Low][Over] = 0;

            eWorkload[Yes][High][Free] = 0;
            eWorkload[Yes][High][Low] = 0;
            eWorkload[Yes][High][High] = 0.8;
            eWorkload[Yes][High][Over] = 0.2;

            eWorkload[Yes][Over][Free] = 0;
            eWorkload[Yes][Over][Low] = 0;
            eWorkload[Yes][Over][High] = 0;
            eWorkload[Yes][Over][Over] = 1;
/* --- */
            eWorkload[No][Free][Free] = 0.4;
            eWorkload[No][Free][Low] = 0.3;
            eWorkload[No][Free][High] = 0.25;
            eWorkload[No][Free][Over] = 0.05;
            
            eWorkload[No][Low][Free] = 0;
            eWorkload[No][Low][Low] = 0.4;
            eWorkload[No][Low][High] = 0.3;
            eWorkload[No][Low][Over] = 0.3;

            eWorkload[No][High][Free] = 0;
            eWorkload[No][High][Low] = 0;
            eWorkload[No][High][High] = 0.4;
            eWorkload[No][High][Over] = 0.6;

            eWorkload[No][Over][Free] = 0;
            eWorkload[No][Over][Low] = 0;
            eWorkload[No][Over][High] = 0;
            eWorkload[No][Over][Over] = 1;
            
        }

    public void eval(String hostid, String sensorName) 
        {
            double value;
            double constancy;


            value = ((Double) dataM.getSensorValue(hostid,sensorName)).doubleValue();
            setConstancy(((Double) dataM.getConstancy(hostid,sensorName)).doubleValue());

            workload[Free]=workload[Low]=workload[High]=workload[Over]=0;

            if (value >= 0 && value < 25)
                sensorState = Over;
            else if (value >= 25 && value < 50)
                sensorState = High;
            else if (value >=50 && value < 75)
                sensorState = Low;
            else if (value >=75 && value <=100)
                sensorState = Free;

            workload[sensorState]=1;
        
            propagateProb(hostid);
            System.out.println("BayesNetwork: eval(" + hostid + "," + sensorName + ")");
        }

    private void setConstancy (double value)
        {
            if (value > 1) {
                value =1;
            } else if (value < 0) {
                value = 0;
            }
        
            constancy[Yes] = value;
            constancy[No] = 1 - value;
        }

        
    private void propagateProb (String hostid)
        {
            double SPr[] = new double[4];
            
            SPr[Free] = constancy[No] * workload[sensorState] * eWorkload[No][sensorState][Free] +
                constancy[Yes] * workload[sensorState] * eWorkload[Yes][sensorState][Free];
            SPr[Low] = constancy[No] * workload[sensorState] * eWorkload[No][sensorState][Low] +
                constancy[Yes] * workload[sensorState] * eWorkload[Yes][sensorState][Low];
            SPr[High] = constancy[No] * workload[sensorState] * eWorkload[No][sensorState][High] +
                constancy[Yes] * workload[sensorState] * eWorkload[Yes][sensorState][High];
            SPr[Over] = constancy[No] * workload[sensorState] * eWorkload[No][sensorState][Over] +
                constancy[Yes] * workload[sensorState] * eWorkload[Yes][sensorState][Over];

            Vector data = new Vector(4);
            data.add(new Double(SPr[Free]));
            data.add(new Double(SPr[Low]));
            data.add(new Double(SPr[High]));
            data.add(new Double(SPr[Over]));
            
            ug.eval(hostid, data);
        }
}
