// Copyright 2002 Rodrigo Araujo Real
//		  Luciano Cavalheiro da Silva
//		  Iara Augustin
//		  Adenauer Correa Yamin
//                Gustavo Frainer
//		  Claudio Fernando Resin Geyer
//
// This file is part of TiPS.
//
// TiPS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TiPS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TiPS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: BayesNetworkImpl2.java 83 2003-12-17 18:04:29Z rreal $
//

/**
 * @file   BayesNetworkImpl2.java
 * @author Rodrigo Real
 * @date   Tue Nov 25 17:28:59 2003

 * @brief BayesNetwork implemented as the way it was disscussed with
 * Luis Paulo Santos, from University of Minho. in this
 * implementation, the constancy is separeted from the CPU data, the
 * utility node 'connects' the data nodes
 *
 * $Id: BayesNetworkImpl2.java 83 2003-12-17 18:04:29Z rreal $
 * 
 */


package org.isam.exehda.services.tips;

import java.util.*;

public class BayesNetworkImpl2
    implements BayesNetwork
{
    private final static int Free=0;
    private final static int Low=1;
    private final static int High=2;
    private final static int Over=3;

    private final static int Yes=1;
    private final static int No=0;
    
/** Attributes */
    private int sensorState;

    private double constancy[] = new double[2];
    private double sConstancy[] = new double[2];
    private double cpu[] = new double[4];
    private double sCpu[] = new double[4];

        /* [cpu][sCpu] */
    private double sCpuT[][] = new double[4][4];
        /* [constancy][sConstancy] */
    private double sConstancyT[][] = new double[2][2];

    private DataManager dataM;
    private UtilityGen ug;
    
    public BayesNetworkImpl2(DataManager dataM, UtilityGen ug) 
        {
            System.out.println("Starting BayesNetwork (CVS $Revision: 83 $) ...");
            this.dataM = dataM;
            this.ug = ug;
            
            constancy[Yes] = 0.5;
            constancy[No] = 0.5;

            sConstancyT[Yes][Yes] = 0.9;
            sConstancyT[Yes][No] = 0.1;
            sConstancyT[No][No] = 0.9;
            sConstancyT[No][Yes] = 0.1;
            
            cpu[Free] = 0.25;
            cpu[Low] = 0.25;
            cpu[High] = 0.25;
            cpu[Over] = 0.25;

            sCpuT[Free][Free] = 0.8;
            sCpuT[Free][Low] = 0.1;
            sCpuT[Free][High] = 0.1;
            sCpuT[Free][Over] = 0;
            
            sCpuT[Low][Free] = 0.05;
            sCpuT[Low][Low] = 0.8;
            sCpuT[Low][High] = 0.1;
            sCpuT[Low][Over] = 0.05;

            sCpuT[High][Free] = 0;
            sCpuT[High][Low] = 0.1;
            sCpuT[High][High] = 0.8;
            sCpuT[High][Over] = 0.1;

            sCpuT[Over][Free] = 0;
            sCpuT[Over][Low] = 0;
            sCpuT[Over][High] = 0.05;
            sCpuT[Over][Over] = 0.95;            
        }

    public void eval(String hostid, String sensorName) 
        {
            double value;
            double constancy;


            value = ((Double) dataM.getSensorValue(hostid,sensorName)).doubleValue();
            setConstancy(((Double) dataM.getConstancy(hostid,sensorName)).doubleValue());

            sCpu[Free]=sCpu[Low]=sCpu[High]=sCpu[Over]=0.0;

            if (value >= 0 && value < 25)
                sensorState = Over;
            else if (value >= 25 && value < 50)
                sensorState = High;
            else if (value >=50 && value < 75)
                sensorState = Low;
            else if (value >=75 && value <=100)
                sensorState = Free;

            sCpu[sensorState]=1.0;
        
            propagateProb(hostid);
//            System.out.println("BayesNetwork: eval(" + hostid + "," + sensorName + ")");
        }

    private void setConstancy (double value)
        {
            if (value > 1) {
                value =1;
            } else if (value < 0) {
                value = 0;
            }
        
            sConstancy[Yes] = value;
            sConstancy[No] = 1 - value;
        }

        
    private void propagateProb (String hostid)
        {
            double finalCpu[] = new double[4];
            double finalConstancy[] = new double[2];
            double normalizationFact;

            finalCpu[Free] = sCpuT[Free][sensorState] * cpu[Free];
            finalCpu[Low] = sCpuT[Low][sensorState] * cpu[Low];
            finalCpu[High] = sCpuT[High][sensorState] * cpu[High];
            finalCpu[Over] = sCpuT[Over][sensorState] * cpu[Over];
            
            normalizationFact = finalCpu[Free] + finalCpu[Low] + finalCpu[High] + finalCpu[Over];

            finalCpu[Free] = finalCpu[Free] / normalizationFact;
            finalCpu[Low] = finalCpu[Low] / normalizationFact;
            finalCpu[High] = finalCpu[High] / normalizationFact;
            finalCpu[Over] = finalCpu[Over] / normalizationFact;

            
            finalConstancy[Yes] = sConstancyT[Yes][Yes] * sConstancy[Yes] + sConstancyT[No][Yes] * sConstancy[No] ;
            finalConstancy[No] = 1 - finalConstancy[Yes];

            BayesNetworkData2 bayesData = new BayesNetworkData2(finalCpu,finalConstancy[Yes]);
            
            ug.eval(hostid, bayesData);
        }
}
