// Copyright 2002 Rodrigo Araujo Real
//		  Luciano Cavalheiro da Silva
//		  Iara Augustin
//		  Adenauer Correa Yamin
//                Gustavo Frainer
//		  Claudio Fernando Resin Geyer
//
// This file is part of TiPS.
//
// TiPS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TiPS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TiPS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: CompManagerImpl.java 283 2004-10-15 20:19:08Z rreal $
//

package org.isam.exehda.services.tips;

import java.net.*;
import java.lang.reflect.*;

public class CompManagerImpl
    extends java.rmi.server.UnicastRemoteObject
    implements CompManager
{
    private DataManager dataM;
    private BayesNetwork bn;
    private Learner l;
    private UtilityGen ug;
    private TipsCellScheduler tips;


    public CompManagerImpl(TipsCellScheduler tips) 
        throws java.rmi.RemoteException 
        {
            this.tips = tips;
            System.out.println("CompManager ($Revision: 283 $) started");
        }
    
    public void loadComponent(String url, String className)
        throws java.rmi.RemoteException, java.net.MalformedURLException 
        {
            System.out.println("Loading new Class from: " + url);

                // download and instantiate the Class object
            Class c = null;
            URLClassLoader cLoader = new URLClassLoader(new URL[] {new URL(url)});
            try {
                c = Class.forName(className, true, cLoader);
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }

                // verify which component should be switched by the
                // verification of the interface name
            try {
                if ( BayesNetwork.class.isAssignableFrom(c) ) {
                        // must change the BayesNetwork component
                    System.out.println("Loading " + (c.getInterfaces()[0]).toString());
                    Constructor bnConst = c.getDeclaredConstructor(new Class[] {DataManager.class, UtilityGen.class} );
                    bn = (BayesNetwork) bnConst.newInstance(new Object[] {dataM,ug});
                    tips.setBayesNetwork(bn);
                    
                } else if (Learner.class.isAssignableFrom(c) ) {
                        // must change the Learner component
                    System.out.println("Loading " + (c.getInterfaces()[0]).toString());
                    Constructor lConst = c.getDeclaredConstructor(new Class[] {DataManager.class, BayesNetwork.class} );
                    l = (Learner) lConst.newInstance(new Object[] {dataM,bn});
                    tips.setLearner(l);
                    
                } else if ( UtilityGen.class.isAssignableFrom(c) ) {
                        // must change the UtilityGen component
                    System.out.println("Loading " + (c.getInterfaces()[0]).toString());
                    Constructor ugConst = c.getDeclaredConstructor(new Class[] {DataManager.class} );
                    ug = (UtilityGen) ugConst.newInstance(new Object[] {dataM});
                    tips.setUtilityGen(ug);
                }
            } catch(Exception e) {
                e.printStackTrace();
            }            
        }
}

    
