// Copyright 2002 Rodrigo Araujo Real
//		  Luciano Cavalheiro da Silva
//		  Iara Augustin
//		  Adenauer Correa Yamin
//                Gustavo Frainer
//		  Claudio Fernando Resin Geyer
//
// This file is part of TiPS.
//
// TiPS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// TiPS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with TiPS; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
// $Id: DataManager.java 83 2003-12-17 18:04:29Z rreal $
//

/**
 * @file   DataManager.java
 * @author Rodrigo Real
 * @date   Fri Aug  1 14:05:48 2003
 * 
 * @brief Interface for building DataManagers for TiPS. DataManagers
 * manages the information that is important to the scheduler.
 * $Id: DataManager.java 83 2003-12-17 18:04:29Z rreal $
 * 
 */


package org.isam.exehda.services.tips;

import java.util.*;

public interface DataManager 
{
        /** 
         * Returns the last received value of the sensor identified by
         * sensorName value for the hostid host
         * 
         * @param hostid Identification of the host
         * @param sensorName Identification of the sensor
         * 
         * @return an Object which corresponds to the last received data
         */
    public Object getSensorValue(String hostid, String sensorName);

        /** 
         * Get an object with the history of the readings of the
         * hostid, sensor pair
         * 
         * @param hostid  Identification of the host
         * @param sensorName Identification of the sensor
         * 
         * @return an Object with the history of the received data
         */
    public Object getSensorHistory(String hostid, String sensorName);
    public void setSensorValue(String hostid, String sensorName, Object value);

    public boolean hasConstancy(String hostid, String sensorName);
    public Double getConstancy(String hostid, String sensorName);
    public void setConstancy(String hostid, String sensorName, Double value);

    public Double getUtility(String hostid);
    public void setUtility(String hostid, Double utility);
    public Hashtable getUtilityTable();
}

