// Copyright 2002 Marcelo Augusto Cardozo Junior,
// 		  Luciano Cavalheiro da Silva
//		  Rodrigo Araujo Real
//		  Adenauer Correa Yamin
//		  Claudio Fernando Resin Geyer
//
// This file is part of EXEHDA-CC.
//
// EXEHDA-CC is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// EXEHDA-CC is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.tips;

/**
* Abstraction for a tuple as defined in the Linda Model
*
*/
public class Tuple implements java.io.Serializable {
        /** element holder for the tuple */
    java.util.Vector elements;
    
        /**
        * Constructor code
        */
    public Tuple() 
        {
            elements = new java.util.Vector();
        }

    private Tuple(java.util.Vector e) 
        {
            elements = (java.util.Vector) e.clone();
        }
    
    
        /**
        * Append an element to the tuple
        *
        * @param o the element to be appended
        */
    public Tuple add( Object o )
        {
            elements.add(o);
            return this;
        }

        /**
        * Return an element from the tuple
        *
        * @param index index of the element
        * @return Element at given index
        * @exception java.util.NoSuchElementException if the index is invalid
        */
    public Object get ( int index ) throws java.util.NoSuchElementException
        {
            return(elements.get(index));
        }
        /**
        * Returns the tuple's length
        *
        * @return The length of the tuple
        */
    public int length() 
        {
            return(elements.size());
        }

        /**
        * Compares this tuple to the given pattern starting with the
        * tuple element at the given index.
        *
        * If the given pattern is longer that this tuple, the matchig is
        * said to fail.
        * 
        * @param pattern The pattern to be matched.
        * @param startIndex The index from which the matching should start.
        * @return true if the tuple matches the pattern, false otherwise.
        */
    public boolean matchesPattern(Tuple pattern, int startIndex)
        {
                // if the pattern if longer that the tuple, matching fails
            if (pattern.length()>this.length()){
                return false;
            }
                // for each element in the pattern, starting at startIndex,
            for (int i=startIndex; i<pattern.length(); i++) {       
                    // compare it to the corresponding element in the tuple                
                if ( !this.get(i).equals(pattern.get(i)) ){
                        // matching fails
                    return false;
                }
            }//for
                // successfull matching
            return true;
        }//matchesPattern

        /**
        * String representation for this tuple
        */
    public String toString()
        {
            return elements.toString();
        }

    public Object clone() 
        {
            return(new Tuple(elements));            
        }
    
}

