
package exehda.mw;

import java.net.URL;
import java.net.URLClassLoader;
import java.net.JarURLConnection;
import java.net.MalformedURLException;
import java.util.jar.Attributes;
import java.lang.reflect.*;

/**
 * 
 * 
 */
public class Starter
    extends Thread 
{    

        /** 
         * Starter entry point
         * 
         * @param args command-line arguments
         */
    public static void
    main(String[] args) throws Exception
        {
            URLClassLoader appClassLoader = new URLClassLoader(
                new URL[]{ 
                    new URL(args[0])});

                // skip parameter #0
            String[] appargs = new String[args.length-1];
            System.arraycopy(args, 1, appargs, 0, appargs.length);
        
            if (args[1].equals("-master")) {
                    /*
                     * Install rmi security manager
                     */
                System.setProperty("java.rmi.server.codebase",args[0]);
                System.setProperty("java.security.policy","java.policy");
                System.out.println("Application Codebase: "+args[0]);
            
                System.setSecurityManager(new java.rmi.RMISecurityManager());
                    /*
                     * create a rmiregistry if one is not alredy running
                     */
                try {
                    java.rmi.registry.LocateRegistry.createRegistry(1099);
                    System.out.println("Created registry.");
                } catch (java.rmi.RemoteException re){
                    System.out.println("Registry already running.");
                }                                                     
            } else {
                System.out.println(args[1]);
            }
            startApp(args[0], appargs, appClassLoader);

        }
    

        /** 
         * Application code runs under the created URL class loader since the thread
         * itself was loaded using that class loader. This way application classes would
         * be found without using explicit loadClass(calls);
         * 
         * @param appURL 
         * @param appArgs 
         * @param appClassLoader 
         */
    public static void 
    startApp(String appURL, String[] appArgs, ClassLoader appClassLoader )
        { 
            try {
                    // look up for the main class attribute (application entry point)
                URL appJarURL = new URL("jar","",appURL+"!/");
                JarURLConnection uc = (JarURLConnection) appJarURL.openConnection();
                Attributes jarAttr = uc.getMainAttributes();
            
                String mcn = (jarAttr == null ?
                              null : jarAttr.getValue(Attributes.Name.MAIN_CLASS));
            
                    // if it was found try to run the main method on that class
                if (mcn != null) {
                        // load the main class
                    Class mainClass = appClassLoader.loadClass(mcn);
                
                        // invoke main on the main class
                    Method m = mainClass.getMethod("main", new Class[]{String[].class});
                    m.invoke(null,new Object[]{appArgs});
                        // at this point application should be running
                }
                else {
                    System.out.println("No Main-Class attribute found in the jar file.");
                    System.exit(1);
                }
            }
            catch (java.lang.reflect.InvocationTargetException ite){
                System.out.println("nested: "+ite.getTargetException());
                ite.getTargetException().printStackTrace();
            }
            catch (Exception e){
                System.out.println("--"+e+"--");
            }
        }
}
