import org.apache.tools.ant.Project;
import org.apache.tools.ant.Target;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.MacroInstance;
import org.apache.tools.ant.taskdefs.Available;
import org.apache.tools.ant.types.Path;
import java.io.File;



/**
 * Generates on-the-fly fetch,build and deploy targets based on defined module
 * properties.
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-06-14 15:52:52 -0300 (Tue, 14 Jun 2005) $ $Revision: 905 $
 */
public class ModulesSetup
    extends Task
{
    private final String SRC_PATH_CHECK = "/CVS/Root".replace('/', File.separatorChar);
    
    private final String DFLT_PREFIX = "module.default";
    private final String MODULE_NAME=".name";
    private final String MODULE_PKG=".pkg.name";
    private final String MODULE_PKG_PREFIX=".pkg.prefix";
    private final String MODULE_CVS_ROOT=".cvs.root";
    private final String MODULE_CVS_PKG=".cvs.pkg";
    private final String MODULE_CVS_TAG=".cvs.tag";
    private final String MODULE_BUILD_TARGET=".build.target";
    private final String MODULE_BUILD_ENABLED=".build.enabled";
    private final String MODULE_BUILD_DEBUG=".build.debug";
    private final String MODULE_DEPLOY_TARGET=".deploy.target";
    private final String MODULE_DEPLOY_DISTFILE=".deploy.distfile";
    private final String MODULE_CLASSPATH=".classpath";
    
    
    public void
    execute()
        {
            Target build = (Target) getProject().getTargets().get("build");
            Target fetch = (Target) getProject().getTargets().get("fetch");
            Target deploy = (Target) getProject().getTargets().get("deploy");
            
            log("Detecting modules...");

            for ( ; hasMoreModules(); nextModule() ) {

                if ( isModuleEnabled() ) {
                    log("Configuring module '" + getModuleName() + "'...");

                    createModuleFetchTask(fetch);
                    createModuleBuildTask(build);
                    createModuleDeployTask(deploy);
                }
                else {
//                    log("Skiping module '" + getModuleName() + "'...");
                }
            }
                // Task's log method
            log("Module detection concluded.");
        }
    
    private final boolean
    hasMoreModules()
        {
            return getProject().getProperty(mprefix+MODULE_NAME) != null;
        }

    private final void
    nextModule()
        {
            ++idx;
            this.mprefix = "module."+idx;
        }

    private final String
    getModuleName()
        {
            return getProject().getProperty(mprefix+MODULE_NAME);
        }

    private final String
    getModulePkg()
        {
            return getProject().getProperty(mprefix+MODULE_PKG);
        }

    private final boolean
    isModuleEnabled()
        {
            String p = getProject().getProperty(mprefix+MODULE_BUILD_ENABLED);
            if ( p == null ) {
                p = getProject().getProperty(DFLT_PREFIX+MODULE_BUILD_ENABLED);
            }

            return Boolean.valueOf(p).booleanValue();
        }

    private final String
    getModuleBuildTarget()
        {
            String p = getProject().getProperty(mprefix+MODULE_BUILD_TARGET);
            if ( p == null ) {
                p = getProject().getProperty(DFLT_PREFIX+MODULE_BUILD_TARGET);
            }
            return p;
        }

    private final String
    getModuleBuildDebug()
        {
            String p = getProject().getProperty(mprefix+MODULE_BUILD_DEBUG);
            if ( p == null ) {
                p = getProject().getProperty(DFLT_PREFIX+MODULE_BUILD_DEBUG);
            }
            return p;
        }

    private final String
    getModuleClassPath()
        {
            String p = getProject().getProperty(mprefix+MODULE_CLASSPATH);
            if ( p == null ) {
                p = getProject().getProperty(DFLT_PREFIX+MODULE_CLASSPATH);
            }
            
            return p;
        }

    private final String
    getModuleDeployTarget()
        {
            String p = getProject().getProperty(mprefix+MODULE_DEPLOY_TARGET);
            if ( p == null ) {
                p = getProject().getProperty(DFLT_PREFIX+MODULE_DEPLOY_TARGET);
            }
            return p;
        }

    private final String
    getModuleDeployDistFile()
        {    
            String p = getProject().getProperty(mprefix+MODULE_DEPLOY_DISTFILE);
            if ( p == null ) {
                String prefix = getProject().getProperty(
                    mprefix+MODULE_PKG_PREFIX);
                if ( prefix == null ) {
                    prefix = getProject().getProperty(
                        DFLT_PREFIX+MODULE_PKG_PREFIX);
                    if ( prefix == null ) {
                        prefix = "";
                    }
                }
                p = prefix + getModulePkg();
            }
            return p.endsWith(".jar") ? p : (p + ".jar");
        }
    
    private final String
    getModuleCVSRoot()
        {
            String p = getProject().getProperty(mprefix+MODULE_CVS_ROOT);
            if ( p == null ) {
                p = getProject().getProperty(DFLT_PREFIX+MODULE_CVS_ROOT);
            }
            return p;
        }
    private final String
    getModuleCVSPkg()
        {
            String p = getProject().getProperty(mprefix+MODULE_CVS_PKG);
            if ( p == null ) {
                p = getProject().getProperty(DFLT_PREFIX+MODULE_CVS_PKG);
            }
            return p;
        }

    private final String
    getModuleCVSTag()
        {
            String p = getProject().getProperty(mprefix+MODULE_CVS_TAG);
            if ( p == null ) {
                p = getProject().getProperty(DFLT_PREFIX+MODULE_CVS_TAG);
            }
            return p;
        }

    private final void
    createModuleFetchTask(Target fetch)
        {

            Available src_present = new Available();
            src_present.setProject(getProject());            
            src_present.setFile( new java.io.File(
                                     getProject().getProperty("srcdir"),
                                     getModulePkg()+SRC_PATH_CHECK));

            MacroInstance m = null;
            
            if ( src_present.eval() ) {
                m = (MacroInstance) getProject().createTask("module-update");
                
                m.setDynamicAttribute("module.name", getModuleName());
                m.setDynamicAttribute("module.pkg.name", getModulePkg());
                m.setDynamicAttribute("module.cvs.tag", getModuleCVSTag());
            }
            else {
                m = (MacroInstance) getProject().createTask("module-install");
                
                m.setDynamicAttribute("module.name", getModuleName());
                m.setDynamicAttribute("module.pkg.name", getModulePkg());
                m.setDynamicAttribute("module.cvs.root", getModuleCVSRoot());
                m.setDynamicAttribute("module.cvs.pkg", getModuleCVSPkg());
                m.setDynamicAttribute("module.cvs.tag", getModuleCVSTag());
            }

            fetch.addTask(m);
        }
    
    private final void
    createModuleBuildTask(Target build)
        {
            MacroInstance m = (MacroInstance)
                getProject().createTask("module-build");

            m.setDynamicAttribute("module.name", getModuleName());
            m.setDynamicAttribute("module.pkg.name", getModulePkg());
            m.setDynamicAttribute("module.build.target", getModuleBuildTarget());
            m.setDynamicAttribute("module.build.debug", getModuleBuildDebug());
            m.setDynamicAttribute("module.classpath", getModuleClassPath());

            build.addTask(m);
        }

    private final void
    createModuleDeployTask(Target deploy)
        {
            MacroInstance m = (MacroInstance)
                getProject().createTask("module-deploy");

            m.setDynamicAttribute("module.name", getModuleName());
            m.setDynamicAttribute("module.pkg.name", getModulePkg());
            m.setDynamicAttribute("module.deploy.target", getModuleDeployTarget());
            m.setDynamicAttribute("module.deploy.distfile", getModuleDeployDistFile());
            m.setDynamicAttribute("module.classpath", getModuleClassPath());
            
            deploy.addTask(m);
        }
    
    private int idx = 0;
    private String mprefix = "module."+idx;
}
