// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.cc;

import org.isam.exehda.Exehda;
import org.isam.exehda.services.CCManager;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Worb;
import java.util.Hashtable;

public class CCManagerImpl
    extends AbstractService
    implements CCManager
{
    private static final int HASH_LEVELS = 2;
    private long lastId = 0;
    private Hashtable spacesByName;

    public CCManagerImpl()
        {
            spacesByName = new Hashtable();
        }

    public void
    start()
        {
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            worb.exportService(this, CCManager.class, CCManager.SERVICE_NAME);
        }

    public void
    stop()
        {
                // TODO: unexport service
        }
    
    public TupleSpace
    createSpace()
        {
            String id = genSpaceId();

            HashTreeSpace space = new HashTreeSpace( HASH_LEVELS, id );
            spacesByName.put(id, space);

            TupleSpace ts = new TupleSpace(id);
            ts.setManager(this);
            
            return ts;
        }
    
    public void
    destroySpace(TupleSpace space)
        {
            HashTreeSpace hts = findSpace(space);
            throw new NullPointerException("Not yet implemented");
        }
    
    public Tuple
    in(Object spaceId, Tuple pattern, boolean remove, long timeout)
        {
            HashTreeSpace hts = findSpace(spaceId);
            
            if ( remove ) {
                return hts.take(pattern, timeout);
            }
            else {
                return hts.read(pattern, timeout);
            }
        }
    
    public void
    out(Object spaceId, Tuple t)
        {
            HashTreeSpace hts = findSpace(spaceId);

            hts.put(t);
        }
    
    public TupleSpace
    joinSpace(Object spaceName)
        {
                // espaos de tuplas so recursos registrados na CIB
            throw new NullPointerException("Not yet implemented");
        }
    
    public void
    leaveSpace(TupleSpace space)
        {
            throw new NullPointerException("Not yet implemented");
        }

    private final String genSpaceId()
        {
            return "Space"+(lastId++);
        }

    private final HashTreeSpace findSpace(Object spaceId)
        {
            HashTreeSpace hts = (HashTreeSpace) spacesByName.get(spaceId);

            if ( hts == null ) {
                throw new IllegalArgumentException("No such space");
            }

            return hts;
        }
}

