// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.ox;

import org.isam.exehda.Exehda;
import org.isam.exehda.ObjectId;
import org.isam.exehda.HostId;
import org.isam.exehda.ApplicationId;
import org.isam.exehda.services.AbstractService;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.OXManager;
import org.isam.exehda.services.OXManager.OXHandle;
import org.isam.exehda.services.OXManager.MasterOXHandle;
import org.isam.exehda.services.OXManager.HomeOXHandle;
import org.isam.exehda.services.OXManager.ProxyOXHandle;
import org.isam.exehda.services.OXManager.IllegalOXHandleException;
import org.isam.exehda.services.CellInformationBase;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameComponent;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.Worb;
import java.util.Hashtable;

/**
 * @author Implemented by frainer, last modified by $Author: lucc $
 * @version $Date: 2005-04-08 15:24:11 -0300 (Fri, 08 Apr 2005) $ $Revision: 734 $ 
 */
public class OXManagerImpl
    extends AbstractService
    implements OXManager
{    
    final static String PROP_OX_URI = "contactAddress";
    final static String PROP_CIB_URI = "contactAddress";
    

    
    public void
    start()
        {
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            worb.exportService(this,
                               OXManager.class,
                               OXManager.SERVICE_NAME);
        }

    public void
    stop()
        {
                //
                // TODO: unexport service
                //
        }
    
        /**
         * Creates a new instance of OXManagerImpl
         */
    public OXManagerImpl()
        {
            handles = new Hashtable();
            masterHandles = new Hashtable();
            timeBetweenUpdates = 1000;
            localClient = new OXManagerClientImpl(this);
            flag = false;
        }
    
        /**
         * Create handle is used to create a new OXHandle for the OX
         * with the given ObjectId. Wheter a new MasterOXHanlde or a new
         * ProxyOXhandle will be created depends on wheter an OXHandle had
         * already been created for the given OX or not
         *
         * @param oid a <code>ObjectId</code> representing the OX to which the OXHandle
         * to be created will refer
         * @return a new <code>OXHandle</code>
         */
    public OXHandle
    createHandle(org.isam.exehda.ObjectId oid)
        {
            if ( isLocalAccess() && (!flag) ) {
                flag = true;
                OXHandle aux = localClient.createHandle(oid);
                flag = false;
                return (aux);
            }
            
            HomeOXHandle homeHandle = new HomeOXHandle(oid);
            handles.put(oid,homeHandle);
            
                //System.out.println("**************adding:"+oid);
//             return (new ProxyOXHandle(oid,timeBetweenUpdates));
            return new ProxyOXHandle(oid);
        }
    

    
        /**
         * Grant ownership invalidates the ownership of the given OXHandle, allowing a
         * new node to request the ownership of this OXHanlde. While the OXHandle
         * ownership is undefined all calls to that handle get queed until a new
         * ownership is requested.  To be used when an OXhandle migrates from one node to
         * another.
         *
         * @param ox a <code>OXHandle</code> whose ownership is being granted
         * @return a <code>OXhandle</code>
         */
    public OXHandle
    grantOwnership(OXHandle ox)
        throws IllegalOXHandleException //HandleNotFoundException, NotMasterHandleException
        {
                //
                // TO DO: review this implementation
                //
            throw new NullPointerException("Not yet implemented");
            
//             Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
//             if ( (worb.getClientHost() == null) && (!flag) ) {
//                 flag = true;
//                 OXHandle aux =localClient.grantOwnership(ox);
//                 flag = false;
//                 return(aux);
//             }
//             if ( !ox.getId().getHost().getCellName().equals(HostId.getLocalHost().getCellName()) ) {
//                 String contactAddress =
//                     getOXMContactAddress(ox.getId().getHost().getCellName());
//                 OXManager cellManager =
//                     (OXManager)worb.lookupService(contactAddress, OXManager.class);
                
//                 cellManager.grantOwnership(ox);
//             }
//             if ( ox instanceof MasterOXHandle ) {
//                     //System.out.println("**************granting:"+ox.getId());
//                 if ( handles.containsKey(ox.getId()) ) {
//                         //System.out.println("**************granting:"+ox.getId());
//                     HomeOXHandle homeHandle = (HomeOXHandle)handles.get(ox.getId());
//                     homeHandle.setOwner(null);
//                     double code = java.lang.Math.random();
//                     homeHandle.setOwnershipCode(""+code);
//                     ox.setOwnershipCode(""+code);
//                     OXHandle proxy = new ProxyOXHandle(ox.getId());
//                     proxy.setOwnershipCode(""+code);
//                     masterHandles.put(ox.getId(), ox);

//                         //
//                         // FIX ME: senseless since ox is actually just a reference
//                         //
// //                     ox = new InvalidOXHandle();
                    
//                         //System.out.println("**************granting:"+proxy);
//                     return(proxy);
//                 }
//                 else {
//                     throw new HandleNotFoundException(); 
//                 }
//             }
//             else{
//                 throw new NotMasterHandleException();
//             }
       
        }
    
        /**
         * Realease handle takes away the given handle from the handle list.
         * Only to be uswed when the OX is no longer necessary.
         *
         * @param ox a <code>OXHandle</code> that is to be realeased
         * @return void
         */
    public void
    releaseHandle(OXHandle ox)
        {
                //
                // TO DO: add implementation
                //
            throw new NullPointerException("Not yet implemented");
        }
    
        /**
         * requestWnershp grants the ownership of the given OXHandle
         * to the calling HostId as long as that OXHandle is not currently owned
         * by another host.
         *
         * @param ox a <code>OXHandle</code> whose ownership is beign requested
         * @return a <code>OXHandle</code> value
         */
    public OXHandle
    requestOwnership(OXHandle ox)
        throws IllegalOXHandleException //HandleNotFoundException, WrongHandleException, NotMasterHandleException, WrongOwnershipCodeException
        {
                //
                // TO DO: review this implementation
                //
            throw new NullPointerException("Not yet implemented");
            
//             Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
//             if ( (worb.getClientHost() == null)&&(!flag) ) {
//                 flag = true;
//                 OXHandle aux =localClient.requestOwnership(ox);
//                 flag = false;
//                 return(aux);
//             }
//             if( !ox.getId().getHost().getCellName().equals(HostId.getLocalHost().getCellName()) ) {
           
//                 String contactAddress = getOXMContactAddress(ox.getId().getHost().getCellName());
//                 OXManager cellManager = (OXManager)worb.lookupService(contactAddress,
//                                                                       OXManager.class);
//                 cellManager.requestOwnership(ox);
            
//             }
//             if((handles.containsKey(ox.getId()))&&(masterHandles.containsKey(ox.getId()))){
//                 HomeOXHandle homeHandle = (HomeOXHandle)handles.get(ox.getId()); 
//                 if(homeHandle.getOwner()==null){
//                         //System.out.println(homeHandle.getOwnershipCode()+"-"+ox.getOwnershipCode());
//                     if(homeHandle.getOwnershipCode().equals(ox.getOwnershipCode())){
//                             //teste
//                         homeHandle.setOwner((HostId)ox.getOwner());
//                         MasterOXHandle master = (MasterOXHandle)masterHandles.get(ox.getId());
//                         master.setAttribute("owner", ox.getOwner());

//                             // FIX ME: senseless since ox is actually just a reference
// //                         ox = new InvalidOXHandle();
                        
//                         return(master);
//                     }
//                     else{
//                         throw new WrongOwnershipCodeException();
//                     }
//                 }
//                 else{
//                     throw new WrongHandleException();
//                 }
//             }
//             else{
//                 throw(new HandleNotFoundException());  
//             }
            
        }
    
    
        /**
         * OXhandle updates the attributes of the given ox. The attributes to be updated
         * are defined in a String array. What type of update is to be done (read or write)
         * is defined in an byte array.
         *
         * @param ox a <code>OXHanlde</code> whose attributes are to be updated
         * @param att an <code>String[]</code> defining what attributes are to be updated
         * @return a <code>OXHandle</code> that is the OXHandle passed as parameter with
         * it's attributes updated
         */
    public OXHandle
    updateAttributes(OXHandle ox, String[] att, byte[] op)
        throws IllegalOXHandleException
        {
            log(Sev.DEBUG, "updateAttributes()", "entering");
                //
                // TO DO: review this implementation  <============== STOPED HERE =================
                //
            throw new NullPointerException("Not yet implemented");
            
//             Worb worb = getServiceWorb();
            
//             if ( (worb.getClientHost() == null) && (!flag) ){
//                 flag = true;
//                 OXHandle aux = localClient.updateAttributes(ox,att,op);
//                 flag = false;
//                 return aux;
//             }
            
//             if ( !ox.getId().getHost().getCellName().equals(HostId.getLocalHost().getCellName())) {
           
//                 String contactAddress = getOXMContactAddress(ox.getId().getHost().getCellName());
//                 OXManager cellManager = (OXManager)worb.lookupService(contactAddress, OXManager.class);
//                 cellManager.updateAttributes(ox,att,op);
            
//             }
            
//             if ( handles.containsKey(ox.getId()) ) {
//                 HomeOXHandle homeHandle = (HomeOXHandle)handles.get(ox.getId()); 
//                 if(homeHandle.getOwner()!=null){
//                         //System.out.println("*******************N?o co");
//                     OXManager oxManager;
               
//                     String oxUri = Exehda.getServiceProperty(OXManager.SERVICE_NAME,
//                                                              PROP_OX_URI,
//                                                              homeHandle.getOwner().getName()+"/ox");
//                     oxManager = (OXManager)worb.lookupService(oxUri, OXManager.class);
//                     return(oxManager.updateAttributes(ox,att,op));
//                 }
//                 else if ( masterHandles.containsKey(ox.getId()) ) {
//                         //System.out.println("*******************N?o continha 2 ");
//                     MasterOXHandle mHandle = (MasterOXHandle)masterHandles.get(ox.getId());
                    
//                     for (int i = 0; i< att.length; i++) {
//                         if ( op[i] == ATT_WRITE ) {
//                             mHandle.setAttribute(att[i], ox.getAttribute(att[i]));
//                         }
//                         else if ( op[i] == ATT_READ ) {
//                             ox.setAttribute(att[i], mHandle.getAttribute(att[i]));
//                         }
//                     }
                    
//                     OXHandle aux = ox;

//                         // FIX ME: senseless assigning any value to ox since its actually just a reference
// //                     ox = new InvalidOXHandle();
//                     return(aux);
//                 }
//                 else {
//                         //System.out.println("*******************N?o continha");
//                 }
//             }
//             else {
//                 throw new HandleNotFoundException();  
//             }
            
//             return null;

//             log(Sev.DEBUG, "updateAttributes()", "leaving");
        }

        ///////////////////////////////////////////////////////////////////////
        // PRIVATE STUFF
        ///////////////////////////////////////////////////////////////////////

    private final Worb
    getServiceWorb()
        {
            return (Worb) Exehda.getService(Worb.SERVICE_NAME);
        }

    
        /**
         * Detects whether the current method invocation is a local or a remote acess.
         *
         * @return true if its a local access, false if its coming through the Worb
         * service.
         */
    private final boolean
    isLocalAccess()
        {
            return getServiceWorb().getClientHost() == null;
        }

//     private final void
//     enterHandleUpdate(ObjectId oxID)
//         {
//         }

//     private final void
//     leaveHandleUpdate(ObjectId oxID)
//         {
//         }

//     private final boolean
//     isHandleUpdateInCourse( ObjectId oxID )
//         {
//         }
    
        /**
         * Returns the contact address of the base OXManger of the given cell.
         *
         * @param cell a <code>String</code> with the name of the cell whose OXM contact
         * address is to be found.
         * @return a <code>String</code> with the contact address of the desired OXM
         */
    private String
    getOXMContactAddress(String cell)
        {
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);
            String cibCellUri = Exehda.getServiceProperty(CellInformationBase.SERVICE_NAME,
                                                          PROP_CIB_URI,
                                                          "//localhost:1980/cib");

            CellInformationBase cibCell = (CellInformationBase)
                worb.lookupService(cibCellUri, CellInformationBase.class);
            ResourceName[] results =
                cibCell.selectByName("oxm",new NameSpace("cell",cell,null),-1);
            
            if (results.length > 0) {
                return(cibCell.getAttribute(results[0],"contactAddress"));
            }
            return(null);
        }


    

    private Hashtable handles;
    private Hashtable masterHandles;
    private long timeBetweenUpdates;
    private OXManagerClientImpl localClient;
    private boolean flag;
}
