package dimi;

import dimi.message.Message;
import dimi.message.MessageHeader;
import dimi.message.MessageListener;
import dimi.ConsumerEntryPoint;
import dimi.name.ConsumerEntryPointName;

import java.rmi.RemoteException;

/**
 * A implementacao do ponto de entrada ao usuario consumidor
 * de um canal. Um usuario real deve estender esta classe e implementar
 * o metodo disseminate(Message) para receber mensagens.
 * @author mcmoraes
 */
public class ConsumerEntryPointImpl extends UserEntryPointImpl implements ConsumerEntryPoint{
    
    private MessageListener messageListener = null;
    
    /**
     * Creates a new instance of ConsumerEntryPointImpl
     * @throws RemoteException if failed to export object
     */
    ConsumerEntryPointImpl(ConsumerEntryPointName name, MessageListener messageListener) throws RemoteException{
        super(name);
        this.messageListener = messageListener;
        setActive(true);
    }
    
    /**
     * Chamado toda vez um mensagem nova e entregue pelo canal.
     * @param message a nova mensagem
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public final synchronized void disseminate(Message message) throws RemoteException{
        if(!isActive()){
            throw new RemoteException();
        }
        if(message.getHeader().getType() == MessageHeader.DESTROY_CHANNEL){
            destroy();
        }
        messageListener.onMessage(message);
    }
    
    /**
     * Finaliza a operacao desse ponto de entrada.
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public final synchronized void destroy() throws RemoteException{
        setActive(false);
    }
    
    /**
     * Getter for property messageListener.
     * @return Value of property messageListener.
     */
    public MessageListener getMessageListener() {
        return messageListener;
    }
        
}
