package dimi;

import java.util.Set;
import java.util.List;
import java.rmi.RemoteException;
import dimi.exception.CantFindChannelException;
import dimi.exception.CantFindDBaseException;
import dimi.exception.CantEnterChannelException;
import dimi.configuration.DisseminationUnitConfiguration;
import dimi.configuration.ChannelConfiguration;
import dimi.name.ChannelName;
import dimi.name.DNodeName;
import dimi.channel.ChannelData;
import dimi.name.DBaseName;

/**
 * Principal interface do servico. Sua funcao principal funcao e permitir a 
 * comunicacao e conexao entre disseminadores a fim de criar e manter canais
 * que sejam coerentes com a rede fisica subjacente.
 * @author  mcmoraes
 */
public interface DBase extends DisseminationUnit{
    
    /**
     * Obtem os filhos do disseminador em um canal.
     * @return os filhos do disseminador no canal
     * @param channelName o nome do canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public DBase[] getChildren(ChannelName channelName) throws RemoteException;
    
    /**
     * Obtem os pais do disseminador em um canal.
     * @return os pais do disseminador no canal
     * @param channelName o nome do canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public DBase[] getParents(ChannelName channelName) throws RemoteException;
    
    /**
     * Obtem os pais o os filhos da unidade de disseminacao em um canal.
     * @param channelName o nome do canal
     * @throws RemoteException if failed to export object
     * @return os pais e os filhos da unidade de disseminacao em um canal
     */    
    public DBase[] getRelatives(ChannelName channelName) throws RemoteException;
    
    /**
     * Obtem as unidades de disseminacao conhecidas em um canal.
     * @param channelName o nome do canal
     * @throws RemoteException if failed to export object
     * @return as unidades de disseminacao conhecidas em um canal
     */    
    public DBase[] getNeighbours(ChannelName channelName) throws RemoteException;
    
    /**
     * Entra em um canal e assume um disseminador como filho nesse canal.
     * @param desiredEntryPointsNumber o numero desejado de pais no canal
     * @param disseminator o disseminador que sera filho no canal
     * @param channelName o nome do canal
     * @throws CantFindChannelException se nao foi possivel encontrar o canal
     * @throws CantFindPeerException se nao foi possivel encontrar o ponto de entrada no canal
     * @throws CantEnterChannelException se nao foi possivel entrar no canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public ChannelData enterChannel(ChannelName channelName, int desiredEntryPointsNumber,Disseminator disseminator) throws RemoteException,CantFindChannelException,CantFindDBaseException,CantEnterChannelException;

    /**
     * Torna-se filho da unidade de disseminacao em um canal.
     * @param disseminator o disseminador a tornar-se filho
     * @param channelName o nome do canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     * @throws CantEnterChannelException se nao foi possivel entrar no canal
     */    
    public void connect(DBase dbase,ChannelName channelName) throws RemoteException,CantEnterChannelException;

    /**
     * Obtem um nome unico para um dnode desse dbase.
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     * @return um nome unico para um dnode desse dbase
     */    
    public DNodeName getDNodeName() throws RemoteException;
    
    /**
     * Obtem a lista parcial de ascendencia em um canal.
     * @param channelName o canal desejado
     * @throws RemoteException if failed to export object
     * @return a lista parcial de ascendencia no canal especificado
     */    
    public DBaseName[] getPartialAscendenceList(ChannelName channelName) throws RemoteException;

}
