package dimi;

import java.util.Set;
import java.rmi.RemoteException;

import dimi.configuration.DisseminationUnitConfiguration;
import dimi.message.buffer.MessageBuffer;
import dimi.name.DisseminationUnitName;
import dimi.exception.CantFindChannelException;
import dimi.message.Message;
import dimi.configuration.ChannelConfiguration;
import dimi.message.buffer.LastDropDisseminationUnitMessageBuffer;
import dimi.channel.DisseminationUnitChannel;
import dimi.name.ChannelName;
import dimi.channel.ChannelData;
import dimi.channel.Channel;

/**
 * Implementacao da interface Disseminator.
 * @author  mcmoraes
 */
public abstract class DisseminationUnitImpl extends DimiComponentImpl implements DisseminationUnit{
    
    //atributos - INICIO
    private DisseminationUnitConfiguration configuration = null;
    private MessageBuffer messageBuffer = null;
    private ChannelManager channelManager = null;
    //atributos - FIM
    
    //construtores - INICIO
    /**
     * Creates a new instance of Disseminator
     * @param disseminationUnitConfiguration a configuracao da unidade de disseminacao
     * @param name o nome da unidade de disseminacao
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    protected DisseminationUnitImpl(DisseminationUnitName name, DisseminationUnitConfiguration disseminationUnitConfiguration) throws RemoteException{
        super(name);
        this.configuration = disseminationUnitConfiguration;
        this.messageBuffer = new LastDropDisseminationUnitMessageBuffer(this,-1);
    }
    //construtores - FIM

    //getters e setters - INICIO
    /**
     * Getter for property messageBuffer.
     * @return Value of property messageBuffer.
     */
    public final MessageBuffer getMessageBuffer() {
        return messageBuffer;
    }
    
    /**
     * Setter for property messageBuffer.
     * @param messageBuffer
     */
    protected final void setMessageBuffer(MessageBuffer messageBuffer) {
        this.messageBuffer = messageBuffer;
    }

    /**
     * Getter for property channelManager.
     * @return Value of property channelManager.
     */
    public final ChannelManager getChannelManager() {
        return channelManager;
    }
    
    /**
     * Setter for property channelManager.
     * @param channelManager New value of property channelManager.
     */
    protected final void setChannelManager(ChannelManager channelManager) {
        this.channelManager = channelManager;
    }
    //getters e setters - FIM
    
    //interface Disseminator - INICIO
    /**
     * Dissemina uma mensagem em um canal.
     * @param message a mensagem a ser disseminada
     * @throws CantFindChannelException se o canal nao for encontrado
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */   
    public final void disseminate(Message message) throws RemoteException,CantFindChannelException{
System.out.println("Message Arrived: " + message.getHeader().toString());
        DisseminationUnitChannel channel = channelManager.getChannel(message.getHeader().getChannelName());
        if(channel!=null){
            channel.disseminate(message);
        }
        else{
            throw new CantFindChannelException();
        }
    }
    
    //interface Disseminator - FIM
    
    //interface DisseminationUnit - INICIO
    /**
     * Obtem a configuracao da unidade de disseminacao
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     * @return a configuracao da unidade de disseminacao
     */
    public final DisseminationUnitConfiguration getConfiguration() throws RemoteException{
        return configuration;
    }

    /**
     *
     * @param channelName
     * @throws RemoteException
     * @return
     */    
    public ChannelData getChannelData(ChannelName channelName) throws RemoteException{
        DisseminationUnitChannel disseminationUnitChannel = getChannelManager().getChannel(channelName);
        if(disseminationUnitChannel != null){
            return new ChannelData(disseminationUnitChannel.getNames()[Channel.ROOT_NAME],disseminationUnitChannel.getConfiguration());
        }
        return null;
    }

    /**
     * Obtem a configuracao de um canal.
     * @param channelName o nome do canal
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     * @return a configuracao do canal
     */
    public final ChannelConfiguration getChannelConfiguration(ChannelName channelName) throws RemoteException{
        DisseminationUnitChannel channel = channelManager.getChannel(channelName);
        if(channel != null){
            return channel.getConfiguration();
        }
        return null;
    }
    
    /**
     * Termina o servico, liberando todas as estruturas envolvidas.
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public abstract void destroy() throws RemoteException;
   
    //interface DisseminationUnit - FIM
}