package dimi;

import dimi.channel.EntryPointChannel;
import dimi.name.DimiName;

import java.rmi.RemoteException;

/**
 * Representa um ponto de entrada de um usuario (produtor ou consumidor) no servico.
 * @author mcmoraes
 */
public abstract class UserEntryPointImpl extends DimiComponentImpl implements UserEntryPoint{
    
    //atributos - INICIO
    /**
     * Indica se o ponto de entrada do usuario e valido.
     */    
    private boolean active;
    /**
     * O canal do ponto de entrada do usuario.
     */    
    private EntryPointChannel entryPointChannel = null;
    //atributos - FIM
    
    //construtores - INICIO
    /**
     * Creates a new instance of ConsumerEntryPointImpl
     * @param name o nome do ponto de entrada do usuario
     * @throws RemoteException if failed to export object
     */
    protected UserEntryPointImpl(DimiName name) throws RemoteException{
        super(name);
        this.active = false;
    }
    //construtores - FIM
    
    //setters e getters - INICIO
    /**
     * Getter for property entryPointChannel.
     * @return Value of property entryPointChannel.
     */
    public final EntryPointChannel getEntryPointChannel() {
        return entryPointChannel;
    }
    
    /**
     * Setter for property entryPointChannel.
     * @param entryPointChannel New value of property entryPointChannel.
     */
    protected final void setEntryPointChannel(EntryPointChannel entryPointChannel) {
        this.entryPointChannel = entryPointChannel;
    }
    
    /**
     * Getter for property active.
     * @return Value of property active.
     */
    public final boolean isActive() {
        return active;
    }
    
    /**
     * Setter for property active.
     * @param active New value of property active.
     */
    protected final void setActive(boolean active) {
        this.active = active;
    }
    //setters e getters - FIM
    
    //interface UserEntryPoint - BEGIN
    /**
     * Inutiliza este ponto de entrada. Se for de um produtor, destroi o canal. Se
     * for de um consumidor termina o consumo.
     * @throws RemoteException if failed to export object
     */    
    public abstract void destroy() throws RemoteException;
    //interface UserEntryPoint - FIM
}
