package dimi.channel.connection;

import java.util.Date;
import java.net.URL;
import java.rmi.RemoteException;

import dimi.Disseminator;
import dimi.message.Message;
import dimi.message.MessageHeader;
import dimi.exception.CantFindChannelException;
import dimi.name.DimiName;

/**
 * Representa uma ligacao entre dois disseminadores em um canal.
 * @author mcmoraes
 */
public abstract class Connection {

    private Date timeOfLastRealMessageReceived = null;
    private long numberOfLastRealMessageReceived;
    private boolean processFilter;
    private boolean input;
    private Disseminator disseminator = null;
    private DimiName disseminatorName = null;
    private boolean active = true;
    
    /**
     * Creates a new instance of Connection
     * @param input se a conexao e de entrada ou de saida
     * @param disseminator o disseminador com o qual sera feita a conexao
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */
    public Connection(boolean input,Disseminator disseminator) throws RemoteException{
        this.input = input;
        this.disseminator = disseminator;
        this.disseminatorName = disseminator.getName();
    }

    /**
     * Getter for property input.
     * @return Value of property input.
     */
    public boolean isInput() {
        return input;
    }    

    /**
     * Getter for property disseminator.
     * @return Value of property disseminator.
     */
    public Disseminator getDisseminator() {
        return disseminator;
    }
        
    /**
     * Getter for property timeOfLastRealMessageReceived.
     * @return Value of property timeOfLastRealMessageReceived.
     */
    public java.util.Date getTimeOfLastRealMessageReceived() {
        return timeOfLastRealMessageReceived;
    }
    
    /**
     * Setter for property timeOfLastRealMessageReceived.
     * @param timeOfLastRealMessageReceived New value of property timeOfLastRealMessageReceived.
     */
    public void setTimeOfLastRealMessageReceived(java.util.Date timeOfLastRealMessageReceived) {
        this.timeOfLastRealMessageReceived = timeOfLastRealMessageReceived;
    }
    
    /**
     * Getter for property processFilter.
     * @return Value of property processFilter.
     */
    public boolean isProcessFilter() {
        return processFilter;
    }
    
    /**
     * Setter for property processFilter.
     * @param processFilter New value of property processFilter.
     */
    public void setProcessFilter(boolean processFilter) {
        this.processFilter = processFilter;
    }
        
    /**
     * Monta o cabecalho da mensagem de saida do canal
     * @return o cabecalho da mensagem de saida do canal
     */    
    protected abstract MessageHeader mountLeaveMessageHeader();
    
    /**
     * Destroi a conexao
     * @throws RemoteException indicate that either a communication failure or a protocol error has occurred
     */    
    public final void destroy() throws RemoteException{
        MessageHeader messageHeader = mountLeaveMessageHeader();
        messageHeader.setOriginDateTime(new Date());
        messageHeader.setProductionDateTime(new Date());
        messageHeader.setType(MessageHeader.LEAVE);
        Message message = new Message(messageHeader);
        try{
            getDisseminator().disseminate(message);
        }
        catch(CantFindChannelException re){re.printStackTrace();}
        active = false;
    }
    
    /**
     * Getter for property numberOfLastRealMessageReceived.
     * @return Value of property numberOfLastRealMessageReceived.
     */
    public long getNumberOfLastRealMessageReceived() {
        return numberOfLastRealMessageReceived;
    }
    
    /**
     * Setter for property numberOfLastRealMessageReceived.
     * @param numberOfLastRealMessageReceived New value of property numberOfLastRealMessageReceived.
     */
    public void setNumberOfLastRealMessageReceived(long numberOfLastRealMessageReceived) {
        this.numberOfLastRealMessageReceived = numberOfLastRealMessageReceived;
    }
    
    /**
     * Obtem a representacao em String desta conexao.
     * @return a representacao em String desta conexao
     */    
    public final String toString(){
        return disseminatorName.toString();
    }
    
    /**
     * Getter for property disseminatorName.
     * @return Value of property disseminatorName.
     */
    public dimi.name.DimiName getDisseminatorName() {
        return disseminatorName;
    }
    
    /**
     * Getter for property active.
     * @return Value of property active.
     */
    public boolean isActive() {
        return active;
    }
    
    /**
     * Setter for property active.
     * @param active New value of property active.
     */
    public void setActive(boolean active) {
        this.active = active;
    }
    
}
