package dimi.configuration;

import java.net.URL;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import java.io.IOException;

import org.apache.commons.jxpath.JXPathContext;

import java.io.Serializable;

import dimi.name.ChannelName;

/**
 * Configuracao de um canal.
 * @author mcmoraes
 */
public class ChannelConfiguration implements Serializable{

    private String filter = null;
    private String description = null;
    private int priority;
    private long maximumSilentInterval;
    private long heartBeatPeriod;

    /**
     * Cria uma nova instancia de ChannelConfiguration.
     */    
    public ChannelConfiguration(){
    }
    
    /**
     * Cria uma nova instancia de ChannelConfiguration.
     * @param xmlFileName o nome do arquivo
     * @throws ParserConfigurationException if a DocumentBuilder cannot be created which satisfies the configuration requested.
     * @throws SAXException If any parse errors occur
     * @throws IOException If any IO errors occur
     */    
    public ChannelConfiguration(String xmlFileName) throws ParserConfigurationException,
                                                                SAXException,
                                                                IOException{
                                                                    
        
        loadFromXMLFile(xmlFileName);
    }

    /**
     * Getter for property description.
     * @return Value of property description.
     */
    public java.lang.String getDescription() {
        return description;
    }
    
    /**
     * Setter for property description.
     * @param description New value of property description.
     */
    public void setDescription(java.lang.String description) {
        this.description = description;
    }
    
    /**
     * Getter for property filter.
     * @return Value of property filter.
     */
    public java.lang.String getFilter() {
        return filter;
    }
    
    /**
     * Setter for property filter.
     * @param filter New value of property filter.
     */
    public void setFilter(java.lang.String filter) {
        this.filter = filter;
    }
    
    /**
     * Getter for property priority.
     * @return Value of property priority.
     */
    public int getPriority() {
        return priority;
    }
    
    /**
     * Setter for property priority.
     * @param priority New value of property priority.
     */
    public void setPriority(int priority) {
        this.priority = priority;
    }
    
    /**
     * Ajusta os valores da configuracao a partir de um arquivo XML.
     * @param xmlFileName o nome do arquivo
     * @throws ParserConfigurationException if a DocumentBuilder cannot be created which satisfies the configuration requested.
     * @throws SAXException If any parse errors occur
     * @throws IOException If any IO errors occur
     */    
    public void loadFromXMLFile(String xmlFileName) throws ParserConfigurationException,
                                                    SAXException,
                                                    IOException{

        
        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(xmlFileName);
        Node node = doc.getDocumentElement();
        
        JXPathContext context = JXPathContext.newContext(node);

        String filter = (String)context.getValue("filter");
        this.filter = filter;
        
        String description = (String)context.getValue("description");
        this.description = description;
        
        String priority = (String)context.getValue("priority");
        this.priority = Integer.valueOf(priority).intValue();

        String maximumSilentInterval = (String)context.getValue("maximum-silent-interval");
        this.maximumSilentInterval = Integer.valueOf(maximumSilentInterval).intValue();

        String heartBeatPeriod = (String)context.getValue("heart-beat-period");
        this.heartBeatPeriod = Integer.valueOf(heartBeatPeriod).intValue();

    }
    
    /**
     * Getter for property maximumSilentInterval.
     * @return Value of property maximumSilentInterval.
     */
    public long getMaximumSilentInterval() {
        return maximumSilentInterval;
    }
    
    /**
     * Setter for property maximumSilentInterval.
     * @param maximumSilentInterval New value of property maximumSilentInterval.
     */
    public void setMaximumSilentInterval(long maximumSilentInterval) {
        this.maximumSilentInterval = maximumSilentInterval;
    }
    
    /**
     * Representacao em String dessa configuracao
     * @return representacao em String dessa configuracao
     */    
    public String toString(){
        StringBuffer result = new StringBuffer();
        result.append("channel configurations::").append(" ");
        result.append("filter:").append(" ").append(filter).append(";").append(" ");
        result.append("description:").append(" ").append(description).append(";").append(" ");
        result.append("priority:").append(" ").append(priority).append(";").append(" ");
        result.append("heart-beat period:").append(" ").append(heartBeatPeriod).append(";").append(" ");
        result.append("maximum silent interval:").append(" ").append(maximumSilentInterval).append(";").append(" ");
        return result.toString();
    }

    /**
     * Getter for property heartBeatPeriod.
     * @return Value of property heartBeatPeriod.
     */
    public long getHeartBeatPeriod() {
        return heartBeatPeriod;
    }
    
    /**
     * Setter for property heartBeatPeriod.
     * @param heartBeatPeriod New value of property heartBeatPeriod.
     */
    public void setHeartBeatPeriod(long heartBeatPeriod) {
        this.heartBeatPeriod = heartBeatPeriod;
    }
        
}
