package dimi.configuration;

import java.util.List;
import java.util.Vector;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import java.io.IOException;

import org.apache.commons.jxpath.JXPathContext;

import java.io.Serializable;

/**
 * Configuracao de um disseminador.
 * @author  mcmoraes
 */
public class DisseminationUnitConfiguration implements Serializable {
    
    private List filterList = null;
    
    /**
     * Cria uma nova instancia de DisseminatorConfiguration.
     */    
    public DisseminationUnitConfiguration(){}
    
    /**
     * Cria uma nova instancia de DisseminatorConfiguration.
     * @param xmlFileName o nome do arquivo
     * @throws ParserConfigurationException if a DocumentBuilder cannot be created which satisfies the configuration requested.
     * @throws SAXException If any parse errors occur
     * @throws IOException If any IO errors occur
     */    
    public DisseminationUnitConfiguration(String xmlFileName) throws ParserConfigurationException, SAXException, IOException{
        
        loadFromXMLFile(xmlFileName);
    }

    /**
     * Getter for property filterList.
     * @return Value of property filterList.
     */
    public java.util.List getFilterList() {
        return filterList;
    }
    
    /**
     * Setter for property filterList.
     * @param filterList New value of property filterList.
     */
    public void setFilterList(java.util.List filterList) {
        this.filterList = filterList;
    }
    
    /**
     * Ajusta os valores da configuracao a partir de um arquivo XML.
     * @param xmlFileName o nome do arquivo
     * @throws ParserConfigurationException if a DocumentBuilder cannot be created which satisfies the configuration requested.
     * @throws SAXException If any parse errors occur
     * @throws IOException If any IO errors occur
     */    
    public void loadFromXMLFile(String xmlFileName) throws ParserConfigurationException,
                                                    SAXException,
                                                    IOException{
        
                
        Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(xmlFileName);
        Node node = doc.getDocumentElement();
        
        JXPathContext context = JXPathContext.newContext(node);
        
        filterList = new Vector();
        Iterator filters = context.iterate("filter-list/filter");
        while(filters.hasNext()){
            String filter = (String)filters.next();
            filterList.add(filter);
        }

    }
    
    /**
     * Representacao em String dessa configuracao
     * @return representacao em String dessa configuracao
     */    
    public String toString(){
        StringBuffer result = new StringBuffer();
        result.append("disseminator configurations::").append(" ");
        result.append("filters:").append(" ");
        Iterator it = filterList.iterator();
        while(it.hasNext()){
            result.append(it.next()).append(",");
        }
        return result.toString();
    }
    
}
