/*
 * ResourceMatcher.java
 *
 * Created on 16 de Julho de 2004, 09:24
 */

package org.isam.exehda.services.perdis;

import org.isam.exehda.Exehda;
import org.isam.exehda.ResourceName;
import org.isam.exehda.ResourceName.NameSpace;
import org.isam.exehda.services.Logger.Sev;
import org.isam.exehda.services.Logger;
import org.isam.exehda.services.DiscovererService;
import org.isam.exehda.services.CellInformationBase;
import org.isam.util.xml.XmlElement;

import java.util.Vector;
import java.util.Enumeration;


/**
 *
 * @author  guilherme
 */
public class ResourceMatcher {
    
    private CellInformationBase cib;
    private NameSpace nameSpace;
    private Logger logger;

    /**
     * Creates a new instance of ResourceMatcher 
     */
    public ResourceMatcher(CellInformationBase cib, NameSpace nameSpace) {
        
        this.cib = cib; 
        this.nameSpace = nameSpace;
        this.logger = ((Logger) Exehda.getService(Logger.SERVICE_NAME));
    }
    
    /**
     * match
     */
    public XmlElement match(XmlElement xmlResearch) throws java.io.IOException{
        
        boolean result = true;
        Vector vecResults = new Vector();
        
        Vector vec = XmlElement.selectByPath(xmlResearch, "/QUERY/TYPE", null);
        XmlElement xmlType = (XmlElement) vec.firstElement();
        String typeTmp = (String) xmlType.getContents();
        
        ResourceName [] rn = this.cib.selectByType(typeTmp, this.nameSpace, -1);
        this.logger.log(Logger.Sev.DEBUG, "ResourceMatcher", "match", "########findByType achou " + rn.length + " recurso(s)###############", null);
        
        for (int i = 0; i < rn.length; i++){
            result = true;
            vec = XmlElement.selectByPath(xmlResearch, "/QUERY/CRITERIA", null);
            for (int j = 0; j < vec.size(); j++){
                if (!matchCriteria((XmlElement)vec.elementAt(j), rn[i])){
                    result = false;
                    break;
                }
            }
            
            if (result) {
                vec = XmlElement.selectByPath(xmlResearch, "/QUERY/BRANCHES", null);            
                if (vec.size() > 0){
                    if (!matchBranches((XmlElement)vec.firstElement(), rn[i])){
                        result = false;
                    }
                }
            }

            if (result) {
                vecResults.add(rn[i]);
                this.logger.log(Logger.Sev.DEBUG, "ResourceMatcher", "match", "######### Recurso adicionado = " + rn[i].getDN() + "\n", null);
            }
        }

        this.logger.log(Logger.Sev.DEBUG, "ResourceMatcher", "match", "####### Total = " + vecResults.size() + " recurso(s) selecionados\n", null);
        return buildQueryResult(vecResults);
    }
       
    /**
     * matchCriteria
     */
    private boolean matchCriteria(XmlElement xmlCriteria, ResourceName resource){

        String nameCrit = xmlCriteria.getAttribute("name", null);
        String valCib = this.cib.getAttribute(resource, nameCrit);
        
        String valCrit = xmlCriteria.getAttribute("value", null);
        String opCrit = xmlCriteria.getAttribute("op", null);
        
        if (valCib == null) {
            return false;
        }

        if (opCrit.equalsIgnoreCase("equ")){
          if (valCrit.equalsIgnoreCase(valCib)){
            return true;
          }
          else return false;
        }

        else if (opCrit.equalsIgnoreCase("neq")){
          if (valCrit.equalsIgnoreCase(valCib)){
            return false;
          }
          else return true;
        }

        else if (opCrit.equalsIgnoreCase("gre")){
          if (Integer.parseInt(valCrit) < Integer.parseInt(valCib)){
            return true;
          }
          else return false;
        }

        else if (opCrit.equalsIgnoreCase("geq")){
          if (Integer.parseInt(valCrit) <= Integer.parseInt(valCib)){
            return true;
          }
          else return false;
        }

        else if (opCrit.equalsIgnoreCase("les")){
          if (Integer.parseInt(valCrit) > Integer.parseInt(valCib)){
            return true;
          }
          else return false;
        }

        else if (opCrit.equalsIgnoreCase("leq")){
          if (Integer.parseInt(valCrit) >= Integer.parseInt(valCib)){
            return true;
          }
          else return false;
        }

        else if (opCrit.equalsIgnoreCase("mul")){
            Vector values = getValues(valCrit);
            for (int i = 0; i < values.size(); i++){
                if (((String)values.elementAt(i)).equalsIgnoreCase(valCib)){
                    return true;
                }
            }
            return false;
        }

        else if (opCrit.equalsIgnoreCase("ran")){
            Vector values = getValues(valCrit);
            if ((Integer.parseInt(valCib)>=Integer.parseInt(values.firstElement().toString())) && ((Integer.parseInt(valCib)<=Integer.parseInt(values.lastElement().toString())))){
                return true;
            }
            else return false;
        }
        return true;
    }
    
    /**
     * matchBranches
     */
    private boolean matchBranches(XmlElement xmlBranches, ResourceName resource){

        Vector vec = XmlElement.selectByPath(xmlBranches, "/BRANCH", null);
        for (int i = 0; i<vec.size(); i++){
            if (matchBranch((XmlElement)vec.elementAt(i),resource)){
                return true;
            }
        }
        return false;
    }
    
    /**
     * matchBranch
     */
    private boolean matchBranch(XmlElement xmlBranch, ResourceName resource){

        boolean result = true;
        Vector vecBranch = XmlElement.selectByPath(xmlBranch, "/CRITERIA", null);       

        for (int j=0;j<vecBranch.size();j++){
            if (!matchCriteria((XmlElement)vecBranch.elementAt(j), resource)) {
                result = false;
                break;
            }
        }

        if (result) {
            vecBranch = XmlElement.selectByPath(xmlBranch, "/BRANCHES", null);                   
            for (int j=0;j<vecBranch.size();j++){
                result = result && matchBranches((XmlElement)vecBranch.elementAt(j), resource);
            }
        }

        return result;        
    }

    /**
     * getValues
     */
    private static Vector getValues(String values){

        String auxResult="";
        Vector result=new Vector();
        for (int i=0; i<values.length();i++){
            if (values.charAt(i)==';'){
                result.add(auxResult);
                auxResult="";
            }
            else {
                auxResult+=values.charAt(i);
            }
        }
        result.add(auxResult);
        return result;
    }
    
    /**
     * buildQueryResult
     */
    private XmlElement buildQueryResult(Vector vecResults) {

        XmlElement xmlQueryResult = XmlElement.createDocRoot();
        xmlQueryResult = XmlElement.createElement("QUERYRESULT");  
        
        for (int j = 0; j < vecResults.size(); j++) {

            String attributes[] = this.cib.getAttributes((ResourceName)vecResults.elementAt(j));

            XmlElement xmlResource = XmlElement.createElement("RESOURCE");
            XmlElement xmlProperties = XmlElement.createElement("PROPERTIES");

            for (int i = 0; i < attributes.length; i++) {
                
                if (attributes[i].equalsIgnoreCase("TYPE")){
                    String attribType = this.cib.getAttribute((ResourceName)vecResults.elementAt(j),attributes[i]);
                    XmlElement xmlType = XmlElement.createElement("TYPE");
                    XmlElement xmlTypeData = XmlElement.createPCData(attribType);
                    xmlType.addChild(xmlTypeData);
                    xmlResource.addChild(xmlType);

                } else if (attributes[i].equalsIgnoreCase("NAME")){
                    String attribName = this.cib.getAttribute((ResourceName)vecResults.elementAt(j),attributes[i]);
                    XmlElement xmlName = XmlElement.createElement("NAME");
                    XmlElement xmlNameData = XmlElement.createPCData(attribName);
                    xmlName.addChild(xmlNameData);
                    xmlResource.addChild(xmlName);

                } else if (attributes[i].equalsIgnoreCase("DESCRIPTION")){
                    String attribDescription = this.cib.getAttribute((ResourceName)vecResults.elementAt(j),attributes[i]);
                    XmlElement xmlDescription = XmlElement.createElement("DESCRIPTION");
                    XmlElement xmlDescriptionData = XmlElement.createPCData(attribDescription);
                    xmlDescription.addChild(xmlDescriptionData);
                    xmlResource.addChild(xmlDescription);

                } else if (attributes[i].equalsIgnoreCase("URL")){
                    String attribURL = this.cib.getAttribute((ResourceName)vecResults.elementAt(j),attributes[i]);
                    XmlElement xmlURL = XmlElement.createElement("URL");
                    XmlElement xmlURLData = XmlElement.createPCData(attribURL);
                    xmlURL.addChild(xmlURLData);
                    xmlResource.addChild(xmlURL);
                }
            }

            for (int i = 0; i< attributes.length;i ++) {
                if ((!(attributes[i].equalsIgnoreCase("TYPE"))) && (!(attributes[i].equalsIgnoreCase("NAME"))) && (!(attributes[i].equalsIgnoreCase("DESCRIPTION"))) && (!(attributes[i].equalsIgnoreCase("URL")))) {
                    XmlElement xmlProperty = XmlElement.createElement("PROPERTY");
                    String propertyAux = this.cib.getAttribute((ResourceName)vecResults.elementAt(j),attributes[i]);
                    xmlProperty.setAttribute("name", attributes[i]);
                    xmlProperty.setAttribute("value", propertyAux);
                    xmlProperties.addChild(xmlProperty);
                }
            }
            xmlResource.addChild(xmlProperties);
            xmlQueryResult.addChild(xmlResource);
        }
        return xmlQueryResult;
    }
    
    /**
     * narrowResults
     */
    public XmlElement narrowResults(XmlElement xmlResults, int maxResults) {
    
        Vector vecSrc = XmlElement.selectByPath(xmlResults,"/RESOURCE",null);
        
        if ((vecSrc.size() <= maxResults) || (maxResults == 0)) {
            return xmlResults;
        }
        
        //Build the new QueryResult
        XmlElement newXmlResults = XmlElement.createDocRoot();
        newXmlResults = XmlElement.createElement("QUERYRESULT");
        
        for (int i = 0; i < maxResults; i++) {
            newXmlResults.addChild((XmlElement) vecSrc.elementAt(i));
        }
        
        return newXmlResults;
    }
}
