// Copyright 2003 ISAM Team <http://www.inf.ufrgs.br/~isam/>
//
// This file is part of ISAM.
//
// ISAM is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// ISAM is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with ISAM; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

package org.isam.exehda.services.primos.exec;

import org.isam.exehda.HostId;
import org.isam.exehda.ObjectId;
import org.isam.exehda.services.Executor;
import org.isam.exehda.services.Executor.SchedulingHeuristic;
import java.util.Vector;

/**
 * This scheduling heuristic supports explicit selection of target hosts for both remote
 * instantiation and migration through the <code>hint</code> parameter.
 *
 * If hint is not provided (i.e. it is null) or it is not of type HostId, this heuristic
 * falls back to the semantic defined in <code>LocalHostSchedHeuristic</code>.
 *
 * @see LocalHostSchedHeuristic
 *
 * @author last modified by $Author: lucc $
 * @version $Date: 2005-03-30 12:52:08 -0300 (Wed, 30 Mar 2005) $ $Revision: 666 $
 */
class HostIdSchedHeuristic
    extends LocalHostSchedHeuristic
{
    public HostId
    chooseCreationHost(String clsName,
                       Object[] params,
                       Object hint,
                       Vector avoidedHosts)
        {
            if ( hint != null && hint instanceof HostId ) {
                return (HostId) hint;
            }
            else {
                return super.chooseCreationHost(clsName,
                                                params,
                                                hint,
                                                avoidedHosts);
            }
        }

    public HostId
    chooseMigrationHost(ObjectId ox,
                        Object hint,
                        java.util.Vector avoidedHosts)
        {
            if ( hint != null && hint instanceof HostId ) {
                return (HostId) hint;
            }
            else {
                return super.chooseMigrationHost(
                    ox, hint, avoidedHosts);
            }
        }
}

