
package exehda.mw;

import java.util.Random;
import java.net.InetAddress;
import org.isam.exehda.services.Worb;
import org.isam.exehda.Exehda;

/**
 * 
 * 
 */
public class JobWorker
    implements Runnable, java.io.Serializable

{
    private static      long id = 0;
    private static      int alreadyExistsOne = 0; 
    private JobMaster   jobMaster;
    private JobCriteria criteria;
    private String name;

        /** 
         * Constructor
         * 
         * @param masterContactAddress contact address for the master object
         * @param jobCriteria criteria to be used in job size dynamic adaptation
         * 
         * @return 
         */
    public 
    JobWorker(String masterContactAddress, String jobCriteria)
        {
           
            Worb worb = (Worb) Exehda.getService(Worb.SERVICE_NAME);

	    

            this.jobMaster = (JobMaster)
                worb.lookupService(masterContactAddress, JobMaster.class);
            
            try{
                this.criteria = (TimeGoalCriteria) Class.forName(jobCriteria).newInstance();
            }catch(Exception e){
                e.printStackTrace();
                this.criteria = new FixedJobCriteria();
            }
		
            try {
                InetAddress localhost = InetAddress.getLocalHost();
                name = localhost.toString() + "#" + id++;
                    //this.setName(localhost.toString() + "#" + id++);
            }
            catch (java.net.UnknownHostException uhe){
                    //this.setName("Unknown Host ("+System.getProperty("os.name")+")" + "#" + id++);
            }
        }

        //public static JobWorker
        //getCurrentWorker()
        //{
       
        //return this;
        //}
    
    
        /** 
         * Worker entry point
         * 
         */
    public void
    run()
        {
            System.out.println("Worker <"+name+"> running...");
            try {
                Object jobParams = null;
                long elapsed_cpu = 0;
                long elapsed_init= 0;
                long elapsed_net = 0;
                
                    //if there isn't another jobmaster in this node
                if(alreadyExistsOne == 0){
                    alreadyExistsOne = 1;
                        //communicate the creation sucess to the master
                    this.jobMaster.getJob(new WorkerCreationStatus(true));
                }
                else{
                        //if there already is another JobWworker in this node then tell the master and die
                    this.jobMaster.getJob(new WorkerCreationStatus(false));
                    return;
                }

            
                while (true) {
                
                        /*
                         * Get a Job from the server
                         */
                    elapsed_net = -System.currentTimeMillis();
                    Job job = jobMaster.getJob(jobParams);
                    elapsed_net += System.currentTimeMillis();
                
                    if (job == null) {
                        System.out.println("Worker: Master.getJob() returned null");
                        alreadyExistsOne = 0;
                        break;
                    }

                        /*
                         * Let the job to set up the environtment without affecting
                         * the mesuared computing time.
                         */
                    elapsed_init = -System.currentTimeMillis();
                    job.init();
                    elapsed_init += System.currentTimeMillis();
                
                        /*
                         * While there are pending iterations, executes the Job
                         */
                    while ( job.hasNext() ) {
                        elapsed_cpu = -System.currentTimeMillis();
                        job.run();
                        elapsed_cpu += System.currentTimeMillis();
                        job.next();
                    }
                    job.setElapsedTime(elapsed_cpu);
                    job.setWorkerName(name);                
                
                        /*
                         * Publish the results;
                         */
                    elapsed_net -= System.currentTimeMillis();
                    jobMaster.publishResult(job);
                    elapsed_net += System.currentTimeMillis();

                        /*
                         * Guess the best job size for the next job
                         */
                    System.out.println("Elapsed cpu =" + elapsed_cpu);
                    
                    jobParams = criteria.guessNewJobSize(job, elapsed_cpu, job.getTimeGoal());
                }
                
            }
            catch( java.rmi.ConnectException ce ) {
                System.out.println(ce);
                ce.printStackTrace();
            }
            catch(Exception e) {
            
                e.printStackTrace();
            }
            System.out.println("Slave done.");
        }
}
